pm_idt:		equ 0x100000
pm_entry:	equ 0x101000

bits 16
align 2

com32_pmidt:
		dw 8*256		
		dd pm_idt		

com32_rmidt:
		dw 0ffffh		
		dd 0			

is_com32_image:
		call highmemsize	
		call comboot_setup_api	
		mov edi,pm_entry	
		xchg eax,edx		
		shl eax,16
		mov ax,dx
		call load_high
		call crlf

com32_start:
		mov ebx,com32_call_start	

com32_enter_pm:
		cli
		mov [SavedSSSP],sp
		mov [SavedSSSP+2],ss
		cld
		call a20_test
		jnz .a20ok
		call enable_a20

.a20ok:
		lgdt [bcopy_gdt]	
		lidt [com32_pmidt]	
		mov eax,cr0
		or al,1
		mov cr0,eax		
		jmp 20h:.in_pm
		bits 32

.in_pm:
		xor eax,eax		
		mov fs,eax
		mov gs,eax
		mov al,28h		
		mov es,eax
		mov ds,eax
		mov ss,eax
		mov esp,[PMESP]		
		jmp ebx			

com32_call_start:
		mov esp,[word HighMemSize]
		mov eax,0x00200000+((pm_idt+8*256)&0x0000ffff)
		mov ebx,0x0000ee00+((pm_idt+8*256)&0xffff0000)
		xor ecx,ecx
		inc ch				
		push ecx

.make_idt:
		stosd
		add eax,8
		xchg eax,ebx
		stosd
		xchg eax,ebx
		loop .make_idt
		pop ecx
		mov eax,0e900b060h
		mov ebx,com32_handle_interrupt-(pm_idt+8*256+8)

.make_ijb:
		stosd
		sub [edi-2],cl			
		xchg eax,ebx
		stosd
		sub eax,8
		xchg eax,ebx
		loop .make_ijb
		push dword com32_farcall	
		push dword (1 << 16)		
		push dword (comboot_seg << 4)	
		push dword com32_intcall	
		movzx esi,word [word CmdOptPtr]
		push esi			
		push dword 5			
		sti				
		call pm_entry			

com32_exit:
		mov bx,com32_done	

com32_enter_rm:
		cli
		cld
		mov [PMESP],esp		
		xor esp,esp		
		jmp 08h:.in_pm16	
		bits 16

.in_pm16:
		mov ax,18h		
		mov es,ax
		mov ds,ax
		mov ss,ax
		mov fs,ax
		mov gs,ax
		lidt [com32_rmidt]	
		mov eax,cr0
		and al,~1
		mov cr0,eax
		jmp 0:.in_rm

.in_rm:					
		mov ax,cs		
		mov ds,ax
		mov es,ax
		mov fs,ax
		mov gs,ax
		lss sp,[SavedSSSP]	
		jmp bx			

com32_done:
		call disable_a20
		sti
		jmp enter_command

com32_int_rm:
		pushf				
		push cs				
		push word .cont			
		push dword edx			
		retf				

.cont:		
		mov ebx,com32_int_resume
		jmp com32_enter_pm		

com32_sys_rm:
		pop gs
		pop fs
		pop es
		pop ds
		popad
		popfd
		mov [cs:Com32SysSP],sp
		retf				

.return:
		mov sp,[cs:Com32SysSP]
		pushfd
		pushad
		push ds
		push es
		push fs
		push gs
		mov ebx,com32_sys_resume
		jmp com32_enter_pm
		bits 32

com32_handle_interrupt:
		movzx eax,al
		xor ebx,ebx		
		mov edx,[ebx+eax*4]	
		mov bx,com32_int_rm
		jmp com32_enter_rm	

com32_int_resume:
		popad
		iret

com32_farcall:
		pushfd				
		pushad				
		mov eax,[esp+10*4]		
		jmp com32_syscall

com32_intcall:
		pushfd				
		pushad				
		movzx eax,byte [esp+10*4]	
		mov eax,[eax*4]			

com32_syscall:
		cld
		movzx edi,word [word SavedSSSP]
		movzx ebx,word [word SavedSSSP+2]
		sub edi,54		
		mov [word SavedSSSP],di
		shl ebx,4
		add edi,ebx		
		mov esi,[esp+11*4]	
		xor ecx,ecx
		mov cl,11		
		rep movsd
		stosd			
		mov eax,com32_sys_rm.return	
		stosd			
		mov eax,[edi-12]	
		and eax,0x200cd7	
		mov [edi-12],eax	
		stosw			
		mov bx,com32_sys_rm
		jmp com32_enter_rm	

com32_sys_resume:
		movzx esi,word [word SavedSSSP]
		movzx eax,word [word SavedSSSP+2]
		mov edi,[esp+12*4]	
		shl eax,4
		add esi,eax		
		and edi,edi		
		jnz .do_copy

.no_copy:	
		mov edi,esi		

.do_copy:	
		xor ecx,ecx
		mov cl,11		
		rep movsd		
		add dword [word SavedSSSP],54	
		popad
		popfd
		ret
		bits 16



