%define		P_FLAGS		word [bp+44]
%define		P_FLAGSL	byte [bp+44]
%define		P_FLAGSH	byte [bp+45]
%define		P_CS		word [bp+42]
%define		P_IP		word [bp+40]
%define		P_DS		word [bp+38]
%define		P_ES		word [bp+36]
%define		P_FS		word [bp+34]
%define		P_GS		word [bp+32]
%define		P_EAX		dword [bp+28]
%define		P_AX		word [bp+28]
%define		P_HAX		word [bp+30]
%define		P_AL		byte [bp+28]
%define		P_AH		byte [bp+29]
%define		P_ECX		dword [bp+24]
%define		P_CX		word [bp+24]
%define		P_HCX		word [bp+26]
%define		P_CL		byte [bp+24]
%define		P_CH		byte [bp+25]
%define		P_EDX		dword [bp+20]
%define		P_DX		word [bp+20]
%define		P_HDX		word [bp+22]
%define		P_DL		byte [bp+20]
%define		P_DH		byte [bp+21]
%define		P_EBX		dword [bp+16]
%define		P_BX		word [bp+16]
%define		P_HBX		word [bp+18]
%define		P_BL		byte [bp+16]
%define		P_BH		byte [bp+17]
%define		P_EBP		dword [bp+8]
%define		P_BP		word [bp+8]
%define		P_HBP		word [bp+10]
%define		P_ESI		dword [bp+4]
%define		P_SI		word [bp+4]
%define		P_HSI		word [bp+6]
%define		P_EDI		dword [bp]
%define		P_DI		word [bp]
%define		P_HDI		word [bp+2]

comboot_too_large:
		mov si,err_comlarge
		call cwritestr
		jmp enter_command

is_comboot_image:
		and dx,dx
		jnz comboot_too_large
		cmp ax,0ff00h		
		jae comboot_too_large
		call comboot_setup_api
		mov cx,comboot_seg
		mov es,cx
		mov bx,100h		
		mov cx,[ClustPerMoby]	
		call getfssec
		xor di,di
		mov cx,64		
		xor eax,eax		
		rep stosd
		mov word [es:0], 020CDh	
		int 12h			
		shl ax,6		
		mov word [es:02h],ax

%ifndef DEPEND
%if real_mode_seg != comboot_seg
%error "This code assumes real_mode_seg == comboot_seg"
%endif
%endif
		mov cx,125		
		mov si,[CmdOptPtr]
		mov di,081h		
		mov al,' '		
		stosb

comboot_cmd_cp:	
		es lodsb
		and al,al
		jz comboot_end_cmd
		stosb
		loop comboot_cmd_cp

comboot_end_cmd: 
		mov al,0Dh		
		stosb
		mov al,126		
		sub al,cl
		mov [es:80h], al	
		mov [SavedSSSP],sp
		mov [SavedSSSP+2],ss	
		mov ax,es
		mov ds,ax
		mov ss,ax
		xor sp,sp
		push word 0		
		jmp comboot_seg:100h	

comboot_return:	
		cli			
		xor ax,ax
		jmp comboot_exit

comboot_setup_api:
		mov di,4*0x20		
		mov eax,comboot_return	
		stosd
		mov ax,comboot_int21	
		stosd
		mov ax,comboot_int22	
		stosd
		mov ax,comboot_bogus
		mov cx,29		
		rep stosd
		ret

comboot_int21:	
		cli
		push ds
		push es
		push fs
		push gs
		pushad
		cld
		mov bp,cs
		mov ds,bp
		mov es,bp
		mov bp,sp			
		mov cx,int21_count
		mov si,int21_table

.again:		
		lodsb
		cmp al,P_AH
		lodsw
		loopne .again
		call ax			

comboot_resume:
		setc P_FLAGSL		
		popad
		pop gs
		pop fs
		pop es
		pop ds
		iret

comboot_bogus:	
		cli			
		mov ax,err_notdos

comboot_exit:
		mov bx,enter_command	

comboot_exit_special:
		xor dx,dx
		mov ds,dx
		mov es,dx
		lss sp,[SavedSSSP]
		sti
		cld
		and ax,ax
		je .nomsg
		mov si,KernelCName
		call cwritestr
		xchg si,ax
		call cwritestr

.nomsg:		
		jmp bx

comboot_getkey:				
		call vgashowcursor
		call comboot_getchar
		call vgahidecursor
		call writechr
		clc
		ret

comboot_writechr:			
		mov al,P_DL
		call writechr
		clc
		ret

comboot_writeserial:			
		mov al,P_DL
		call write_serial
		clc
		ret

comboot_getkeynoecho:			
		call comboot_getchar
		clc
		ret

comboot_writestr:			
		mov es,P_DS
		mov si,P_DX

.loop:		
		es lodsb
		cmp al,'$'		
		je .done
		call writechr
		jmp short .loop

.done:		
		clc
		ret

comboot_checkkey:			
		cmp byte [APIKeyFlag],00h
		jnz .waiting
		call pollchar

.waiting:	
		setz al
		dec al			
		mov P_AL,al
		clc
		ret

comboot_checkver:			
		mov P_EAX,'SY' << 16
		mov P_EBX,'SL' << 16
		mov P_ECX,'IN' << 16
		mov P_EDX,'UX' << 16
		clc
		ret

comboot_getchar:
		cmp byte [APIKeyFlag],00h
		jne .queued
		call getchar		
		and al,al		
		jnz .done
		mov [APIKeyWait],ah	
		inc byte [APIKeyFlag]	

.done:		
		mov P_AL,al
		ret

.queued:	
		mov al,[APIKeyWait]
		dec byte [APIKeyFlag]
		jmp .done

comboot_int22:
		cli
		push ds
		push es
		push fs
		push gs
		pushad
		cld
		mov bp,cs
		mov ds,bp
		mov es,bp
		mov bp,sp			
		cmp ax,int22_count
		jb .ok
		xor ax,ax			

.ok:
		xchg ax,bx
		add bx,bx
		call [bx+int22_table]
		jmp comboot_resume		

comapi_err:
		stc
		ret

comapi_get_version:
		mov P_AX,int22_count
		mov P_CX,(VER_MAJOR << 8)+VER_MINOR
		mov P_DX,my_id
		mov P_BX,cs	
		mov P_ES,ds

comapi_nop:
		clc
		ret

comapi_writestr:
		mov ds,P_ES
		mov si,P_BX
		call writestr
		clc
		ret

comapi_run:
		mov ds,P_ES
		mov si,P_BX
		mov di,command_line

.copyloop:
		lodsb
		stosb
		and al,al
		jnz .copyloop
		xor ax,ax
		mov bx,load_kernel		
		jmp comboot_exit_special	

comapi_run_default:
		mov bx,auto_boot
		jmp comboot_exit_special

comapi_textmode:
		call vgaclearmode
		clc
		ret

comapi_open:
		push ds
		mov ds,P_ES
		mov si,P_SI
		mov di,InitRD
		push di
		call mangle_name
		pop di
		pop ds
		call searchdir
		jz .err
		mov P_AX,ax
		mov P_HAX,dx
		mov ax,[ClustSize]
		mov P_CX,ax
		clc
		ret

.err:
		stc
		ret

comapi_read:
		mov ax,P_BX
		mov si,P_SI
		call getfssec
		jnc .noteof
		xor si,si		

.noteof:	
		mov P_SI,si
		ret

comapi_close:
		clc
		ret

%if IS_PXELINUX
comapi_pxecall:
		mov bx,P_BX
		mov es,P_ES
		mov di,P_DI
		call pxenv
		mov P_AX,ax
		clc
		ret
%else
comapi_pxecall	equ comapi_err			
%endif

comapi_derinfo:
		mov P_AL,my_id
%if IS_SYSLINUX || IS_MDSLINUX
		mov al,[bsDriveNumber]
		mov P_DL,al
		mov P_ES,cs
		mov P_BX,PartInfo
%elif IS_PXELINUX
		mov ax,[APIVer]
		mov P_DX,ax
		mov ax,[StrucPtr]
		mov P_BX,ax
		mov ax,[StrucPtr+2]
		mov P_ES,ax
		mov ax,[InitStack]
		mov P_SI,ax
		mov ax,[InitStack+2]
		mov P_FS,ax
%elif IS_ISOLINUX
		mov al,[DriveNo]
		mov P_DL,al
		mov P_ES,cs
		mov P_BX,spec_packet
%endif
		clc
		ret

comapi_serialcfg:
		mov ax,[SerialPort]
		mov P_DX,ax
		mov ax,[BaudDivisor]
		mov P_CX,ax
		mov ax,[FlowControl]
		or al,ah
		mov ah,[FlowIgnore]
		shr ah,4
		mov P_BX,ax
		clc
		ret

comapi_cleanup:
%if IS_PXELINUX
		test dl,3
		setnz [KeepPXE]
		call unload_pxe
%elif IS_SYSLINUX || IS_MDSLINUX
		mov eax,[OrigFDCTabPtr]
		mov [fdctab],eax
%endif
		xor ax,ax
		xor dx,dx
		int 13h
		clc
		ret

%macro 		int21 2
		db %1
		dw %2
%endmacro

int21_table:
		int21	00h, comboot_return
		int21	01h, comboot_getkey
		int21	02h, comboot_writechr
		int21	04h, comboot_writeserial
		int21	08h, comboot_getkey
		int21	09h, comboot_writestr
		int21	0Bh, comboot_checkkey
		int21	30h, comboot_checkver
		int21	4Ch, comboot_return
		int21	 -1, comboot_bogus
int21_count	equ ($-int21_table)/3
		align 2, db 0

int22_table:
		dw comapi_err			
		dw comapi_get_version		
		dw comapi_writestr		
		dw comapi_run			
		dw comapi_run_default		
		dw comapi_textmode		
		dw comapi_open			
		dw comapi_read			
		dw comapi_close			
		dw comapi_pxecall		
		dw comapi_derinfo		
		dw comapi_serialcfg		
		dw comapi_cleanup
int22_count	equ ($-int22_table)/2

APIKeyWait	db 0
APIKeyFlag	db 0
