
static const char drdevice_c[] = "$Header:";



#include <dirent.h>


#include "thincore/sys.h"
#include "thincore/memory.h"
#include "thincore/net.h"
#include "thincore/strbase.h"
#include "thincore/ndmpserver.h"
#include "ndmp/ndmps.h"
#include "../dr/drdefs.h"
#include "drdevice.h"
#include "drdaemon.h"
#include "../dr/sundisklabel.h"
#include <sys/stat.h>
#include <errno.h>


#define DRDEV_IS_DISK(x)  ((NULL == x) ? FALSE : ((('s' == (x)[0]) || ('h' == (x)[0])) && ('d' == (x)[1]) && (3 == StrLength(x))))

typedef struct   dirent  droDirEntryS;


typedef struct PartitionTypeS
{
  ByteT  cType;
  char  *pstrName;
} DrPartitionTypeT;

/*
** ============================================================================
** Global Data
** ============================================================================
*/
DrPartitionTypeT gatPartitionType[] =
{
  {0x00, "Empty"},
  {0x01, "FAT12"},
  {0x02, "XENIX root"},
  {0x03, "XENIX usr"},
  {0x04, "FAT16 <32M"},
  {0x05, "Extended"},
  {0x06, "FAT16"},
  {0x07, "NTFS"},
  {0x08, "AIX"},
  {0x09, "AIX bootable"},
  {0x0A, "OS/2 Boot Manager"},
  {0x0B, "Win95 FAT32"},
  {0x0C, "Win95 FAT32 (LBA)"},
  {0x0E, "Win95 FAT16 (LBA)"},
  {0x0F, "Extended Win95 (LBA)"},
  {0x10, "OPUS"},
  {0x11, "Hidden FAT12"},
  {0x12, "Compaq diagnostics"},
  {0x14, "Hidden FAT16 <32M"},
  {0x16, "Hidden FAT16"},
  {0x17, "Hidden NTFS"},
  {0x18, "AST Windows swapfile"},
  {0x1B, "Hidden Win95 FAT32"},
  {0x1C, "Hidden Win95 FAT32 (LBA)"},
  {0x1E, "Hidden Win95 FAT16 (LBA)"},
  {0x24, "NEC DOS"},
  {0x39, "Plan 9"},
  {0x3C, "PartitionMagic recovery"},
  {0x40, "Venix 80286"},
  {0x41, "PPC PReP Boot"},
  {0x42, "SFS"},
  {0x4D, "QNX4.x"},
  {0x4E, "QNX4.x 2nd part"},
  {0x4F, "QNX4.x 3rd part"},
  {0x50, "OnTrack DM"},
  {0x51, "OnTrack DM6 Aux1"},
  {0x52, "CP/M"},
  {0x53, "OnTrack DM6 Aux3"},
  {0x54, "OnTrackDM6"},
  {0x55, "EZ-Drive"},
  {0x56, "Golden Bow"},
  {0x5C, "Priam Edisk"},
  {0x61, "SpeedStor"},
  {0x63, "GNU HURD or SysV"},
  {0x64, "Novell Netware 286"},
  {0x65, "Novell Netware 386"},
  {0x70, "DiskSecure Multi-Boot"},
  {0x75, "PC/IX"},
  {0x80, "Old Minix"},
  {0x81, "Minix / old Linux"},
  {0x82, "Linux swap / Others"},
  {0x83, "Linux"},
  {0x84, "OS/2 hidden C: drive"},
  {0x85, "Linux extended"},
  {0x86, "NTFS volume set"},
  {0x87, "NTFS volume set"},
  {0x8E, "Linux LVM"},
  {0x93, "Amoeba"},
  {0x94, "Amoeba BBT"},
  {0x9F, "BSD/OS"},
  {0xA0, "IBM Thinkpad hibernation"},
  {0xA5, "BSD/386"},
  {0xA6, "OpenBSD"},
  {0xA7, "NeXTSTEP"},
  {0xB7, "BSDI fs"},
  {0xB8, "BSDI swap"},
  {0xC1, "DRDOS/sec (FAT-12)"},
  {0xC4, "DRDOS/sec (FAT-16 < 32M)"},
  {0xC6, "DRDOS/sec (FAT-16)"},
  {0xC7, "Syrinx"},
  {0xDA, "Non-FS data"},
  {0xDB, "CP/M / CTOS / ..."},
  {0xDE, "Dell Utility"},
  {0xE1, "DOS access"},
  {0xE3, "DOS R/O"},
  {0xE4, "SpeedStor"},
  {0xEB, "BeOS fs"},
  {0xEE, "EFI GPT"},
  {0xEF, "EFI (FAT-12/16/32)"},
  {0xF1, "SpeedStor"},
  {0xF4, "SpeedStor"},
  {0xF2, "DOS secondary"},
  {0xFD, "Linux raid autodetect"},
  {0xFE, "LANstep"},
  {0xFF, "BBT"},
  { 0, 0 }
};

/*
** Global Variable:
**   gaacSunVTOCSliceTypes
**
** Description:
**   This array is used to translate Sun slice types into strings describing
**   the slice type.  For out of range values use:
**   <netvault>/disaster/unix/linux/drdevice.c gatPartitionType
*/

char *gaacSunVTOCSliceTypes[] =
{
  "NoType", /* partition could still have size and contents */
  "Boot", /* is this always UFS? */
  "Root", /* is this always UFS? */
  "Swap",
  "Usr", /* is this always UFS? */
  "Whole Disk",
  "Stand",
  "Var", /* is this always UFS? */
  "Home", /* is this always UFS? */
  "AltSect",
  "Cache",
  "Reserved"
};


INTERNAL BooleanT fncbLookExt3 (int, uint64_t, uint64_t);
INTERNAL BooleanT fncbLookExt2 (int, uint64_t, uint64_t);
INTERNAL BooleanT fncbLookNtfs (int, uint64_t, uint64_t);
INTERNAL BooleanT fncbLookSwap (int, uint64_t, uint64_t);
INTERNAL BooleanT fncbLookFat  (int, uint64_t, uint64_t);


INTERNAL GPTFsRecT  gtGPTFsRec[] = 
{
  { "ext3",       fncbLookExt3 }, 
  { "ext2",       fncbLookExt2 }, 
  { "NTFS",       fncbLookNtfs }, 
  { "Linux-Swap", fncbLookSwap }, 
  { "FAT",        fncbLookFat  }, 
  { 0, 0}
};







/*
** ============================================================================
** ============================================================================
*/
EXTERN void droTraceLocal (const char *pstrFormat, ...)
{
	va_list   args;
	char      strBuf [1024];

	MemZero   (strBuf, sizeof (strBuf));
	va_start  (args,   pstrFormat);
	vsnprintf (strBuf, sizeof (strBuf) - 1, pstrFormat, args);
	va_end    (args);

	TRACE ((1,0,ALWAYS,strBuf));
}









/*
** ============================================================================
** ============================================================================
*/
EXTERN void droHexDump (ByteT *pcBR, long lSize)
{ 
  long lRow = 1;
  long lSeg = 1;
  long lIdx = 0;

  FILE *pIO = fopen ("zzz_dump", "a");

  fprintf (pIO, "=========\n");
  fprintf (pIO, "%2ld  ", lRow++);
  
  while (lIdx < lSize)
  {
    fprintf (pIO, "%02x ", pcBR [lIdx++]);
  
    if (lIdx == (lSeg * 16))
    {
      fprintf (pIO, "\n%2ld  ", lRow++);
      lSeg++;
    }
  }
  fprintf (pIO, "\n");
  fclose (pIO);
}   







/*
** ============================================================================
** ============================================================================
*/
EXTERN BooleanT droSyncDevice (const char *pstrDev)
{
  BooleanT  bSuccess = FALSE;
  BooleanT  bOpen    = FALSE;
  long      lFd      = 0;
	long      lErr     = 0;
	long      lRc      = 0;

	if (TRUE == StrEmpty (pstrDev))
	{
		DRTRACE("ERROR: Failed to sync device. Input device name empty");
		goto exit_bypass;
	}
		
	DRTRACE("Begin sync device: %s", pstrDev);

	lFd  = open64 (pstrDev, O_WRONLY);
	lErr = errno;

	if (-1 == lFd)
	{
		DRTRACE("ERROR: Failed to sync device. Open error: [%s]", DR_STR_ERR (lErr));
		goto exit_bypass;
	}

  bOpen = TRUE;

	lRc  = ioctl (lFd, DRO_IOCTL_REREAD_PARTITION, NULL);
	lErr = errno;
	
	if (-1 == lRc)
	{
		DRTRACE("ERROR: Failed to sync device. IOCTL error: [%s]", DR_STR_ERR (lErr));
		goto exit_bypass;
	}
		
	DRTRACE("Ask kernel to re-read partition table success. Sync device OK: [%s]", pstrDev);
	
	bSuccess = TRUE;

exit_bypass:

	if (TRUE == bOpen)
	{
		close (lFd);
	}

	return bSuccess;
}







/*
** ============================================================================
** ============================================================================
*/
EXTERN BooleanT droGetSectorSizeViaFd (long lFd, long *plSecSize)
{
  BooleanT  bSuccess = FALSE;
	long      lErr     = 0;
	long      lRc      = 0;
  long      lSecSize = DR_SECTOR_BLKSIZE;

	DRTRACE("Begin get sector size via FD: [%ld]", lFd);

	if (0 > lFd)
	{
		DRTRACE("ERROR: Failed to get sector size via FD. Invalid input FD: [%ld]", lFd);
		goto exit_bypass;
	}

	lRc  = ioctl (lFd, DRO_IOCTL_SECTOR_SIZE, &lSecSize);
	lErr = errno;
	
	if (-1 == lRc)
	{
		DRTRACE("ERROR: Failed to get sector size via FD. IOCTL error: [%s]", DR_STR_ERR (lErr));
		goto exit_bypass;
	}

	if (NULL != plSecSize)
	{
		*plSecSize = lSecSize;
	}
		
	DRTRACE("Get sector size via FD OK. FD: [%ld], SectorSize: [%ld]", lFd, lSecSize);

	bSuccess = TRUE;
	
exit_bypass:

	return bSuccess;
}









/*
** ============================================================================
** ============================================================================
*/
EXTERN BooleanT droGetDeviceSizeViaFd (long lFd, uint64 *pllSize)
{
  BooleanT  bSuccess = FALSE;
  uint64    llSize   = 0;
	long      lErr     = 0;
	long      lRc      = 0;

	DRTRACE("Begin get device size via FD: [%ld]", lFd);

	if (0 > lFd)
	{
		DRTRACE("ERROR: Failed to get device size via FD. Invalid input FD: [%ld]", lFd);
		goto exit_bypass;
	}

	lRc  = ioctl (lFd, DRO_IOCTL_GET_SIZE_64, &llSize);
	lErr = errno;
	
	if (-1 == lRc)
	{
		DRTRACE("ERROR: Failed to get device size via FD. IOCTL error: [%s]", DR_STR_ERR (lErr));
		goto exit_bypass;
	}

	if (NULL != pllSize)
	{
		*pllSize = llSize;
	}
		
	DRTRACE("Get device size via FD OK. FD: [%ld], Size: [%llu]", lFd, llSize);

	bSuccess = TRUE;
	
exit_bypass:

	return bSuccess;
}








/*
** ============================================================================
** ============================================================================
*/
EXTERN BooleanT droGetSectorCountViaFd (long lFd, uint64 *pllSecCount)
{
  BooleanT  bSuccess = FALSE;
	long      lSecSize = 0;
  uint64    llSecCnt = 0;
  uint64    llSize   = 0;

	DRTRACE("Begin get sector count via FD: [%ld]", lFd);

	if (0 > lFd)
	{
		DRTRACE("ERROR: Failed to get sector count via FD. Invalid input FD: [%ld]", lFd);
		goto exit_bypass;
	}

	if (TRUE != droGetSectorSizeViaFd (lFd, &lSecSize))
	{
		DRTRACE("ERROR: Failed to get sector count via FD. Get sector size has error");
		goto exit_bypass;
	}

	if (TRUE != droGetDeviceSizeViaFd (lFd, &llSize))
	{
		DRTRACE("ERROR: Failed to get sector count via FD. Get device size has error");
		goto exit_bypass;
	}

  llSecCnt = llSize / ((uint64) lSecSize);

	if (NULL != pllSecCount)
	{
		*pllSecCount = llSecCnt;
	}
		
	DRTRACE("Get sector count via FD OK. FD: [%ld], SecCount: [%llu]", lFd, llSecCnt);

	bSuccess = TRUE;
	
exit_bypass:

	return bSuccess;
}













/*
** ============================================================================
** ============================================================================
*/
EXTERN long droGetSectorSize (const char *pstrDev)
{
  BooleanT  bOpen    = FALSE;
  long      lFd      = 0;
	long      lErr     = 0;
	long      lRc      = 0;
  long      lSecSize = DR_SECTOR_BLKSIZE;

	if (TRUE == StrEmpty (pstrDev))
	{
		DRTRACE("ERROR: Failed to get device sector size. Input device name empty");
		goto exit_bypass;
	}
		
	DRTRACE("Begin get device sector size: %s", pstrDev);

	lFd  = open64 (pstrDev, O_RDONLY);
	lErr = errno;

	if (-1 == lFd)
	{
		DRTRACE("ERROR: Failed to get device sector size. Open error: [%s]", DR_STR_ERR (lErr));
		goto exit_bypass;
	}

  bOpen = TRUE;

	lRc  = ioctl (lFd, DRO_IOCTL_SECTOR_SIZE, &lSecSize);
	lErr = errno;
	
	if (-1 == lRc)
	{
		DRTRACE("ERROR: Failed to get device sector size. IOCTL error: [%s]", DR_STR_ERR (lErr));
		goto exit_bypass;
	}
		
	DRTRACE("Get device sector size OK. Device: %s, SectorSize: %ld", pstrDev, lSecSize);
	
exit_bypass:

	if (TRUE == bOpen)
	{
		close (lFd);
	}

	return lSecSize;
}








/*
** ============================================================================
**
** Public Function:
**   droGetMBR
**
** Description:
**   Gets the MBR from a device given a device file name.
**   Gets disk label and boot sectors (1st-16th device sectors) for Solaris.
**   Gets MBR (first device sector) for DOS/Other.
**
** Arguments:
**   pstrDev - device file name string
**   ptData - OUT - pointer result structure; droGetMBR gets memory for data
**
** Globals:
**   gtMachineArch - machine architecture => OS & disk format
**
** Results:
**   TRUE and valid MBR in ptData on success
**   FALSE and undefined ptData contents on failure
**
** ============================================================================
*/
EXTERN BooleanT droGetMBR( const char *pstrDev, XdrDataFieldT *ptData )
{
  uint32    uiMBRSize = (DR_ARCH_SOLARIS == gtMachineArch ?  DR_SOLARIS_MBR_SIZE : DR_SECTOR_BLKSIZE );
  BooleanT   bSuccess   = FALSE;
  int        lFd       = open64 (pstrDev, O_RDONLY);
  ByteT     *pcMBR;


  TRACE((9,0,ALWAYS,"Allocating memory for MBR in droGetMBR - %d", uiMBRSize ));

  pcMBR = (ByteT *)MemAllocate( uiMBRSize );

  TRACE((10,0,ALWAYS,"Begin droGetMBR %u bytes for device: %s",uiMBRSize,pstrDev));

  if ( -1 == lFd )
  {
    TRACE((11,0,ALWAYS,"Failed to open device: %s. %s",
      pstrDev,strerror(errno)));
  }
  else if ( uiMBRSize != read ( lFd, pcMBR, uiMBRSize ) )
  {
    MemFree( &pcMBR );
    TRACE((12,0,ALWAYS,"Failed to read %i bytes from device: %s. %s",
      uiMBRSize, pstrDev, strerror(errno) ));
  }
  else
  {
    ptData->lSize = uiMBRSize;
    ptData->pData = pcMBR;
    bSuccess       = TRUE;
    close (lFd);
  }

  TRACE((13,0,ALWAYS,"End droGetMBR for device: %s. %s",
    pstrDev, STR_BOOLEAN(bSuccess)));

  return bSuccess;
}






/*
** ============================================================================
**
** Public Function:
**
** Description:
**   gets size of partition that file system is on in bytes
**   size is limited to unsigned int 32 * block size (typically 512)
**   = 2 terabytes ... only known OS with bigger plans is Solaris 10 
**   although some managed file systems have bigger capacities
**
** Arguments:
**   pllSize - OUT - size in bytes of partition
**   lFd - open file descriptor for partition device file
**
** Returns:
**   TRUE and byte size on success
**   FALSE and unmodified byte size on failure
**
** ============================================================================
*/
EXTERN BooleanT droGetDeviceSize (const char *pstrDev, int64 *pllSize)
{
  BooleanT  bSuccess = FALSE;
  BooleanT  bOpen    = FALSE;
  uint64    llSize   = 0;
  long      lFd      = 0;
	long      lErr     = 0;
	long      lRc      = 0;

	if (TRUE == StrEmpty (pstrDev))
	{
		DRTRACE("ERROR: Failed to get device size. Input device name empty");
		goto exit_bypass;
	}
		
	DRTRACE("Begin get device size: %s", pstrDev);

	lFd  = open64 (pstrDev, O_RDONLY);
	lErr = errno;

	if (-1 == lFd)
	{
		DRTRACE("ERROR: Failed to get device size. Open error: [%s]", DR_STR_ERR (lErr));
		goto exit_bypass;
	}

  bOpen = TRUE;

	lRc  = ioctl (lFd, DRO_IOCTL_GET_SIZE_64, &llSize);
	lErr = errno;
	
	if (-1 == lRc)
	{
		DRTRACE("ERROR: Failed to get device size. IOCTL error: [%s]", DR_STR_ERR (lErr));
		goto exit_bypass;
	}

	if (NULL != pllSize)
	{
		*pllSize = llSize;
	}
		
	DRTRACE("Get device size OK. Device: [%s], Size: [%s]", pstrDev, droGroupDigit (llSize));

	bSuccess = TRUE;
	
exit_bypass:

	if (TRUE == bOpen)
	{
		close (lFd);
	}

	return bSuccess;
}







/*
** ============================================================================
** ============================================================================
*/
INTERNAL BooleanT droGetDeviceGeoDetail (int lFd, DrGeometryT *psGeo)
{
	BooleanT   bSuccess  = FALSE;
  long       lRC1      = 0;
  long       lRC2      = 0;
  long       lErr      = 0;
  long       lSecSize  = 0;
  int64      llSecCnt  = 0;
  int64      llSize    = 0;
  struct     hd_geometry  tGeo;

	if (0 > lFd)
	{
		DRTRACE("ERROR: Failed to get device geometry. Invalid in FD");
		goto exit_bypass;
	}
	
	if (NULL == psGeo)
	{
		DRTRACE("ERROR: Failed to get device geometry. Return geometry is NULL");
		goto exit_bypass;
	}
  
	lRC1 = ioctl (lFd, DRO_IOCTL_SECTOR_CNT, &llSecCnt);

  lRC2 = ioctl (lFd, HDIO_GETGEO, &tGeo);

  if (0 == lRC2)
  {
    psGeo->ulHeads   = tGeo.heads;
    psGeo->ulSectors = tGeo.sectors;
    psGeo->ullOffset = tGeo.start;

    if (0 == psGeo->ulHeads * psGeo->ulSectors)
    {
      lRC2 = -1;
    }
  }
  if (0 == lRC1 && 0 == lRC2)
  {
    psGeo->ulCylinders = llSecCnt / (psGeo->ulHeads * psGeo->ulSectors);
  }
  else if (0 == lRC1)
  {
    psGeo->ulHeads     = 1;
    psGeo->ulCylinders = 1;
    psGeo->ulSectors   = llSecCnt;
  }

  lRC1 = ioctl (lFd, DRO_IOCTL_SECTOR_SIZE, &lSecSize);
	lErr = errno;

	if (-1 == lRC1)
	{
		DRTRACE("ERROR: Failed to get device geometry. IOCTL get sector size error: [%s]",
		DR_STR_ERR (lErr));
		goto exit_bypass;
	}

	lRC1 = ioctl (lFd, DRO_IOCTL_GET_SIZE_64, &llSize);
	lErr = errno;

	if (-1 == lRC1)
	{
		DRTRACE("ERROR: Failed to get device geometry. IOCTL get size error: [%s]",
		DR_STR_ERR (lErr));
		goto exit_bypass;
	}

	psGeo->ulSectors = llSize / (int64) lSecSize;
  psGeo->ullSize   = llSize;
	
  bSuccess = TRUE;

exit_bypass:
    
  return bSuccess;
}








/*
** ============================================================================
** ============================================================================
*/

/*
** Public Function:
**   GetSolarisDeviceGeometry
**
** Description:
**   Read Solaris disk label to get device information.
**   This could be changed to do various ioctls to get IDE/SCSI device
**   geometries, but that is for a later version.
**   XXX GetSolarisDeviceGeometry - use /dev/raw/?, use ioctls not disk label?
**
** Arguments:
**   lFd - file descriptor for device file of target disk
**   psGeo - OUT - geometry results including total blocks,
**     cylinders, heads, and sectors/track.
**
** Results:
**   TRUE and valid disk geometry on success, and
**   FALSE with undefined disk geometry otherwise.
*/
EXTERN BooleanT GetSolarisDeviceGeometry( int lFd, DrGeometryT *psGeo )
{
  ByteT strBuf[DR_SECTOR_BLKSIZE];
  BooleanT bResult = FALSE;

  if ( DR_SECTOR_BLKSIZE != read( lFd, strBuf, DR_SECTOR_BLKSIZE ) )
  {
    TRACE((18,0,ALWAYS,"Failed to read disk label. %s", strerror(errno)));
  }
  else
  {
    psGeo->ulHeads     = ((SunDiskLabelT *)strBuf)->usHeads;
    psGeo->ulCylinders = ((SunDiskLabelT *)strBuf)->usDataCyl;
    psGeo->ulSectors   = ((SunDiskLabelT *)strBuf)->usSect;
    psGeo->ullSize     = (int64)DR_SECTOR_BLKSIZE * psGeo->ulHeads *
       psGeo->ulCylinders * psGeo->ulSectors;

    bResult = TRUE;
  }

  return bResult;
}


/*
** ============================================================================
** ============================================================================
*/
EXTERN BooleanT GetPartitionType (char **pstrType, ByteT cType)
{
  DrPartitionTypeT  *ptType = gatPartitionType;
  BooleanT          bFound  = FALSE;

  while (TRUE != bFound && NULL != ptType->pstrName)
  {
    if (ptType->cType == cType)
    {
      *pstrType = StrDuplicate (ptType->pstrName);
      bFound = TRUE;
    }
    ptType++;
  }
  return bFound;
}

/*
** ============================================================================
** ============================================================================
*/

/*
** Public Function:
**   GetSolarisPartitionTypeString
**
** Description:
**   Gets a descriptive string for a partition type ID.  If the type is
**   out of the range of Solaris values, an attempt is made to map the
**   ID to other known types.
**
** Arguments:
**   pstrType - OUT - pointer to copy of descriptive string
**   usID - ID number of type to lookup
**
** Returns:
**   TRUE if a type string was found and copied successfully for output
**   FALSE if the type is outside of known bounds, "Unknown" returned in
**     string
*/

EXTERN BooleanT GetSolarisPartitionTypeString( char **pstrType, ushort usID )
{
  BooleanT bFound = FALSE;

  if ( SDL_ST_MAX < usID )
  {
    if ( 256 > usID )
    {
      /*
      ** defer to the big list of partition types if the type id is bigger
      ** than the solaris type range, but still in the range where it could
      ** be from a linux setup
      */
      bFound = GetPartitionType( pstrType, (ByteT)usID );
    }
    else
    {
      *pstrType = StrDuplicate( "Unknown" );
    }
  }
  else
  {
    /*
    ** the partition type is in the range of known Solaris types
    */
    bFound = TRUE;
    *pstrType = StrDuplicate( gaacSunVTOCSliceTypes[usID] );
  }

  return bFound;
}

/*
** ============================================================================
** ============================================================================
*/
EXTERN BooleanT ValidateSignature (ByteT *pstr)
{
  return (0x55 == pstr[510] && 0xaa == pstr[511]);
}

/*
** ============================================================================
** ============================================================================
*/

/*
** Private Function:
**   ValidateSolarisSignature
**
** Description:
**   Checks bytes near end of block which indicate block is a valid Solaris
**   disk label.
**   Does not check internal signature because sparc linux does not set it
**   correctly.
**   Does not perform checksum of sector, but probably should.
**
** Arguments:
**   pStr - points to an array of bytes representing the disk label
**
** Returns:
**   TRUE if the sector appears to be a Solaris disk label
**   FALSE otherwise
*/
EXTERN BooleanT ValidateSolarisSignature (ByteT *pStr)
{
  return ( (0xda == pStr[508] && 0xbe == pStr[509]) );
}

/*
** ============================================================================
** ============================================================================
*/

/*
** Private Function:
**   CopySolarisPT
**
** Description:
**   Copies just the partition table information from one disk label to
**   another.  No checksum, sector signature, or format checking is done.
**
** Arguments:
**   pSrcStr - IN - pointer to buffer of at least SDL_SIZE bytes containing PT
**   pDestStr - OUT - pointer to buffer of at least SDL_SIZE bytes for PT
*/
INTERNAL void CopySolarisPT( ByteT *pSrcStr, ByteT *pDestStr )
{
  SunDiskLabelT *ptSrcSDL = (SunDiskLabelT *)pSrcStr;
  SunDiskLabelT *ptDestSDL = (SunDiskLabelT *)pDestStr;
  int32 iIndex;

  for ( iIndex = 0; SDL_SLICE_COUNT > iIndex; iIndex++ )
  {
    ptDestSDL->atSliceAttrs[iIndex].usSliceType =
       ptSrcSDL->atSliceAttrs[iIndex].usSliceType;

    ptDestSDL->atSliceAttrs[iIndex].usSliceFlags =
       ptSrcSDL->atSliceAttrs[iIndex].usSliceFlags;

    ptDestSDL->atSliceGeom[iIndex].uiStartCyl =
       ptSrcSDL->atSliceGeom[iIndex].uiStartCyl;

    ptDestSDL->atSliceGeom[iIndex].uiSectors =
       ptSrcSDL->atSliceGeom[iIndex].uiSectors;
  }

  /*
  ** also copy the endmark from the source to preserve the validity checking
  ** of the partition table
  */
  ptDestSDL->usEndMark = ptSrcSDL->usEndMark;

}

/*
** ============================================================================
** ============================================================================
*/

/*
** Private Function:
**   UpdateSolarisDiskLabelEndmark
**
** Description:
**   Put the signature words into the disk label.  This doesn't make the
**   disk label valid, but it does allow our routines to read the partition
**   table without complaining.
**
** Arguments:
**   pDestStr - IO - pointer to disk label which must be SDL_SIZE bytes or
**     else!
*/
INTERNAL void UpdateSolarisDiskLabelEndmark( ByteT *pTargetStr )
{
  SunDiskLabelT *ptSDL = (SunDiskLabelT *)pTargetStr;

  ptSDL->usEndMark = SDL_ENDMARK;

  return;
}

/*
** ============================================================================
** ============================================================================
*/

/*
** Private Function:
**   UpdateSolarisDiskLabelChecksum
**
** Description:
**   Calculate checksum of disklabel passed in an set it in the structure
**   The checksum doesn't include the last 2 bytes which are the checksum
**   itself.
**
** Arguments:
**   pDestStr - IO - pointer to disk label which must be SDL_SIZE bytes or
**     else!  This pointer is re-cast like crazy to access the disk label
**     structure and calculate the XOR checksum one ushort at a time.
*/
INTERNAL void UpdateSolarisDiskLabelChecksum( ByteT *pDestStr )
{
  ushort *pusShorts = (ushort *)(((SunDiskLabelT *)pDestStr) + 1) - 2;
  ushort usCkSum;

  for ( usCkSum = 0; pusShorts >= (ushort *)pDestStr; pusShorts-- )
  {
    usCkSum ^= *pusShorts;
  }

  ((SunDiskLabelT *)pDestStr)->usChecksum = usCkSum;
}

/*
** ============================================================================
** ============================================================================
*/

/*
** Private Function:
**   GeometryMatchSolaris
**
** Description:
**   If sectors per cylinder don't match, there is a problem.
**   The disk labels had better be big enough!
**
** Arguments:
**   pDL1Str - IN - a bunch of bytes which will be cast to a SunDiskLabelT
**   pDL2Str - IN - a bunch of bytes which will be cast to a SunDiskLabelT
**
** Return:
**   TRUE if labels compatible
**   FALSE otherwise
*/
INTERNAL BooleanT GeometryMatchSolaris( ByteT *pDL1Str, ByteT *pDL2Str )
{
  SunDiskLabelT *ptDL1 = (SunDiskLabelT *)pDL1Str;
  SunDiskLabelT *ptDL2 = (SunDiskLabelT *)pDL2Str;

  return ( ptDL1 && ptDL2 &&
    (ptDL1->usHeads * ptDL1->usSect) == (ptDL2->usHeads * ptDL2->usSect) );
}

/*
** ============================================================================
** ============================================================================
*/

/*
** Public Function:
**   MergeSolarisDiskLabel
**
** Description:
**   Blindly merges requested data from source buffer to target buffer.
**   Recalculates checksum of sector and peace be with you.
**   XXX This assumes Solaris disk label format 1 (the only known to exist).
**
** Arguments:
**   pSrcStr - points to an array of bytes representing the source disk label
**   pDestStr - points to an array of bytes representing the target disk label
**   bLabel - merge the label in around the partition table if true, otherwise
**     merge the partition table into the disk label
**
** Results:
**   TRUE if merge successful
**   FALSE otherwise
*/
EXTERN BooleanT MergeSolarisDiskLabel (
    ByteT *pSrcStr,
    ByteT *pDestStr,
    BooleanT bLabel
    )
{
  /*
  ** This will be plenty fast, so it might as well be easy too.
  **
  ** Either copy the partition table to the dest, OR
  ** copy the partition table from the dest, blast in the whole source
  ** and then restore the dest partition table
  */

  ByteT pTemp[SDL_SIZE];
  BooleanT bResult = TRUE;

  if ( TRUE == bLabel )
  {
    /*
    ** restore partition table case
    */

    /* check to see if disk geometry is compatible with partition table */
    if ( TRUE == (bResult = GeometryMatchSolaris( pSrcStr, pDestStr )) )
    {
      CopySolarisPT( pDestStr, pTemp );
      MemCopy( pDestStr, pSrcStr, SDL_SIZE );
      CopySolarisPT( pTemp, pDestStr );

      /*
      ** give the dest a valid solaris disk label signature so we can read
      ** the partition table. this assumes that what we just restored was
      ** valid solaris partition table data.  users can do whatever they want.
      */
      UpdateSolarisDiskLabelEndmark( pDestStr );
    }
  }
  else
  {
    /*
    ** restore disk label case
    */
    CopySolarisPT( pSrcStr, pDestStr );
  }

  UpdateSolarisDiskLabelChecksum( pDestStr );

  return bResult;
}

/*
** ============================================================================
** ============================================================================
*/
INTERNAL BooleanT DeviceIsCdrom (const char *pstrDev)
{
  BooleanT      bCdrom   = FALSE;
  FILE          *pFd     = NULL;
  char          strBuf[128];
  struct stat    tStat;

  TRACE((19,0,ALWAYS,"Begin DeviceIsCdrom for Device: %s", pstrDev));

  if (0 == strncmp ("/dev/hd", pstrDev, 7))
  {
    MemZero (strBuf, sizeof(strBuf));

    snprintf (strBuf, sizeof (strBuf), "/proc/ide/%s/media", pstrDev + 5);

    pFd = fopen (strBuf, "r");

    if (NULL != pFd && NULL != fgets (strBuf, sizeof (strBuf), pFd))
    {
      if (0 == strncmp (strBuf, "cdrom", 5))
      {
        bCdrom = TRUE;
      }
      fclose (pFd);
    }
    else
    {
      if (0 == stat (pstrDev, &tStat))
      {
        if (0 == (tStat.st_mode & 0222))
        {
          bCdrom = TRUE;
        }
      }
    }
  }
  TRACE((20,0,ALWAYS,"End DeviceIsCdrom for Device: %s. %s", pstrDev, STR_BOOLEAN(bCdrom)));

  return bCdrom;
}




/*
** ============================================================================
** ============================================================================
*/
EXTERN char *drdevStrip (char *pstrInput)
{
  char      *pstr = pstrInput;
  long      lLen  = 0;
  long      i     = 0;
  BooleanT  bLoop = TRUE;

  if (NULL != pstrInput && 0 < (lLen = StrLength (pstrInput)))
  {
    while (0 != *pstr && TRUE == bLoop)
    {
      if (' ' != *pstr)
      {
        bLoop = FALSE;
        pstr--;
      }
      pstr++;
    }

    for (i = 0; i < (lLen + 1); i++)
    {
      pstrInput[i] = pstr[i];
    }

    bLoop = TRUE;
    lLen  = StrLength (pstrInput);
    pstr  = pstrInput + lLen - 1;

    for (i = 0; i < lLen && TRUE == bLoop; i++)
    {
      if (' ' != *pstr && '\n' != *pstr && '\r' != *pstr)
      {
        bLoop = FALSE;
      }
      else
      {
        pstr--;
      }
    }
    pstr++;
    *pstr = 0;
  }

  return pstrInput;
}









/*
** ============================================================================
** ============================================================================
*/
INTERNAL BooleanT bmrStrHasDigit (char *pstrInput)
{
	BooleanT  bRC = FALSE;

	if (NULL != pstrInput)
	{
		while (0 != *pstrInput)
		{
			if (TRUE == StrIsDigit (*pstrInput))
			{
				bRC = TRUE;
			}

			pstrInput++;
		}
	}


	return bRC;
}







/*
** ============================================================================
**
**  major minor  #blocks    name
**
**  7     0      83288      loop0
**  8     0      8388608    i2o/hda
**  8     1      8385898    i2o/hda1
**
**  8     0      1048576    sda
**  8    16      104448     sdb
**  8    17      104432     sdb1
**  65   128     104448     sdy
**  65   144     104448     sdz
**  65   160     104448     sdaa
**  65   176     104448     sdab
**
**  69   0       1745000    cciss/c0d0       #Controller 0, disk 0
**  69   0       1500       cciss/c0d0p1     #Controller 0, disk 0, partition 1
**
**  69   0       150000     ida/c0d0
**  69   0       1500       ida/c0d0p1
**  69   0       1500       ida/c0d0p2
**
** ============================================================================
*/
EXTERN BooleanT GetServerDevices (XdrT tXdr, XdrArrayT *ptArray)
{
	BooleanT   bRC      = FALSE;
	BooleanT   bFound   = FALSE;
	long       lRC      = 0;
	long       lError   = 0;
	long       lNum1    = 0;
	long       lNum2    = 0;
	long       lNum3    = 0;
	FILE       *phFile  = NULL;
	char       *pstr    = NULL;
	char       cChar1   = 0;
	char       cChar2   = 0;
	char       cLine    = 0;
	char       strLine [256];
	char       strName [256];
	char       strDev  [256];
	char       strTmp  [32];

	phFile = fopen ("/proc/partitions", "r");
	lError = errno;

	if (NULL == phFile)
	{
		DRTRACE("ERROR: Failed to get list of disk devices. Open /proc/partitions error");
		DRTRACE("ERROR: Error: [%ld], Msg: [%s]", lError, strerror (lError));
		goto bmrExitError;
	}

	while (NULL != fgets (strLine, sizeof (strLine), phFile))
	{
		bFound = FALSE;

		MemZero (strName, sizeof (strName));

		if (4 == sscanf (strLine, "%ld %ld %ld %[^'\n']", &lNum1, &lNum2, &lNum3, strName))
		{
			pstr = StrNextCharMatch (strName, '/');

			if (NULL == pstr)
			{
				lRC = sscanf (strName, "%c%c%s%c", &cChar1, &cChar2, strTmp, &cLine);

				if (('s' == cChar1 || 'h' == cChar1) && 'd' == cChar2 && TRUE != bmrStrHasDigit (strTmp))
				{
					bFound = TRUE;
				}
			}
			else
			{
				pstr++;

				lRC = sscanf (pstr, "%c%ld%c%ld%c", &cChar1, &lNum1, &cChar2, &lNum2, &cLine);
				
				if (4 == lRC && 'c' == cChar1 && 'd' == cChar2)
				{
					bFound = TRUE;
				}
				
				lRC = sscanf (pstr, "%c%c%s%c", &cChar1, &cChar2, strTmp, &cLine);

				if (('s' == cChar1 || 'h' == cChar1) && 'd' == cChar2 && TRUE != bmrStrHasDigit (strTmp))
				{
					bFound = TRUE;
				}
			}
		}

		if (TRUE == bFound)
		{
			drdevStrip (strName);
			
			MemZero (strDev, sizeof (strDev));

			snprintf (strDev, sizeof (strDev) - 1, "/dev/%s", strName);
			
			if (TRUE != DeviceIsCdrom (strDev))
			{
				DrDeviceInfoT  tInfo = XdrNew (tXdr, DR_DEVICE_INFO);
				tInfo->pstrName     = StrDuplicate (strDev);
				XdrArrayAdd (ptArray, &tInfo, 0);
	
				bRC = TRUE;
			}
		}
	}

bmrExitError:

	if (NULL != phFile)
	{
		fclose (phFile);
	}

	return bRC;
}








/*
** ====================================================================================================
** ====================================================================================================
*/
INTERNAL BooleanT droGetMpathVolumeHandle
(
	char     *pstrDiskHandle,
	char     *pstrVolHandle,
	long     lHandleLen,
	int64    llSizeIn,
	int64    llOffsetIn
)
{
	BooleanT        bRC      = FALSE;
	BooleanT        bLoop    = TRUE;
	int64           llOffset = 0;
	int64           llSize   = 0;
	long            lErr     = 0;
	droDirEntryS    *psEntry = NULL;
	DIR             *pDir    = NULL;
	char            *pstrDir = "/dev/mapper/";

	DRTRACE("Get Mpath volume handle. VolSize: [%s], VolOffset: [%s]",
	droGroupDigit (llSizeIn), droGroupDigit (llOffsetIn));

	pDir = opendir (pstrDir);
	lErr = errno;

	if (NULL == pDir)
	{
		DRTRACE("ERROR: Failed to get multipath volume handle. Open error: [%s] [%s]",
		DROVSTR (pstrDir), DROSYSERR (lErr));
		goto droExit;
	}

	while (TRUE == bLoop)
	{
		psEntry = readdir (pDir);
		lErr    = errno;
		
		if (NULL == psEntry)
		{
			if (0 != lErr)
			{
				DRTRACE("ERROR: Failed to get multipath volume handle. Read error: [%s] [%s] [%ld]",
				DROVSTR (pstrDir), DROSYSERR (lErr), lErr);
			}
			goto droExit;
		}

		if (TRUE != StrEqual (psEntry->d_name, ".") && TRUE != StrEqual (psEntry->d_name, ".."))
		{
			MemZero (pstrVolHandle, lHandleLen);

			snprintf (pstrVolHandle, lHandleLen, "%s%s", pstrDir, psEntry->d_name);

			if (NULL != strstr (pstrVolHandle, pstrDiskHandle))
			{
				if (TRUE != droGetDeviceGeometry (pstrVolHandle, NULL, NULL, NULL, &llOffset))
				{
					DRTRACE("ERROR: Failed to get multipath volume handle. Get geometry error: [%s]",
					DROVSTR (pstrVolHandle));
					goto droExit;
				}

				if (TRUE != droGetDeviceSize (pstrVolHandle, &llSize))
				{
					DRTRACE("ERROR: Failed to get multipath volume handle. Get size error: [%s]",
					DROVSTR (pstrVolHandle));
					goto droExit;
				}
				
				DRTRACE("Test Handle: [%s], Size: [%s], Offset: [%s]", 
				DROVSTR (pstrVolHandle), droGroupDigit (llSize), droGroupDigit (llOffset));

				if (llSizeIn == llSize && llOffsetIn == llOffset)
				{
					bLoop = FALSE;

					DRTRACE("Found volume handle OK");
					DRTRACE("  Disk Handle:   [%s]", DROVSTR (pstrDiskHandle));
					DRTRACE("  Volume Handle: [%s]", DROVSTR (pstrVolHandle));
					DRTRACE("  Volume Size:   [%s]", droGroupDigit (llSize));
					DRTRACE("  Volume Offset: [%s]", droGroupDigit (llOffset));
				}
			}
		}
	}

	bRC = TRUE;

droExit:

	if (NULL != pDir)
	{
		closedir (pDir);
	}

	return bRC;
}








/*
** ====================================================================================================
** ====================================================================================================
*/
INTERNAL BooleanT droGetVolumeHandle
(
	FILE       *pFd,
	char       *pstrDiskHandle,
	char       *pstrVolHandle,
	long       lHandleLen,
	int64      llSizeIn,
	uint64     ullOffset
)
{
  BooleanT   bFound  = FALSE;
  char       strBuf [512];

	DRTRACE("Get volume handle belong to Disk: [%s], VolSize: [%s], VolOffset: [%s]",
	DROVSTR (pstrDiskHandle), droGroupDigit (llSizeIn), droGroupDigit (ullOffset));

	if (TRUE == StrEmpty (pstrDiskHandle))
	{
		DRTRACE("ERROR: Failed to get volume handle. Input disk handle is EMPTY");
		goto droExit;
	}

	if (NULL != strstr (pstrDiskHandle, "mpath"))
	{
		bFound = droGetMpathVolumeHandle (pstrDiskHandle,
		                                  pstrVolHandle,
																			lHandleLen,
																			llSizeIn,
																			ullOffset);

		goto droExit;
	}

  fseek (pFd, 0L, SEEK_SET);
 
  while (FALSE == bFound && NULL != fgets (strBuf, sizeof (strBuf), pFd))
  {
    char  strName [128];
    long  lMajor  = 0;
    long  lMinor  = 0;
    int64 llSize  = 0;

    if (4 == sscanf (strBuf," %ld %ld %llu %[^\n ]", &lMajor,&lMinor,&llSize,strName))
    {
      MemZero (pstrVolHandle, lHandleLen);

      sprintf (pstrVolHandle, "/dev/%s", strName);

      if (NULL != strstr (pstrVolHandle, pstrDiskHandle) && llSizeIn == llSize)
      {
        int lPtFd;
        struct hd_geometry ptgeo;
        lPtFd = open64 (pstrVolHandle, O_RDONLY);
        if ( -1 == lPtFd)
        {
          TRACE((158,0,ALWAYS,"Failed to open File: %s. %s", pstrVolHandle, strerror(errno)));
        }
        else if ( 0 != ioctl(lPtFd, HDIO_GETGEO, &ptgeo))
        {
          TRACE((159,0,ALWAYS,"Failed to ioctl on Device %s. %s",pstrVolHandle,  strerror(errno)));
        }
        else if (ptgeo.start == ullOffset)
				{
          bFound = TRUE;
				}

        if ( -1 != lPtFd)
				{
          close(lPtFd);
				}
      }
    }
  }

droExit:

  return bFound;
}





/*
** ============================================================================
** ============================================================================
*/
INTERNAL BooleanT  GPTGetPartitionName (
                  long        *pl_maj,
                  long        *pl_min,
                  FILE        *pFd,
                  const char  *pstrDev,
                  char        *pstrPtName,
                  long        lPtLen,
                  int64       llSizeIn)
{
  BooleanT  bFound  = FALSE;
  char       strBuf[512];

  fseek (pFd, 0L, SEEK_SET);
 
  while (FALSE == bFound && NULL != fgets (strBuf, sizeof (strBuf), pFd))
  {
    char  strName [128];
    long  lMajor  = 0;
    long  lMinor  = 0;
    int64 llSize  = 0;

    if (4 == sscanf (strBuf," %ld %ld %llu %[^\n ]", &lMajor,&lMinor,&llSize,strName))
    {
      MemZero (pstrPtName, lPtLen);

      sprintf (pstrPtName, "/dev/%s", strName);

      if (NULL != strstr (pstrPtName, pstrDev) && llSizeIn == llSize)
      {
        if (*pl_min < lMinor)
        {
          bFound  = TRUE;
          *pl_maj = lMajor;
          *pl_min = lMinor;
        }
      }
    }
  }
  return bFound;
}





/*
** ============================================================================
** ============================================================================
*/

/*
** Private Function:
**   GetSolarisPartitionName
**
** Description:
**   Get the next device name of a partition from the current
**   location in the /proc/partitions input stream given a
**   device and a partition size in blocks ...
**   This should only be used to enumerate all partitions, not to find
**   an individual one.
**
** Arguments:
**   pFd - pointer to stream data from /proc/partitions
**   pstrDev - the device file name under linux for sparc for the target disk
**   pstrPtName - OUT - buffer for resulting device name
**   lPtLen - the length of the output buffer
**   llSizeIn - the size of the partition to return
*/
INTERNAL BooleanT  GetSolarisPartitionName (
                  FILE        *pFd,
                  const char  *pstrDev,
                  char        *pstrPtName,
                  long        lPtLen,
                  int64       llSizeIn)
{
  BooleanT  bFound  = FALSE;
  char       strBuf[512];
 
  TRACE((25,0,ALWAYS,"Begin GetSolarisPartitionName for device: %s Size: 0x%llx", 
  pstrDev, llSizeIn));

  while (FALSE == bFound && NULL != fgets (strBuf, sizeof (strBuf), pFd))
  {
    char  strName [128];
    long  lMajor  = 0;
    long  lMinor  = 0;
    int64 llSize  = 0;

    if (4 == sscanf (strBuf," %ld %ld %llu %[^\n ]", &lMajor,&lMinor,&llSize,strName))
    {
      MemZero (pstrPtName, lPtLen);

      sprintf (pstrPtName, "/dev/%s", strName);

      if (NULL != strstr (pstrPtName, pstrDev) && llSizeIn == llSize)
      {
        bFound = TRUE;
      }
    }
  }
  TRACE((26,0,ALWAYS,"End GetSolarisPartitionName with: %s", STR_BOOLEAN(bFound)));

  return bFound;
}








/*
** ====================================================================================================
** ====================================================================================================
*/
INTERNAL BooleanT droReadMbrVolumeTableRecursive
(
	XdrT           tXdr,
	int            lFd,
	FILE           *pFd,
	char           *pstrDiskHandle,
	DrGeometryT    *psGeo,
	int64          llBase,
	int64          llOffset,
	long           lIndex,
	XdrArrayT      *ptPtEntries,
	XdrArrayT      *ptSecEntries
)
{
  BooleanT          bSuccess = FALSE;
  ByteT             strBuf[DR_SECTOR_BLKSIZE];

  DRTRACE("Enter read BMR volume table recursive. Base: [0x%llx], Offset: [0x%llx]",
	llBase, llOffset);

  if (TRUE != DrSeek (lFd, llOffset))
  {
    TRACE((30,0,ALWAYS,"Failed to seek to position: 0x%llx", llOffset));
  }
  else if (DR_SECTOR_BLKSIZE != read (lFd, strBuf, DR_SECTOR_BLKSIZE))
  {
    TRACE((31,0,ALWAYS,"Failed to Read 512 Bytes. %s", strerror(errno)));
  }
  else if (TRUE != ValidateSignature (strBuf))
  {
    TRACE((32,0,ALWAYS,"Invalid Signature: 0x%02x 0x%02x",strBuf[510],strBuf[511]));
  }
  else
  {
    BooleanT bFailed = FALSE;
    long     i = 0;

    if (NULL != ptSecEntries)
    {
      DrDataT        tEntry    = XdrNew (tXdr, DR_DATA);
      DrBootSectorT  *ptRecord = NULL;

      tEntry->tData.lSize = sizeof (DrBootSectorT);
      tEntry->tData.pData = MemAllocate (sizeof(DrBootSectorT));

      ptRecord = tEntry->tData.pData;

      ptRecord->llOffset = llOffset;

      MemCopy (ptRecord->cBuffer, strBuf, DR_SECTOR_BLKSIZE);

      XdrArrayAdd (ptSecEntries, &tEntry, 0);
    }

    if (NULL != ptPtEntries)
    {
      for (i=0; FALSE == bFailed && i<4; i++)
      {
        DrPartitionT *psPt = PT_OFFSET(strBuf, i);

        if (0x00 != psPt->cType && (0 == llBase || TRUE != IS_EXTENDED(psPt->cType)))
        {
          char strVolHandle [256];
          
          if (TRUE != droGetVolumeHandle (
                      pFd,
                      pstrDiskHandle, 
                      strVolHandle,
                      sizeof (strVolHandle),
                      IS_EXTENDED(psPt->cType) ? 1 : PT_NUMBER_SECTORS(psPt) / 2, 
                      llBase != 0  ? (llOffset/DR_SECTOR_BLKSIZE) + PT_RELATIVE_SECTOR(psPt) : PT_RELATIVE_SECTOR(psPt)))
          {
            DRTRACE("ERROR: Failed to get Volume Handle for Disk: [%s]"
            " Where Partition Size is: %llu", pstrDiskHandle ,PT_NUMBER_SECTORS (psPt) / 2);

            bFailed = TRUE;
          }
          else
          {
            uint64            ullSecRel       = PT_RELATIVE_SECTOR(psPt);
            uint64            ullSecNum       = PT_NUMBER_SECTORS(psPt);
            uint64            ullOffsetSector = llOffset / DR_SECTOR_BLKSIZE;
            DrPartitionInfoT  tInfo           = XdrNew (tXdr, DR_PARTITION_INFO);

            if(TRUE == IS_EXTENDED(psPt->cType))
            {
              tInfo->lKind = eDrPartitionKindExtended;
            }
            else if(0 == llBase)
            {
              tInfo->lKind =  eDrPartitionKindPrimary;
            }
            else
            {
              tInfo->lKind = eDrPartitionKindLogical;
            }

            tInfo->pstrName  = StrDuplicate (strVolHandle);
            tInfo->ullSize   = ullSecNum * DR_SECTOR_BLKSIZE;
            tInfo->ullOffset = llOffset + (ullSecRel * DR_SECTOR_BLKSIZE);
            tInfo->ulEnd     = (unsigned long)(((ullOffsetSector + ullSecRel + ullSecNum) / 
                               SECTOR_PER_TRACK) / psGeo->ulHeads);
            tInfo->ulStart   = (unsigned long)(((ullOffsetSector + ullSecRel) / 
                               SECTOR_PER_TRACK) / psGeo->ulHeads);
            tInfo->bActive   = (psPt->cBootFlag == 0x80);
            tInfo->lType     = psPt->cType;
            tInfo->lIndex    = lIndex++;

            GetPartitionType (&tInfo->pstrType, psPt->cType);

            DRTRACE("");
            DRTRACE("Volume of Disk: %s", DROVSTR (pstrDiskHandle));
            DRTRACE("  ID:       [%ld]",  tInfo->lIndex);
            DRTRACE("  Handle:   [%s]",   DROVSTR (strVolHandle));
            DRTRACE("  Type:     [%s]",   DROVSTR (tInfo->pstrType));
            DRTRACE("  Size:     [%s]",   droGroupDigit (tInfo->ullSize));
            DRTRACE("  Offset:   [%s]",   droGroupDigit (tInfo->ullOffset));
            DRTRACE("  Active:   [%s]",   STR_BOOLEAN (tInfo->bActive));

            XdrArrayAdd (ptPtEntries, &tInfo, 0);
          }
        }
      }
    }

    for (i=0; FALSE == bFailed && i<4; i++)
    {
      DrPartitionT *psPt = PT_OFFSET(strBuf, i);

      if (TRUE == IS_EXTENDED (psPt->cType))
      {
        bFailed = (FALSE == droReadMbrVolumeTableRecursive (
                            tXdr,
                            lFd,
                            pFd,
                            pstrDiskHandle,
                            psGeo,
                            (0 == llBase) ? PT_RELATIVE_OFFSET(psPt) : llBase,
                            llBase + PT_RELATIVE_OFFSET(psPt),
                            (0 == llBase) ? 5 : lIndex,
                            ptPtEntries,
                            ptSecEntries));
      }
    }
    bSuccess = (FALSE == bFailed);
  }
  
	return bSuccess;
}





/*
** ============================================================================
** ============================================================================
*/
INTERNAL BooleanT fncbLookExt (int lFd, uint64_t llStart, uint64_t llEnd, BooleanT bExt3)
{
  BooleanT  bSuccess = FALSE;
  ByteT     strBuf[1024];

  MemZero (strBuf, sizeof (strBuf));

  if (TRUE != DrSeek (lFd, (llStart + 2) * 512))
  {
    TRACE((41,0,ALWAYS,"Failed to seek to postion: 0x%llx", (llStart + 2) * 512));
  }
  else if (sizeof (strBuf) != read (lFd, strBuf, sizeof (strBuf)))
  {
    TRACE((42,0,ALWAYS,"Failed to Read Ext2 SuperBlock. %s", strerror(errno)));
  }
  else
  {
    uint16_t lMagic   = 0;
    uint32_t lJournal = 0;

    memcpy (&lMagic,   strBuf + 0x38, 2);
    memcpy (&lJournal, strBuf + 0x5c, 4);

    bExt3 ? (bSuccess = (lMagic == 0xef53) & ((lJournal & 0x4) != 0)) : 
            (bSuccess = (lMagic == 0xef53));
  }
  return bSuccess;
}




/*
** ============================================================================
** ============================================================================
*/
INTERNAL BooleanT fncbLookExt3 (int lFd, uint64_t llStart, uint64_t llEnd)
{
  return fncbLookExt (lFd, llStart, llEnd, TRUE);
}


/*
** ============================================================================
** ============================================================================
*/
INTERNAL BooleanT fncbLookExt2 (int lFd, uint64_t llStart, uint64_t llEnd)
{
  return fncbLookExt (lFd, llStart, llEnd, FALSE);
}


/*
** ============================================================================
** ============================================================================
*/
INTERNAL BooleanT fncbLookNtfs (int lFd, uint64_t llStart, uint64_t llEnd)
{
  BooleanT  bSuccess = FALSE;
  char     strBuf [512];

  if (TRUE != DrSeek (lFd, llStart * 512))
  {
    TRACE((43,0,ALWAYS,"Failed to seek to postion: 0x%llx", llStart * 512));
  }
  else if (sizeof (strBuf) != read (lFd, strBuf, sizeof (strBuf)))
  {
    TRACE((44,0,ALWAYS,"Failed to Read NTFS Buffer"));
  }
  else
  {
    bSuccess = ! strncmp (strBuf + 3, "NTFS", 4);
  }
  return bSuccess;
}



/*
** ============================================================================
** ============================================================================
*/
INTERNAL BooleanT fncbLookSwap (int lFd, uint64_t llStart, uint64_t llEnd)
{
  BooleanT  bSuccess = FALSE;
  long      lSize    = getpagesize ();

  if (0 == lSize)
  {
    TRACE((45,0,ALWAYS,"Failed to to get PageSize for swap"));
  }
  else if (TRUE != DrSeek (lFd, llStart * 512))
  {
    TRACE((46,0,ALWAYS,"Failed to seek to postion: 0x%llx", llStart * 512));
  }
  else
  {
    char *pstrBuf = (char *) MemAllocate (lSize);
    char *pstr    = pstrBuf;

    if (NULL == pstrBuf)
    {
      TRACE((47,0,ALWAYS,"Failed to get memory for Swap Buffer"));
    }
    else if (lSize != read (lFd, pstrBuf, lSize))
    {
      TRACE((48,0,ALWAYS,"Failed to Read Swap Buffer"));
    }
    else
    {
      pstr += lSize - 10;
      
      bSuccess = (! strncmp (pstr, "SWAP-SPACE", 10)) | (! strncmp (pstr, "SWAPSPACE2", 10));
    }
    MemFree (&pstrBuf);
  }
  return bSuccess;
}



/*
** ============================================================================
** ============================================================================
*/
INTERNAL BooleanT fncbLookFat (int lFd, uint64_t llStart, uint64_t llEnd)
{
  BooleanT  bSuccess = FALSE;
  ByteT     strBuf [512];

  if (TRUE != DrSeek (lFd, llStart * 512))
  {
    TRACE((49,0,ALWAYS,"Failed to seek to postion: 0x%llx", llStart * 512));
  }
  else if (sizeof (strBuf) != read (lFd, strBuf, sizeof (strBuf)))
  {
    TRACE((50,0,ALWAYS,"Failed to Read FAT Buffer"));
  }
  else
  {
    uint16_t  lSig     = 0;
    uint8_t   lSys     = 0;
    uint16_t  lSecSize = 0;

    memcpy (&lSig,     strBuf + 0x1fe, 2);
    memcpy (&lSecSize, strBuf + 0x00b, 2);

    lSys = strBuf[0x3];
      
    bSuccess = (lSig == 0xaa55) & (lSecSize == 0x200) & (0 != lSys);
  }
  return bSuccess;
}


/*
** ============================================================================
** ============================================================================
*/
INTERNAL void GPTDisplayPartitionTable (
              BooleanT      bPrimary,
              GPTHdrT       *ptHdr,
              int            lFd,
              FILE          *pFd,
              const char    *pstrDev,
              DrGeometryT    *psGeo)
{
  BooleanT   bContinue = TRUE;
  uint64     llPos     = ptHdr->llLbaPtEntry * 512;
  int        lCount    = ptHdr->lEntryCnt;
  int        i;
  GPTEntryT  tEnt;
  char       strPrint[256];
  long       l_maj = -1;
  long       l_min = -1;
  
  TRACE((51,0,ALWAYS,"Enter Display GPT Partition Table"));

  bPrimary ?
  sprintf(strPrint,"*** GPT Partition HDR [PRIMARY], Size: %d, Offset: 0x%llx",
  (int) sizeof (GPTHdrT), (uint64) 512) :
  sprintf(strPrint,"*** GPT Partition HDR [BACKUP], Size: %d, Offset: 0x%llx",
  (int) sizeof (GPTHdrT), (uint64) (psGeo->ullSize - 512));

  TRACE((52,0,ALWAYS,""));
  TRACE((53,0,ALWAYS, strPrint));
  TRACE((54,0,ALWAYS,"    Signature:       0x%llx", GUID_SIG (ptHdr->llSig)));
  TRACE((55,0,ALWAYS,"    HDR Size:        0x%0x",  ptHdr->lHdrSize));
  TRACE((56,0,ALWAYS,"    HDR CRC:         0x%0x",  ptHdr->lHdrCrc));
  TRACE((57,0,ALWAYS,"    HDR LBA Primary: 0x%llx", ptHdr->llLbaPrime));
  TRACE((58,0,ALWAYS,"    HDR LBA Backup:  0x%llx", ptHdr->llLbaBack));
  TRACE((59,0,ALWAYS,"    LBA Use First:   0x%llx", ptHdr->llUlbaFirst));
  TRACE((60,0,ALWAYS,"    LBA Use Last:    0x%llx", ptHdr->llUlbaLast));
  TRACE((61,0,ALWAYS,"    LBA Entry Pos:   0x%llx", ptHdr->llLbaPtEntry));
  TRACE((62,0,ALWAYS,"    Entry Count:     0x%0x",  ptHdr->lEntryCnt));
  TRACE((63,0,ALWAYS,"    Entry Size:      0x%0x",  ptHdr->lEntrySize));
  TRACE((64,0,ALWAYS,"    Entry CRC:       0x%0x",  ptHdr->lEntryCrc));
  TRACE((65,0,ALWAYS,"    IO_SIZE:         0x%llx", psGeo->ullSize));
  TRACE((66,0,ALWAYS,"    IO_TYPE:         %s",((psGeo->ullSize / 512) % 2) ? "Odd":"Even"));
  TRACE((67,0,ALWAYS,""));

  for (i=0; i<lCount && TRUE == bContinue; i++)
  {
    MemZero (&tEnt, sizeof (GPTEntryT));
    
    llPos = (uint64) (1024 + (i * sizeof (GPTEntryT)));

    if (TRUE != DrSeek (lFd, llPos))
    {
      TRACE((68,0,ALWAYS,"Failed to seek to position: 0x%llx", llPos));
      bContinue = FALSE;
    }
    else if (sizeof (GPTEntryT) != read (lFd, &tEnt, sizeof (GPTEntryT)))
    {
      TRACE((69,0,ALWAYS,"Failed to Read GPT Entry. %d", errno));
      bContinue = FALSE;
    }
    else
    {
      uint64 llTypeH = 0;
      uint64 llTypeL = 0;
      uint64 llPtH   = 0;
      uint64 llPtL   = 0;
          
      memcpy (&llTypeH, ((char *) &tEnt.tGuidType), 8);
      memcpy (&llTypeL, ((char *) &tEnt.tGuidType) + 8, 8);

      if (TRUE != PT_UNUSE (llTypeH, llTypeL))
      {
        GPTFsRecT  *ptGPTFsRec = &gtGPTFsRec[0];
        BooleanT   bFound      = FALSE;
        char       strDev [128];
        char       strFs  [128];

        MemZero (strDev, sizeof (strDev));
        MemZero (strFs,  sizeof (strFs));
        memcpy (&llPtH, ((char *) &tEnt.tGuidPt), 8);
        memcpy (&llPtL, ((char *) &tEnt.tGuidPt) + 8, 8);

        while (NULL != ptGPTFsRec->pstrFs && TRUE != bFound)
        {
          bFound = ptGPTFsRec->fncbFsLookup (lFd, tEnt.llLbaStart, tEnt.llLbaEnd);

          if (TRUE == bFound)
          {
            break;
          }
          ptGPTFsRec++;
        }
        sprintf(strFs,"%s", (TRUE == bFound && NULL != ptGPTFsRec->pstrFs) ?
        ptGPTFsRec->pstrFs : "UNKNOWN");
        
        if (TRUE !=  GPTGetPartitionName (
                    &l_maj,
                    &l_min,
                    pFd,
                    pstrDev, 
                    strDev,
                    sizeof (strDev),
                    ((tEnt.llLbaEnd - tEnt.llLbaStart) + 1) / 2))
        {
          strcpy (strDev, "UNKNOWN");
        }
        
        TRACE((70,0,ALWAYS,"    GPT Entry [%d], Size: %u, Offset: 0x%llx", i, sizeof (GPTEntryT), llPos));
        TRACE((71,0,ALWAYS,"    EFI System:   %s",         STR_BOOLEAN (PT_EFI(llTypeH, llTypeL))));
        TRACE((72,0,ALWAYS,"    GUID Type:    0x%llx%llx", GUID_SIG (llTypeH), GUID_SIG (llTypeL)));
        TRACE((73,0,ALWAYS,"    GUID Pt:      0x%llx%llx", GUID_SIG (llPtH),   GUID_SIG (llPtL)));
        TRACE((74,0,ALWAYS,"    LBA Start:    0x%llx",     tEnt.llLbaStart));
        TRACE((75,0,ALWAYS,"    LBA End:      0x%llx",     tEnt.llLbaEnd));
        TRACE((76,0,ALWAYS,"    Filesystem:   %s",         strFs));
        TRACE((77,0,ALWAYS,"    Device Name:  %s",         strDev));
        TRACE((78,0,ALWAYS,""));
      }
    }
  }
}







/*
** ============================================================================
** ============================================================================
*/
INTERNAL BooleanT droReadGptVolumeTable
(
	XdrT          tXdr,
	int           lFd,
	FILE          *pFd,
	const char    *pstrDev,
	DrGeometryT   *psGeo,
	XdrArrayT     *ptPtEntries
)
{
  BooleanT   bSuccess = FALSE;
  GPTHdrT    *ptHdrP  = (GPTHdrT *)   MemAllocate (sizeof (GPTHdrT));
  GPTHdrT    *ptHdrB  = (GPTHdrT *)   MemAllocate (sizeof (GPTHdrT));
  GPTEntryT  *ptEnt   = (GPTEntryT *) MemAllocate (sizeof (GPTEntryT));
  LastSecT   tLast; 

  tLast.lBlock  = 0;
  tLast.lSize   = sizeof (GPTHdrT);
  tLast.pstrBuf = (ByteT *) ptHdrB;

  if (NULL == ptHdrP)
  {
    TRACE((80,0,ALWAYS,"No memory for GPT HDR Primary"));
  }
  else if (NULL == ptHdrB)
  {
    TRACE((81,0,ALWAYS,"No memory for GPT HDR Backup"));
  }
  else if (NULL == ptEnt)
  {
    TRACE((82,0,ALWAYS,"No memory for GPT Entry"));
  }
  else if (TRUE != DrSeek (lFd, 512))
  {
    TRACE((83,0,ALWAYS,"Failed to seek to position: 512"));
  }
  else if (sizeof (GPTHdrT) != read (lFd, ptHdrP, sizeof (GPTHdrT)))
  {
    TRACE((84,0,ALWAYS,"Failed to Read GPT Hdr. %s", strerror(errno)));
  }
  else if (0 != DRioctl (lFd, _IO(0x12,108), &tLast))
  {
    TRACE((85,0,ALWAYS,"ERROR: Failed to ask Kernel to get last sector"));
  }
  else
  {
    uint64 llPos  = ptHdrP->llLbaPtEntry * 512;
    int    lCount = ptHdrP->lEntryCnt;
    int    lIdx   = 1;
    int    i;
    long   l_maj  = -1;
    long   l_min  = -1;

    GPTDisplayPartitionTable (TRUE,  ptHdrP, lFd, pFd, pstrDev, psGeo);
    GPTDisplayPartitionTable (FALSE, ptHdrB, lFd, pFd, pstrDev, psGeo);

    bSuccess = TRUE;
    
    for (i=0; i<lCount && TRUE == bSuccess; i++)
    {
      MemZero (ptEnt, sizeof (GPTEntryT));
    
      llPos = (uint64) (1024 + (i * sizeof (GPTEntryT)));
            
      if (TRUE != DrSeek (lFd, llPos))
      {
        TRACE((86,0,ALWAYS,"Failed to seek to position: 0x%llx", llPos));
        bSuccess = FALSE;
      }
      else if (sizeof (GPTEntryT) != read (lFd, ptEnt, sizeof (GPTEntryT)))
      {
        TRACE((87,0,ALWAYS,"Failed to Read GPT Entry. %d", errno));
        bSuccess = FALSE;
      }
      else
      {
        uint64 llTypeH = 0;
        uint64 llTypeL = 0;
            
        memcpy (&llTypeH, ((char *) &ptEnt->tGuidType), 8);
        memcpy (&llTypeL, ((char *) &ptEnt->tGuidType) + 8, 8);

        if (TRUE != PT_UNUSE (llTypeH, llTypeL))
        {
          GPTFsRecT  *ptGPTFsRec = &gtGPTFsRec[0];
          BooleanT   bFound      = FALSE;
          char       strDev [128];
          
          MemZero (strDev, sizeof (strDev));

          while (NULL != ptGPTFsRec->pstrFs && TRUE != bFound)
          {
            bFound = ptGPTFsRec->fncbFsLookup (lFd, ptEnt->llLbaStart, ptEnt->llLbaEnd);

            if (TRUE == bFound)
            {
              break;
            }
            ptGPTFsRec++;
          }

          if (TRUE !=  GPTGetPartitionName (
                      &l_maj,
                      &l_min,
                      pFd,
                      pstrDev, 
                      strDev,
                      sizeof (strDev),
                      ((ptEnt->llLbaEnd - ptEnt->llLbaStart) + 1) / 2))
          {
            TRACE((88,0,ALWAYS,"Failed to get Partition Device Name from PROC. Offset: 0x%llx",
            ptEnt->llLbaStart));
            bSuccess = FALSE;
          }
          else
          {
            DrPartitionInfoT  tInfo = XdrNew (tXdr, DR_PARTITION_INFO);

            (TRUE == bFound && NULL != ptGPTFsRec->pstrFs) ?
            (tInfo->pstrType = StrDuplicate (ptGPTFsRec->pstrFs)) :
            (tInfo->pstrType = StrDuplicate ("UNKNOWN"));

            tInfo->pstrName   = StrDuplicate (strDev);
            tInfo->lKind      = eDrPartitionKindPrimary;
            tInfo->ullSize     = (ptEnt->llLbaEnd - ptEnt->llLbaStart + 1) * 512;
            tInfo->ullOffset  = ptEnt->llLbaStart * 512;
            tInfo->ulEnd       = ptEnt->llLbaEnd / 1024;
            tInfo->ulStart     = ptEnt->llLbaStart / 1024;
            tInfo->bActive     = FALSE;
            tInfo->lType       = 0xee;
            tInfo->lIndex     = lIdx++;

            XdrArrayAdd (ptPtEntries, &tInfo, 0);
          }
        }
      }
    }
  }
  MemFree (&ptHdrP);
  MemFree (&ptHdrB);
  MemFree (&ptEnt);

  return bSuccess;
}





/*
** ============================================================================
** ============================================================================
*/
INTERNAL BooleanT GPTGetPartitionTable (
                  XdrT          tXdr,
                  int            lFd,
                  const char    *pstrDev,
                  XdrArrayT      *ptArray)
{
  BooleanT   bSuccess = FALSE;
  GPTHdrT    tHdrP;
  GPTHdrT    tHdrB;
  ByteT      strPmbr[512];
  LastSecT   tLast; 
    
  tLast.lBlock  = 0;
  tLast.lSize   = sizeof (GPTHdrT);
  tLast.pstrBuf = (ByteT *) &tHdrB;

  TRACE((89,0,ALWAYS,"Enter Backup GPT Get Partition Table"));
  
  if (TRUE != DrSeek (lFd, 0))
  {
    TRACE((90,0,ALWAYS,"Failed to seek to position: 0"));
  }
  else if (sizeof (strPmbr) != read (lFd, strPmbr, sizeof (strPmbr)))
  {
    TRACE((91,0,ALWAYS,"Failed to Read PMBR. %s", strerror(errno)));
  }
  else if (TRUE != DrSeek (lFd, 512))
  {
    TRACE((92,0,ALWAYS,"Failed to seek to position: 512"));
  }
  else if (sizeof (GPTHdrT) != read (lFd, &tHdrP, sizeof (GPTHdrT)))
  {
    TRACE((93,0,ALWAYS,"Failed to Read Primary GPT Hdr. %s", strerror(errno)));
  }
  else if (0 != DRioctl (lFd, _IO(0x12,108), &tLast))
  {
    TRACE((94,0,ALWAYS,"ERROR: Failed to ask Kernel to get last sector"));
  }
  else if (tHdrB.llSig != tHdrP.llSig)
  {
    TRACE((95,0,ALWAYS,"ERROR: Primary GUID Sig is not the same as a Backup GUID Sig"));
    TRACE((96,0,ALWAYS,"       Please fix this before backup can happen"));
    TRACE((97,0,ALWAYS,"       Primary Sig: 0x%llx", GUID_SIG (tHdrP.llSig)));
    TRACE((98,0,ALWAYS,"       Backup Sig:  0x%llx", GUID_SIG (tHdrB.llSig)));
  }
  else if (tHdrB.lEntryCnt != tHdrP.lEntryCnt)
  {
    TRACE((99,0,ALWAYS,"ERROR: Primary GUID Entry Count is not the same as a Backup GUID Entry Count"));
    TRACE((100,0,ALWAYS,"       Please fix this before backup can happen"));
    TRACE((101,0,ALWAYS,"       Primary Sig: %u", tHdrP.lEntryCnt));
    TRACE((102,0,ALWAYS,"       Backup Sig:  %u", tHdrB.lEntryCnt));
  }
  else
  {
    DrDataT  ptData = NULL;
    long     lCount  = (tHdrP.lEntryCnt + ((- tHdrP.lEntryCnt) & 3)) / 4;
    long     i;

    TRACE((103,0,ALWAYS,"Partiton entry count: %ld", tHdrP.lEntryCnt));
    TRACE((104,0,ALWAYS,"Partiton Used Buffer: %ld", lCount));

    TRACE((105,0,ALWAYS,"Backup PMBR"));

    ptData = XdrNew (tXdr, DR_DATA);
    ptData->tData.lSize  = sizeof (strPmbr);
    ptData->tData.pData  = MemAllocate (sizeof (strPmbr));
    MemCopy (ptData->tData.pData, strPmbr, sizeof (strPmbr));
    XdrArrayAdd  (ptArray, &ptData, 0);
    
    TRACE((106,0,ALWAYS,"Backup GPT Primary HDR"));

    ptData = XdrNew (tXdr, DR_DATA);
    ptData->tData.lSize  = sizeof (GPTHdrT);
    ptData->tData.pData  = MemAllocate (sizeof (GPTHdrT));
    MemCopy (ptData->tData.pData, &tHdrP, sizeof (GPTHdrT));
    XdrArrayAdd  (ptArray, &ptData, 0);
    
    TRACE((107,0,ALWAYS,"Backup GPT Backup HDR"));
    
    ptData = XdrNew (tXdr, DR_DATA);
    ptData->tData.lSize  = sizeof (GPTHdrT);
    ptData->tData.pData  = MemAllocate (sizeof (GPTHdrT));
    MemCopy (ptData->tData.pData, &tHdrB, sizeof (GPTHdrT));
    XdrArrayAdd  (ptArray, &ptData, 0);
  
    if (0 == lCount)
    {
      TRACE((108,0,ALWAYS,"ERROR: Parition count is ZERO"));
    }
    else if (TRUE != DrSeek (lFd, 1024))
    {
      TRACE((109,0,ALWAYS,"ERROR: Failed to seek to position: 1024"));
    }
    else
    {
      ByteT strBuf[512];

      bSuccess = TRUE;
      
      TRACE((110,0,ALWAYS,"Begin Backup GPT PRIMARY Entries at Offset: 0x%llx", (uint64) 1024));

      for (i=0; i<lCount && TRUE == bSuccess; i++)
      {
        if (sizeof (strBuf) != read (lFd, strBuf, sizeof (strBuf)))
        {
          TRACE((111,0,ALWAYS,"ERROR: Failed to read GPT Parition Table buffer of 512 bytes"));
          bSuccess = FALSE;
        }
        else
        {
          ptData = XdrNew (tXdr, DR_DATA);
          
          ptData->tData.lSize = sizeof (strBuf);
          ptData->tData.pData = MemAllocate (sizeof (strBuf));
          MemCopy (ptData->tData.pData, strBuf, sizeof (strBuf));
          XdrArrayAdd  (ptArray, &ptData, 0);
        }
      }
      
      TRACE((112,0,ALWAYS,"Begin Backup GPT BACKUP Entries at Offset: 0x%llx", 
      (uint64) tHdrB.llLbaPtEntry * 512));
      
      if (TRUE != DrSeek (lFd, tHdrB.llLbaPtEntry * 512))
      {
        TRACE((113,0,ALWAYS,"ERROR: Failed to seek to Backup First Partition Entry: 0x%llx",
        tHdrB.llLbaPtEntry * 512));
      }
      else
      {
        for (i=0; i<lCount && TRUE == bSuccess; i++)
        {
          if (sizeof (strBuf) != read (lFd, strBuf, sizeof (strBuf)))
          {
            TRACE((114,0,ALWAYS,"ERROR: Failed to read GPT Parition Table buffer of 512 bytes"));
            bSuccess = FALSE;
          }
          else
          {
            ptData = XdrNew (tXdr, DR_DATA);
            
            ptData->tData.lSize  = sizeof (strBuf);
            ptData->tData.pData  = MemAllocate (sizeof (strBuf));
            MemCopy (ptData->tData.pData, strBuf, sizeof (strBuf));
            XdrArrayAdd  (ptArray, &ptData, 0);
          }
        }
      }
    }
  }
  TRACE((115,0,ALWAYS,"Total number of buffer: %ld", XdrArrayCount (*ptArray)));
  TRACE((116,0,ALWAYS,"Leave GPT Get Parition Table Primary and Backup. (%s)", 
  STR_BOOLEAN(bSuccess)));

  return bSuccess;
}







/*
** ============================================================================
** ============================================================================
*/
/*
** Private Function:
**   droReadSolarisVolumeTable
**
** Descrtiption:
**   This function has dual purpose driven by the inputs.
**   - If a partition table result list pointer is passed in, the partition
**     table is analysed and added to the result list.
**   - If a sector entry result list pointer is passed in, the entire disk
**     label and boot sectors are added to the list.
**   For Solaris devices, the only supported partition table is in the
**   disk label (sector 0 of the device) and the only supported boot sectors
**   are sectors 1-15 (the 2nd-16th sectors on the disk).
**   Attempts to read disk labels and boot sectors from partition devices
**   will probably result in undesirable output ... use the device file name.
**   XXX it would be better if this routine was only used to read the disk
**   XXX label (512 bytes) and a 'get bootstrap'  routine was used to get
**   XXX the 8192-512b that is the boot program ... and even another routine
**   XXX to just get the partition table (about 96 bytes)
**
** Arguments:
**   tXdr - OUT - pointer to hang result structure on
**   lFd - open file descriptor to device file for target disk
**   pFd - open stream pointer to /proc/partitions file
**   pstrDev - file name of device representing target disk
**   psGeo - disk geometry as read from solaris disk label (C/H/S/Bytes)
**   base, offset, index - legacy to DOS partition scheme, may be useful later
**     unused for now, but extended parts in Sol10 or LVM might need this
**     set to 0,0,1 for calls now
**   ptPtEntries - I/O - if not NULL, add partition results to this list
**   ptSecEntries - I/O - if not NULL add boot record result to this list
**
** Note:
**   There are some leftover offset variables from the DOS implementation.
**   These may come in handy for extended Solaris functionality SOON.
**
** Results:
**   TRUE and valid data partition table and/or disk label data in tXdr and
**     respective lists
**   FALSE on a number of I/O errors failing to read data from the device
**     result structures are unchanged
*/
INTERNAL BooleanT droReadSolarisVolumeTable(
                  XdrT          tXdr,
                  int            lFd,
                  FILE          *pFd,
                  const char    *pstrDev,
                  DrGeometryT    *psGeo,
                  XdrArrayT      *ptPtEntries,
                  XdrArrayT      *ptSecEntries)
{
  BooleanT   bSuccess = FALSE;
  ByteT     strBuf[DR_SOLARIS_MBR_SIZE];
  
  TRACE((117,0,ALWAYS,"Begin droReadSolarisVolumeTable for device: %s",pstrDev));

  if (TRUE != DrSeek (lFd, 0))
  {
    TRACE((118,0,ALWAYS,"Failed to seek to position: 0x0"));
  }
  else if (DR_SOLARIS_MBR_SIZE != read( lFd, strBuf, DR_SOLARIS_MBR_SIZE ))
  {
    TRACE((119,0,ALWAYS,"Failed to read %i bytes from device. %s",
      DR_SOLARIS_MBR_SIZE, strerror(errno)));
  }
  else if (TRUE != ValidateSolarisSignature( strBuf ))
  {
    TRACE((120,0,ALWAYS,"Invalid Solaris disk label signature: 0x%02x 0x%02x",
      strBuf[508], strBuf[509]));
  }
  else
  {
    BooleanT bFailed = FALSE;
    uint32 i = 0;

    if ( NULL != ptSecEntries )
    {
      DrDataT tEntry = XdrNew( tXdr, DR_DATA );
      DrSolarisBootSectorT *ptRecord = NULL;

      tEntry->tData.lSize = sizeof( DrSolarisBootSectorT );

      TRACE((121,0,ALWAYS,"Allocating memory in droReadSolarisVolumeTable - %d",
        tEntry->tData.lSize ));

      tEntry->tData.pData = MemAllocate( tEntry->tData.lSize );
      ptRecord            = tEntry->tData.pData;
      ptRecord->llOffset  = 0;

      MemCopy( ptRecord->cBuffer, strBuf, DR_SOLARIS_MBR_SIZE );
      XdrArrayAdd( ptSecEntries, &tEntry, 0 );
    }

    /*
    ** Get device file names for slices(partitions)
    */
    if ( NULL != ptPtEntries )
    {
      SunDiskLabelT *ptSunDL = (SunDiskLabelT *)strBuf;
      uint32 iIndex = 1;
      
      for ( i = 0; FALSE == bFailed && SDL_SLICE_COUNT > i; i++ )
      {
        /*
        ** only report names for slices with non-zero lengths
        */
        if ( 0 != ptSunDL->atSliceGeom[i].uiSectors )
        {
          char strPtName [128];

          /*
          ** sequentially read partition names from /proc/partitions
          ** and match expected partition sizes to verify
          */
          if ( TRUE != GetSolarisPartitionName( pFd, pstrDev, strPtName,
            sizeof( strPtName ), ptSunDL->atSliceGeom[i].uiSectors / 2 ) )
          {
            TRACE((122,0,ALWAYS,"ERROR: Failed to get %s %s %s %lu",
              "Partition name of Device:", pstrDev,
              "Where Partition Size is:",
              ptSunDL->atSliceGeom[i].uiSectors / 2 ));

            bFailed = TRUE;
          }
          else
          {
            /*
            ** partition name retrieval was successful, so get geometry
            ** and attributes
            */

            DrPartitionInfoT tInfo = XdrNew( tXdr, DR_PARTITION_INFO );

            uint64 ullSecRel = ptSunDL->usHeads * ptSunDL->usSect *
              ptSunDL->atSliceGeom[i].uiStartCyl;

            /* XXX make new Kind for slices ? */
            tInfo->lKind = eDrPartitionKindPrimary;
            tInfo->pstrName = StrDuplicate (strPtName);

            /*
            ** calculate byte offset and size of slice
            */
            tInfo->ullOffset = (uint64)ullSecRel * DR_SECTOR_BLKSIZE;
            tInfo->ullSize = ptSunDL->atSliceGeom[i].uiSectors *
              (uint64)DR_SECTOR_BLKSIZE;

            /*
            ** calculate start and end cylinders
            */
            tInfo->ulStart = ptSunDL->atSliceGeom[i].uiStartCyl;
            tInfo->ulEnd = tInfo->ulStart +
              (ptSunDL->atSliceGeom[i].uiSectors / ptSunDL->usHeads /
               ptSunDL->usSect );

            /*
            ** boot partition is a special type in Solaris disk label
            */
            tInfo->bActive =
              (SDL_ST_BOOT == ptSunDL->atSliceAttrs[i].usSliceType);

            tInfo->lType = ptSunDL->atSliceAttrs[i].usSliceType |
              DR_PART_TYPE_PLATFORM_SOLARIS;
            tInfo->lIndex = iIndex++;

            GetSolarisPartitionTypeString( &tInfo->pstrType, tInfo->lType );

            TRACE((123,0,ALWAYS,"*** Partition Info ***"));
            TRACE((124,0,ALWAYS,"    Name:   %s",    tInfo->pstrName));
            TRACE((125,0,ALWAYS,"    Type:   %s",    tInfo->pstrType));
            TRACE((126,0,ALWAYS,"    Size:   %#llx", tInfo->ullSize));
            TRACE((127,0,ALWAYS,"    Offset: %#llx", tInfo->ullOffset));

            XdrArrayAdd (ptPtEntries, &tInfo, 0);
          }
        }
      }
    } /* NULL != ptEntries */

    bSuccess = (FALSE == bFailed);
  }

  TRACE((128,0,ALWAYS,"End droReadSolarisVolumeTable for Device: %s. %s",
    pstrDev, STR_BOOLEAN(bSuccess)));

  return bSuccess;
}









/*
** ============================================================================
** ====================================================================================================
** ====================================================================================================
*/
INTERNAL BooleanT droIsDiskGpt (int lFd, char *pstrDiskHandle)
{
  BooleanT   bRC    = FALSE;
  uint64     llSigP = 0;
  uint64     llSigB = 0;
  LastSecT   tLastSec;

	DRTRACE("Determine disk style for disk device: [%s]", DROVSTR (pstrDiskHandle));

  tLastSec.lBlock  = 0;
  tLastSec.lSize   = 8;
  tLastSec.pstrBuf = (ByteT *) &llSigB;

  if (TRUE != DrSeek (lFd, 512))
  {
    DRTRACE("ERROR: Failed to seek to postition: 512, of device: %s", pstrDiskHandle);
  }
  else if (sizeof (uint64) != read (lFd, &llSigP, sizeof (uint64)))
  {
    DRTRACE("ERROR: Failed to read GPT Signature of device: %s", pstrDiskHandle);
  }
  else if (0 != DRioctl (lFd, _IO(0x12,108), &tLastSec))
  {
    DRTRACE("ERROR: Failed to ask Kernel to get last sector");
  }
  else
  {
    bRC = (GUID_SIG (llSigP) == GPT_GUID_SIG) & (GUID_SIG (llSigB) == GPT_GUID_SIG);
  }

	DRTRACE("Disk device: [%s] is [%s]", DROVSTR (pstrDiskHandle), bRC ? "GPT" : "MBR");

  return bRC;
}




/*
** ============================================================================
** ============================================================================
*/

/*
** Public Function:
**   GetSectorsBR
**
** Description:
**   Get the boot record for a device given its device file name
**
** Arguments:
**   tXdr - OUT - buffer for boot record for device
**   pstrDev - string containing device file name for partition or full device
**   tReply - OUT - holds the partition or list of partitions
**
** Globals:
**   gtMachineArch - machine architecture => OS & disk format
**
** Returns:
**   TRUE and valid partition reply on success
**   FALSE and undefined partition reply on failure
*/

EXTERN BooleanT GetSectorsBR(
  XdrT tXdr,
  char *pstrDev,
  DrGetPartitionTableReplyT tReply
)
{
  BooleanT   bSuccess   = FALSE;
  int        lFd       = open64 (pstrDev, O_RDONLY);

  TRACE((134,0,ALWAYS,"Begin GetSectorsBR for device: %s", pstrDev));

  if (-1 == lFd)
  {
    TRACE((135,0,ALWAYS,"Failed to open device: %s. %s",
      pstrDev, strerror(errno)));
    tReply->lError = NDMP_IO_ERR;
  }
  else if (DR_ARCH_SOLARIS == gtMachineArch)
  {
    if (TRUE != (bSuccess = droReadSolarisVolumeTable (
                            tXdr, 
                            lFd, 
                            NULL, 
                            pstrDev, 
                            NULL, 
                            NULL, 
                            &tReply->oRecords)))
    {
      TRACE((136,0,ALWAYS,"Failed to read Solaris partition data for device: %s", pstrDev));
      tReply->lError = NDMP_IO_ERR;
      XdrDestroyArray (&tReply->oRecords);
    }
  }
  else if (TRUE == droIsDiskGpt (lFd, pstrDev))
  {
    if (TRUE != (bSuccess = GPTGetPartitionTable (
                            tXdr, 
                            lFd, 
                            pstrDev, 
                            &tReply->oRecords)))
    {
      TRACE((137,0,ALWAYS,"Failed to get GPT partition table for device: %s", pstrDev));
      tReply->lError = NDMP_IO_ERR;
      XdrDestroyArray (&tReply->oRecords);
    }
  }
  else if (TRUE != (bSuccess =  droReadMbrVolumeTableRecursive ( 
                                tXdr, 
                                lFd, 
                                NULL, 
                                pstrDev, 
                                NULL, 
                                0, 
                                0, 
                                1, 
                                NULL, 
                                &tReply->oRecords)))
  {
    TRACE((138,0,ALWAYS,"Failed to read Lagacy partition table for device: %s", pstrDev));
    tReply->lError = NDMP_IO_ERR;
    XdrDestroyArray (&tReply->oRecords);
  }
  else
  {
    close(lFd);
    bSuccess = TRUE;
  }
  TRACE((139,0,ALWAYS,"End GetSectorsBR for device: %s. %s",
  pstrDev, STR_BOOLEAN(bSuccess)));

  return bSuccess;
}








/*
** ====================================================================================================
**
** Public Function:
**   droGetAllVolume
**
** Description:
**   Gets list of partitions available for backup on device.
**
** Arguments:
**   tXdr - OUT - storage pointer for partitions result structure
**   pstrDev - device file name string of device to get partitions for
**   oReply - OUT - success value and list head to hang results on
**
** Globals:
**   gtMachineArch - machine architecture => OS & disk format
**
** Returns:
**   TRUE and valid partition reply on success
**   FALSE and undefined partition reply on failure
**
** ====================================================================================================
*/
EXTERN BooleanT droGetAllVolume (char *pstrDiskHandle, XdrT tXdr, DrPartitionListReplyT oReply)
{
  BooleanT      bRC  = FALSE;
  long          lFd  = -1;
	long          lErr = 0;
  FILE          *pFd = NULL;
  DrGeometryT   sGeo;

	DRTRACE("Enter get all volume for Disk: [%s]", DROVSTR (pstrDiskHandle));

	MemZero (&sGeo, sizeof (sGeo));

	if (TRUE == StrEmpty (pstrDiskHandle))
	{
		DRTRACE("ERROR: Failed to get all volume. Input disk handle is EMPTY");
		goto droExit;
	}

	lFd  = open64 (pstrDiskHandle, O_RDONLY);
	lErr = errno;
	
	DRTRACE("Open disk device OK: [%s]", pstrDiskHandle);
  
	if (-1 == lFd)
	{
		DRTRACE("ERROR: Failed to get all volume. Open error: [%s] [%s]",
		pstrDiskHandle, DROSYSERR (lErr));
		goto droExit;
	}

  if (DR_ARCH_SOLARIS == gtMachineArch)
	{
		if (TRUE != GetSolarisDeviceGeometry (lFd, &sGeo))
		{
			DRTRACE("ERROR: Failed to get all volume. Get disk SUN geometry error: [%s]",
			pstrDiskHandle);
			goto droExit;
		}
	}
	else
	{
		if (TRUE != droGetDeviceGeoDetail (lFd, &sGeo))
		{
			DRTRACE("ERROR: Failed to get all volume. Get disk geometry error: [%s]",
			pstrDiskHandle);
			goto droExit;
		}
	}
	
  pFd  = fopen ("/proc/partitions", "r");
	lErr = errno;

	if (NULL == pFd)
	{
		DRTRACE("ERROR: Failed to get all volume. Open /proc/partitions error: [%s]",
		DROSYSERR (lErr));
		goto droExit;
	}
	
	DRTRACE("Open proc interface OK: /proc/partitions");
	
	if (DR_ARCH_SOLARIS == gtMachineArch)
	{
		if (TRUE != droReadSolarisVolumeTable (tXdr,
		                                       lFd,
																					 pFd,
																					 pstrDiskHandle,
																					 &sGeo,
																					 &oReply->oPartitions,
																					 NULL))
		{
			XdrDestroyArray (&oReply->oPartitions);

			DRTRACE("ERROR: Failed to get all volume. Read SUN volume table error: [%s]",
			pstrDiskHandle);

			goto droExit;
		}
	}
	else if (TRUE == droIsDiskGpt (lFd, pstrDiskHandle))
	{
		if (TRUE != droReadGptVolumeTable (tXdr,
		                                   lFd,
																			 pFd,
																			 pstrDiskHandle,
																			 &sGeo,
																			 &oReply->oPartitions))
		{
			XdrDestroyArray (&oReply->oPartitions);
			
			DRTRACE("ERROR: Failed to get all volume. Read GPT volume table error: [%s]",
			pstrDiskHandle);

			goto droExit;
		}
	}
	else if (TRUE != droReadMbrVolumeTableRecursive (tXdr,
	                                                 lFd,
																									 pFd,
																									 pstrDiskHandle,
																									 &sGeo,
																									 0,
																									 0,
																									 1,
																									 &oReply->oPartitions,
																									 NULL))
	{
		XdrDestroyArray (&oReply->oPartitions);
			
		DRTRACE("ERROR: Failed to get all volume. Read MBR volume recursive table error: [%s]",
		pstrDiskHandle);

		goto droExit;
	}

	oReply->ullSize      = sGeo.ullSize;
	oReply->ulSectorSize = DR_SECTOR_BLKSIZE;

	DRTRACE("Get all volume for disk OK. Disk: [%s], VolumeCnt: [%ld]",
	pstrDiskHandle, XdrArrayCount (oReply->oPartitions));

	bRC = TRUE;

droExit:

	if (-1 != lFd)
	{
		close (lFd);
	}

	if (NULL != pFd)
	{
		fclose (pFd);
	}

	if (TRUE != bRC)
	{
    oReply->lError = NDMP_IO_ERR;
	}

	return bRC;
}











/*
** ============================================================================
** ============================================================================
*/
EXTERN int DRioctl (int lFd, int lCode, void *pVoid)
{
  int       lRC     = -1;
  LastSecT  *ptLast = (LastSecT *) pVoid;
  struct    utsname tInfo;
  int       lMaj = 0;
  int       lMin = 0;
  int       lRel = 0;
  int       lKver = 0;
      
  if (3 == sscanf (tInfo.release, "%d.%d.%d", &lMaj, &lMin, &lRel))
  {
    lKver = KERNEL_VER(lMaj,lMin,lRel);
  }
  else
  {
    lKver = KERNEL_VER(2,6,0);
  }

  if (lKver < KERNEL_VER(2,6,0))
  {
    TRACE((150,0,ALWAYS,"Kernel Version: 2.4. Use IOCTL to get/set last sector"));
    lRC = ioctl (lFd, lCode, pVoid);
  }
  else
  {
    DrGeometryT sGeo;

    TRACE((151,0,ALWAYS,"Kernel Version: 2.6. Use conventional method to get/set last sector"));
    
    if (TRUE != droGetDeviceGeoDetail (lFd, &sGeo))
    {
      TRACE((152,0,ALWAYS,"ERROR: Failed to get device GEO"));
    }
    else if (TRUE != DrSeek (lFd, sGeo.ullSize - 512))
    {
      TRACE((153,0,ALWAYS,"Failed to seek to POS: 0x%llx", sGeo.ullSize - 512));
    }
    else
    {
      if (lCode == _IO(0x12,108))
      {
        TRACE((154,0,ALWAYS,"Read last sector. Size: %ld", ptLast->lSize));

        if (ptLast->lSize != read (lFd, ptLast->pstrBuf, ptLast->lSize))
        {
          TRACE((155,0,ALWAYS,"Failed to Read 512 Bytes. %s", strerror(errno)));
        }
        else
        {
          lRC = 0;
        }
      }
      else if (lCode == _IO(0x12,109))
      {
        TRACE((156,0,ALWAYS,"Write last sector. Size: %ld", ptLast->lSize));

        if (ptLast->lSize != write (lFd, ptLast->pstrBuf, ptLast->lSize))
        {
          TRACE((157,0,ALWAYS,"Failed to Write 512 Bytes. %s", strerror(errno)));
        }
        else
        {
          lRC = 0;
        }
      }
    }
  }
  return lRC;
}



