#ifndef DRDEVICE_H
#define DRDEVICE_H

static const char drdevice_h[] = "$Header:";




#ifdef EXTERN
#undef EXTERN
#endif

#define EXTERN                        extern
#define DRO_IOCTL_REREAD_PARTITION    _IO  (0x12, 95)
#define DRO_IOCTL_SECTOR_CNT          _IO  (0x12, 96)
#define DRO_IOCTL_SECTOR_SIZE         _IO  (0x12, 104)
#define DRO_IOCTL_GET_SIZE_64         _IOR (0x12, 114, size_t)
#define DRO_IOCTL_GEOMETRY            0x0301
#define DR_TCP_PORT                   10666
#define DR_DEF_BLKSIZE                32768
#define MAX_DEVICE_NUM                20
#define MAX_PARTITION_NUM             60
#define DEF_PARTITION_NUM             4
#define DEVICE_NAME_LENGTH            26
#define DEVICE_SIZE_STR_LENGTH        30
#define PARTITION_TYPE_LENGTH         50
#define SECTOR_PER_TRACK              63
#define DR_TAG_SERVER_NAME            1
#define DR_TAG_IP_ADDRESS             2
#define DR_TAG_PORT_NUMBER            3
#define DR_TAG_RECOVER_PT             5
#define DR_TAG_RECOVER_MBR            6
#define DR_STR_BUFFER                 1024
#define DR_STNZ_IP_ADDRESS            "Addresses"
#define DR_STNZ_PORT_NUMBER           "Port"
#define DR_PROC_PT_FILE               "/proc/partitions"
#define DR_COPYRIGHT_STRING           "(c) Copyright BakBone Software, Inc. 2001"
#define DR_4BYTES_VALUE(p)            (((unsigned long)p[0])        | \
                                      (((unsigned long)p[1]) <<  8) | \
																			(((unsigned long)p[2]) << 16) | \
                                      (((unsigned long)p[3]) << 24))



#define DROVSTR(s)              ((NULL != (char *)(s)) ? (s) : "NULL")
#define DROSYSERR(s)            strerror(s)
#define ACTIVE_FLAG             0x80
#define EXTENDED                0x05
#define WIN98_EXTENDED          0x0F
#define LINUX_PARTITION         0x81
#define LINUX_SWAP              0x82
#define LINUX_NATIVE            0x83
#define LINUX_EXTENDED          0x85
#define LINUX_LVM               0x8E
#define LINUX_RAID              0xFD

#define IS_EXTENDED(x)        ((x) == EXTENDED || (x) == WIN98_EXTENDED || (x) == LINUX_EXTENDED)
#define PT_OFFSET(p, n)       ((DrPartitionT *)((p) + 0x1BE + (n) * sizeof(DrPartitionT)))
#define PT_RELATIVE_SECTOR(p) ((int64) DR_4BYTES_VALUE((p)->cSecRelative))
#define PT_RELATIVE_OFFSET(p) (PT_RELATIVE_SECTOR(p) * DR_SECTOR_BLKSIZE)
#define PT_NUMBER_SECTORS(p)  ((int64) DR_4BYTES_VALUE((p)->cSecNumber))
#define KERNEL_VER(a,b,c)     (((a) << 16) + ((b) << 8) + (c))


#define DRTRACE(...)             droTraceLocal (__VA_ARGS__)
#define DR_STR(x)                ((NULL != (char *)(x)) ? (x) : "NULL")
#define DR_STR_ERR(x)            strerror(x)




typedef struct DrPartitionS
{
  ByteT cBootFlag;
  ByteT cHdrStart;
  ByteT cSecStart;
  ByteT cCylStart;
  ByteT cType;
  ByteT cHdrEnd;
  ByteT cSecEnd;
  ByteT cCylEnd;
  ByteT cSecRelative[4];
  ByteT cSecNumber[4];
} DrPartitionT;

typedef struct DrBootSectorS
{
  uint64 llOffset;
  ByteT  cBuffer[DR_SECTOR_BLKSIZE];
} DrBootSectorT;

typedef struct DrSolarisBootSectorS
{
  uint64 llOffset;
  ByteT  cBuffer[DR_SOLARIS_MBR_SIZE];
} DrSolarisBootSectorT;

typedef struct DrProgressS
{
  char  acTransferType[64];
  char  acCurrentItem[64];
  int64 llBytesProcessed;
  int64 llBytesLeft;
} DrProgressT;

typedef struct DrGeometryS
{
  uint64 ullSize;
  uint64 ullOffset;
  uint32 ulCylinders;
  uint32 ulHeads;
  uint32 ulSectors;
} DrGeometryT;



/*
** =============
** GPT Supported
** =============
*/

#define GUID_SIG(x)    ((uint64)(ntohl((long)((x >> 32) & 0xffffffff))) | (((uint64)(ntohl((long)(x & 0xffffffff)))) << 32))
#define PT_EFI(h,l)    ((GUID_SIG(h) == 0x28732ac11ff8d211LL) && (GUID_SIG(l) == 0xba4b00a0c93ec93bLL))
#define PT_UNUSE(h,l)  ((GUID_SIG(h) == 0x0) && (GUID_SIG(l) == 0x0))
#define GPT_GUID_SIG   0x4546492050415254LL


typedef struct
{
  uint32_t  lTimeH;
  uint16_t  lTimeM;
  uint16_t  lTimeL;
  uint8_t   lClockH;
  uint8_t   lClockL;
  uint8_t   cNode[6];
} GPTguidT;


typedef struct
{ 
  uint64_t  lReq:1;
  uint64_t  lResv:47; 
  uint64_t  lSpec:16;
} __attribute__ ((packed)) GPTAttrT;
  
  
typedef struct 
{ 
  uint64_t  llSig;
  uint32_t  lVer;
  uint32_t  lHdrSize;
  uint32_t  lHdrCrc;
  uint32_t  lRes;
  uint64_t  llLbaPrime;
  uint64_t  llLbaBack;
  uint64_t  llUlbaFirst;
  uint64_t  llUlbaLast;
  GPTguidT  tGuid;
  uint64_t  llLbaPtEntry;
  uint32_t  lEntryCnt;
  uint32_t  lEntrySize;
  uint32_t  lEntryCrc;
  uint8_t   cReserved [420];
} __attribute__ ((packed)) GPTHdrT;
  
  
typedef struct
{ 
  GPTguidT   tGuidType;
  GPTguidT   tGuidPt;
  uint64_t   llLbaStart;
  uint64_t   llLbaEnd;
  GPTAttrT   llAttr;
  ByteT      strPtName[72];
} __attribute__ ((packed)) GPTEntryT;


typedef struct
{ 
  char      *pstrFs;
  BooleanT  (*fncbFsLookup) (int, uint64_t, uint64_t);
} GPTFsRecT;


typedef struct
{
  uint   lBlock;
  size_t lSize;
  ByteT  *pstrBuf;
} LastSecT;
                   



/*
** ============================================================================
** Extern Routines Prototype
** ============================================================================
*/
EXTERN void     droTraceLocal           (const char *, ...);
EXTERN BooleanT droSyncDevice           (const char *);
EXTERN BooleanT droGetMBR               (const char *, XdrDataFieldT * );
EXTERN long     droGetSectorSize        (const char *);
EXTERN BooleanT droGetDeviceSize        (const char *, int64 *);
EXTERN BooleanT droGetSectorSizeViaFd   (long, long *);
EXTERN BooleanT droGetDeviceSizeViaFd   (long, uint64 *);
EXTERN BooleanT droGetSectorCountViaFd  (long, uint64 *);
EXTERN BooleanT droGetAllVolume         (char *, XdrT, DrPartitionListReplyT);
EXTERN BooleanT droGetDeviceGeometry    (char *, long *, long *, long *, int64 *);
EXTERN char     *droGroupDigit          (int64);





EXTERN int      DRioctl (int, int, void *);
EXTERN BooleanT DrSeek( int, uint64 );
EXTERN BooleanT ValidateSignature( ByteT * );
EXTERN BooleanT GetPartitionType( char **pstrType, ByteT );
EXTERN BooleanT GetServerDevices( XdrT, XdrArrayT * );
EXTERN BooleanT GetSectorsBR( XdrT, char *, DrGetPartitionTableReplyT );
EXTERN BooleanT MergeSolarisDiskLabel( ByteT *, ByteT *, BooleanT );
EXTERN BooleanT ValidateSolarisSignature( ByteT * );
EXTERN void TraceMessageInline (long, long, long, const char *, ...);
EXTERN void droHexDump (ByteT *, long);

EXTERN int open64 (__const char *__file, int __oflag, ...);
EXTERN __off64_t lseek64 (int, __off64_t, int);


#endif
