
#ifndef	DR_DAEMON
#define	DR_DAEMON
#endif

static const char drtrace_c[] = "$Header:";

#include "thincore/sys.h"
#include <errno.h>

static FILE *gTraceIO = NULL;
static long  glLevel = ALWAYS;


/*
** ============================================================================
**
** Public Function:
**  NvTraceToFile
**
** Description:
**  Open file for writing to as trace file, or trace to stderr if no file
**  is specified (NULL) or if trace file is not usable.
**
** Parameters:
**  *pstrFile - pointer to string containing file name to open
**  
** Return Values:
**  void
**
** Side Effects:
**  gTraceIO gets set to write stream for file or stderr.
**  Error message is printed to stderr if file is specified and not usable.
**
** To Do:
**  Wrap system calls with library functions.
** 
** ============================================================================
*/
extern void NvTraceToFile( const char *pstrFile )
{
	if ( NULL != gTraceIO && stderr != gTraceIO )
	{
		/* close stream previously used for tracing */
		fclose( gTraceIO );
	}

	if ( NULL == pstrFile )
	{
		/* use stderr if no file is specified */
		gTraceIO = stderr;
	}
	else
	{
		int iFd = open( pstrFile, O_WRONLY | O_CREAT | O_TRUNC, 0666 );

		if ( 0 <= iFd )
		{
			FILE *pTrace = fdopen ( iFd, "w" );

			if ( NULL != pTrace )
			{
				gTraceIO = pTrace;
			}
			else
			{
				close( iFd );
				fprintf( stderr, " Failed open trace file - %d %s\n", errno,
					strerror( errno ) );
				gTraceIO = stderr;
			}
		}
		else
		{
			fprintf( stderr, " Failed open trace file descriptor - %d %s\n", errno,
				strerror( errno ) );
			gTraceIO = stderr;
		}
	}
}


/*
** ============================================================================
** ============================================================================
*/
extern long NvTraceSetLevel (long lLevel)
{
	long lResult 	= glLevel;
	glLevel 			= lLevel;
	return lResult;
}



/*
** ============================================================================
** ============================================================================
*/

extern void NvTrace (const char *pstrModuleId, long lId, long lCode, long lLevel, const char *pstrFormat, va_list args)
{
	if (NULL != gTraceIO && glLevel >= lLevel)
	{
		struct timeval sTime;

		gettimeofday(&sTime, NULL);

		fprintf(gTraceIO, "%ld:%ld.%03ld ",
						(long)getpid(),
						(long)(sTime.tv_sec % 1000),
						(long)(sTime.tv_usec / 1000));
		vfprintf(gTraceIO, pstrFormat, args);
		fprintf(gTraceIO, "\n");
		fflush(gTraceIO);
	}
}

extern void TraceMessageInline
(
    long           lTraceId,
    long           lStatusCode,
    long    eLevel,
    const char    *pstrFormat,
    ... /* varargs */
)
{
     static const char *pstrModuleId = "VaultDR";
     va_list            args;
           va_start(args, pstrFormat);
           NvTrace(pstrModuleId, lTraceId, lStatusCode, eLevel, pstrFormat, args);
           va_end(args);
}

