static const char callbacks_c[] = "$Header:";
#include "../../x86/config/drtrace.h"
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "netcfg.h"
#include <errno.h>
#include <sys/types.h>
#include <signal.h>
#include <sys/wait.h>
#include <unistd.h>

#define SERVER_IP_ENTRY "server_address_entry"
#define PING_STATUS_ENTRY "ping_status_entry"
#define IP_ADDRESS_ENTRY  "ip_address_entry"
#define NETMASK_ENTRY	"netmask_entry"
#define GATEWAY_ENTRY	"gateway_entry"
#define MAIN_WINDOW 	"main_window"

pid_t lDrDaemonPid = -1;

BooleanT ValidateIfParm (char *);

void PoopulateInterfaces (GtkWidget *);

MODULE("DRGUI")

void main_quit ()
{
   if ( -1 != lDrDaemonPid )
   {
		/* 
		 *Remove the handler that we set for SIGCHLD , 
		 * so that it does not go to the SIGNAL handler
		 * It would not have caused any harm , but just for safety
		 */
      struct sigaction act;
      act.sa_handler = SIG_IGN;
      act.sa_flags = 0;
      sigemptyset (&act.sa_mask);

      sigaction (SIGCHLD,&act,NULL);

		TRACE (("Killing Drdaemon ..."));		
	
      if ( -1 == kill (lDrDaemonPid,SIGTERM))
      {
         TRACE (("Failed to kill Drdaemon Error : %s",errno));
      }
   }
   gtk_main_quit();
}

void quick_message (gchar *message) {

   GtkWidget *dialog, *label;
   
   /* Create the widgets */
   
   dialog = gtk_dialog_new_with_buttons ("Message",
                                         NULL,
                                         GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
                                         GTK_STOCK_OK,
                                         GTK_RESPONSE_NONE,
                                         NULL);
   label = gtk_label_new (message);
   
   /* Ensure that the dialog box is destroyed when the user responds. */
   
   g_signal_connect_swapped (dialog,
                             "response", 
                             G_CALLBACK (gtk_widget_destroy),
                             dialog);

   /* Add the label, and show everything we've added to the dialog. */

   gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),
                      label);
   gtk_widget_show_all (dialog);
}

BooleanT quit_confirm ()
{
	GtkWidget *dialog, *label;
	BooleanT bClose = FALSE;

	dialog = gtk_dialog_new_with_buttons ("Quit",
					       NULL,
					       GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_MODAL,
					       GTK_STOCK_CANCEL,
					       GTK_RESPONSE_CANCEL,
					       GTK_STOCK_QUIT,
					       GTK_RESPONSE_CLOSE,
					       NULL);
	
	label = gtk_label_new ("Closing this application will abort all running backup / restore jobs.\n Are you sure you want to Quit ?");

	gtk_container_add (GTK_CONTAINER (GTK_DIALOG(dialog)->vbox),label);
	gtk_widget_show (label);


	switch (gtk_dialog_run (GTK_DIALOG (dialog)))
	{
		case GTK_RESPONSE_CLOSE:
			bClose = TRUE;
			break;
				
		default:
			bClose = FALSE;
	}
		
	gtk_widget_destroy (dialog);	
	return bClose;
}


void sigchld_handler(int signum, siginfo_t *siginfo,void *arg)
{
	int status;
	waitpid ( siginfo->si_pid, &status,WNOHANG);
	/* If the signal was sent by the child that we spawned then we can reste the PID */
	if ( lDrDaemonPid == siginfo->si_pid )
	{
		quick_message ("Fatal Error !\nDRDaemon was killed due to some reason \nPlease press the start button to restart\nPlease report this problem");
		lDrDaemonPid = -1;
	}
}

void
on_quit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if ( TRUE == quit_confirm () )
	{
		main_quit();
	}
}


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *aboutdialog1;
	const gchar *authors[] = {
		        "Bakbone DR Team",
			NULL};

//	GtkWidget *main_window = lookup_widget (GTK_WIDGET(menuitem),MAIN_WINDOW);
	/* This can be replaces with some translators */
/*	gchar *translators = _("translator-credits");

	aboutdialog1 = gtk_about_dialog_new ();
	gtk_about_dialog_set_version (GTK_ABOUT_DIALOG (aboutdialog1), "4.0");
	gtk_about_dialog_set_name (GTK_ABOUT_DIALOG (aboutdialog1), _("VaultDR Client"));
	gtk_about_dialog_set_comments (GTK_ABOUT_DIALOG (aboutdialog1), _("Gui for VaultOS"));
	gtk_about_dialog_set_website (GTK_ABOUT_DIALOG (aboutdialog1), "http://www.bakbone.com/products/disaster_recovery");
  	gtk_about_dialog_set_website_label (GTK_ABOUT_DIALOG (aboutdialog1), _("Bakbone"));
    	gtk_about_dialog_set_authors (GTK_ABOUT_DIALOG (aboutdialog1), authors);
      	//gtk_about_dialog_set_translator_credits (GTK_ABOUT_DIALOG (aboutdialog1), translators);
	gtk_widget_show_all (aboutdialog1);
*/
}


void
on_ping_server_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *serverEntry = lookup_widget (GTK_WIDGET (button),SERVER_IP_ENTRY);
	GtkWidget *statusEntry = lookup_widget (GTK_WIDGET (button),PING_STATUS_ENTRY);

	if ( NULL == serverEntry || NULL == statusEntry )	
	{
		TRACE (("Unable to find widget for Server IP address entry %s or status entry %s", SERVER_IP_ENTRY, PING_STATUS_ENTRY));
	}
	else
	{
		const gchar *ip = gtk_entry_get_text (GTK_ENTRY (serverEntry));

		if ( 0 >= strlen (ip)	|| TRUE != ValidateIfParm (ip))
		{
			quick_message ("ERROR: Please Enter a valid Address");
		}
		else
		{
			if (TRUE == DrPg (ip))
			{
				gtk_entry_set_text (GTK_ENTRY(statusEntry),"Reachable");
			}
			else
			{
				gtk_entry_set_text (GTK_ENTRY(statusEntry),"Unrechable");
			}
		}
	}
}


void
on_main_window_destroy                 (GtkObject       *object,
                                        gpointer         user_data)
{
	main_quit();
}


void
on_Auto_configure_button_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
	char  strParms[4][32] = {'\0'};
	char  *pstrParms[4];
	long i = 0 , lNumIf = 0;

	GtkWidget *server_entry = lookup_widget ( GTK_WIDGET (button),SERVER_IP_ENTRY);
	GtkWidget *ip_entry = lookup_widget (GTK_WIDGET (button),IP_ADDRESS_ENTRY);
	GtkWidget *netmask_entry = lookup_widget (GTK_WIDGET (button),NETMASK_ENTRY);
	GtkWidget *gateway_entry = lookup_widget (GTK_WIDGET (button),GATEWAY_ENTRY);
	GtkWidget *MainWindow = lookup_widget (GTK_WIDGET (button),MAIN_WINDOW);

	TRACE (("Autoconfigure Network"));

	if ( NULL == server_entry || NULL == ip_entry || NULL == netmask_entry || NULL == gateway_entry )
	{
		TRACE (("Unable to find handle for one of the text boxes, its a bug"));
	}
	else
	{
		strcpy ( strParms[0], gtk_entry_get_text (GTK_ENTRY (ip_entry)));
		strcpy ( strParms[1], gtk_entry_get_text (GTK_ENTRY (netmask_entry)));
		strcpy ( strParms[2], gtk_entry_get_text (GTK_ENTRY (gateway_entry)));
		strcpy ( strParms[3], gtk_entry_get_text (GTK_ENTRY (server_entry)));	
		
		/* 
		 * First check if we can ping the server woth existing settings , Configure if ping fails
		 */	
		if ( TRUE == DrPg (strParms[3]))
		{
			char strMsg [512] = {'\0'};
			sprintf (strMsg, "Able to Ping Server %s with current network configuration\nNetwork Configuration not changed", strParms[3]);
			quick_message (strMsg);
		}
		else
		{
			lNumIf = GetNicInterfaces ();

			for ( i = 0 ; i < 4 ; i++ )
			{
				pstrParms[i] = strParms[i];
			}	
		
			ValidateAndConfigIf (NULL, pstrParms, lNumIf);

			if ( NULL != MainWindow )
			{
				PoopulateInterfaces (MainWindow);
			}
			else
			{
				TRACE (("Failed to get handle for main window, so could not update interfaces"));
			}
		}
	}
}

void
on_start_button_clicked                (GtkButton       *button,
                                        gpointer         user_data)
{
	/* 
	 * WE are not really checking if the drdaemon is running now 
	 *	Could be that somebody just killed it, but thats not a valid test case
	 * We can Handle SIGCHLD here .. but keep it for later ..
	 */

	if (-1 != lDrDaemonPid )
	{
		quick_message ("DR Daemon was already started");
	}
	else if (0 == (lDrDaemonPid = fork ()))
	{
		if ( -1 == execl ("/bin/drdaemon_gui","drdaemon_gui",NULL))
		{
			TRACE (("Failed to exec drdaemon , check if it is in /bin/drdaemon_gui"));
			quick_message ("Failed to start /bin/drdaemon_gui\nCheck the permissions on this file");
		}
	}
	else if ( 0 < lDrDaemonPid )
	{
		TRACE (("DrDaemon Started with Pid = %d", lDrDaemonPid));

		/* Parent Continue */
		struct sigaction act;
		act.sa_sigaction = sigchld_handler;
		act.sa_flags = SA_SIGINFO;
		sigemptyset (&act.sa_mask);
	
		sigaction (SIGCHLD,&act,NULL);
	}
}


gboolean
on_main_window_delete_event            (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
  /* If we return a TRUE here the application will not get a destroy event 
   * If we return a FALSE then th application gets a destroy event
   */
  return (!quit_confirm ());
}

