static const char msg_c[] = "$Header:";

#include <sys/ioctl.h>
#include <unistd.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <net/if.h>
#include <signal.h>
#include <errno.h>
#include "thincore/sys.h"
#include "thincore/memory.h"
#include "thincore/net.h"
#include "thincore/strbase.h"
#include "../../messages.h"

#define LISTEN_PORT 17000

#include <gtk/gtk.h>
#include "support.h"

#define MESSAGE_VIEW "messages_textview"
#define PROGRESS_BAR "progressbar1"
#define INTERFACE_LISTVIEW "treeview1"
enum
{
   INTERFACE_COLUMN,
   IP_COLUMN,
	NETMASK_COLUMN,
	STATUS_COLUMN,
   N_COLUMNS
};

static NetAddressT gServerAddress;

void InitiateInterfaceList (GtkWidget *MainWindow)
{

	GtkWidget *interface_list;
	GtkTreeIter iter_level1;  /* Parent iter */
	GtkTreeIter iter_level2;  /* Child iter  */
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkWidget *scrollWindow;
	GtkListStore *store = gtk_list_store_new (N_COLUMNS,       /* Total number of columns */
                                          G_TYPE_STRING,   /* Interface              */
                                          G_TYPE_STRING,   /* IP        	          */
                                          G_TYPE_STRING,   /* Netmask						*/
                                          G_TYPE_STRING); /*  Status	   	      */	

	TRACE (("Initializing the interface list"));

	/* Create the new List with Model */
	interface_list = lookup_widget ( GTK_WIDGET (MainWindow), INTERFACE_LISTVIEW);
	gtk_tree_view_set_model (GTK_TREE_VIEW (interface_list), GTK_TREE_MODEL (store));

	/* 
 	 * Add the 4 Columns to the list
 	 */
	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Interface", renderer, "text", INTERFACE_COLUMN, NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (interface_list), column);

	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("IP Address", renderer, "text", IP_COLUMN, NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (interface_list), column);

	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Netmask", renderer, "text", NETMASK_COLUMN, NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (interface_list), column);

	renderer = gtk_cell_renderer_text_new ();
	column = gtk_tree_view_column_new_with_attributes ("Status", renderer, "text", STATUS_COLUMN, NULL);
	gtk_tree_view_append_column (GTK_TREE_VIEW (interface_list), column);
}


void PoopulateInterfaces (GtkWidget *MainWindow)
{
	GtkWidget *interface_list = lookup_widget ( GTK_WIDGET (MainWindow), INTERFACE_LISTVIEW);
	GtkListStore *store = (GtkListStore *)gtk_tree_view_get_model (	GTK_TREE_VIEW (interface_list) );
	long index = 1;
	BooleanT bContinue = TRUE;
	int s = socket (PF_INET, SOCK_STREAM, 0);
	
	TRACE (("Populate Interfaces List"));	

	if ( -1 == s)
	{
		TRACE (("Failed to create Socket"));
		bContinue = FALSE;
	}
	else if ( NULL == store )
	{
		TRACE (("Failed to get the strore for the treeview"));
		bContinue = FALSE;
	}
	else
	{
		gtk_list_store_clear (store);
	}

	while (bContinue && NULL != store )
	{
		struct ifreq ifr;

		ifr.ifr_ifindex = index++;
		
		if (0 > ioctl (s, SIOCGIFNAME, &ifr))
		{
			bContinue = FALSE;
		}
		else
		{ 
			char strAddress [32] = {'\0'};
			char strNetmask [32] = {'\0'};
			char strStatus [16] = {'\0'};
			GtkTreeIter iter_level1;

			if (0 > ioctl (s, SIOCGIFADDR, &ifr))
			{
				TRACE (( "Failed to find the IP address for %s", ifr.ifr_name));
			}
			else
			{
				char *ip = NULL;
				struct sockaddr_in *addr = (struct sockaddr_in *) &ifr.ifr_addr;	
				ip = inet_ntoa (addr->sin_addr);

				if ( ip != NULL )
				{
					strcpy ( strAddress,ip );
				}
			
				if ( 0 > ioctl (s,SIOCGIFNETMASK,&ifr))
				{
					TRACE (("Failed to get netmask for %s", ifr.ifr_name));
        			}
				else
				{
					struct sockaddr_in *netmask = (struct sockaddr_in *) &ifr.ifr_addr;
					char *ip = NULL;
					ip = inet_ntoa(netmask->sin_addr);

					if ( NULL != ip )
					{
						strcpy ( strNetmask, ip );
					}
				}
		
				if ( 0 > ioctl (s, SIOCGIFFLAGS, &ifr))
				{
					TRACE (("Failed to get flags for %s",ifr.ifr_name));
				}
				else
				{
					strcpy ( strStatus, (IFF_UP & ifr.ifr_flags) ? "Active" : "InActive");
				}

				/* Eliminate all interfaces except  "eth" interfaces */
				if ( NULL != strstr ( ifr.ifr_name, "eth" ))
				{
					TRACE (("Found Interface : %s , IP : %s , Netmask : %s , Status : %s",ifr.ifr_name,strAddress,strNetmask,strStatus));
					/* Add a row to the listview */
					gtk_list_store_append (store, &iter_level1);  /* Acquire a top-level iterator */
					gtk_list_store_set (store, &iter_level1,
								INTERFACE_COLUMN, ifr.ifr_name,
								IP_COLUMN, strAddress,
								NETMASK_COLUMN, strNetmask,
								STATUS_COLUMN, strStatus,
								-1);
				}
			}
		}
	}
}


BooleanT RecieveMessage ( NetSocketT   tSocket, GuiMessage **msg)
{
	/* First Recieve the header and find the lenght of the packet */
	long lBytesReceived = 0;
	GuiMessage oHeader;
	BooleanT bSuccess = TRUE;

	TRACE (("Waiting for message"));
	if ( NET_ERROR_NONE != NetTcpReceiveBlocking(&lBytesReceived,  tSocket, &oHeader ,sizeof (GuiMessage ), NULL))
	{
		TRACE (("NetTcpRecieveBlocking Failed to recieve Header"));
		bSuccess = FALSE;
	}
	else
	{
		char *pstrMessage = (char *)malloc (oHeader.lLength);

		if (NET_ERROR_NONE != NetTcpReceiveBlocking(&lBytesReceived,  tSocket, pstrMessage ,oHeader.lLength, NULL))
		{
			TRACE (("NetTcpRecieveBlocking Failed to recieve Message Body"));
		}
		else
		{
			*msg = (GuiMessage*)malloc (sizeof (GuiMessage) + oHeader.lLength);
			memcpy ( *msg, &oHeader,sizeof (GuiMessage));
			strcpy((*msg)->strData , pstrMessage);
		}
	}
	return bSuccess;
}


void* MessageLoop (void *arg)
{
	GtkWidget *MainWindow = (GtkWidget *)arg;
	BooleanT bContinue = TRUE;
	static NetSocketT   gtListenSocket = NET_SOCKET_INVALID;
	char address[32] = {'\0'};
	struct sigaction act;
	NetInitializeIpAddress(&gServerAddress, INADDR_LOOPBACK, LISTEN_PORT);
	
	gdk_threads_enter();
	InitiateInterfaceList (MainWindow);
	PoopulateInterfaces (MainWindow);
	gdk_flush();
	gdk_threads_leave();

	/* Ignore SIG_PIPE */
	act.sa_handler = SIG_IGN;
	act.sa_flags = 0;
	sigemptyset (&act.sa_mask);
	sigaction (SIGPIPE,&act,NULL);


	if (FALSE == NetOpen(&gtListenSocket, NET_TYPE_TCP))
	{
		TRACE (("Failed to create listening socket"));
	}
	else if (FALSE == NetSetReuseAddress(gtListenSocket))
	{
		TRACE (("Failed to set reuse address on listening socket"));
	}
	else if (FALSE == NetBind(gtListenSocket, &gServerAddress))
	{
		TRACE (("Failed to bind address to listening socket"));
	}
	else if (FALSE == NetListen(gtListenSocket, 5))
	{
		TRACE (("Failed to listen on  socket"));
	}
	else
	{
		TRACE (("Starting Message Loop ..."));
		while (bContinue)
		{
			NetSocketT  tNewSocket = NET_SOCKET_INVALID;
			NetAddressT tFrom;
			
			if (NET_ERROR_NONE != NetAccept(&tNewSocket, gtListenSocket, &tFrom))
			{
				TRACE (("Accept Failed"));
				bContinue = FALSE;
			}
			else
			{				
				GuiMessage *msg = NULL;
				GtkTextIter start, end;
				GtkWidget *MessagePanel = lookup_widget(GTK_WIDGET(MainWindow), MESSAGE_VIEW);
				GtkWidget *ProgressBar = lookup_widget (GTK_WIDGET(MainWindow), PROGRESS_BAR);
				GtkTextBuffer *text_buffer = NULL;
				gdk_threads_enter();
				text_buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (MessagePanel));
								
				/* Clear the Text View and progress Bar */
				gtk_progress_bar_update ( GTK_PROGRESS_BAR (ProgressBar), 0.0);
				gtk_text_buffer_get_end_iter (text_buffer, &end);
				gtk_text_buffer_get_start_iter (text_buffer, &start);
				gtk_text_buffer_delete (text_buffer,&start,&end);
				gdk_flush ();
				gdk_threads_leave();
				TRACE (("Drdaemon Connected"));

				if ( NULL == MessagePanel || NULL == ProgressBar || NULL == text_buffer )
				{
					TRACE (("Failed to get Handles to Widgets: MessagePanel , textview"));
				}
				else
				{
					BooleanT bRecieve = TRUE;
					while (bRecieve)
					{ 
						msg = NULL;
						if (TRUE != RecieveMessage (tNewSocket,&msg))
						{
							bRecieve = FALSE;
							TRACE (("Failed to Recieve Message"));
						}
						else
						{
							long temp = 0 ;
							switch ( msg->lType )
							{
								case PROGRESS_UPDATE_MESSAGE:
								{
									temp = atol (msg->strData);
									gdk_threads_enter();
									gtk_progress_bar_update ( GTK_PROGRESS_BAR (ProgressBar), (float)temp / 100.0);
									gdk_flush();
									gdk_threads_leave();
									break;
								}

								case TRACE_MESSAGE:
								{
									GtkTextIter cursor;
									gdk_threads_enter();
									gtk_text_buffer_get_end_iter (text_buffer, &cursor);
									gtk_text_buffer_insert (text_buffer, &cursor, msg->strData, -1 );

									gtk_text_buffer_get_end_iter (text_buffer, &cursor);
									gtk_text_buffer_insert (text_buffer, &cursor, "\n", -1);
									gtk_text_view_scroll_to_iter (GTK_TEXT_VIEW (MessagePanel),&cursor,0.0,TRUE,1.0,0.0);
									gdk_flush();
                                                                        gdk_threads_leave();

									break;
								}
							}
						}

						if ( NULL != msg)
						{
							free ( msg );
						}
					}	
				}
			}
		}
	}
}
