static const char drstart_c[] = "$Header:";
/*
 * This is a program to start drdaemon, so that when drdaemon exits , we can fork a shell.
 * We had to write this program that almost does everything a shell does, so that when user presses Ctrl^C
 * only drdaemon dies. If we had started drdaemon from python scripts, 
 * the Ctrl^C signal will go to all the processes associated with that terminal and the CD reboots.
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>
#include <termios.h>
#include <string.h>

pid_t shell_pgid;
struct termios s_mode;
int s_term;
int interactive;

void
init_shell ()
{

  /* See if we are running interactively.  */
  s_term = STDIN_FILENO;
  interactive = isatty (s_term);

  if (interactive)
  {
	 /* Loop until we are in the foreground.  */
	 while (tcgetpgrp (s_term) != (shell_pgid = getpgrp ()))
		kill (-shell_pgid, SIGTTIN);

	 /* Ignore interactive and job-control signals.  */
	 signal (SIGINT, SIG_IGN);
	 signal (SIGQUIT, SIG_IGN);
	 signal (SIGTSTP, SIG_IGN);
	 signal (SIGTTIN, SIG_IGN);
	 signal (SIGTTOU, SIG_IGN);
	 signal (SIGCHLD, SIG_IGN);

	 /* Put ourselves in our own process group.  */
	 shell_pgid = getpid ();
	 if (setpgid (shell_pgid, shell_pgid) < 0)
	 {
		printf ("Failed to put the shell in its own process group :: %s\n",
				  strerror (errno));
		exit (1);
	 }

	 /* Grab control of the terminal.  */
	 tcsetpgrp (s_term, shell_pgid);

	 /* Save default terminal attributes for shell.  */
	 tcgetattr (s_term, &s_mode);
  }
  else
	 printf ("The shell is not interactive");

}


void
launch_process (pid_t pgid,
					 int infile, int outfile, int errfile, int foreground)
{
  pid_t pid;
  if (interactive)
  {
	 /* Put the process into the process group and give the process group
	    the terminal, if appropriate.
	    This has to be done both by the shell and in the individual
	    child processes because of potential race conditions.  */
	 pid = getpid ();
	 if (pgid == 0)
		pgid = pid;
	 setpgid (pid, pgid);
	 if (foreground)
		tcsetpgrp (s_term, pgid);

	 /* Set the handling for job control signals back to the default.  */
	 signal (SIGINT, SIG_DFL);
	 signal (SIGQUIT, SIG_DFL);
	 signal (SIGTSTP, SIG_DFL);
	 signal (SIGTTIN, SIG_DFL);
	 signal (SIGTTOU, SIG_DFL);
	 signal (SIGCHLD, SIG_DFL);
  }

  if (infile != STDIN_FILENO)
  {
	 dup2 (infile, STDIN_FILENO);
	 close (infile);
  }
  if (outfile != STDOUT_FILENO)
  {
	 dup2 (outfile, STDOUT_FILENO);
	 close (outfile);
  }
  if (errfile != STDERR_FILENO)
  {
	 dup2 (errfile, STDERR_FILENO);
	 close (errfile);
  }


  if (0 != system ("reset; /bin/drdaemon"))
	 printf ("Failed to start drdaemon :%s", strerror (errno));
//  if ( -1 == execl ("/bin/drdaemon","drdaemon",NULL))
//    printf ("Failed to exec /usr/bin/drdaemon :: %s",strerror(errno));
  exit (1);
}


int
wait_for_job (pid_t pid)
{
  int stat_val = 0;
  return waitpid (pid, &stat_val, 0);
}

void
put_job_in_foreground (pid_t pgid)
{
  tcsetpgrp (s_term, pgid);
  wait_for_job (pgid);

  /* Put the shell back in the foreground.  */
  tcsetpgrp (s_term, shell_pgid);

  /* Restore the shell's terminal modes.  */
  tcsetattr (s_term, TCSADRAIN, &s_mode);
}


void
launch_job (pid_t pgid, int foreground)
{
  pid_t pid;

  pid = fork ();
  if (pid == 0)
  {
	 launch_process (pgid, STDIN_FILENO, STDOUT_FILENO, STDERR_FILENO,
						  foreground);
  }
  else if (pid < 0)
  {
	 printf ("fork failed : %s\n", strerror(errno));
	 exit (1);
  }
  else
  {
	 /* parent process.  */
	 if (interactive)
	 {
		if (!pgid)
		  pgid = pid;
		setpgid (pid, pgid);
	 }
  }


  if (!interactive)
	 wait_for_job (pgid);
  else if (foreground)
	 put_job_in_foreground (pgid);
}

int
main ()
{

  init_shell ();
  launch_job (0, 1);
  if (-1 == execl ("/usr/bin/sh", "/usr/bin/sh", NULL))
	 printf ("failed to start shell .. exiting\n");
  exit (0);
}
