#!/bin/bash

# This script creates a customized CentOS 5.4 live Cd with Netvault Server, VaultDR Server and drdaemon
# Usage  sudo ./make_fc_live.sh CentOS-5.4-i386-LiveCD.iso
# 
# *** Place netvault distribution and VaultDR npk file in the current directory and change the NV_BUILD_DIR, NV_DIST and DR_SERVER_NPK vars ***
#
# Check the following files when upgrading to a new revision
#
# 1.) isolinux.cfg
# 2.) %gconf-tree.xml
# 3.) ../rc.local
# 4.) default.session
# 5.) Live user name below
# 6.) centos-live

NV_BUILD_DIR="/root/nv8/build/"
ISO_IMAGE="$1"
CDMNT="cdmnt"
CD_COPY="cdcopy"
SQUASH_MNT="ex_mnt"
EX_SQUASH="ex_squash"
MNTDIR="fsmnt"
NV_DIST="netvault-R2010MAY04-HALO-Linux.tar.gz"
DR_SERVER_NPK="drc-2-6-1-2.npk"
NV_PARAM="../nv_parameters"
RC_LOCAL="../rc.local"
NV_INSTALL_SCRIPT="../nvinstall.sh"
DRDAEMON_PATH="$NV_BUILD_DIR/vaultos/unix/linux"
VAULTOS_LIVE_CD="VaultOS-live-centos.iso"
CHROOT="/usr/sbin/chroot"
#LICENSE_FILE="rawlicense.dat"
NV_LICENSE="nvlicense.dat" 
DRC_LICENSE="drc_14day.dat"

DISABLE_SERVICES="iptables ip6tables cups sendmail auditd bluetooth isdn NetworkManager hplip"

UNWANTED_PACKAGES="thunderbird openoffice.org-core"

function clean_up ()
{

umount $MNTDIR/proc $MNTDIR/sys $MNTDIR/dev > /dev/null 2>&1
umount $MNTDIR > /dev/null 2>&1
umount $SQUASH_MNT > /dev/null 2>&1
umount $CDMNT > /dev/null 2>&1

rm -fr  $CDMNT $CD_COPY $EX_SQUASH $MNTDIR $SQUASH_MNT 
}


function failure()
{
	echo -e "$@" 1>&2

	# [TODO] : Clean up
	exit 2
}

function install_package ()
{
	$CHROOT $MNTDIR/ /usr/bin/yum -y install "$@"
}

function remove_package ()
{
	$CHROOT $MNTDIR/ /usr/bin/yum -y remove "$@"
}

function remove_package_group ()
{
	echo "Remove Group : $@"
	$CHROOT $MNTDIR/ /usr/bin/yum -y  groupremove "$@"
}

function disable_service ()
{
	$CHROOT $MNTDIR/ /sbin/chkconfig --level 345 $1 off
}

function enable_service ()
{
	$CHROOT $MNTDIR/ /sbin/chkconfig --level 345 $1 on
}

function yum_clean ()
{
	$CHROOT $MNTDIR/ /usr/bin/yum  -y clean all
}

function customize_boot
{
	cp -f isolinux.cfg $CD_COPY/isolinux || failure "Failed to customize boott options"
	cp vaultos.png $CD_COPY/isolinux/splash.jpg || failure "Failed to ccopy boot image"
}

function free_space
{
	echo "Free space on the filesystem ..."
	dd if=/dev/zero of=$MNTDIR/delete_me
	rm -f $MNTDIR/delete_me
}

function customize_cd
{
#  Customizations to add drdaemon ...
echo "Adding drdaemon to the CD ..."
cp $DRDAEMON_PATH/drdaemon ../VaultDr_client.sh  $MNTDIR/bin/ || failure "Failed to copy Drdaemon to the CD"
chmod 0777 $MNTDIR/bin/drdaemon $MNTDIR/bin/VaultDr_client.sh || failure "Failed to set the right permissions for binaries"

# Setting permissions so that drdaemon always runs with root permissions
chmod a+s $MNTDIR/bin/drdaemon $MNTDIR/bin/VaultDr_client.sh || failure "Failed to set the right permissions for binaries"
mkdir $MNTDIR/trace || failure "Failed to create /trace directory in the CD"
chmod 0777 $MNTDIR/trace

# Copy all the netvault stuff
cp $NV_PARAM  $NV_DIST $DR_SERVER_NPK $NV_LICENSE $DRC_LICENSE $MNTDIR/root/ || failure "Failed to copy netvault files"
cp -f $NV_INSTALL_SCRIPT $MNTDIR/usr/bin/ || failure "Failed to copy $NV_INSTALL_SCRIPT"
cp -f $RC_LOCAL $MNTDIR/etc/rc.local || failure "Failed to copy rc.local"
chmod 0777 $MNTDIR/usr/bin/nvinstall.sh $MNTDIR/etc/rc.local ||  failure "Failed to set permissions on scripts"

# Create Udevstart as it does not exist on Ubuntu , we just make a script with equivalent commands
echo "/sbin/udevtrigger" > $MNTDIR/bin/udevstart || failure "Failed to create /bin/udevstart"
echo "/sbin/udevsettle" >> $MNTDIR/bin/udevstart || failure "Failed to write to /bin/udevstart"
echo "exit 0" >> $MNTDIR/bin/udevstart || failure "Failed to write to /bin/udevstart"
chmod 0777 $MNTDIR/bin/udevstart

# To Autostart drdaemon when the user logs in
cp default.session $MNTDIR/usr/share/gnome/default.session || failure "Failed to add drdaemon at startup"

# Special Desktop customizations ...
echo "Applying Desktop customizations ..."

cp -f %gconf-tree.xml $MNTDIR/etc/gconf/gconf.xml.defaults/ || failure "Failed to customize Desktop"

#rm -f $MNTDIR/usr/share/applications/liveinst.desktop

#Changing the Background Image of the desktop -- currently changed with gconf file
#cp $DESKTOP_BACKGROUND $MNTDIR/$BACKGROUND_LOCATION
#$CHROOT $MNTDIR gconftool-2 --direct --config-source xml:readwrite:/etc/gconf/gconf.xml.defaults --type string --set /desktop/gnome/background/picture_filename $BACKGROUND_LOCATION
cp -f nvbu_wall.png $MNTDIR/usr/share/backgrounds/ || faliure "Failed to copy Wallpaper"

# Launcher stuff
cp nvgui.desktop drdaemon.desktop $MNTDIR/usr/share/applications/ || failure "Failed to copy desktop files"
cp -f nvlogo.png $MNTDIR/usr/share/icons/ || failure "Failed to copy nvlogo"

# Before we customize we need to set the resolve.conf right
cp $MNTDIR/etc/resolv.conf $MNTDIR/etc/resolv.conf_bak > /dev/null 2>&1
cp /etc/resolv.conf $MNTDIR/etc/resolv.conf

#CentOS special - remove documentation folder from Desktop
cp -f centos-live $MNTDIR/etc/init.d/ || failure "Failed to copy live user script"

# Set auto Login
sed -i -e 's/TimedLoginDelay=[0-9][0-9]/TimedLoginDelay=1/' $MNTDIR/etc/init.d/centos-live || failure "Failed to set Timed Login"

# disable screensaver locking
$CHROOT $MNTDIR /usr/bin/gconftool-2 --direct --config-source=xml:readwrite:/etc/gconf/gconf.xml.defaults -s -t bool /apps/gnome-screensaver/lock_enabled false >/dev/null

# Enable sudo for live user
#sed -i 's/^.*wheel.*NOPASSWD.*$/\%centos  \tALL=(ALL)\t NOPASSWD: ALL/' $MNTDIR/etc/sudoers

# Remove any lock file for Yum transactions
rm -f $MNTDIR/var/lib/rpm/__db*

# Remove software groups
remove_package_group "Games and Entertainment"
#remove_package_group "Printing Support"
remove_package_group "Office/Productivity"
#remove_package_group "Mail Server"
#remove_package_group "Sound and Video"
remove_package_group "Graphics"
#remove_package_group "Graphical Internet"

# Remove unwanted large packages
for pack in $UNWANTED_PACKAGES
do
remove_package $pack
done

rm -fr $MNTDIR/usr/share/doc

# Install needed packages
install_package libXp
#cp -a /usr/lib/libXp.so* $MNTDIR/usr/lib/

# Clean yum -- Last step in package add / remove
yum_clean

# Disable Services
for i in $DISABLE_SERVICES
do
disable_service $i
done

# Disable gnome-volume manager
rm -f $MNTDIR/usr/share/gnome/autostart/gnome-volume-manager.desktop || failure "Failed to disable gnome-volume-manager"

enable_service network
# Enable Services
#for i in $ENABLE_SERVICES
#do
#enable_service $i
#done

# reset resolv.conf
rm $MNTDIR/etc/resolv.conf
mv $MNTDIR/etc/resolv.conf_bak $MNTDIR/etc/resolv.conf > /dev/null 2>&1

echo "Customizing the bootloader ..."
customize_boot

}

##############     MAIN     #################
START=`date`

clean_up
rm -f $VAULTOS_LIVE_CD

mkdir $CDMNT $CD_COPY $EX_SQUASH $MNTDIR $SQUASH_MNT || failure "Failed to create base dirs"

# Mount CD
mount $ISO_IMAGE $CDMNT -o loop|| faliure "Failed to mount cd - $ISO_IMAGE"

echo "Copying CD ..."
rsync --exclude=/LiveOS/squashfs.img -a $CDMNT/ $CD_COPY/ || failure "Failed to extract contents of the ISO Image"

echo "Extracting Squashfs ..."
mount $CDMNT/LiveOS/squashfs.img $SQUASH_MNT/ -o loop -t squashfs || failure "Failed to mount squashfs"
rsync -a $SQUASH_MNT/ $EX_SQUASH/ || failure "Failed to copy squashfs contensts"
umount $SQUASH_MNT
#unsquashfs -f -d $EX_SQUASH $CDMNT/LiveOS/squashfs.img || failure "Failed to UNsquash filesystem"

umount $CDMNT

mount $EX_SQUASH/LiveOS/ext3fs.img $MNTDIR -o loop,rw || failure "Failed to mount ext3 filesystem"

mount -t proc none $MNTDIR/proc
mount -t sysfs none $MNTDIR/sys
mount --bind /dev/ $MNTDIR/dev

customize_cd

umount $MNTDIR/proc
umount $MNTDIR/sys
umount $MNTDIR/dev

free_space

umount $MNTDIR || failure "Failed to unmount ext3 fs"

/sbin/mksquashfs $EX_SQUASH $CD_COPY/LiveOS/squashfs.img || failure "Failed to create squashfs filesystem"

mkisofs -D -r -V "VaultOS" -cache-inodes -J -l -b isolinux/isolinux.bin \
	-c isolinux/boot.cat -no-emul-boot -boot-load-size 4 \
	-boot-info-table -o $VAULTOS_LIVE_CD $CD_COPY || failure "Failed to create ISO Image"

clean_up

echo "CentOS Based VaultOS created : $VAULTOS_LIVE_CD"
echo "Start = $START     , End = `date`"
exit 0
