#!/bin/sh

# Script to build the VaultOS cd from RHEL5.2  CD1 ( x86 and ia64 )
# Arguments - $1 = path to RHEL5 disc 1 iso image
# 	    - $2 = platform ( ia64 / x86 )

PLATFORM="x86"
VERSION="5.1"
RHEL_IMAGE=$1
EXTRACTED_CD=extracted_cd
CD_MNT=cd_mnt
FS_MNT=fs.mnt
FS_NEW=fs.new
NEW_IMAGE=vaultos_x86_v5.0_RHEL5.iso
INITRD_MNT=initrd.mnt
DRDAEMON_PATH=$NV_BUILD_DIR/vaultos/unix/linux
DRSTART_PATH=$NV_BUILD_DIR/vaultos/unix/linux/livecd
BOOT_IMG_MOUNT=boot.img.mnt
BOOT_IMG_NEW=boot.img.new
BOOT_IMG_TMP=boot.img.tmp
BOOT_IMG_NEW_MNT=boot.img.new.mnt

failure ()
{
echo "$1"
echo "Exiting ..."
exit 1
}

customize_initrd()
{
echo "Cusomtizing initrd ..."
cp $1 initrd.img.gz || failure "Failed to get the initrd image"
gunzip initrd.img.gz || failure "Failed to gunzip initrd iamge"
cd $INITRD_MNT/
cpio -idv < ../initrd.img
cd -

# Place All Customizations to initrd here ... ( base directory - $INITRD_MNT )
cp -f $DRDAEMON_PATH/drdaemon $INITRD_MNT/bin/ || failure "Failed to copy drdaemon into initrd image"
chmod 0777 $INITRD_MNT/bin/drdaemon || failure "Failed to change permissions on drdaemon"
mkdir $INITRD_MNT/trace || failure "Failed to create the /trace directory"
chmod 0777 $INITRD_MNT/trace
cp /sbin/udevd /sbin/udevtrigger /sbin/udevsettle $INITRD_MNT/bin/ || failure "Failed to copy udevtrigger/settle/udevd to initrd"
echo "/bin/udevtrigger" >> $INITRD_MNT/bin/udevstart
echo "/bin/udevsettle" >> $INITRD_MNT/bin/udevstart
chmod 0777 $INITRD_MNT/bin/udevstart || failure "Failed to set execute on udevstart"
cp -f /sbin/ethtool $INITRD_MNT/sbin/ || failure "Failed to copy ethtool"

# Replace product name
sed -i 's/.*Red Hat.*/VaultOS/g' $INITRD_MNT/.buildstamp

# To look at Initrd image ------ ANALYSE 
#echo "Exiting to have a look at initrd"
#exit 1

# Repack ...
cd $INITRD_MNT
find . | cpio -o -H newc | gzip -9 > ../initrd.img.gz || failure "Failed to re-create initrd"
cd -
mv -f initrd.img.gz $1 || failure "Failed to put initrd back in place"
}

customize_filesystem()
{

# Mount file system
echo "Unpacking the filesytem ..."
mount -o loop -t squashfs $EXTRACTED_CD/images/stage2.img $FS_MNT || failure "Failed to mount the stage2.img (cramfs filesystem) from the cd image"
rsync -a $FS_MNT/ $FS_NEW/ || failure "Failed to copy the mounted filesystem"

# To look at filesystem and extracted CD ------ ANALYSE 
#echo "Exiting to have a look at the filesystem"
#exit 1

umount $FS_MNT

# Customize the FileSystem - base folder -- $FS_NEW

echo "Making Customizations ..."
# Compile the python script and copy it.
rm -f rescue.pyc > /dev/null 2>&1
python -c "import py_compile;py_compile.compile('rescue.py')"
[ -f rescue.pyc ] || failure "Failed to compile rescue.py"

# Compile our start program and copy it
cp -f rescue.py rescue.pyc $FS_NEW/usr/lib/anaconda/ || failure "failed to copy py files"
cp -f $DRSTART_PATH/drstart $FS_NEW/usr/bin/ || failure "Failed to copy drstart"
chmod 0777 $FS_NEW/usr/bin/drstart $FS_NEW/usr/lib/anaconda/rescue.py*

# Change Product Name
sed -i 's/.*Red Hat.*/VaultOS/g' $FS_NEW/.buildstamp
# -----  Customizations end -----

# Repack
echo "Repacking the filesystem ..."
mksquashfs fs.new stage2.img || failure "Failed to create the compressed cramfs image"
echo "Creating the new CD ..."
mv -f stage2.img $EXTRACTED_CD/images/stage2.img || failure "Failed to copy the compressed filesystem to the extracetd cd"
}


# MAIN ====>>

if [ $2 = "ia64" ]; then
echo "Platform = ia64"
PLATFORM="ia64"
else
echo "Platform = x86"
PLATFORM="x86"
fi

NEW_IMAGE="vaultos_""$PLATFORM""_v""$VERSION""_""RHEL5.iso"

# Check if squashfs tools are present
if [ ! -e /sbin/mksquashfs ]; then 
failure "Cannot find mksquashfs, Install appropriate rpms before proceeding"
fi

# Cleanup
umount $CD_MNT $FS_MNT $INITRD_MNT > /dev/null 2>&1
rm -fr $EXTRACTED_CD $CD_MNT $FS_MNT $FS_NEW $NEW_IMAGE $INITRD_MNT initrd.img $BOOT_IMG_NEW_MNT $BOOT_IMG_MOUNT $BOOT_IMG_NEW_MNT $BOOT_IMG_TMP  > /dev/null 2>&1

# Mount the CD
echo "Start =>"
echo "Copying the Original CD ..."
mkdir $EXTRACTED_CD $CD_MNT $FS_MNT $FS_NEW $INITRD_MNT
mount -o loop $RHEL_IMAGE $CD_MNT || failure "Failed to mount the cd image, check if you are root and if the image is present"
rsync -a $CD_MNT/ $EXTRACTED_CD/ || failure "Failed to make a copy of the mounted cd image"

customize_filesystem

if [ $PLATFORM = "ia64" ]; then
# Open up boot.img
echo "Opening boot.img ..."
mkdir $BOOT_IMG_MOUNT $BOOT_IMG_TMP $BOOT_IMG_NEW_MNT || failure "Failed to create dirs for boot.img"
mount -o loop $CD_MNT/images/boot.img $BOOT_IMG_MOUNT || failure "Failed to mount boot.img"
rsync -a $BOOT_IMG_MOUNT/ $BOOT_IMG_TMP/ || failure "Failed to copy contents of boot.img"
umount $BOOT_IMG_MOUNT
fi

# customize the initrd image
if [ $PLATFORM = "ia64" ]; then
customize_initrd $BOOT_IMG_TMP/efi/boot/initrd.img
cp -f $BOOT_IMG_TMP/efi/boot/initrd.img $BOOT_IMG_TMP/ || Failure "Failed to copy initrd.img outside"
else
customize_initrd $EXTRACTED_CD/isolinux/initrd.img
fi


if [ $PLATFORM = "ia64" ]; then
# Customize boot.img
cp -f elilo.conf $BOOT_IMG_TMP/efi/boot/ || failure "Failed to copy elilo.conf"
cp -f elilo.conf $BOOT_IMG_TMP/ || failure "Failed to copy elilo.conf outside"

# Pack up boot.img
echo "Packing boot.img ..."
dd if=/dev/zero of=$BOOT_IMG_NEW bs=1024 count=`du -s $BOOT_IMG_TMP | cut -f1`
mkdosfs $BOOT_IMG_NEW || failure "Failed to create dosfs"
mount -o loop $BOOT_IMG_NEW $BOOT_IMG_NEW_MNT || failure "Failed to mount new dosfs for boot.img"
rsync -a $BOOT_IMG_TMP/ $BOOT_IMG_NEW_MNT/ || failure "Failed to copy all files to new boot.img"
umount $BOOT_IMG_NEW_MNT
cp -f $BOOT_IMG_NEW $EXTRACTED_CD/images/boot.img || failure "Failed to copy new boot.img"
fi

umount $CD_MNT

if [ $PLATFORM != "ia64" ]; then
# Place Customizations to the CD itself here .. ( base directory $EXTRACTED_CD )
cp -f isolinux_rhel.cfg $EXTRACTED_CD/isolinux/isolinux.cfg || failure "Failed to copy isolinux.cfg"
cp -f vaultos.msg $EXTRACTED_CD/isolinux/vaultos.msg || failure "Failed to copy vaultos.msg"
fi

cp -f ks.cfg $EXTRACTED_CD/ || failure "Failed to copy ks.cfg"

#  Remove unwanted stuff
rm -fr $EXTRACTED_CD/*.html $EXTRACTED_CD/README* \
$EXTRACTED_CD/EULA $EXTRACTED_CD/GPL $EXTRACTED_CD/RPM-GPG-KEY* \
$EXTRACTED_CD/RE*  \
$EXTRACTED_CD/images/xen $EXTRACTED_CD/images/pxeboot $EXTRACTED_CD/images/minstg2.img \
$EXTRACTED_CD/images/diskboot.img \
$EXTRACTED_CD/Cluster* $EXTRACTED_CD/Server $EXTRACTED_CD/VT 

# Final stage - create the iso

if [ $PLATFORM = "ia64" ]; then
# IA64 Cmd
mkisofs -o "$NEW_IMAGE"  -J -r -A "VaultOS" -V "VaultOS" -no-emul-boot  -b images/boot.img -c boot.cat $EXTRACTED_CD || failure "Failed to create the new iso image"
else
# x86 Cmd
mkisofs -o "$NEW_IMAGE" -T -J -r -A "VaultOS" -V "VaultOS" -boot-info-table -no-emul-boot  -b isolinux/isolinux.bin -c isolinux/boot.cat $EXTRACTED_CD || failure "Failed to create the new iso image"
fi

# Cleanup
rm -fr $EXTRACTED_CD $CD_MNT $FS_MNT $FS_NEW $INITRD_MNT $BOOT_IMG_NEW $BOOT_IMG_MOUNT $BOOT_IMG_TMP $BOOT_IMG_NEW_MNT initrd.img rescue.pyc > /dev/null 2>&1
exit 0
