#!/bin/bash 

# This script creates a customized Ubuntu live CD with Netvault Server, VaultDR Server and drdaemon
# Currently it creates a live CD for x86  and ppc from a Ubuntu 8.04 Live CD.
# Usage ./make_ub_live.sh ubuntucd.iso [x86/ppc]
#
# ***  Place netvault distribution and VaultDr npk file in the current directory and change NV_BUILD_DIR, NV_DIST and DR_SERVER_NPK vars ***
#
# NOTE : Not tested with ppc with new changes
# 
# Check the following files when upgrading to a new Ubuntu revision
#
# 1.) isolinux.cfg
# 2.) %gconf-tree.xml
# 3.) rc.local
#

NV_BUILD_DIR="/home/sapan/Mainline/build/"
NV_DIST="netvault-R2008SEP26-GENUS-Linux.tar.gz"
DR_SERVER_NPK="drc-2-4-1-2.npk"
NV_PARAM="nv_parameters"
RC_LOCAL="rc.local"
NV_INSTALL_SCRIPT="nvinstall.sh"
PLATFORM="x86"
MNTDIR=mnt
CDMNT=cdmnt
ISO_IMAGE="$1"
EXTRACTED_CD=extracted_cd
EXTRACTED_FS=extracted_fs
NEW_EXTRACTED_FS=extracted_fs.new
VAULTOS_LIVE_CD="VaultOS-live-$PLATFORM.iso"
BOOTIMG_MNTDIR=boot.img.mnt
ELILO_PATH=efi/boot/elilo.conf
DESKTOP_BACKGROUND=vaultos_background.png
BACKGROUND_LOCATION=/usr/share/backgrounds/vaultos_background.png
DRDAEMON_PATH="$NV_BUILD_DIR/vaultos/unix/linux"
FS_MNT=fs_mnt

function failure()
{
        echo -e "$@" 1>&2

	# Unmount Everything
	umount $MNTDIR/proc > /dev/null 2>&1
	umount $MNTDIR/sys > /dev/null 2>&1
	umount $MNTDIR/dev > /dev/null 2>&1
	umount $MNTDIR $FS_MNT > /dev/null 2>&1
	umount $BOOTIMG_MNTDIR > /dev/null 2>&1

        exit 2
}

function init_check()
{

# Avoiding Unicode  ... Not Really sure if this is okay
export LC_ALL=C

# Clean up
umount $MNTDIR/dev $MNTDIR/proc $MNTDIR/sys > /dev/null 2>&1
umount $CDMNT $MNTDIR $FS_MNT > /dev/null 2>&1
rm -fr $NEW_EXTRACTED_FS $EXTRACTED_CD $MNTDIR $FS_MNT $VAULTOS_LIVE_CD $BOOTIMG_MNTDIR $CDMNT > /dev/null 2>&1

# Checks -
which mksquashfs > /dev/null 2>&1 || failure "Cannot find mksquashfs Please install squashfs-tools package"

[ -f $ISO_IMAGE ] || failure "Cannot find ubuntu live cd.\n Download it from http://www.ubuntu.com/getubuntu/download"

[ -f $NV_DIST ] || failure "Netvault Dist file $NV_DIST not found, set NV_DIST var"
[ -f $DR_SERVER_NPK ] || failure "Netvault Dist file $DR_SERVER_NPK not found, set DR_SERVER_NPK var"
[ -f $DRDAEMON_PATH/drdaemon ] || failure "Drdaemon not found, please compile"

}

function customize_boot
{
# Customizing Boot for x86 only
if [ "$PLATFORM" = "x86" ]; then
	cp -f isolinux.cfg $EXTRACTED_CD/isolinux/ || failure "Failed to customize boot options"
echo "en" > $EXTRACTED_CD/isolinux/lang || failure "Failed to set defualt GFXBOOT language"
fi

}

function free_space
{
echo "Trying to free space on the CD ..."
# Delete Windows realated programs
rm -fr $EXTRACTED_CD/programs $EXTRACTED_CD/bin

# Remove unwanted things from the Filesystem
rm -fr $MNTDIR/usr/share/doc 
rm -fr $MNTDIR/usr/share/example-content 
rm -fr $MNTDIR/usr/lib/openoffice

}

function customize_cd
{
#  Customizations to add drdaemon ...
echo "Adding drdaemon to the CD ..."
cp $DRDAEMON_PATH/drdaemon VaultDr_client.sh  $MNTDIR/bin/ || failure "Failed to copy Drdaemon to the CD"
chmod 0777 $MNTDIR/bin/drdaemon $MNTDIR/bin/VaultDr_client.sh || failure "Failed to set the right permissions for binaries"

# Setting permissions so that drdaemon always runs with root permissions
chmod a+s $MNTDIR/bin/drdaemon $MNTDIR/bin/VaultDr_client.sh || failure "Failed to set the right permissions for binaries"
mkdir $MNTDIR/trace || failure "Failed to create /trace directory in the CD"
chmod 0777 $MNTDIR/trace

# Copy all the netvault stuff
cp $NV_PARAM  $NV_DIST $DR_SERVER_NPK $MNTDIR/root/ || failure "Failed to copy netvault files"
cp -f $NV_INSTALL_SCRIPT $MNTDIR/usr/bin/ || failure "Failed to copy $NV_INSTALL_SCRIPT"
cp -f $RC_LOCAL $MNTDIR/etc/rc.local || failure "Failed to copy rc.local"
chmod 0777 $MNTDIR/usr/bin/$NV_INSTALL_SCRIPT $MNTDIR/etc/rc.local ||  failure "Failed to set permissions on scripts"

# Create Udevstart as it does not exist on Ubuntu , we just make a script with equivalent commands
echo "/sbin/udevtrigger" > $MNTDIR/bin/udevstart || failure "Failed to create /bin/udevstart"
echo "/sbin/udevsettle" >> $MNTDIR/bin/udevstart || failure "Failed to write to /bin/udevstart"
echo "exit 0" >> $MNTDIR/bin/udevstart || failure "Failed to write to /bin/udevstart"
chmod 0777 $MNTDIR/bin/udevstart

# To Autostart drdaemon when the user logs in
cp default.session $MNTDIR/usr/share/gnome/default.session || failure "Failed to add drdaemon at startup"

# Special Desktop customizations ...
echo "Applying Desktop customizations ..."

# Removing extra Items from the Desktop, But they are links that are created Dynamically , so lets destroy the source so there are no links on desktop
rm -fr $MNTDIR/usr/share/example-content $MNTDIR/usr/share/applications/ubiquity-gtkui.desktop

cp -f %gconf-tree.xml $MNTDIR/etc/gconf/gconf.xml.defaults/ || failure "Failed to customize Desktop"

#Changing the Background Image of the desktop -- Not changing for now
#cp $DESKTOP_BACKGROUND $MNTDIR/$BACKGROUND_LOCATION
#chroot $MNTDIR gconftool-2 --direct --config-source xml:readwrite:/etc/gconf/gconf.xml.defaults --type string --set /desktop/gnome/background/picture_filename $BACKGROUND_LOCATION

# Before we customize we need to set the resolve.conf right
cp $MNTDIR/etc/resolv.conf $MNTDIR/etc/resolv.conf_bak > /dev/null 2>&1
cp /etc/resolv.conf $MNTDIR/etc/resolv.conf

# This script can be actually used to remove / install packages and other customizations
cp customize $MNTDIR/ || failure "Failed to copy cusomtization script"

chmod 0777 $MNTDIR/customize
echo "Running customization script ... "
chroot $MNTDIR ./customize || failure "Customize script failed"
rm $MNTDIR/customize

# reset resolv.conf
rm $MNTDIR/etc/resolv.conf
mv $MNTDIR/etc/resolv.conf_bak $MNTDIR/etc/resolv.conf > /dev/null 2>&1

echo "Customizing the bootloader ..."
customize_boot

# Try to free space on the CD
free_space

}


############################### MAIN START ###############################
START=`date`
if [ $# -le 1 ]; then
PLATFORM="x86"
fi
echo "Creating Live CD for $PLATFORM ... "

#  Initialization
init_check

# Mount the image:
mkdir $MNTDIR $CDMNT || failure "Failed to create mnt/cdmnt directory."
mount $ISO_IMAGE $CDMNT -o loop  || failure "Failed to mount iso image."

# Extract everything to a temporary directory:
echo "Copying CD ..."
rsync --exclude=/casper/filesystem.squashfs -a $CDMNT/ $EXTRACTED_CD/ || failure "Failed to extract the contents of the ISO Image"

# mount the uncompressed filesystem:
echo "Copying livecd filesystem ..."
mkdir $FS_MNT
mount -t squashfs $CDMNT/casper/filesystem.squashfs $FS_MNT -o loop || failure "Failed to mount the filesystem"
rsync -a $FS_MNT/ $MNTDIR/ || failure "Failed to copy the filesystem"
umount $FS_MNT/

# We are done with the original CD and can unmount it:
umount $CDMNT

# Mounting proc and sysfs
mount -t proc none $MNTDIR/proc
mount -t sysfs none $MNTDIR/sys
mount --bind /dev/ $MNTDIR/dev

# For looking at the filesystem -- Debugging
#echo " Exiting to look at the File System on CD. Look into the mnt folder"
#exit 0

############## Customize the CD ###############
customize_cd

# Unmount Everything
umount $MNTDIR/proc
umount $MNTDIR/sys
umount $MNTDIR/dev

# Regenrate Manifest
chmod +w $EXTRACTED_CD/casper/filesystem.manifest
chroot $MNTDIR dpkg-query -W --showformat='${Package} ${Version}\n' > $EXTRACTED_CD/casper/filesystem.manifest
cp $EXTRACTED_CD/casper/filesystem.manifest $EXTRACTED_CD/casper/filesystem.manifest-desktop
sed -ie '/ubiquity/d' $EXTRACTED_CD/casper/filesystem.manifest-desktop

# Regenerate the checksum
rm $EXTRACTED_CD/md5sum.txt
(cd $EXTRACTED_CD && find . -type f -print0 | xargs -0 md5sum > md5sum.txt)

# Compress the filesystem and put it back on the CD
echo "Compressing the Filesystem and putting it back into the CD ..."
rm -f $EXTRACTED_CD/casper/filesystem.squashfs
mksquashfs $MNTDIR $EXTRACTED_CD/casper/filesystem.squashfs -nolzma || failure "Failed to create compressed fs"

echo "Creating the actual ISO Image ... "

if [ "$PLATFORM" = "x86" ]; then

mkisofs -D -r -V "VaultOS" -cache-inodes -J -l -b isolinux/isolinux.bin \
	-c isolinux/boot.cat -no-emul-boot -boot-load-size 4 \
	-boot-info-table -o $VAULTOS_LIVE_CD $EXTRACTED_CD || failure "Failed to create ISO Image"

fi

if [ "$PLATFORM" = "ppc" ]; then

mkisofs -r -V "VaultOS" \
            --netatalk -hfs -probe -map hfs.map \
            -chrp-boot -iso-level 2 -part -no-desktop \
            -hfs-bless extracted_cd/install \
            -hfs-volid Ubuntu/PowerPC_edgy \
            -o $VAULTOS_LIVE_CD $EXTRACTED_CD || failure "Failed to create ISO Image"
fi

echo "Created VaultOS Live CD for $PLATFORM --  $VAULTOS_LIVE_CD !!!!"
# Clean up
echo "Cleaning up ..."
rm -fr $NEW_EXTRACTED_FS $EXTRACTED_CD $MNTDIR $FS_MNT $CDMNT

END=`date`
echo "Start time = $START"
echo "End   time = $END"
