#!/bin/sh

PATH=$PATH:/bin:/sbin:/usr/bin:/usr/sbin
export PATH


# ==========================================================================
# ==========================================================================
VersionCompare()
{
	LVAL=$1;
	RVAL=$2;
	Return=0;

	LLEN=`perl -e 'print length $ARGV[0];' $LVAL`
	RLEN=`perl -e 'print length $ARGV[0];' $RVAL`

	if [ $LLEN -ne $RLEN ]
	then
		if [ $LLEN -gt $RLEN ]
		then
			RVAL=`printf "%s.0" $RVAL`
		else
			LVAL=`printf "%s.0" $LVAL`
		fi
	fi

	DFLVAL=`echo "$LVAL" | awk -F. '{print $1$2$3}'`
	DFRVAL=`echo "$RVAL" | awk -F. '{print $1$2$3}'`
	if [ $DFLVAL -gt $DFRVAL ]
	then
			Return=1;
	elif [ $DFLVAL -lt $DFRVAL ]
	then
			Return=-1;
	fi
		echo $Return;
}

# ==========================================================================
# ==========================================================================
CleanOutputDirectory()
{
	echo
	echo "Clearing cdimage from build dir."
	rm -rf cdimage
	rm -rf usbimage
	rm -rf vaultfs
}

# ==========================================================================
# ==========================================================================
ValidateRamDiskSize ()
{
	echo
	echo "Validate RAMDISK size for current system"

	if strRamDisk=`dmesg | grep 'RAMDISK driver initialized'`
	then
		lKernelVer=`uname -r | awk -F. '{print $1"."$2}'`
	elif strRamDisk=`dmesg | grep 'RAM disk driver initialized'`
	then
		lKernelVer=2.2
	else
		lKernelVer=Unknown
	fi

	if [ "Unknown" = "$lKernelVer" ]
	then
		echo
		echo "WARNING: Don't know RAMDISK size, required RAMDISK size of 65536K"
		echo
		return 1
	elif [ "2.4" = "$lKernelVer" ]
	then
		strRamDiskSize=`echo $strRamDisk | cut -d" " -f8`
	elif [ "2.2" = "$lKernelVer" ]
	then
		strRamDiskSize=`echo $strRamDisk | cut -d" " -f9`
	fi

	if [[ "65536K" < "$strRamDiskSize" ]]
	then
		echo
		echo "ERROR: Current RAMDISK size is: $strRamDiskSize. Required RAMDISK size is: 65536K"
		echo "       To modify RAMDISK size using LILO: edit file /etc/lilo.conf, add a line"
		echo "       ramdisk=65536. Run /sbin/lilo and reboot the system. If running different"
		echo "       Boot Loader feed ramdisk_size=65536 to kernel's ARGS"
		echo
		exit 1
	fi
	return 0
}



# ==========================================================================
# ==========================================================================
ValidateNasmCompiler ()
{
	echo
	echo "Validate NASM Netwide Assembler Compiler for current system"

	if [ -f /usr/bin/nasm ]
	then
		strNasmVer=`/usr/bin/nasm -v | cut -d" " -f3`
	elif strNasmVer=`rpm -qa | grep nasm`
	then
		strNasmVer=`echo $strNasmVer | cut -d"-" -f2`
	else
		echo
		echo "ERROR: No Netwide Assembler Compiler (nasm) found in this system. NASM compiler"
		echo "       verison 0.98.34 (prefer) required to make VaultDR bootable CDROM ISO"
		echo "       image. The nasm-0.98.34-1 RPM is available every where in the net to"
		echo "       download and install in your system"
		echo
		exit 1
	fi

	COMPRESULT=`VersionCompare "0.98.34" $strNasmVer`
	if [ $COMPRESULT = 1 ]
	then
		echo
		echo "WARNING: Current NASM version is: $strNasmVer. Prefer: 0.98.34 or later"
		echo
	fi
}



# ==========================================================================
# ==========================================================================
ValidateMkisofs ()
{
	echo
	echo "Validate mkisofs - Create hybrid ISO9660/JOLIET/HFS for current system"

	if [ -f /usr/bin/mkisofs ]
	then
		strMkisofsVer=`/usr/bin/mkisofs -version | cut -d" " -f2`
	elif strMkisofsVer=`rpm -qa | grep mkisofs`
	then
		strMkisofsVer=`echo $strMkisofsVer | cut -d"-" -f2`
	else
		echo
		echo "ERROR: mkisofs - Create hybrid ISO9660/JOLIET/HFS filesystem is not found in this"
		echo "       system. mkisofs version 1.14 or later is required to make VaultDR bootable"
		echo "       CDROM ISO image. mkisofs RPM is available every where in the net to download"
		echo "       and install in your system"
		echo
		exit 1
	fi

	COMPRESULT=`VersionCompare "1.14" $strMkisofsVer`
	if [ $COMPRESULT = 1 ]
	then
		echo
		echo "WARNING: Current mkisofs version is: $strMkisofsVer. Prefer: 1.14 or later"
		echo
	fi
}

# ==========================================================================
# Executes and Checks the exit status variable $? and exits script if fails.
#   Parameter: 1 - Command to Execute
#	             2 - Custom error msg if failed
# ==========================================================================
CheckReturnCode()
{
	echo "Executing: '$1' ..."
	$1 > /dev/null 2>&1  #Execute whatever was passed as 1st parameter to function
	
	if [ $? != 0 ]       #Check return Exit status
	then
		if [ -z "$2" ]     #Check if we wanted to output a custom error msg.
		then
			echo "'$1' failed for some reason...please fix"
			echo ""
		else
			echo "FAILURE: $2" #Output a custom error msg
			echo ""
		fi
		exit 1
	else
		echo "'$1' succeeded."
		echo ""
	fi
}


# ==========================================================================
# PrepareOutputDirectory - Extracts the vaultOS filesystem from the vaultfs.tar.gz
#   file. 
# ==========================================================================
PrepareOutputDirectory()
{

	# VaultOS building using kernel source 2.4/2.6
	echo;
	echo "The Machine Kernel version is: $VaultOSKernel.";
	echo "Enter the  choice for kernel source version used for VaultOS[1/2]";
	echo "1. 2.4 Kernel Source";
	echo "2. 2.6 Kernel Source";
	read KernelVer;
	VaultOSKernel=2.4
	if [ 2 -eq $KernelVer ]
	then
		strKernelSrc=kernel_x86_2695
		strModulesDir=/lib/modules/2.6.9-5.EL
		VaultOSKernel=2.6
		strTarfile="vaultfs_2.6.tgz"
		strBootloaderDir="bootloader_2.6"
	elif [ 1 -ne $KernelVer ]
	then
		echo "Incorrect Input Value";
		exit 1;
	fi

	echo "****************The Enviroment Selected**********************"
	echo "Kernel Version: $VaultOSKernel"
	echo "Kernel SourceName: $strKernelSrc"
	echo "Module Dir: $strModulesDir"
	echo "*************************************************************"

	TARFILE="$sourceRoot/$strTarfile"
	if [ TRUE == $untarVaultOS ]
	then
		if [ -f $TARFILE ]
		then 
			echo
			echo "Found Filesystem file $TARFILE"
			CheckReturnCode "tar xfz $TARFILE"
		else
			echo "Could not find the $TARFILE filesystem file. Check it out from CVS"
		fi
	fi

	TARFILE="$sourceRoot/bootloader_$VaultOSKernel.tgz"
		echo "Copying Bootloader file."
		if [ -f $TARFILE ]
		then 
			echo
			echo "Found Filesystem file $TARFILE"
			CheckReturnCode "tar xfz $TARFILE"
		else
			echo "Could not find the $TARFILE filesystem file. Check it out from CVS"
		fi
	echo "Creating cdimage inside builddir"
	if [ ! -d cdimage ]
	then
		CheckReturnCode "mkdir cdimage" "Error in creating cdimage dir"
	else
		echo "Directory cdimage Already exists"
	fi
	if [ ! -d usbimage ]
	then
		CheckReturnCode "mkdir usbimage" "Error in creating cdimage dir"
	else
		echo "Directory usbimage Already exists"
	fi
}

# ==========================================================================
# BuildDrExecutables - Build the drdameon, drnic, drscsi, etc... that are the
#   whole point of this boot image.
# ==========================================================================
BuildDrExecutables()
{
	if [ TRUE == $buildDrExes ]
	then
		strHomeDir=$PWD
		echo "Current Dir = $PWD"
		CheckReturnCode "cd $strHomeDir"
		CheckReturnCode "cd ../../.."
		if [ ! -f makefile ] || [ ! -f x86/config/makefile ]
		then
			echo "Could not find one of the drdaemon make file or x86/config/Makefile"
			exit 1
		fi
		echo "$PWD"
		CheckReturnCode "make" "Making DrDaemon executable"
		CheckReturnCode "cp -df drdaemon $strHomeDir/vaultfs/bin/" "Copying DrDaemon executable to vaultfs"
		CheckReturnCode "cd x86/config"
		CheckReturnCode "make" "Making drnic, drscsi... config executable"
		CheckReturnCode "cp -df drdaemon_ncr $strHomeDir/vaultfs/bin/"
		CheckReturnCode "cp -df drinit $strHomeDir/vaultfs/bin/"
		CheckReturnCode "cp -df $sourceRoot/../../config/drmktmp_ncr.sh $strHomeDir/vaultfs/bin/"
		CheckReturnCode "cp -df drnic $strHomeDir/vaultfs/bin/"
		if [ 2.6 == "$VaultOSKernel" ]
		then
		CheckReturnCode "cp -df $sourceRoot/../../config/drnic_26.txt $strHomeDir/vaultfs/bin/drnic.txt"
		CheckReturnCode "cp -df $sourceRoot/../../config/drscsi_26.txt $strHomeDir/vaultfs/bin/drscsi.txt"
		else
		CheckReturnCode "cp -df $sourceRoot/../../config/drnic.txt $strHomeDir/vaultfs/bin/"
		CheckReturnCode "cp -df $sourceRoot/../../config/drscsi.txt $strHomeDir/vaultfs/bin/"
		fi
		CheckReturnCode "cp -df drprobe $strHomeDir/vaultfs/bin/"
		CheckReturnCode "cp -df drscsi $strHomeDir/vaultfs/bin/"
		CheckReturnCode "cp -df drxrecv $strHomeDir/vaultfs/bin/"
		CheckReturnCode "cp -df drxsend $strHomeDir/vaultfs/bin/"
		CheckReturnCode "cp -df drzero $strHomeDir/vaultfs/bin/"
		CheckReturnCode "cd $strHomeDir"

		# Copy necessary libraries into lib directory.
		CheckReturnCode "cp -f /sbin/modprobe         $strHomeDir/vaultfs/bin/"
		CheckReturnCode "cp -f /sbin/depmod           $strHomeDir/vaultfs/bin/"
		CheckReturnCode "cp -f /lib/ld-linux.so.2     $strHomeDir/vaultfs/lib/"
		CheckReturnCode "cp -f /usr/lib/libcrack.so.2 $strHomeDir/vaultfs/lib/"
		CheckReturnCode "cp -f /lib/libcrypt.so.1     $strHomeDir/vaultfs/lib/"
		CheckReturnCode "cp -f /lib/libc.so.6         $strHomeDir/vaultfs/lib/"
		CheckReturnCode "cp -f /lib/libdl.so.2        $strHomeDir/vaultfs/lib/"
		CheckReturnCode "cp -f /lib/libm.so.6         $strHomeDir/vaultfs/lib/"
		CheckReturnCode "cp -f /usr/lib/libncurses.so.5 $strHomeDir/vaultfs/lib/"
		CheckReturnCode "cp -f /lib/libnsl.so.1       $strHomeDir/vaultfs/lib/"
		CheckReturnCode "cp -f /lib/libnss_files.so.2 $strHomeDir/vaultfs/lib/"
		CheckReturnCode "cp -f /lib/libpam_misc.so.0  $strHomeDir/vaultfs/lib/"
		CheckReturnCode "cp -f /lib/libpam.so.0       $strHomeDir/vaultfs/lib/"
		CheckReturnCode "cp -f /lib/libproc-3.2.3.so $strHomeDir/vaultfs/lib/"
		#CheckReturnCode "cp -f /lib/libproc.so.2.0.11 $strHomeDir/vaultfs/lib/"
		CheckReturnCode "cp -f /usr/lib/libreadline.so.4.3 $strHomeDir/vaultfs/lib/"
		CheckReturnCode "cp -f /lib/libresolv.so.2    $strHomeDir/vaultfs/lib/"
		CheckReturnCode "cp -f /lib/libtermcap.so.2   $strHomeDir/vaultfs/lib/"
		CheckReturnCode "cp -f /lib/libutil.so.1      $strHomeDir/vaultfs/lib/"
		CheckReturnCode "cp -f /lib/libpthread-0.10.so $strHomeDir/vaultfs/lib/libpthread.so.0"
		CheckReturnCode "cp -f /usr/lib/libz.so $strHomeDir/vaultfs/lib/libz.so.1"
		CheckReturnCode "cd $strHomeDir"
	fi
}


# ==========================================================================
# BuildKernelAndModules - calls the build command on the kernel_x86_2418
#   directory which should be on same level as nv604 root.
# ==========================================================================
BuildKernelAndModules ()
{
	echo;
	echo "Building the Kernel Image";
	echo;
	strHomeDir=$PWD;

	cd ../../../../../../../$strKernelSrc;
	strKernelDir=$PWD;
	echo "Enter Location of kernel [Default=$strKernelDir]";
	
	if [ FALSE == $useDefaultPaths ]   #Check if user wants to use default
	then
		read strUserKernelDir            #Get path from user
		if [ "" != "$strUserKernelDir" ] #Maybe user wants to use default
		then
			strKernelDir=strUserKernelDir; #User overrode default
			CheckReturnCode "cd $strUserKernelDir"
		fi
	fi
	
	if [ -f Makefile ]                 #Ensure makefile exists in kernel source dir
	then
		echo "Makefile found!"
		if [ TRUE == $buildKernel ]      #Does user want to build the kernel?
		then
			#Build kernel with config file
			echo "***********************************"
			echo "Building Kernel Image..."
			CheckReturnCode "make mrproper"
			#Use our custom configuration file.
			CheckReturnCode "cp -df $sourceRoot/zconfig .config"
			CheckReturnCode "cp -df $sourceRoot/misc.config arch/i386/boot/compressed/misc.c"
			CheckReturnCode "make oldconfig"
			CheckReturnCode "make dep"
			CheckReturnCode "make bzImage"
		fi
		CheckReturnCode "cp -df arch/i386/boot/bzImage $strHomeDir/cdimage/vaultos"
		if [ TRUE == $buildModules ]     #Does user want to build the modules?
		then
			echo "Enter location of the installed modules dir {Default=$strModulesDir}"
			if [ FALSE == $useDefaultPaths ] #User wants to get prompted for Module Dir
			then
				read strUserModulesDir;
				if [ "" != "$strUserModulesDir" ]
				then
					strModulesDir=$strUserModulesDir;
				fi
			fi
			echo "**********************************"
			echo "Building Modules... $PWD"
			CheckReturnCode "make modules"
			#CheckReturnCode "make modules_install"
		fi
		CheckReturnCode "$sourceRoot/cp.pl $strModulesDir/drivers/net $strHomeDir/vaultfs/mods/net/ \.[k]o"
		CheckReturnCode "$sourceRoot/cp.pl $strModulesDir/drivers/scsi $strHomeDir/vaultfs/mods/scsi/ \.[k]o"
		CheckReturnCode "$sourceRoot/cp.pl $strModulesDir/drivers/block $strHomeDir/vaultfs/mods/scsi/ \.[k]o"
		CheckReturnCode "$sourceRoot/cp.pl $strModulesDir/drivers/message $strHomeDir/vaultfs/mods/scsi/ \.[k]o"
		CheckReturnCode "$sourceRoot/cp.pl $strModulesDir/drivers/addon/bcm5700 $strHomeDir/vaultfs/mods/net/ \.[k]o"
		CheckReturnCode "$sourceRoot/cp.pl $strModulesDir/drivers/addon/e1000 $strHomeDir/vaultfs/mods/net/ \.[k]o"
		CheckReturnCode "$sourceRoot/cp.pl $strModulesDir/drivers/addon/e100 $strHomeDir/vaultfs/mods/net/ \.[k]o"
		CheckReturnCode "$sourceRoot/cp.pl $strModulesDir/drivers/ide $strHomeDir/vaultfs/mods/autoload/ \.[k]o"
		CheckReturnCode "/bin/cp $strModulesDir/drivers/scsi/qla2xxx/qla2xxx.ko  $strHomeDir/vaultfs/mods/autoload/"
		#Copy any custom modules from the 'extramodules' directory
		if [ TRUE == $copyExtraMods ]
			then
				CheckReturnCode "$sourceRoot/cp.pl $strHomeDir/extramodules/net $strHomeDir/vaultfs/mods/net/ \.[k]o"
				CheckReturnCode "$sourceRoot/cp.pl $strHomeDir/extramodules/scsi $strHomeDir/vaultfs/mods/scsi/ \.[k]o"
				CheckReturnCode "$sourceRoot/cp.pl $strHomeDir/extramodules/autoload $strHomeDir/vaultfs/mods/autoload/ \.[k]o"
		fi
	else
		echo "Kernel Makefile Not Found"
		exit 1
	fi
	#Now set the current directory back to the starting point
	CheckReturnCode "cd $strHomeDir"
}


# ==========================================================================
# ==========================================================================
ValidateMisc ()
{
	#Copy the bootloader config to the output directory.
	CheckReturnCode "cp $strBootloaderDir/isolinux.cfg cdimage/isolinux.cfg"
	if [ ! -f cdimage/isolinux.cfg ]
	then
		echo
		echo "ERROR: Can not find file isolinux.cfg in directory ./cdimage"
		echo
		exit 1
	fi
	
	if [ ! -f cdimage/vaultos ]
	then
		echo
		echo "ERROR: Can not find file VaultDR Kernel (vaultos) in ./cdimage"
		echo
		exit 1
	fi
	
	if [ ! -f /root/mp ]
	then
		mkdir -p /root/mp > /dev/null 2>&1
	fi
}


# ==========================================================================
# ==========================================================================
CompileAndMakeIso ()
{
	echo
	echo "Compiling Boot Loader"
	strHomeDir=$PWD
	strErrorMsg="ERROR: Failed to compile Boot Loader. Validate source code"
	cd $strBootloaderDir;
	CheckReturnCode "nasm -O99 -f bin -DDATE_STR="'0x3f8de1cc'" -DHEXDATE="0x3f8de1cc" -l isolinux.lst -o isolinux.bin isolinux.asm" "$strErrorMsg"
	CheckReturnCode "mv isolinux.bin ../cdimage/" "$strErrorMsg"
	cd ..;

	umount /root/mp	 > /dev/null 2>&1
	umount /dev/ram0 > /dev/null 2>&1

	echo
	echo "Initialize and make RAMDISK filesystem"
	dd if=/dev/zero of=/dev/ram0 bs=1024 count=65536 > /dev/null 2>&1
	mke2fs /dev/ram0 -b 1024 65536 > /dev/null 2>&1

	if ! mount -t ext2 /dev/ram0 /root/mp > /dev/null 2>&1
	then
		echo
		echo "ERROR: Failed to mount /dev/ram0 to /root/mp mount point"
		echo
		exit 1
	fi
	echo
	echo "Migrate VaultDR CDROM filesystem to RAMDISK filesystem"
	strErrorMsg="ERROR: Copying the file system"
	cd vaultfs;
	find . > zlist
	cpio -o --format=newc < zlist > drfs
	gzip -f drfs
	echo
	echo "Making final VaultDR/RAMDISK filesystem"
	CheckReturnCode "mv drfs.gz ../cdimage/drfs" "$strErrorMsg"
	cd ..
	#cp -Rdp vaultfs/* /root/mp > /dev/null 
	#umount /root/mp	 > /dev/null 2>&1
	#umount /dev/ram0 > /dev/null 2>&1
	#dd if=/dev/ram0 of=drfs bs=1024 count=65536 > /dev/null 2>&1
	#gzip drfs > /dev/null
	
	echo
	echo "Making VaultDR bootable CDROM ISO image"
	strErrorMsg="ERROR: Failed to make VaultDR bootable CDROM ISO image"
	CheckReturnCode "mkisofs -o cdimage/drx86.iso -b isolinux.bin -c boot.cat -no-emul-boot -boot-load-size 4 -boot-info-table cdimage" "$strErrorMsg"
	
	echo
	echo "Completed making VaultDR bootable CDROM ISO image (drx86.iso) successfully!"
	echo "Compilation output are stored in $PWD/cdimage "
	echo
}


# ==========================================================================
# USB Compilation
# ==========================================================================
CompileAndMakeUSBIso ()
{

	if [ 2.6 == "$VaultOSKernel" ]
	then
  	strHomeDir=$sourceRoot
		echo $strHomeDir
  	#cd boot_usb_grub;
		echo " You Want to Building USB Image [y/n]"
		read Ans
		if [ 'y' == $Ans ] || [ 'Y' == $Ans ]
		then
			strErrorMsg="ERROR: Failed to Build ISO Image."
			echo "Please insert the USB key ..."
			read strUserKernelDirp
			echo "path = $PWD"
			echo "ret Value:" 
			sh $strHomeDir/boot_usb_grub/zdr_make $PWD;
			if [ 1 -eq $? ]
			then
				echo "ERROR: Failed to make bootable USB ISO image"
			else
				echo "USB Boot Image Stored in $PWD/usbimage."
			fi	
		else
			exit 0
		fi
	fi
  echo

}



# ==========================================================================
# Main
# ==========================================================================
trap 'trap "" 1 2 3 9 15;
  echo 
  echo "WARN: Terminated automation of making VaultDR CDROM ISO process."
  echo 
  exit 1;
  exit 2' 1 2 3 15
	
# Get Command line Arguments
useDefaultPaths=FALSE;
buildKernel=TRUE;
buildModules=TRUE;
buildDrExes=TRUE;
copyExtraMods=TRUE;
untarVaultOS=TRUE;
sourceRoot=
VaultOSKernel=`uname -r | awk -F. '{print $1"."$2}'` 
strTarfile="vaultfs.tar.gz"
strBootloaderDir="bootloader_2.4"
strKernelSrc=kernel_x86_2418
strModulesDir=/lib/modules/2.4.18-3custom;

while getopts Dekmtvs: opt
do
	case "$opt" in
		D) echo "Using default paths"; useDefaultPaths=TRUE;;
		e) echo "Don't copy Extra Modules to image"; copyExtraMods=FALSE;;
		k) echo "Not Building the Kernel"; buildKernel=FALSE;;
		m) echo "Not Building the Modules";buildModules=FALSE;;
		t) echo "Not Untar'ing the VaultOS FS";untarVaultOS=FALSE;;
		v) echo "Not Building the Dr exe's";buildDrExes=FALSE;;
		s) sourceRoot=$OPTARG;;
		\?) echo "Unknown Parameter:"
				echo "Usage: autobuild.sh [-Dekmtv]"
        echo "   -D => Use Default Paths (no prompt)"
				echo "   -e => Don't copy Extra Modules from extramodules dir"
				echo "   -k => Don't build kernel"
				echo "   -m => Don't build the modules"
				echo "   -t => Don't untar vaultos.tar.gzip"
				echo "   -v => Don't build the dr executables";
				echo "   -s <DIR>=> DIR is the source code directory of this module."; exit 1;;
	esac
done

if [ -n $sourceRoot ]
then
		echo "Invoked by make with $sourceRoot";
fi
CleanOutputDirectory
ValidateRamDiskSize       #Ramsize needs to be 64Mb
ValidateNasmCompiler      
ValidateMkisofs
PrepareOutputDirectory    #Extract the filesystem file to copy modules into.
# Build the kernel before the DR binaries, because drdevice.c requires the version.h in  
# the kernel source that is created during 'make dep' stage.
BuildKernelAndModules     #Build the kernel and modules and copy them to vaultfs
BuildDrExecutables        #Build the drdaemon, drnic, drscsi files, and copy them to vaultfs
ValidateMisc
CompileAndMakeIso
CompileAndMakeUSBIso
