%ifndef HAVE_SPECIAL_APPEND
%macro SPECIAL_APPEND 0
%endmacro
%endif

%ifndef HAVE_UNLOAD_PREP
%macro UNLOAD_PREP 0
%endmacro
%endif

is_linux_kernel:
		cmp dx,80h			
		ja kernel_corrupt
		and dx,dx
		jnz kernel_sane
		cmp ax,1024			
		jb kernel_corrupt

kernel_sane:	
		push ax
		push dx
		push si
		push word real_mode_seg
		pop es
		movzx eax,ax			
		shl edx,16			
		or eax,edx
		mov [KernelSize],eax
		xor edx,edx
		div dword [ClustSize]		
		add edx,byte -1			
		adc eax,byte 0			
		mov [KernelClust],eax
		call abort_check		
		mov ecx,[ClustPerMoby]
		shr ecx,1			
		cmp ecx,[KernelClust]
		jna .normalkernel
		mov ecx,[KernelClust]

.normalkernel:
		sub [KernelClust],ecx
		xor bx,bx
		pop si                          
		call getfssec
		cmp word [es:bs_bootsign],0AA55h
		jne kernel_corrupt		
		push si
		call highmemsize

construct_cmdline:
		mov di,[CmdLinePtr]
		mov si,boot_image        	
		mov cx,boot_image_len
		rep movsb
		mov si,KernelCName       	
		mov cx,[KernelCNameLen]
		rep movsb
		mov al,' '                      
		stosb
		SPECIAL_APPEND			
		mov si,[CmdOptPtr]              
		mov cx,(kern_cmd_len+3) >> 2
		rep movsd
		mov si,cmd_line_here
		mov byte [initrd_flag],0
		push es				
		pop ds

get_next_opt:   
		lodsb
		and al,al
		jz cmdline_end
		cmp al,' '
		jbe get_next_opt
		dec si
		mov eax,[si]
		cmp eax,'vga='
		je is_vga_cmd
		cmp eax,'mem='
		je is_mem_cmd
%if IS_PXELINUX
		cmp eax,'keep'			
		jne .notkeep
		cmp dword [si+3],'ppxe'
		jne .notkeep
		cmp byte [si+7],' '		
		ja .notkeep
		or byte [cs:KeepPXE],1

.notkeep:
%endif
		push es                         
		push cs
		pop es                          
		mov di,initrd_cmd
		mov cx,initrd_cmd_len
		repe cmpsb
		jne not_initrd
		mov di,InitRD
		push si                         
		call mangle_name                
		pop si
		cmp byte [es:InitRD],NULLFILE	
		seta byte [es:initrd_flag]	

not_initrd:	
		pop es                          

skip_this_opt:  
		lodsb                           
		cmp al,' '
		ja skip_this_opt
		dec si
		jmp short get_next_opt

is_vga_cmd:
		add si,4
		mov eax,[si-1]
		mov bx,-1
		cmp eax,'=nor'			
		je vc0
		dec bx				
		cmp eax,'=ext'			
		je vc0
		dec bx				
		cmp eax,'=ask'			
		je vc0
		call parseint                   
		jc skip_this_opt		

vc0:		
		mov [bs_vidmode],bx		
		jmp short skip_this_opt

is_mem_cmd:
		add si,4
		call parseint
		jc skip_this_opt		
%if HIGHMEM_SLOP != 0
		sub ebx,HIGHMEM_SLOP
%endif
		mov [cs:HighMemSize],ebx
		jmp short skip_this_opt

cmdline_end:
		push cs                         
		pop ds
		sub si,cmd_line_here
		mov [CmdLineLen],si		
		mov dword [RamdiskMax], HIGHMEM_MAX	
		cmp dword [es:su_header],HEADER_ID	
		jne old_kernel		
		cmp word [es:su_version],0200h	
		jb old_kernel		
		cmp word [es:su_version],0201h	
		jb new_kernel                   
		mov word [es:su_heapend],linux_stack	
		or byte [es:su_loadflags],80h	
		cmp word [es:su_version],0203h	
		jb new_kernel			
		mov eax,[es:su_ramdisk_max]
		mov [RamdiskMax],eax		

new_kernel:
		mov byte [es:su_loader],my_id	
		movzx ax,byte [es:bs_setupsecs]	
		mov [SetupSecs],ax
		mov al,[es:su_loadflags]
		mov [LoadFlags],al

read_kernel:
		mov eax,[HighMemSize]
		sub eax,100000h
		cmp eax,[KernelSize]
		jb no_high_mem
		movzx esi,word [SetupSecs]
		inc si
		shl si,9
		mov ecx,8000h
		sub ecx,esi
		push ecx
		add esi,(real_mode_seg << 4)
		mov edi,100000h 
		call bcopy
		call abort_check
		pop ecx
		pop si
		mov eax,[KernelSize]
		sub eax,8000h
		jbe high_load_done
		call load_high

high_load_done:
		mov ax,real_mode_seg
		mov es,ax

load_initrd:
		test byte [initrd_flag],1
		jz nk_noinitrd
		push es                         
		push ds
		pop es                          
		mov si,InitRD
		mov di,InitRDCName
		call unmangle_name              
		sub di,InitRDCName
		mov [InitRDCNameLen],di
		mov di,InitRD
		call searchdir                  
		pop es
		jz initrd_notthere
		mov [es:su_ramdisklen1],ax	
		mov [es:su_ramdisklen2],dx
		mov edx,[HighMemSize]		
		dec edx
		mov eax,[RamdiskMax]		
		cmp edx,eax
		jna memsize_ok
		mov edx,eax			

memsize_ok:
		inc edx
		xor dx,dx			
		sub edx,[es:su_ramdisklen]	
		xor dx,dx			
		mov [es:su_ramdiskat],edx	
		call loadinitrd			
		jmp short initrd_end

initrd_notthere:
		mov si,err_noinitrd
		call cwritestr
		mov si,InitRDCName
		call cwritestr
		mov si,crlf_msg
		jmp abort_load

no_high_mem:    
		mov si,err_nohighmem		
		jmp abort_load

initrd_end:
nk_noinitrd:
		call abort_check        	
		mov si,strLoadMsg
		call cwritestr
		call vgaclearmode		
		UNLOAD_PREP			
		mov ax,real_mode_seg
		mov fs,ax
		cli				
		test byte [LoadFlags],LOAD_HIGH
		jz need_high_cmdline
		cmp word [fs:su_version],0202h	
		jb need_high_cmdline
		mov dword [fs:su_cmd_line_ptr],(real_mode_seg << 4) + cmd_line_here
		jmp short in_proper_place

need_high_cmdline:
		mov ax,9000h
		mov es,ax
		mov si,cmd_line_here
		mov di,si
		mov [fs:kern_cmd_magic],word CMD_MAGIC 
		mov [fs:kern_cmd_offset],di	
		mov cx,[CmdLineLen]
		add cx,byte 3
		shr cx,2			
		fs rep movsd
		push fs
		pop es
		test byte [LoadFlags],LOAD_HIGH
		jnz in_proper_place		
		mov ax,9000h
		mov es,ax
		mov cx,[SetupSecs]
		inc cx				
		shl cx,7			
		xor si,si
		xor di,di
		fs rep movsd			
		mov di,[SetupSecs]
		inc di				
		mov cx,32768/512		
		sub cx,di			
		shl di,9			
		shl cx,7			
		xor eax,eax
		rep stosd			
		mov ecx,[KernelSize]
		mov esi,100000h
		mov edi,10000h
		call bcopy

in_proper_place:
		cmp word [es:bs_rootdev],byte 0
		jne root_not_floppy
		mov word [es:bs_rootdev],0200h

root_not_floppy:
%if IS_SYSLINUX || IS_MDSLINUX
		lgs si,[cs:fdctab]
		mov di,linux_fdctab
		mov cx,6			
		gs rep movsw
		mov [cs:fdctab],word linux_fdctab 
		mov [cs:fdctab+2],es
%endif

kill_motor:
		xor ax,ax
		xor dx,dx
		int 13h
%ifdef debug
		xor ax,ax
		int 16h
%endif
		cli
		mov bx,es
		mov ds,bx
		mov fs,bx
		mov gs,bx
		mov ss,bx
		mov sp,linux_stack
		add bx,020h
		push bx
		push word 0h
		retf

old_kernel:
		test byte [initrd_flag],1	
		jz load_old_kernel
		mov si,err_oldkernel
		jmp abort_load

load_old_kernel:
		mov word [SetupSecs],4		
		mov byte [LoadFlags],0		
		jmp read_kernel

loadinitrd:
		push es 
		mov ax,real_mode_seg
		mov es,ax
		mov edi,[es:su_ramdiskat]
		push si
		pop si
		mov eax,[es:su_ramdisklen]
		call load_high
		call crlf
		pop es
		ret



