
#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/mount.h>
#include <dirent.h>
#include <errno.h>
#include "drtrace.h"
#include "drcdrom.h"

MODULE("DRCDROM");

#define DRCDROM_STRSIZE 256

extern DrcdromE LocateAndMountCDROM()
{
	int    lReturnCode   = DRCDROM_NOT_FOUND;
	DIR    *pDir         = NULL;

	TRACE(("INFO: Searching for IDE CDROMs..."));
	if (NULL == (pDir = opendir("/proc/ide")))
	{
		TRACE(("WARN: Failed to open /proc/ide to search for IDE CDROMs"));
	}
	else
	{
		struct		dirent	*pDirEnt	= NULL;
		while (NULL != (pDirEnt = readdir(pDir)))
		{
			if (0 == strncmp("hd", pDirEnt->d_name,2))
			{
				char strIdeDrive[DRCDROM_STRSIZE] = {0};
				char strMediaBuffer[DRCDROM_STRSIZE] = {0};
				FILE *pMediaFile = NULL;
				char strDevName[DRCDROM_STRSIZE] = {0};
				TRACE(("INFO: Examining /proc/ide/%s", pDirEnt->d_name));
				snprintf(strDevName, DRCDROM_STRSIZE, "/dev/%s", pDirEnt->d_name);
				snprintf(strIdeDrive, DRCDROM_STRSIZE, "/proc/ide/%s/media", pDirEnt->d_name);
				if (NULL == (pMediaFile = fopen(strIdeDrive, "r")))
				{
					TRACE(("WARN: Could not open Media File: %s", strIdeDrive));
				}
				else if (0 == fread(strMediaBuffer, 1, sizeof(strMediaBuffer), pMediaFile))
				{
					TRACE(("WARN: Failed to read the opened file: %s", strIdeDrive));
				}
				else if (0 != strncmp("cdrom", strMediaBuffer, 5))
				{
					TRACE(("INFO: '%s' was not a cdrom device - media file='%s'", strIdeDrive, strMediaBuffer));
				}
				else if (-1 == mount (strDevName, "/mp", "iso9660", MS_RDONLY, NULL))
				{
					TRACE(("ERROR: Failed to mount an IDE CDROM ('%s). Verify disk is in drive", strDevName));
					lReturnCode = DRCDROM_NOT_MOUNTED;
				}
				else
				{
					TRACE(("SUCCESS: Mounted IDE CDROM Drive: %s", strDevName));
					lReturnCode = DRCDROM_MOUNTED;
					break;
				}
			}
		}
	}

	if (DRCDROM_MOUNTED != lReturnCode)
	{
		char strSCSIMount[DRCDROM_STRSIZE] = {0};
		long lScsiIndex                    = 0;

		/* Attempt to mount the first 10 usb/scsi cdroms - they share /dev/scdX.
		  '/dev/scd0 ... /dev/scd9' */
		TRACE(("Searching for USB/SCSI CDROMs..."));
		for (lScsiIndex = 0;  lScsiIndex < 10; lScsiIndex++)
		{
			snprintf(strSCSIMount, DRCDROM_STRSIZE, "/dev/scd%ld", lScsiIndex);
			if (-1 == mount (strSCSIMount, "/mp", "iso9660", MS_RDONLY, NULL))
			{
				TRACE(("ERROR: Failed to mount SCSI CDROM ('%s). Verify disk is in drive", strSCSIMount));
			}
			else
			{
				TRACE(("SUCCESS: Mounted SCSI CDROM Drive: %s", strSCSIMount));
				lReturnCode = DRCDROM_MOUNTED;
				break;
			}
		}
	}
	
	return lReturnCode;
}
