
#include <signal.h>
#include <ncurses.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mount.h>
#include <sys/socket.h>
#include <unistd.h>
#include <dirent.h>
#include <fcntl.h>
#include <net/if.h>
#include <errno.h>
#include <curses.h>
#include <ctype.h>
#include <netinet/in.h>
#include <netinet/ether.h>
#include <asm/types.h>
#include <linux/if_packet.h>
#include <linux/filter.h>
#include "drtrace.h"
#include "drmod.h"

MODULE("DR_NCR")

#define		IO_BLKGETSIZE						_IO(0x12,96)
#define		TRUE										1
#define		FALSE										0
#define		MAX_NUM_IF							10
#define		DR_PROC_PT_FILE					"/proc/partitions"
#define		SYSERR(x)								strerror(x)
#define		STR_BOOLEAN(x)					((x)?"TRUE":"FALSE")
#define		STR_BAKBONE							"BakBone Software, Inc."
#define		STR_TITLE								"NCR MPRAS Root Disk Recovery Process"
#define		STR_ROOT_DEV						"Select Root Disk to start Recovery Process"
#define		STR_TEMP_DEV						"Select TEMP Disk to start Recovery Process"
#define		STR_NETWORK							"Network Interface Configuration (%s)"
#define		STR_SERVER_IP						"Enter the IP address of the Server"
#define		STR_AVAIL_NIC						"Available Network Interfaces"
#define		STR_CONFIG_IF						"Configuration Interface (%s)"
#define		STR_OK									"[  OK  ]"
#define		STR_YES									"[  YES  ]"
#define		STR_NO									"[  NO  ]"
#define		STR_INSTALL							"[ Install ]" 
#define		STR_EXIT								"[  Exit  ]"
#define		STR_CONFIG							"[ Config ]"
#define		STR_AUTO								"[  Auto  ]"
#define		STR_CANCEL							"[ Cancel ]"


typedef		unsigned long long			int64;
typedef		unsigned char						ByteT;

static WINDOW	*gpWin							= NULL;
static char 	gstrDevRoot 	[64] = {0};
static char 	gstrDevTmp		[64] = {0};
static char 	gstrServerIp 	[64] = {0};
static attr_t	gtAttribute [6];


typedef enum
{
	DrYes	= 0,
	DrNo	= 1
} DrRequestE;


extern BooleanT DrGz 			(char *);
extern BooleanT DrIfcfg		(char *);
extern BooleanT DrIfRoute	(char *);
extern BooleanT DrPg			(char *);


/*
** =======================================================================================
** =======================================================================================
*/
static long CfgGetNegotiationTime ()
{
	long	lRC = 1;
	int		lFd	= open ("/bin/drnic.cfg", O_RDONLY);
	char	strBuf[16];

	memset (strBuf, 0, sizeof (strBuf));

	if (-1 != lFd && -1 != read (lFd, strBuf, sizeof (strBuf)))
	{
		lRC = atoi (strBuf);
		close (lFd);
	}
	TRACE(("CfgGetNegotiationTime: %ld", lRC));

	return lRC;
}


/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT GetDeviceSize (char *pstrDev, int64 *pllSize)
{
  BooleanT  bSuccess 	= FALSE;
  int       lFd				= open (pstrDev, O_RDONLY);

	TRACE(("Begin GetDeviceSize for Dev: %s", pstrDev));

  if (0 <= lFd)
  {
    unsigned long ulSize = 0;

    if (0 == ioctl (lFd, IO_BLKGETSIZE, &ulSize))
    {
      *pllSize = (int64) ulSize * (int64) 512;
      bSuccess = TRUE;
    }
		else
		{
			TRACE(("Failed to do IOCTL to get device size. %s", SYSERR(errno)));
		}
    close (lFd);
  }
	else
	{
		TRACE(("Failed to do open device to device size. %s", SYSERR(errno)));
	}
	TRACE(("End GetDeviceSize for Dev: %s. %s", pstrDev, STR_BOOLEAN(bSuccess)));

  return bSuccess;
}



/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT DeviceIsCdrom (char *pstrDev)
{
	BooleanT	bCdrom 			= FALSE;
	FILE			*pFd 				= NULL;
	char			strBuf[512] = {0};
	struct 		stat  		tStat;

	TRACE(("Start validate if device is CDROM"));

	if (0 == strncmp ("/dev/hd", pstrDev, 7))
	{
		snprintf (strBuf, sizeof(strBuf), "/proc/ide/%s/media", pstrDev+5);

		pFd = fopen (strBuf, "r");

		if (NULL != pFd && NULL != fgets (strBuf, sizeof (strBuf), pFd))
		{
			if (0 == strncmp (strBuf, "cdrom", 5))
			{
				bCdrom = TRUE;
			}
		}
		else
		{
			if (0 == stat (pstrDev, &tStat))
			{
				if (0 == (tStat.st_mode & 0222))
				{
					bCdrom = TRUE;
				}
			}
		}
		fclose(pFd);
	}
	TRACE(("End validate if device is CDROM. (%s)", STR_BOOLEAN (bCdrom)));

	return bCdrom;
}




/*
** =======================================================================================
** =======================================================================================
*/
static void LoadNcrModules ()
{
	int	lRC = 0;

	fprintf(stderr,"Loading device drivers. Please wait...\n");

	TRACE(("Begin LoadNcrModules"));

	TRACE(("Begin loading module: DAC960.o"));
	DrCmdLoadModule ("/mods/scsi/DAC960.o", &lRC) ? 
	TRACE(("Loading module: DAC960.o Successfully")) :
	TRACE(("Loading module: DAC960.o Failed"));
	
	TRACE(("Begin loading module: megaraid.o"));
	DrCmdLoadModule ("/mods/scsi/megaraid.o", &lRC) ? 
	TRACE(("Loading module: megaraid.o Successfully")) :
	TRACE(("Loading module: megaraid.o Failed"));

	TRACE(("Begin loading module: sym53c416.o"));
	DrCmdLoadModule ("/mods/scsi/sym53c416.o", &lRC) ? 
	TRACE(("Loading module: sym53c416.o Successfully")) :
	TRACE(("Loading module: sym53c416.o Failed"));

	TRACE(("Begin loading module: e100.o"));
	DrCmdLoadModule ("/mods/net/e100.o", &lRC) ? 
	TRACE(("Loading module: e100.o Successfully")) :
	TRACE(("Loading module: e100.o Failed"));

	TRACE(("Begin loading module: e1000.o"));
	DrCmdLoadModule ("/mods/net/e1000.o", &lRC) ? 
	TRACE(("Loading module: e1000.o Successfully")) :
	TRACE(("Loading module: e1000.o Failed"));

	TRACE(("Begin loading module: tulip.o"));
	DrCmdLoadModule ("/mods/net/tulip.o", &lRC) ? 
	TRACE(("Loading module: tulip.o Successfully")) :
	TRACE(("Loading module: tulip.o Failed"));
	
	TRACE(("End LoadNcrModules"));
}




/*
** =======================================================================================
** =======================================================================================
*/
static long GetNicInterfaces ()
{
  long  lInt  = 0;
  FILE  *pFd  = fopen ("/proc/net/dev", "r");

  TRACE(("Begin GetNicInterfaces from file: /proc/net/dev"));

  if (NULL == pFd)
  {
    TRACE(("ERROR: Failed to open file /proc/net/dev. %s", strerror(errno)));
  }
  else
  {
    char  strBuf[256] = {0};
    char  *pstr = NULL;
  
    while (NULL != fgets (strBuf, sizeof (strBuf), pFd))
    {
      pstr = (char *) strtok (strBuf, " ");

      if (NULL != pstr && NULL != strstr (pstr, "eth"))
      {
        TRACE(("Got NIC interface: eth%ld", lInt));
        lInt++;
      }
    }
    fclose (pFd);
  }
  TRACE(("End GetNicInterfaces. Found (%ld) NIC interfaces from /proc/net/dev. %s",
  lInt, (lInt > 0) ? "SUCCESS" : "ERROR"));

  return lInt;
}




/*
** =======================================================================================
** =======================================================================================
*/
static long GetDiskDevices (char *pstrDevs[])
{
	long	lDevCount	= 0;
	FILE	*pFd 			= fopen (DR_PROC_PT_FILE, "r");
	
	TRACE(("Begin GetDiskDevices from file: /proc/partitions"));

	if (NULL == pFd)
	{
		TRACE(("Failed to open file: %s. %s",DR_PROC_PT_FILE, SYSERR(errno)));
	}
	else
	{
		char 		strBuf		[256];
		char 		strName		[128];
		long		lMajor 		= 0;
		long		lMinor 		= 0;
		int64		llSize 		= 0;

		while (NULL != fgets (strBuf, sizeof (strBuf), pFd))
		{
			if (4 == sscanf (strBuf, " %ld %ld %llu %[^\n ]", &lMajor, &lMinor, &llSize, strName))
			{
				char			strDev[128]	= {0};
				BooleanT	bFoundDev		= FALSE;
				char			*pstr				= NULL;

				memset	(strDev, 0, sizeof (strDev));
				sprintf (strDev, "/dev/%s", strName);

				for (pstr=strName; *pstr; pstr++);

				if (NULL != strstr (strName, "ida/") && 'p' != pstr[-2] && llSize > 1440) 
				{
					bFoundDev = TRUE;
				}
				else if (NULL != strstr (strName, "rd/") && 'p' != pstr[-2] && llSize > 1440)
				{
					bFoundDev = TRUE;
				}
				else if (NULL != strstr (strName, "cciss/") && 'p' != pstr[-2] && llSize > 1440)
				{
					bFoundDev = TRUE;
				}
				else if (! isdigit (pstr[-1]) && llSize > 1440)
				{
					bFoundDev = TRUE;
				}

				if (TRUE == bFoundDev && TRUE != DeviceIsCdrom (strDev))
				{
					char	strSize [32] = {0};
					int64	llSize = 0;
					
					if (TRUE == GetDeviceSize (strDev, &llSize))
					{
						sprintf (strSize, "%lld MB", llSize / 0x100000);
					}
					else
					{
						sprintf (strSize, "Unknown");
					}
					pstrDevs [lDevCount] = (char *) malloc (64);

					sprintf (pstrDevs [lDevCount], "%s  %s", strDev, strSize);
					TRACE(("Devices found for this server: %s ", pstrDevs [lDevCount]));
					lDevCount++;
				}
			}
		}
		fclose (pFd);
	}
	TRACE(("End GetDiskDevices - Device Count: %ld", lDevCount));
	
	return lDevCount;
}




/*
** =======================================================================================
** =======================================================================================
*/
static void WinInit ()
{
  WINDOW 	*pWin = NULL;
  int 		lRC 	= 0;
  int 		i 		= 0;

  pWin = initscr ();

  if (NULL != pWin)
	{
		lRC = has_colors ();

		if (0 != lRC)
		{
			start_color ();

			init_pair (1, COLOR_WHITE, 	COLOR_WHITE);
			init_pair (2, COLOR_BLACK, 	COLOR_WHITE);
			init_pair (3, COLOR_WHITE, 	COLOR_RED);
			init_pair (4, COLOR_BLUE, 	COLOR_WHITE);
			init_pair (5, COLOR_YELLOW, COLOR_BLUE);
			init_pair (6, COLOR_WHITE, 	COLOR_BLUE);

			for (i = 0; i < 6; i++)
			{
				gtAttribute[i] = COLOR_PAIR (i + 1) | ((!(i & 1)) ? A_BOLD : 0);
			}
		}
		else
		{
			gtAttribute[0] = gtAttribute[1] = gtAttribute[3] =
			gtAttribute[4] = gtAttribute[5] = A_NORMAL;
			gtAttribute[2] = A_REVERSE;
		}
	}
}



/*
** =======================================================================================
** =======================================================================================
*/
static DrRequestE	DisplayRequest (
									char				*pstrYes,
									char				*pstrNo,
									char 				*pstrMsg, 
									DrRequestE 	eDefaultRequest)
{
	WINDOW		*pWin				= NULL;
	int				lX					= 120;
	int				lY					= 7;
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	int				RC					= 0;
	int				i						= 0;
	int				lKeyPos			= 0;
	BooleanT	bExit				= FALSE;

	clear 		();
	refresh		();
	getmaxyx 	(stdscr, lMaxY, lMaxX);

	if (strlen (pstrMsg) >= lMaxX)
	{
		pstrMsg [lMaxX-6] = '\0';
		lX = strlen (pstrMsg) + 4;
	}
	else if (strlen (pstrMsg) < 46)
	{
		lX = 50;
	}
	else
	{
		lX = strlen (pstrMsg) + 4;
	}
	pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);
	
	wborder (pWin, 
					ACS_VLINE 		| gtAttribute[0], 
					ACS_VLINE 		| gtAttribute[1], 
					ACS_HLINE 		| gtAttribute[0], 
					ACS_HLINE 		| gtAttribute[1], 
					ACS_ULCORNER 	| gtAttribute[0], 
					ACS_URCORNER 	| gtAttribute[0], 
					ACS_LLCORNER 	| gtAttribute[1], 
					ACS_LRCORNER 	| gtAttribute[1]);

	wattrset (pWin, gtAttribute[1]);

	for (i=1; i<lY-1; i++)
	{
		wmove 	(pWin, i, 1);
		whline 	(pWin, ' ', lX-2);
	}
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 2, (lX-strlen (pstrMsg)) / 2, pstrMsg);

	if (DrYes == eDefaultRequest)
	{
		lKeyPos	= 0;
		wattrset 	(pWin, gtAttribute[2]);
		mvwaddstr (pWin, 4, 10, pstrYes);
		wattrset 	(pWin, gtAttribute[3]);
		mvwaddstr (pWin, 4, (lX-strlen (pstrNo))-10, pstrNo);
	}
	else
	{
		lKeyPos	= 1;
		wattrset 	(pWin, gtAttribute[3]);
		mvwaddstr (pWin, 4, 10, pstrYes);
		wattrset 	(pWin, gtAttribute[2]);
		mvwaddstr (pWin, 4, (lX-strlen (pstrNo))-10, pstrNo);
	}
	
	noecho 		();
	keypad 		(pWin, 1);
	cbreak 		();
		
	do
	{
		wrefresh 		(pWin);
		redrawwin 	(pWin);
		RC = wgetch	(pWin);

		switch (RC)
		{
			case '\t':
			case KEY_RIGHT:
			case KEY_LEFT:
			{
				if (0 == lKeyPos)
				{
					lKeyPos = 1;
					wattrset 	(pWin, gtAttribute[3]);
					mvwaddstr (pWin, 4, 10, pstrYes);
					
					wattrset 	(pWin, gtAttribute[2]);
					mvwaddstr (pWin, 4, (lX-strlen (pstrNo))-10, pstrNo);
				}
				else if (1 == lKeyPos)
				{
					lKeyPos = 0;
					wattrset 	(pWin, gtAttribute[2]);
					mvwaddstr (pWin, 4, 10, pstrYes);
					wattrset 	(pWin, gtAttribute[3]);
					mvwaddstr (pWin, 4, (lX-strlen (pstrNo))-10, pstrNo);
				}
				break;
			}
			
			case '\n':
			{
				bExit = TRUE;
				break;
			}
		}
	} while (TRUE != bExit);
	
	clear 	();
	refresh	();
	delwin 	(pWin);

	return lKeyPos;
}



/*
** =======================================================================================
** =======================================================================================
*/
static void DisplayInfo (char *pstrMsg)
{
	WINDOW		*pWin				= NULL;
	int				lX					= strlen (pstrMsg) + 10;
	int				lY					= 5;
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	int				i						= 0;

	clear 		();
	refresh		();
	getmaxyx 	(stdscr, lMaxY, lMaxX);

	pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);
	
	wborder (pWin, 
					ACS_VLINE 		| gtAttribute[0], 
					ACS_VLINE 		| gtAttribute[1], 
					ACS_HLINE 		| gtAttribute[0], 
					ACS_HLINE 		| gtAttribute[1], 
					ACS_ULCORNER 	| gtAttribute[0], 
					ACS_URCORNER 	| gtAttribute[0], 
					ACS_LLCORNER 	| gtAttribute[1], 
					ACS_LRCORNER 	| gtAttribute[1]);

	wattrset (pWin, gtAttribute[1]);

	for (i=1; i<lY-1; i++)
	{
		wmove 	(pWin, i, 1);
		whline 	(pWin, ' ', lX-2);
	}
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 2, (lX-strlen (pstrMsg)) / 2, pstrMsg);
		
	wrefresh (pWin);
	redrawwin (pWin);
}




/*
** =======================================================================================
** =======================================================================================
*/
static void DisplayMsg (char *pstrMsg)
{
	WINDOW		*pWin				= NULL;
	int				lX					= strlen (pstrMsg) + 10;
	int				lY					= 7;
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	int				RC					= 0;
	int				i						= 0;

	clear 		();
	refresh		();
	getmaxyx 	(stdscr, lMaxY, lMaxX);

	pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);
	
	wborder (pWin, 
					ACS_VLINE 		| gtAttribute[0], 
					ACS_VLINE 		| gtAttribute[1], 
					ACS_HLINE 		| gtAttribute[0], 
					ACS_HLINE 		| gtAttribute[1], 
					ACS_ULCORNER 	| gtAttribute[0], 
					ACS_URCORNER 	| gtAttribute[0], 
					ACS_LLCORNER 	| gtAttribute[1], 
					ACS_LRCORNER 	| gtAttribute[1]);

	wattrset (pWin, gtAttribute[1]);

	for (i=1; i<lY-1; i++)
	{
		wmove 	(pWin, i, 1);
		whline 	(pWin, ' ', lX-2);
	}
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 2, (lX-strlen (pstrMsg)) / 2, pstrMsg);
	wattrset 	(pWin, gtAttribute[2]);
	mvwaddstr (pWin, 4, (lX-strlen (STR_OK)) /2, STR_OK);
		
	do
	{
		RC = wgetch	(pWin);
	} while (RC != '\n');

	clear 	();
	refresh	();
	delwin 	(pWin);
}


/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT ValidateNetworkNum (char *pstrNum)
{
	int		bSuccess 	= FALSE;
	long	lClassZ		= 0;
	long	lClassA		= 0;
	long	lClassB		= 0;
	long	lClassC		= 0;

	if (4 == sscanf (pstrNum, "%ld.%ld.%ld.%ld", &lClassZ, &lClassA, &lClassB, &lClassC))
	{
		if (0 <= lClassZ && 255 >= lClassZ &&
				0 <= lClassA && 255 >= lClassA &&
				0 <= lClassB && 255 >= lClassB &&
				0 <= lClassC && 255 >= lClassC)
		{
			bSuccess = TRUE;
		}
	}
	return bSuccess;
}







/*
** =======================================================================================
** =======================================================================================
*/
static void DrawDiskDev (
						WINDOW 	*pWin, 
						char 		*pstrDevs [], 
						long 		lCount)
{
	int		lInitY				= 4;
	int		lInitX				= 4;
	int		i 						= 1;

	wattrset (pWin, gtAttribute [1]);

	for (i=1; i<=lCount; i++)
	{
		wmove 		(pWin, i+lInitY, lInitX);
		whline 		(pWin, ' ', 50-lInitX*2);
		mvwaddstr (pWin, i+lInitY, lInitX, pstrDevs [i-1]);

	}
}




/*
** =======================================================================================
** =======================================================================================
*/
static void DisplayDiskDev (
						WINDOW 		*pWin,
						char 			*pstrDevs [], 
						long 			lCount,
						BooleanT	bRootDisk)
{
	int				lInitY			= 4;
	int				lInitX			= 4;
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	int				lPos				= 1;
	int				lIdx				= 1;
	int				RC					= 0;
	int				lKeyPos			= 1;
	BooleanT	bExit				= FALSE;
		
	getmaxyx 			(pWin, lMaxY, lMaxX);
	DrawDiskDev 	(pWin, pstrDevs, lCount);
	noecho 				();
	keypad 				(pWin, 1);
	cbreak 				();
		
	do
	{
		wattrset 		(pWin, gtAttribute [2]);
		wmove 			(pWin, lPos+lInitY, 2);
		whline 			(pWin, ' ', lMaxX - 4);
		mvwaddstr 	(pWin, lPos+lInitY, lInitX, pstrDevs [lIdx-1]);
		whline			(pWin, ' ',(lInitX - strlen (pstrDevs [lIdx-1])+1) / 2 - 2);
		wrefresh 		(pWin);
		redrawwin 	(pWin);
		RC = wgetch	(pWin);
		wattrset 		(pWin, gtAttribute [1]);
		wmove 			(pWin, lPos+lInitY, 2);
		whline 			(pWin, ' ', lMaxX - 4);
		mvwaddstr 	(pWin, lPos+lInitY, lInitX, pstrDevs [lIdx-1]);

		switch (RC)
		{
			case KEY_UP:
			{
				if (1 < lPos)
				{
					lPos--;
				}
				if (1 < lIdx)
				{
					lIdx--;
				}
				break;
			}

			case KEY_DOWN:
			{
				if (lCount > lPos)
				{
					lPos++;
				}
				if (lCount > lIdx)
				{
					lIdx++;
				}
				break;
			}
	
			case '\t':
			case KEY_RIGHT:
			{
				if (1 == lKeyPos)
				{
					lKeyPos++;
					wattrset 	(pWin, gtAttribute[2]);
					wmove 		(pWin, 17, 5);
					whline 		(pWin, ' ', strlen (STR_INSTALL));
					mvwaddstr (pWin, 17, 5, STR_INSTALL);
				}
				else if (2 == lKeyPos)
				{
					lKeyPos++;
					wattrset 	(pWin, gtAttribute[3]);
					wmove 		(pWin, 17, 5);
					whline 		(pWin, ' ', strlen (STR_INSTALL));
					mvwaddstr (pWin, 17, 5, STR_INSTALL);
	
					wattrset 	(pWin, gtAttribute[2]);
					wmove 		(pWin, 17, 32);
					whline 		(pWin, ' ', strlen (STR_EXIT));
					mvwaddstr (pWin, 17, 32, STR_EXIT);
				}
				else
				{
					lKeyPos = 1;
					wattrset 	(pWin, gtAttribute[3]);
					wmove 		(pWin, 17, 32);
					whline 		(pWin, ' ', strlen (STR_EXIT));
					mvwaddstr (pWin, 17, 32, STR_EXIT);
				}
				break;
			}
			
			case KEY_LEFT:
			{
				if (1 == lKeyPos)
				{
					lKeyPos	= 3;
					wattrset 	(pWin, gtAttribute[2]);
					wmove 		(pWin, 17, 32);
					whline 		(pWin, ' ', strlen (STR_EXIT));
					mvwaddstr (pWin, 17, 32, STR_EXIT);
				}
				else if (3 == lKeyPos)
				{
					lKeyPos--;

					wattrset 	(pWin, gtAttribute[3]);
					wmove 		(pWin, 17, 32);
					whline 		(pWin, ' ', strlen (STR_EXIT));
					mvwaddstr (pWin, 17, 32, STR_EXIT);

					wattrset 	(pWin, gtAttribute[2]);
					wmove 		(pWin, 17, 5);
					whline 		(pWin, ' ', strlen (STR_INSTALL));
					mvwaddstr (pWin, 17, 5, STR_INSTALL);
				}
				else if (2 == lKeyPos)
				{
					lKeyPos--;
					
					wattrset 	(pWin, gtAttribute[3]);
					wmove 		(pWin, 17, 5);
					whline 		(pWin, ' ', strlen (STR_INSTALL));
					mvwaddstr (pWin, 17, 5, STR_INSTALL);
				}
				break;
			}

			case '\n':
			{
				if (3 == lKeyPos)
				{
				}
				else if (2 == lKeyPos || 1 == lKeyPos)
				{
				}
				if (TRUE == bRootDisk)
				{
					strcpy (gstrDevRoot, pstrDevs [lIdx-1]);
				}
				else
				{
					strcpy (gstrDevTmp, pstrDevs [lIdx-1]);
				}

				bExit = TRUE;

				break;
			}
		}
	} while (TRUE != bExit);
}




/*
** =======================================================================================
** =======================================================================================
*/
static void ListDiskDev (
						char			*pstrDevs[], 
						long			lCount,
						BooleanT	bRootDisk)
{
	WINDOW	*pWin			= NULL;
	int			lX				= 50;
	int			lY				= 19;
	int			lMaxX			= 0;
	int			lMaxY			= 0;
	int			i					= 0;

	getmaxyx (stdscr, lMaxY, lMaxX);

	pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);

	wborder (pWin, 
					ACS_VLINE 		| gtAttribute[0], 
					ACS_VLINE 		| gtAttribute[1], 
					ACS_HLINE 		| gtAttribute[0], 
					ACS_HLINE 		| gtAttribute[1], 
					ACS_ULCORNER 	| gtAttribute[0], 
					ACS_URCORNER 	| gtAttribute[0], 
					ACS_LLCORNER 	| gtAttribute[1], 
					ACS_LRCORNER 	| gtAttribute[1]);

	wattrset (pWin, gtAttribute[1]);

	for (i=1; i<lY-1; i++)
	{
		wmove 	(pWin, i, 1);
		whline 	(pWin, ' ', lX-2);
	}
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 1, (lX-strlen (STR_BAKBONE)) / 2, STR_BAKBONE);

	if (TRUE == bRootDisk)
	{
		mvwaddstr (pWin, 2, (lX-strlen (STR_ROOT_DEV)) / 2, STR_ROOT_DEV);
	}
	else
	{
		mvwaddstr (pWin, 2, (lX-strlen (STR_TEMP_DEV)) / 2, STR_TEMP_DEV);
	}
	wattrset 	(pWin, gtAttribute[1]);
	wmove 		(pWin, 3, 1);
	whline 		(pWin, ACS_HLINE, lX-2);
	wmove 		(pWin, lY-3, 1);
	whline 		(pWin, ACS_HLINE, lX-2);
	
	wattrset 	(pWin, gtAttribute[3]);

	wmove 		(pWin, lY-2, 5); 
	whline 		(pWin, ' ', strlen (STR_INSTALL));
	mvwaddstr (pWin, lY-2, 5, STR_INSTALL);
	
	wmove 		(pWin, lY-2, 32);
	whline 		(pWin, ' ', strlen (STR_EXIT));
	mvwaddstr (pWin, lY-2, 32, STR_EXIT);
	
	DisplayDiskDev (pWin, pstrDevs, lCount, bRootDisk);

	delwin		(pWin);
	redrawwin	(stdscr);
	refresh		();
}





/*
** =======================================================================================
** =======================================================================================
*/
static void	GetServerIp (
						WINDOW		*pWin,
						int				lCount,
						char			**pstrLabels,
						char			**pstrArr)
{
	int 	i							= 0;
	int		lMaxX					= 0;
	int		lMaxY					= 0;
	int		lMaxLabelLen 	= 0;
	int		lStrLen				= 0;
	int		lInput				= 0;
	int		lPos					= 0;
	int		lCursePos			= 0;

	getmaxyx (pWin, lMaxY, lMaxX);

	for (i=0; i<lCount; i++)
	{
		if ((lStrLen = strlen (pstrLabels[i])) > lMaxLabelLen)
		{
			lMaxLabelLen = lStrLen;
		}
	}

	for (i=0; i<lCount; i++)
	{
		wattrset 	(pWin, gtAttribute [1]);
		mvwaddstr (pWin, 5 + 2 * i, 2 + lMaxLabelLen - strlen (pstrLabels[i]), pstrLabels[i]);
		waddstr 	(pWin, ": ");
		wattrset 	(pWin, gtAttribute[5]);
		whline 		(pWin, ' ', lMaxX - lMaxLabelLen - 6);
		pstrArr[i][0] = '\0';
	}
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 8, 28, STR_OK);
	wrefresh	(pWin);
	noecho 		();
	keypad 		(pWin, 1);
	cbreak 		();

	lPos = lCursePos = 0;

	do
	{
		if (lPos < lCount)
		{
			wmove 		(pWin, 5 + 2 * lPos, 4 + lMaxLabelLen);
			wattrset 	(pWin, gtAttribute[4]);
			whline 		(pWin, ' ', lMaxX - lMaxLabelLen - 6);
			waddstr 	(pWin, pstrArr[lPos]);
			wmove 		(pWin, 5 + 2 * lPos, 4 + lMaxLabelLen + lCursePos);

			lInput = wgetch (pWin);

			wmove 		(pWin, 5 + 2 * lPos, 4 + lMaxLabelLen);
			wattrset 	(pWin, gtAttribute[5]);
			whline 		(pWin, ' ', lMaxX - lMaxLabelLen - 6);
			waddstr 	(pWin, pstrArr[lPos]);

			switch (lInput)
			{
				case KEY_LEFT:
				{
					if (lCursePos > 0)
					{
						lCursePos--;
					}
					break;
				}
				
				case KEY_RIGHT:
				{
					if (lCursePos < strlen (pstrArr[lPos]))
					{
						lCursePos++;
					}
					break;
				}

				case KEY_HOME:
				{
					lCursePos = 0;
					break;
				}
				
				case KEY_END:
				{
					lCursePos = strlen (pstrArr[lPos]);
					break;
				}
				
				case KEY_BACKSPACE:
				{
					if (lCursePos > 0)
					{
						lCursePos--;
						memmove	(pstrArr[lPos]  + lCursePos, pstrArr[lPos] + lCursePos + 1,
						strlen 	(pstrArr[lPos]) - lCursePos);
					}
					break;
				}
				
				case KEY_DC:
				{
					if (lCursePos < strlen (pstrArr[lPos]))
					{
						memmove (pstrArr[lPos]+lCursePos, pstrArr[lPos]+lCursePos+1, strlen (pstrArr[lPos])-lCursePos);
					}
					break;
				}
				
				case '\n':
				{
					lInput = KEY_DOWN;
					break;
				}

				case ' '...'~':
				{
					if (strlen (pstrArr[lPos]) < lMaxX - lMaxLabelLen - 6)
					{
						memmove (pstrArr[lPos]+lCursePos+1, pstrArr[lPos]+lCursePos, 
						strlen (pstrArr[lPos])-lCursePos+1);
						pstrArr[lPos][lCursePos++] = lInput;
					}
					break;
				}
			}
		}
		else
		{
			wattrset 	(pWin, gtAttribute[2]);
			mvwaddstr (pWin, 8, 28, STR_OK);
			wrefresh 	(pWin);
			
			lInput = wgetch (pWin);

			wattrset 	(pWin, gtAttribute[3]);
			mvwaddstr (pWin, 8, 28, STR_OK);
			wrefresh 	(pWin);
		}

		switch (lInput)
		{
			case KEY_UP:
			{
				lPos--;
				if (lPos < 0)
				{
					lPos = lCount;
				}
				lCursePos = 0;
				break;
			}
			case KEY_DOWN:
			{
				case '\t':
				{
					lPos++;
					lPos %= (lCount + 1);
					lCursePos = 0;
					break;
				}
			}
		}
	}
	while (((lInput != '\n') && (lInput != ' ')) || (lPos != lCount));
}



/*
** =======================================================================================
** =======================================================================================
*/
static void DisplayServerIp ()
{
	WINDOW		*pWin				= NULL;
	int				lX					= 65;
	int				lY					= 10;
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	int				i						= 0;
	BooleanT	bContinue		= TRUE;
	char			*pstrLabels[] = { "IP Address" };
	char			strArr			[1][40];
	char			*pstrArr[]	= { strArr [0] };

	clear 		();
	refresh		();
	getmaxyx (stdscr, lMaxY, lMaxX);

	pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);
	
	wborder (pWin, 
					ACS_VLINE 		| gtAttribute[0], 
					ACS_VLINE 		| gtAttribute[1], 
					ACS_HLINE 		| gtAttribute[0], 
					ACS_HLINE 		| gtAttribute[1], 
					ACS_ULCORNER 	| gtAttribute[0], 
					ACS_URCORNER 	| gtAttribute[0], 
					ACS_LLCORNER 	| gtAttribute[1], 
					ACS_LRCORNER 	| gtAttribute[1]);

	wattrset (pWin, gtAttribute[1]);

	for (i=1; i<lY-1; i++)
	{
		wmove 	(pWin, i, 1);
		whline 	(pWin, ' ', lX-2);
	}
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 1, (lX-strlen (STR_BAKBONE)) / 2, STR_BAKBONE);
	mvwaddstr (pWin, 2, (lX-strlen (STR_SERVER_IP)) / 2, STR_SERVER_IP);
	wattrset 	(pWin, gtAttribute[1]);
	wmove 		(pWin, 3, 1);
	whline 		(pWin, ACS_HLINE, lX-2);
	wmove 		(pWin, 7, 1);
	whline 		(pWin, ACS_HLINE, lX-2);
	
	wattrset 	(pWin, gtAttribute[3]);
	wmove 		(pWin, 8, 28);
	whline 		(pWin, ' ', 8);
	mvwaddstr (pWin, 8, 28, STR_OK);

	while (TRUE == bContinue)
	{
		redrawwin 	(pWin);
		GetServerIp	(pWin, 1, pstrLabels, pstrArr);
		
		if (TRUE == ValidateNetworkNum (pstrArr[0]))
		{
			strcpy (gstrServerIp, pstrArr[0]);
			bContinue = FALSE;
		}
		else
		{
			if (DrYes ==	DisplayRequest (
										STR_YES,
										STR_NO,
										"Invalid IP Address. Do you want to skip config ?",
										DrNo))
			{
				bContinue = FALSE;
			}
		}
	}
	delwin 		(pWin);
	redrawwin	(stdscr);
	refresh		();
	
}





/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT GetBcastAddr (char *pstrIp, char *pstrMask, char *pstrBcast)
{
  BooleanT  bSuccess	= FALSE;
  long      lIpA    	= 0;
  long      lIpB    	= 0;
  long      lIpC    	= 0;
  long      lIpD    	= 0;

	TRACE(("Get Bcast for IP: %s  NetMask: %s", pstrIp, pstrMask));

  if (4 != sscanf (pstrIp, "%ld.%ld.%ld.%ld", &lIpA, &lIpB, &lIpC, &lIpD))
  {
    TRACE(("ERROR: Failed to sscanf to convert IP to Bcast"));
  }
  else
  {
    long  lIp = (lIpA << 24) | (lIpB << 16) | (lIpC << 8) | (lIpD);

    if (4 != sscanf (pstrMask, "%ld.%ld.%ld.%ld", &lIpA, &lIpB, &lIpC, &lIpD))
    {
      TRACE(("ERROR: Failed to sscanf to convert NetMask to Bcast"));
    }
    else
    {
      long  lMask   = (lIpA << 24) | (lIpB << 16) | (lIpC << 8) | (lIpD);
      long  lBcast  = ~lMask | lIp;

      sprintf (pstrBcast,"%ld.%ld.%ld.%ld",
              ((lBcast >> 24) & 0xff),
              ((lBcast >> 16) & 0xff),
              ((lBcast >> 8)  & 0xff),
              (lBcast & 0xff));

      bSuccess = TRUE;
    }
  }
	TRACE(("End get Bcast for IP: %s  NetMask: %s. %s", 
	pstrIp, pstrMask, bSuccess ? "SUCCESS" : "ERROR"));

  return bSuccess;
}



/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT ValidateIfParm (char *pstr)
{
  int   bSuccess  = FALSE;
  long  lIpA   		= 0;
  long  lIpB   		= 0;
  long  lIpC   		= 0;
  long  lIpD   		= 0;

	TRACE(("Validate Interface Parm: %s", pstr));

  if (4 == sscanf (pstr, "%ld.%ld.%ld.%ld", &lIpA, &lIpB, &lIpC, &lIpD))
  {
    if (0x00 <= lIpA && 0xff >= lIpA &&
        0x00 <= lIpB && 0xff >= lIpB &&
        0x00 <= lIpC && 0xff >= lIpC &&
        0x00 <= lIpD && 0xff >= lIpD)
    {
      bSuccess = TRUE;
    }
  }
	TRACE(("End validate Interface Parm: %s. %s", 
	pstr, bSuccess ? "SUCCESS" : "ERROR"));

  return bSuccess;
}




/*
** =======================================================================================
** =======================================================================================
*/
static void ValidateAndConfigIf (char *pstrIf, char **ppstrParms, long lNumIf)
{
	char strMsg[128] = {0};

	if (TRUE != ValidateIfParm (ppstrParms[0]))
	{
		sprintf(strMsg,"ERROR: Invalid input IP Address: %s", ppstrParms[0]);
		DisplayMsg (strMsg);
	}
	else if (TRUE != ValidateIfParm (ppstrParms[1]))
	{
		sprintf(strMsg,"ERROR: Invalid input NetMask: %s", ppstrParms[1]);
		DisplayMsg (strMsg);
	}
	else if (0 < strlen (ppstrParms[2]) && TRUE != ValidateIfParm (ppstrParms[2]))
	{
		sprintf(strMsg,"ERROR: Invalid input GateWay: %s", ppstrParms[2]);
		DisplayMsg (strMsg);
	}
	else if (NULL == pstrIf && TRUE != ValidateIfParm (ppstrParms[3]))
	{
		sprintf(strMsg,"ERROR: Invalid input Server IP: %s", ppstrParms[3]);
		DisplayMsg (strMsg);
	}
	else if (NULL == pstrIf)
	{
		BooleanT	bSuccess				= FALSE;
		char 			strConfig[128] 	= {0};
		char 			strMsg[128]			= {0};
		char			strBcast[32]		= {0};
		int				i;
		int				j;

		TRACE(("Entering Config Auto for all NIC interfaces"));
	
		strcpy (gstrServerIp, ppstrParms[3]);
		memset (strBcast, 0, sizeof (strBcast));
		GetBcastAddr (ppstrParms[0], ppstrParms[1], strBcast);

		for (i=0; i<lNumIf && FALSE == bSuccess; i++)
		{
			memset (strConfig, 0, sizeof (strConfig));
			(0 != strBcast[0]) ? 
			sprintf(strConfig,"eth  %s netmask %s broadcast %s down",ppstrParms[0],ppstrParms[1],strBcast) :
			sprintf(strConfig,"eth  %s netmask %s down",ppstrParms[0],ppstrParms[1]);

			for (j=0; j<lNumIf; j++)
			{
				strConfig[3] = 0x30 + j;
				TRACE(("Bring NIC Interface: eth%c (down)", 0x30 + j));

				if (TRUE != DrIfcfg (strConfig))
				{
					TRACE(("ERROR: Failed to bring NIC Interface: eth%c (down)", 0x30 + j));
				}
			}
			memset (strConfig, 0, sizeof (strConfig));
			(0 != strBcast[0]) ? 
			sprintf(strConfig,"eth  %s netmask %s broadcast %s up",ppstrParms[0],ppstrParms[1],strBcast) :
			sprintf(strConfig,"eth  %s netmask %s up",ppstrParms[0],ppstrParms[1]);

			strConfig[3] = 0x30 + i;
			TRACE(("Bring NIC Interface: eth%c (up)", 0x30 + i));

			if (TRUE != DrIfcfg (strConfig))
			{
				TRACE(("ERROR: Failed to bring NIC Interface: eth%c (up)", 0x30 + i));
			}
			else
			{
				if (0 < strlen (ppstrParms[2]))
				{
					memset (strConfig, 0, sizeof (strConfig));
					sprintf(strConfig, "add default gw %s", ppstrParms[2]);
				
					TRACE(("Add Default Gateway for without specific Dev: %s", ppstrParms[2]));
					
					if (TRUE != DrIfRoute (strConfig))
					{
						TRACE(("ERROR: Failed to Add Default Gateway: %s", ppstrParms[2]));
					}
					else
					{
						TRACE(("Add Default Gateway successfully: %s", ppstrParms[2]));
					}
				}

				sprintf(strMsg,"Contact Server: %s using Interface: eth%c. Please Wait...",
				ppstrParms[3], 0x30 + i);

				DisplayInfo (strMsg);

				sleep (CfgGetNegotiationTime ());

				if (TRUE == DrPg (ppstrParms[3]))
				{
					sprintf(strMsg,"Contact Server: %s Successfully using Interface: eth%c",
					ppstrParms[3], 0x30 + i);
					TRACE((strMsg));
					bSuccess = TRUE;
				}
				else
				{
					TRACE(("WARN: Failed to contact Server: %s Using Interface: eth%c",
					ppstrParms[3], 0x30 + i));
				}
			}
		}
		if (TRUE == bSuccess)
		{
			DisplayMsg (strMsg);
		}
		else
		{
			sprintf(strMsg,"ERROR: Failed contact Server: %s using all interfaces",ppstrParms[3]);
			DisplayMsg (strMsg);
		}
	}
	else if (NULL != pstrIf)
	{
		char 	strConfig[128] 	= {0};
		char	strBcast[32]		= {0};

		memset (strConfig, 0, sizeof (strConfig));
		memset (strBcast,  0, sizeof (strBcast));

		if (TRUE == GetBcastAddr (ppstrParms[0], ppstrParms[1], strBcast))
		{
			sprintf(strConfig, "%s %s netmask %s broadcast %s up",
			pstrIf, ppstrParms[0], ppstrParms[1], strBcast);
		}
		else
		{
			sprintf(strConfig, "%s %s netmask %s up",
			pstrIf, ppstrParms[0], ppstrParms[1]);
		}

		if (TRUE != DrIfcfg (strConfig))
		{
			sprintf(strMsg,"ERROR: Failed to config Interface: %s", pstrIf);
			DisplayMsg (strMsg);
		}
		else
		{
			sprintf(strMsg,"Config Interface: %s successfully", pstrIf);
			DisplayMsg (strMsg);
		}
		
		if (0 < strlen (ppstrParms[2]))
		{
			memset (strConfig, 0, sizeof (strConfig));
			sprintf(strConfig, "add default gw %s dev %s", ppstrParms[2], pstrIf);
			
			if (TRUE != DrIfRoute (strConfig))
			{
				sprintf(strMsg,"ERROR: Failed to Add Default Gateway: %s", ppstrParms[2]);
				DisplayMsg (strMsg);
			}
			else
			{
				sprintf(strMsg,"Add Default Gateway successfully: %s", ppstrParms[2]);
				DisplayMsg (strMsg);
			}
		}
	}
}





/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT GetIfParms (char *pstrIf, char **ppstrTags, char **ppstrParms, long lCount)
{
	WINDOW		*pWin				= NULL;
	int				lX					= 45;
	int				lY					= 14;
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	int				i						= 0;
	int				lTagLen			= strlen (*ppstrTags);
	int				lParmLen		= (lX - lTagLen) - 5;
	int				lParmStart	= lTagLen + 2;
	int				lPos				= 0;
	int				lCursePos		= 0;
	int				lInput			= 0;
	BooleanT	bContinue		= TRUE;
	BooleanT	bSuccess		= FALSE;
	char			strBuf[64];

	clear 	 ();
	refresh	 ();
	getmaxyx (stdscr, lMaxY, lMaxX);

	(NULL == pstrIf) ? (lY = 16) : (lY = 14);

	pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);

	wborder (pWin, 
					ACS_VLINE 		| gtAttribute[0], 
					ACS_VLINE 		| gtAttribute[1], 
					ACS_HLINE 		| gtAttribute[0], 
					ACS_HLINE 		| gtAttribute[1], 
					ACS_ULCORNER 	| gtAttribute[0], 
					ACS_URCORNER 	| gtAttribute[0], 
					ACS_LLCORNER 	| gtAttribute[1], 
					ACS_LRCORNER 	| gtAttribute[1]);

	wattrset (pWin, gtAttribute[1]);

	for (i=1; i<lY-1; i++)
	{
		wmove 	(pWin, i, 1);
		whline 	(pWin, ' ', lX-2);
	}
	memset 		(strBuf, 0, sizeof (strBuf));
	(NULL == pstrIf) ?
	sprintf		(strBuf, STR_CONFIG_IF, "ALL") :
	sprintf		(strBuf, STR_CONFIG_IF, pstrIf);
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 1, (lX/2) - (strlen (STR_BAKBONE) / 2), STR_BAKBONE);
	mvwaddstr (pWin, 2, (lX/2) - (strlen (strBuf) / 2), strBuf);
	wattrset 	(pWin, gtAttribute[1]);
	wmove 		(pWin, 3, 1);
	whline 		(pWin, ACS_HLINE, lX-2);
	
	for (i=0; i<lCount; i++)
	{
		wattrset 	(pWin, gtAttribute [1]);
		mvwaddstr (pWin, 5+2*i, 2, ppstrTags[i]);
		wattrset 	(pWin, gtAttribute[5]);
		whline 		(pWin, ' ', lParmLen);
		memset    (ppstrParms[i], 0, lParmLen);
	}
	wattrset 	(pWin, gtAttribute[1]);
	wmove 		(pWin, lY-3, 1);
	whline 		(pWin, ACS_HLINE, lX-2);
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, lY-2, (lX/4) - (strlen (STR_CONFIG)/2), STR_CONFIG);
	mvwaddstr (pWin, lY-2, (3*lX/4) - (strlen (STR_CANCEL)/2), STR_CANCEL);
	
	wrefresh	(pWin);
	noecho 		();
	keypad 		(pWin, 1);
	cbreak 		();
	
	while (TRUE == bContinue)
	{
		if (lPos < lCount)
		{
			wattrset 	(pWin, gtAttribute[3]);
			mvwaddstr (pWin, lY-2, (3*lX/4) - (strlen (STR_EXIT)/2), STR_CANCEL);
			mvwaddstr (pWin, lY-2, (lX/4) - (strlen (STR_CONFIG)/2), STR_CONFIG);

			wmove 		(pWin, 5 + 2 * lPos, lParmStart);
			wattrset 	(pWin, gtAttribute[4]);
			whline 		(pWin, ' ', lParmLen);
			waddstr 	(pWin, ppstrParms[lPos]);
			wmove 		(pWin, 5 + 2 * lPos, lParmStart + lCursePos);

			lInput = wgetch (pWin);

			wmove 		(pWin, 5 + 2 * lPos, lParmStart);
			wattrset 	(pWin, gtAttribute[5]);
			whline 		(pWin, ' ', lParmLen);
			waddstr 	(pWin, ppstrParms[lPos]);

			switch (lInput)
			{
				case KEY_LEFT:
				{
					if (lCursePos > 0)
					{
						lCursePos--;
					}
					break;
				}
				
				case KEY_RIGHT:
				{
					if (lCursePos < strlen (ppstrParms[lPos]))
					{
						lCursePos++;
					}
					break;
				}

				case KEY_HOME:
				{
					lCursePos = 0;
					break;
				}
				
				case KEY_END:
				{
					lCursePos = strlen (ppstrParms[lPos]);
					break;
				}
				
				case KEY_BACKSPACE:
				{
					if (lCursePos > 0)
					{
						lCursePos--;
						memmove	(
								ppstrParms[lPos] + lCursePos, 
								ppstrParms[lPos] + lCursePos + 1, 
								strlen (ppstrParms[lPos]) - lCursePos);
					}
					break;
				}
				
				case KEY_DC:
				{
					if (lCursePos < strlen (ppstrParms[lPos]))
					{
						memmove (
								ppstrParms[lPos] + lCursePos, 
								ppstrParms[lPos] + lCursePos + 1, 
								strlen (ppstrParms[lPos]) - lCursePos);
					}
					break;
				}
				
				case '\n':
				{
					lInput = KEY_DOWN;
					break;
				}

				case ' '...'~':
				{
					if (strlen (ppstrParms[lPos]) < lMaxX - lTagLen - 6)
					{
						memmove (
								ppstrParms[lPos] + lCursePos + 1, 
								ppstrParms[lPos] + lCursePos, 
								strlen (ppstrParms[lPos]) - lCursePos + 1);

						ppstrParms[lPos][lCursePos++] = lInput;
					}
					break;
				}
			}
		}
		else
		{
			int lTmp	=	(NULL == pstrIf) ? 4 : 3;

			if (lTmp == lPos)
			{
				wattrset 	(pWin, gtAttribute[3]);
				mvwaddstr (pWin, lY-2, (3*lX/4) - (strlen (STR_CANCEL)/2), STR_CANCEL);
				wattrset 	(pWin, gtAttribute[2]);
				mvwaddstr (pWin, lY-2, (lX/4) - (strlen (STR_CONFIG)/2), STR_CONFIG);
			}
			else if (lTmp + 1 == lPos)
			{
				wattrset 	(pWin, gtAttribute[3]);
				mvwaddstr (pWin, lY-2, (lX/4) - (strlen (STR_CONFIG)/2), STR_CONFIG);
				wattrset 	(pWin, gtAttribute[2]);
				mvwaddstr (pWin, lY-2, (3*lX/4) - (strlen (STR_CANCEL)/2), STR_CANCEL);
			}
			lInput = wgetch (pWin);
		}

		switch (lInput)
		{
			case '\t':
			case KEY_DOWN:
			{
				lCursePos = 0;
				lPos++;
				lPos %= (lCount + 2);
				break;
			}

			case KEY_UP:
			{
				lCursePos = 0;

				if (--lPos < 0)
				{
					lPos = lCount + 1;
				}
				break;
			}
				
			case '\n':
			{
				int lTmp	=	(NULL == pstrIf) ? 4 : 3;

				if (lTmp == lPos)
				{
					bSuccess = TRUE;
				}
				bContinue = FALSE;
				break;
			}
		}
	}
	delwin		(pWin);
	redrawwin	(stdscr);
	refresh		();

	return bSuccess;
}



/*
** =======================================================================================
** =======================================================================================
*/
static void DisplayIfMenu (WINDOW *pWin, int lX, int lY, long lNumIf) 
{
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	int				lInitY			= 4;
	int				lInitX			= 4;
	int				lPos				= 1;
	int				lIdx				= 1;
	int				lRC					= 0;
	int				i						= 0;
	int				lKeyPos			= 1;
	BooleanT	bExit				= FALSE;
	char			*pstrIfs[MAX_NUM_IF];

	getmaxyx (pWin, lMaxY, lMaxX);
	wattrset (pWin, gtAttribute [1]);

	for (i=0; i<lNumIf; i++)
	{
		pstrIfs[i] = (char *) malloc (5);
		sprintf		(pstrIfs[i], "eth%d", i);
		wmove 		(pWin, lPos+i+lInitY, lInitX);
		mvwaddstr (pWin, lPos+i+lInitY, lInitX, pstrIfs[i]);
	}
	noecho 	();
	keypad 	(pWin, 1);
	cbreak 	();

	do
	{
		wattrset 		(pWin, gtAttribute [2]);
		wmove 			(pWin, lPos+lInitY, 2);
		whline 			(pWin, ' ', lMaxX - 4);
		mvwaddstr 	(pWin, lPos+lInitY, lInitX, pstrIfs[lIdx-1]);
		whline			(pWin, ' ',(lInitX - strlen (pstrIfs[lIdx-1])+1) / 2 - 2);
		wrefresh 		(pWin);
		redrawwin 	(pWin);
		lRC = wgetch(pWin);
		wattrset 		(pWin, gtAttribute [1]);
		wmove 			(pWin, lPos+lInitY, 2);
		whline 			(pWin, ' ', lMaxX - 4);
		mvwaddstr 	(pWin, lPos+lInitY, lInitX, pstrIfs[lIdx-1]);

		switch (lRC)
		{
			case KEY_UP:
			{
				if (1 < lPos)
				{
					lPos--;
				}
				if (1 < lIdx)
				{
					lIdx--;
				}
				break;
			}

			case KEY_DOWN:
			{
				if (lNumIf > lPos)
				{
					lPos++;
				}
				if (lNumIf > lIdx)
				{
					lIdx++;
				}
				break;
			}
			
			case '\t':
			case KEY_RIGHT:
			{
				if (1 == lKeyPos)
				{
					lKeyPos++;
					wattrset 	(pWin, gtAttribute[2]);
					wmove 		(pWin, 17, (lX/4) - (strlen (STR_CONFIG)/2) - 5);
					whline 		(pWin, ' ', strlen (STR_CONFIG));
					mvwaddstr (pWin, 17, (lX/4) - (strlen (STR_CONFIG)/2) - 5, STR_CONFIG);

				}
				else if (2 == lKeyPos)
				{
					lKeyPos++;
					wattrset 	(pWin, gtAttribute[3]);
					wmove 		(pWin, 17, (lX/4) - (strlen (STR_CONFIG)/2) - 5);
					whline 		(pWin, ' ', strlen (STR_CONFIG));
					mvwaddstr (pWin, 17, (lX/4) - (strlen (STR_CONFIG)/2) - 5, STR_CONFIG);
	
					wattrset 	(pWin, gtAttribute[2]);
					wmove 		(pWin, 17, (lX/2) - (strlen (STR_AUTO)/2));
					whline 		(pWin, ' ', strlen (STR_AUTO));
					mvwaddstr (pWin, 17, (lX/2) - (strlen (STR_AUTO)/2), STR_AUTO);
				}
				else if (3 == lKeyPos)
				{
					lKeyPos++;
					
					wattrset 	(pWin, gtAttribute[3]);
					wmove 		(pWin, 17, (lX/2) - (strlen (STR_AUTO)/2));
					whline 		(pWin, ' ', strlen (STR_AUTO));
					mvwaddstr (pWin, 17, (lX/2) - (strlen (STR_AUTO)/2), STR_AUTO);

					wattrset 	(pWin, gtAttribute[2]);
					wmove 		(pWin, 17, (3*lX/4) - (strlen (STR_EXIT)/2) + 5);
					whline 		(pWin, ' ', strlen (STR_EXIT));
					mvwaddstr (pWin, 17, (3*lX/4) - (strlen (STR_EXIT)/2) + 5, STR_EXIT);
				}
				else
				{
					lKeyPos = 1;
					wattrset 	(pWin, gtAttribute[3]);
					wmove 		(pWin, 17, (3*lX/4) - (strlen (STR_EXIT)/2) + 5);
					whline 		(pWin, ' ', strlen (STR_EXIT));
					mvwaddstr (pWin, 17, (3*lX/4) - (strlen (STR_EXIT)/2) + 5, STR_EXIT);
				}
				break;
			}
			
			case KEY_LEFT:
			{
				if (1 == lKeyPos)
				{
					lKeyPos	= 4;
					wattrset 	(pWin, gtAttribute[2]);
					wmove 		(pWin, 17, (3*lX/4) - (strlen (STR_EXIT)/2) + 5);
					whline 		(pWin, ' ', strlen (STR_EXIT));
					mvwaddstr (pWin, 17, (3*lX/4) - (strlen (STR_EXIT)/2) + 5, STR_EXIT);
				}
				else if (4 == lKeyPos)
				{
					lKeyPos--;

					wattrset 	(pWin, gtAttribute[3]);
					wmove 		(pWin, 17, (3*lX/4) - (strlen (STR_EXIT)/2) + 5);
					whline 		(pWin, ' ', strlen (STR_EXIT));
					mvwaddstr (pWin, 17, (3*lX/4) - (strlen (STR_EXIT)/2) + 5, STR_EXIT);

					wattrset 	(pWin, gtAttribute[2]);
					wmove 		(pWin, 17, (lX/2) - (strlen (STR_AUTO)/2));
					whline 		(pWin, ' ', strlen (STR_AUTO));
					mvwaddstr (pWin, 17, (lX/2) - (strlen (STR_AUTO)/2), STR_AUTO);
				}
				else if (3 == lKeyPos)
				{
					lKeyPos--;
					
					wattrset 	(pWin, gtAttribute[3]);
					wmove 		(pWin, 17, (lX/2) - (strlen (STR_AUTO)/2));
					whline 		(pWin, ' ', strlen (STR_AUTO));
					mvwaddstr (pWin, 17, (lX/2) - (strlen (STR_AUTO)/2), STR_AUTO);
					
					wattrset 	(pWin, gtAttribute[2]);
					wmove 		(pWin, 17, (lX/4) - (strlen (STR_CONFIG)/2) - 5);
					whline 		(pWin, ' ', strlen (STR_CONFIG));
					mvwaddstr (pWin, 17, (lX/4) - (strlen (STR_CONFIG)/2) - 5, STR_CONFIG);
				}
				else if (2 == lKeyPos)
				{
					lKeyPos--;
					
					wattrset 	(pWin, gtAttribute[3]);
					wmove 		(pWin, 17, (lX/4) - (strlen (STR_CONFIG)/2) - 5);
					whline 		(pWin, ' ', strlen (STR_CONFIG));
					mvwaddstr (pWin, 17, (lX/4) - (strlen (STR_CONFIG)/2) - 5, STR_CONFIG);
				}
				break;
			}
			case '\n':
			{
				if (4 == lKeyPos)
				{
					bExit = TRUE;
				}
				else if (3 == lKeyPos)
				{
					char	*pstrTags[] = { "  IP Address: ", "Network Mask: ", "     Gateway: ", "   Server IP: " };
					long	lTagCnt			= sizeof (pstrTags) / sizeof (pstrTags[0]);
					char	strParms[lTagCnt][32];
					char	*pstrParms[lTagCnt];
					int		i;

					for (i=0; i<lTagCnt; i++)
					{
						memset (strParms[i], 0, sizeof (strParms[i]));
						pstrParms[i] = strParms[i];
					}
					if (TRUE == GetIfParms (NULL, pstrTags, pstrParms, lTagCnt))
					{
						ValidateAndConfigIf (NULL, pstrParms, lNumIf);
					}
				}
				else if (2 == lKeyPos || 1 == lKeyPos)
				{
					char	*pstrTags[] = { "  IP Address: ", "Network Mask: ", "     Gateway: " };
					long	lTagCnt			= sizeof (pstrTags) / sizeof (pstrTags[0]);
					char	strParms[lTagCnt][32];
					char	*pstrParms[lTagCnt];
					int		i;

					for (i=0; i<lTagCnt; i++)
					{
						memset (strParms[i], 0, sizeof (strParms[i]));
						pstrParms[i] = strParms[i];
					}
					if (TRUE == GetIfParms (pstrIfs[lIdx-1], pstrTags, pstrParms, lTagCnt))
					{
						ValidateAndConfigIf (pstrIfs[lIdx-1], pstrParms, lNumIf);
					}
				}
				break;
			}
		}
	} while (TRUE != bExit);

	for (i=0; i<lNumIf; i++)
	{
		free (pstrIfs[i]);
	}
}




/*
** =======================================================================================
** =======================================================================================
*/
static void DisplayAvailIf (long lNumIf)
{
	WINDOW	*pWin			= NULL;
	int			lX				= 45;
	int			lY				= 19;
	int			lMaxX			= 0;
	int			lMaxY			= 0;
	int			i					= 0;

	clear 	 ();
	refresh	 ();
	getmaxyx (stdscr, lMaxY, lMaxX);

	pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);

	wborder (pWin, 
					ACS_VLINE 		| gtAttribute[0], 
					ACS_VLINE 		| gtAttribute[1], 
					ACS_HLINE 		| gtAttribute[0], 
					ACS_HLINE 		| gtAttribute[1], 
					ACS_ULCORNER 	| gtAttribute[0], 
					ACS_URCORNER 	| gtAttribute[0], 
					ACS_LLCORNER 	| gtAttribute[1], 
					ACS_LRCORNER 	| gtAttribute[1]);

	wattrset (pWin, gtAttribute[1]);

	for (i=1; i<lY-1; i++)
	{
		wmove 	(pWin, i, 1);
		whline 	(pWin, ' ', lX-2);
	}
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 1, (lX/2) - (strlen (STR_BAKBONE) / 2), STR_BAKBONE);
	mvwaddstr (pWin, 2, (lX/2) - (strlen (STR_AVAIL_NIC) / 2), STR_AVAIL_NIC);
	wattrset 	(pWin, gtAttribute[1]);
	wmove 		(pWin, 3, 1);
	whline 		(pWin, ACS_HLINE, lX-2);
	wmove 		(pWin, 16, 1);
	whline 		(pWin, ACS_HLINE, lX-2);
	
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 17, (lX/4) - (strlen (STR_CONFIG)/2) - 5, STR_CONFIG);
	mvwaddstr (pWin, 17, (lX/2) - (strlen (STR_AUTO)/2), STR_AUTO);
	mvwaddstr (pWin, 17, (3*lX/4) - (strlen (STR_EXIT)/2) + 5, STR_EXIT);

	DisplayIfMenu (pWin, lX, lY, lNumIf);

	delwin		(pWin);
	redrawwin	(stdscr);
	refresh		();
}








/*
** =======================================================================================
** =======================================================================================
*/
static void CreateMainWindow ()
{
	int		lX		= 70;
	int		lY		= 13;
	int		lMaxX	= 0;
	int		lMaxY	= 0;
	int		i			= 0;

	getmaxyx (stdscr, lMaxY, lMaxX);

	gpWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);

	wborder (gpWin, 
					ACS_VLINE 		| gtAttribute[0], 
					ACS_VLINE 		| gtAttribute[1], 
					ACS_HLINE 		| gtAttribute[0], 
					ACS_HLINE 		| gtAttribute[1], 
					ACS_ULCORNER 	| gtAttribute[0], 
					ACS_URCORNER 	| gtAttribute[0], 
					ACS_LLCORNER 	| gtAttribute[1], 
					ACS_LRCORNER 	| gtAttribute[1]);

	wattrset (gpWin, gtAttribute[1]);

	for (i=1; i<lY-1; i++)
	{
		wmove 	(gpWin, i, 1);
		whline 	(gpWin, ' ', lX-2);
	}
	wattrset 	(gpWin, gtAttribute[3]);
	mvwaddstr (gpWin, 1, (lX-strlen (STR_BAKBONE)) / 2, STR_BAKBONE);
	mvwaddstr (gpWin, 2, (lX-strlen (STR_TITLE)) / 2, STR_TITLE);
	wattrset 	(gpWin, gtAttribute[1]);
	
	wmove 		(gpWin, 3, 9);
	whline 		(gpWin, ACS_HLINE, 51);
	whline 		(gpWin, ACS_ULCORNER, 1);
	wmove 		(gpWin, 4, 9);
	wvline 		(gpWin, ACS_VLINE, 1);
	
	wmove 		(gpWin, 3, 60);
	whline 		(gpWin, ACS_URCORNER, 1);
	wmove 		(gpWin, 4, 60);
	wvline 		(gpWin, ACS_VLINE, 1);

	wmove 		(gpWin, 5, 9);
	whline 		(gpWin, ACS_HLINE, 51);
	whline 		(gpWin, ACS_LLCORNER, 1);
	
	wmove 		(gpWin, 5, 60);
	whline 		(gpWin, ACS_LRCORNER, 1);
	
	mvwaddstr (gpWin, 6, 9, "0%");
	mvwaddstr (gpWin, 6, 57, "100%");
	mvwaddstr (gpWin, 6, 33, "50%");
	mvwaddstr (gpWin, 6, 21, "25%");
	mvwaddstr (gpWin, 6, 45, "75%");
	
	wrefresh 	(gpWin);
	redrawwin	(gpWin);
}



/*
** =======================================================================================
** =======================================================================================
*/
static void UpdateProgress (long lPercent)
{
	int	lMod	= lPercent % 2;

	if (0 == lMod)
	{
		wattrset 	(gpWin, gtAttribute[2]);
		wmove 		(gpWin, 4, 9 + (lPercent / 2));
		whline 		(gpWin, ' ', 1);
		wrefresh 	(gpWin);
		redrawwin	(gpWin);
	}
}




/*
** =======================================================================================
** =======================================================================================
*/
static void InitProgress ()
{
	wattrset 	(gpWin, gtAttribute[1]);
	wmove 		(gpWin, 4, 10);
	whline 		(gpWin, ' ', 50);
	wrefresh 	(gpWin);
	redrawwin	(gpWin);
}



/*
** =======================================================================================
** =======================================================================================
*/
static void WinLog (char *pstrMsg, long lIndex)
{
	if (lIndex < 5 )
	{
		wattrset	(gpWin, gtAttribute[3]);
		mvwaddstr	(gpWin, 7 + lIndex, 2, pstrMsg);
		wrefresh	(gpWin);
		redrawwin	(gpWin);
		}
	}



/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT ConnectToServer (char *pstrIp, long *plSock)
{
	BooleanT	bSuccess	= FALSE;
	long 			lIpA			= 0;
	long 			lIpB			= 0;
	long 			lIpC			= 0;
	long 			lIpD			= 0;

	TRACE(("Start connect to Server (%s)", pstrIp));

	if (4 != sscanf (pstrIp, "%ld.%ld.%ld.%ld", &lIpA, &lIpB, &lIpC, &lIpD))
	{
		TRACE(("Failed to sscanf for Server IP address"));
	}
	else
	{
		long 	lIpAddr	= ((lIpA << 24) & 0xff000000) | 
										((lIpB << 16) & 0xff0000) 	|
										((lIpC << 8)  & 0xff00)			|
										((lIpD)  & 0xff);

		TRACE(("Connect the server with IP: 0x%0lx", lIpAddr));

		if (0 == lIpAddr)
		{
			TRACE(("Server IP address is zero"));
		}
		else
		{
			long		lSock	= -1;
			struct	sockaddr_in	tSockAddr;

			memset (&tSockAddr, 0, sizeof (tSockAddr));

			tSockAddr.sin_family			= AF_INET;
			tSockAddr.sin_addr.s_addr	= htonl (lIpAddr);
			tSockAddr.sin_port				= htons (10666);

			if (0 > (lSock = socket (AF_INET, SOCK_STREAM, 0)))
			{
				TRACE(("Failed to create TCP socket for connect to Server"));
			}
			else if (0 >	connect (
										lSock,
										(struct sockaddr *) &tSockAddr,
										sizeof (tSockAddr)))
			{
				long lErr = errno;
				TRACE(("Failed to connect to Server. %s", SYSERR(lErr)));
			}
			else
			{
				*plSock		= lSock;
				bSuccess 	= TRUE;
			}
		}
	}
	return bSuccess;
}




/*
** =================================================================================
** =================================================================================
*/
static BooleanT XferData (long lSock)
{
	BooleanT	bSuccess	= TRUE;
	long 			lFd				= -1;

	TRACE(("Start transfer data with socket (%ld)", lSock));

	if (-1 == (lFd = open ("/mp/zncrf1.gz", O_WRONLY | O_CREAT, 0777)))
	{
		long lErr = errno;
		TRACE(("Failed to open file /mp/zncrf1.gz. %s",  SYSERR (lErr)));
		bSuccess	= FALSE;
	}
	else
	{
		BooleanT	bContinue				= TRUE;
		long			lBytesRecv			= 0;
		long			lWritten				= 0;
		long			lPercent				= 1;
		int64			llSegment				= 0x11e1a3ff / 100;
		int64			llTotal					= 0;
		ByteT			strBuf [32768]	= {0};

		InitProgress ();

		while (TRUE == bContinue)
		{
			lBytesRecv = recv (lSock, strBuf, sizeof (strBuf), MSG_WAITALL);

			if (-1 == lBytesRecv)
			{
				bContinue = FALSE;
				bSuccess 	= FALSE;
				TRACE(("Failed to receive data from server"));
			}
			else if (32768 == lBytesRecv)
			{
				lWritten = write (lFd, strBuf, 32768);
				
				if (-1 == lWritten || 32768 != lWritten)
				{
					TRACE(("Failed to write to device. Bytes written is %ld", lWritten));
					bContinue	= FALSE;
					bSuccess	= FALSE;
				}
				else
				{
					llTotal += (int64) lWritten;

					if ((llTotal >= llSegment * (int64) lPercent) && lPercent < 100)
					{
						UpdateProgress (lPercent++);
					}
				}
			}
			else if (32768 > lBytesRecv && lBytesRecv > 0)
			{
				lWritten = write (lFd, strBuf, lBytesRecv);
				
				if (-1 == lWritten || lBytesRecv != lWritten)
				{
					TRACE(("Failed to write last buffer to device"));
					bContinue	= FALSE;
					bSuccess	= FALSE;
				}
				else
				{
					UpdateProgress (100);
					TRACE(("Transfer data completed successfully. (EOF)"));
				}
				TRACE(("Recv last buffer (EOF)"));
			}
			else
			{
				bContinue = FALSE;
				TRACE(("Enter an unkown recv phase"));
			}
		}
		close (lFd);
	}
	TRACE(("End transfer data from server. %s", STR_BOOLEAN(bSuccess)));

	return bSuccess;
}



/*
** =================================================================================
** =================================================================================
*/
static BooleanT FinalXfer (char *pstrDev)
{
	BooleanT	bSuccess 	= FALSE;
	long 			lFdWrite	= -1;
	long 			lFdRead		= -1;

	if (-1 == (lFdWrite = open (pstrDev, O_WRONLY)))
	{
		long lErr = errno;
		TRACE(("Failed to open device (%s) to recover. %s", pstrDev, SYSERR (lErr)));
	}
	else if (-1 == (lFdRead = open ("/mp/zncrf1", O_RDONLY)))
	{
		long lErr = errno;
		TRACE(("Failed to open file /mp/zncrf1. %s", SYSERR (lErr)));
	}
	else
	{
		BooleanT	bContinue = TRUE;
		long			lPercent	= 1;
		int64			llSegment	= (32768 * 32768 ) / 100;
		int64			llTotal		= 0;
		long			lRead			= 0;
		long			lWrite		= 0;
		long			i					= 0;
		ByteT			strBuf [65536];

		bSuccess = TRUE;
		
		InitProgress ();

		for (i=0; i<16384 && TRUE == bContinue; i++)
		{
			lRead = read (lFdRead, strBuf, sizeof (strBuf));

			if (-1 == lRead)
			{
				TRACE(("Failed to read file /mp/zncrf1"));
				bContinue	= FALSE;
				bSuccess	= FALSE;
			}
			else if (lRead == 65536)
			{
				lWrite = write (lFdWrite, strBuf, sizeof (strBuf));
				
				if (-1 == lWrite || lWrite != 65536)
				{
					TRACE(("Failed to write to %s", pstrDev));
					bContinue	= FALSE;
					bSuccess	= FALSE;
				}
				else
				{
					llTotal += (int64) lWrite;

					if ((llTotal >= llSegment * (int64) lPercent) && lPercent < 100)
					{
						UpdateProgress (lPercent++);
					}
				}
			}
			else
			{
				lWrite = write (lFdWrite, strBuf, lRead);
				bContinue	= FALSE;
				bSuccess	= TRUE;
				UpdateProgress (100);
			}
		}
		close (lFdWrite);
		close (lFdRead);
	}
	return bSuccess;
}



/*
** =================================================================================
** =================================================================================
*/
static void StartProcess (char *argv1, char *argv2, char *argv3) 
{
	long	lSock = -1;

	DisplayMsg ("Make sure the server is ready for recovery process");

	if (TRUE != ConnectToServer (argv3, &lSock))
	{
		DisplayMsg ("ERROR: Failed to connect to Server. See /trace/dr.log");
	}
	else
	{
		char strMsg [256] = {0};

		CreateMainWindow ();

		WinLog("NCR MPRAS Recovery process started", 1);
		sprintf(strMsg,"Connect to server %s at port 10666 successfully", argv3);
		WinLog (strMsg, 2);
		sprintf(strMsg, "Root disk %s will be use for recovery process", argv1);
		WinLog (strMsg, 3);
		WinLog("Transfer data started. Please wait.....",4);

		if (TRUE != XferData (lSock))
		{
			DisplayMsg ("ERROR: Failed to transfer data from Server. See /trace/dr.log");
		}
		else
		{
			WinLog("Transfer data of phase 1 image completed successfuly", 4);
			DisplayInfo ("Uncompress Phase 1 image file. Please wait...");

			if (TRUE != DrGz ("/mp/zncrf1.gz"))
			{
				DisplayMsg ("ERROR: Failed to uncompress Image");
			}
			else
			{
				CreateMainWindow ();

				WinLog("NCR MPRAS Recovery process started", 1);
				sprintf(strMsg,"Transfer uncompress data to root disk");
				WinLog (strMsg, 2);
				sprintf(strMsg, "Root disk %s will be use for recovery process", argv1);
				WinLog (strMsg, 3);
				WinLog("Transfer data started. Please wait.....",4);

				if (TRUE == FinalXfer (argv1))
				{
					DisplayMsg ("Completed transfer data. Please reboot");

					/*
					while (1)
					{
						int lRC = system ("/bin/drreboot");

						if (WIFSIGNALED (lRC) && (WTERMSIG (lRC) == SIGINT || WTERMSIG (lRC) == SIGQUIT))
						{
							break;
						}
					}
					*/

				}
				else
				{
					DisplayMsg ("ERROR: Failed to uncompress image and write to Boot device");
				}
			}
		}
	}
}



/*
** =======================================================================================
** =======================================================================================
*/
int main (int argc, char *argv[]) 
{
	long	lIfCount			= 0;
	long	lDevCount			= 0;
	char 	*pstrDevs[32]	= {0};

	DrTraceInit (TRUE);

	TRACE(("DRDAEMON_NCR process begin"));

	LoadNcrModules ();
	WinInit ();

	if (0 == (lIfCount = GetNicInterfaces ()))
	{
		DisplayMsg ("ERROR: Unable to find any NIC interfaces"); 
	}
	else if (0 == (lDevCount = GetDiskDevices (pstrDevs)))
	{
		DisplayMsg ("ERROR: Unable to find any Disk Devices");
	}
	else
	{
		BooleanT	bContinue 				= TRUE;
		char 			strSelectDev [64] = {0};
		char			strMsg [512]			= {0};
		char			strCmd [512]			= {0};
		
		memset (gstrDevRoot,  0, sizeof (gstrDevRoot));
		memset (gstrDevTmp,   0, sizeof (gstrDevTmp));
		memset (gstrServerIp, 0, sizeof (gstrServerIp));
		
		DisplayAvailIf (lIfCount); 

		while (TRUE == bContinue)
		{
			ListDiskDev (pstrDevs, lDevCount, TRUE);

			strcpy (strSelectDev, (char *) strtok (gstrDevRoot, " "));

			sprintf(strMsg, "Data on (%s) will be lost. Do you want to go ?", strSelectDev);
			
			if (DrYes ==	DisplayRequest (STR_YES, STR_NO, strMsg, DrYes))
			{
				bContinue = FALSE;
			}
		}

		bContinue = TRUE;
		
		while (TRUE == bContinue)
		{
			ListDiskDev (pstrDevs, lDevCount, FALSE);

			strcpy (strSelectDev, (char *) strtok (gstrDevTmp, " "));

			sprintf(strMsg, "Data on (%s) will be lost. Do you want to go ?", strSelectDev);
			
			if (DrYes ==	DisplayRequest (STR_YES, STR_NO, strMsg, DrYes))
			{
				bContinue = FALSE;
			}
		}

		if (0 == strlen (gstrServerIp))
		{
			DisplayServerIp ();
		}
		TRACE(("Root Disk Device is: %s", gstrDevRoot));
		TRACE(("Temp Disk Device is: %s", gstrDevTmp));
		TRACE(("Server IP Address is: %s", gstrServerIp));

		DisplayInfo ("Making temp filesystem. Please wait...");
		sprintf (strCmd, "/bin/drmktmp_ncr.sh %s > /dev/null 2> /dev/null", gstrDevTmp);
		
		if (0 != system (strCmd))
		{
			DisplayMsg ("ERROR: Failed to make Temp Filesystem");
		}
		else if	(0 == strlen (gstrDevRoot) || 
						 0 == strlen (gstrDevTmp) || 
						 0 == strlen (gstrServerIp))
		{
			DisplayMsg ("ERROR: Root disk or Temp disk or Server IP not provided");
		}
		else
		{
			StartProcess (gstrDevRoot, gstrDevTmp, gstrServerIp);
		}
	}
	system ("tset -Q");

	return 0;
}
