
#include "drgz.h"

#define O_BINARY  0
#define RW_USER (S_IRUSR | S_IWUSR) 
#define MAX_PATH_LEN   1024

DECLARE(uch, gBufferIn,  INBUFSIZ +INBUF_EXTRA);
DECLARE(uch, gBufferOut, OUTBUFSIZ+OUTBUF_EXTRA);
DECLARE(uch, gBufferWin, 2L*WSIZE);

int  			glInFd			= -1;
int  			glOutFd			= -1;
off_t 		glBytesIn		= 0;
off_t 		glBytesOut	= 0;
unsigned 	glOutSizeGz	= 0;
unsigned 	glInSizeGz	= 0;
unsigned 	glInPtr			= 0;

char gstrInFile  [MAX_PATH_LEN] = {0};
char gstrOutFile [MAX_PATH_LEN] = {0};

static int LocalUncompress ();
static int (*pMainFunc) () = LocalUncompress;

static int 			GlMethod 		= DEFLATED;
static time_t 	glTimeStamp	= 0;
static int 			glPartNB		= 0;



/*
** =================================================================================
** =================================================================================
*/
static int LocalUncompress ()
{
  ulg lCRC = 0;
  ulg lLen = 0;
  uch strBuf[16];
  int i;

  ConvertCRC (NULL, 0);

  if (GlMethod == DEFLATED)
	{
		int lRC = DrDecompress ();

		if (lRC == 3)
		{
			ZLOGGZ(TRUE,0,"ERROR: Out of memory");
		}
		else if (lRC != 0)
		{
			ZLOGGZ(TRUE,0,"ERROR: Invalid compressed data");
		}
	}
  else
	{
		ZLOGGZ(TRUE,0,"ERROR: Invalid method");
	}
	for (i=0; i<8; i++)
	{
		strBuf[i] = (uch) GET_BYTE ();
	}
	lCRC = LG (strBuf);
	lLen = LG (strBuf + 4);

  if (lCRC != ConvertCRC (gBufferOut, 0))
	{
		ZLOGGZ(TRUE,0,"ERROR: Invalid compressed data - CRC Error");
	}
  if (lLen != (ulg) (glBytesOut & 0xffffffff))
	{
		ZLOGGZ(TRUE,0,"ERROR: Invalid compressed data - Length Error");
	}
  return TRUE;
}




/*
** =================================================================================
** =================================================================================
*/
static int ReachEOF ()
{
	BooleanT bEOF = FALSE;

  if (glInPtr == glInSizeGz)
	{
		if (glInSizeGz != INBUFSIZ || FillBuffer (1) == EOF)
		{
			bEOF = TRUE;
		}
		glInPtr = 0;
	}
  return bEOF;
}




/*
** =================================================================================
** =================================================================================
*/
static void SetOutputFileStat (struct stat *ptStat)
{
	struct utimbuf	tTimeBuf;
	
	if (glTimeStamp != 0 && ptStat->st_mtime != glTimeStamp) 
	{
		ptStat->st_mtime = glTimeStamp;
	}
	tTimeBuf.modtime = ptStat->st_mtime;
	tTimeBuf.actime  = ptStat->st_atime;

	if (-1 == utime (gstrOutFile, &tTimeBuf)) 
	{
		ZLOGGZ(FALSE,errno,"ERROR: Failed to set Access and Mod time for Output File (%s)", gstrOutFile); 
	}
	else if (-1 == chmod (gstrOutFile, ptStat->st_mode & 07777)) 
	{
		ZLOGGZ(FALSE,errno,"ERROR: Failed to chmod Output File (%s)", gstrOutFile); 
	}
	else if (-1 == chown (gstrOutFile, ptStat->st_uid, ptStat->st_gid))
	{
		ZLOGGZ(FALSE,errno,"ERROR: Failed to chown Output File (%s)", gstrOutFile); 
	}
	else if (-1 == unlink (gstrInFile)) 
	{
		ZLOGGZ(FALSE,errno,"ERROR: Failed to unlink Input File (%s)", gstrInFile); 
	}
}




/*
** =================================================================================
** =================================================================================
*/
static int GetMethod ()
{
	uch 	lFlags			= 0;     
	ulg 	lTimeStamp	= 0;     
	int 	lMagic			= 0;   
	char 	strMagic[2]	= {0};

	strMagic[0] = (char) GET_BYTE();
	strMagic[1] = (char) GET_BYTE();
	lMagic 			= 0; 
	GlMethod 		= -1;                 
	glPartNB++;                   

	if (0 == memcmp (strMagic, GZIP_MAGIC, 2) || 0 == memcmp (strMagic, OLD_GZIP_MAGIC, 2)) 
	{
		GlMethod = (int) GET_BYTE();

		if (GlMethod != DEFLATED) 
		{
			ZLOGGZ(TRUE,0,"ERROR: Unknown Method - Not Supported\n");
		}
		lFlags = (uch) GET_BYTE();

		if (0 != (lFlags & ENCRYPTED)) 
		{
			ZLOGGZ(TRUE,0,"ERROR: Unkown Encrypted - Not Supported\n");
		}
		if (0 != (lFlags & CONTINUATION)) 
		{
			ZLOGGZ(TRUE,0,"ERROR: Multiple Compress - Not Supported");
		}
		if (0 != (lFlags & RESERVED)) 
		{
			ZLOGGZ(TRUE,0,"ERROR: Reverse Flag - Not Supported");
		}
		lTimeStamp  =  (ulg) GET_BYTE();
		lTimeStamp |= ((ulg) GET_BYTE()) << 8;
		lTimeStamp |= ((ulg) GET_BYTE()) << 16;
		lTimeStamp |= ((ulg) GET_BYTE()) << 24;

		GET_BYTE();  
		GET_BYTE();  

		if (0 != (lFlags & CONTINUATION)) 
		{
			unsigned lPart = (unsigned) GET_BYTE();
			lPart |= ((unsigned) GET_BYTE()) << 8;
		}
		if (0 != (lFlags & EXTRA_FIELD)) 
		{
			unsigned lLength = (unsigned) GET_BYTE();
			lLength |= ((unsigned) GET_BYTE()) << 8;

			while (lLength--)
			{
				GET_BYTE();
			}
		}

		if (0 != (lFlags & ORIG_NAME)) 
		{
			char c;
			do
			{
				c = GET_BYTE ();
			} while (c != 0);
		}

		if ((lFlags & COMMENT) != 0) 
		{
			while (GET_BYTE() != 0);
		}
	} 
	if (GlMethod >= 0) 
	{
		return GlMethod;
	}

	if (glPartNB == 1) 
	{
		ZLOGGZ(TRUE,0,"ERROR: Invalid compress format - Not Supported");
		return -1;
	}
	else 
	{
		if (strMagic[0] == 0)
		{
			int lByte;

			for (lByte = lMagic; lByte == 0; lByte = TRY_BYTE ())
			{
				continue;
			}
			if (lByte == EOF)
			{
				return -3;
			}
		}
		ZLOGGZ(FALSE,0,"WARNING: Decompression OK, trailing garbage ignored");
		return -2;
	}
}


/*
** =================================================================================
** =================================================================================
*/
static void ProcessInputFile (char *pstrFile)
{
	char		*pstr = NULL;
	struct	stat	tStat;

	strcpy (gstrInFile, pstrFile);
	strcpy (gstrOutFile, pstrFile);

	if (0 != lstat (gstrInFile, &tStat))
	{
		ZLOGGZ(TRUE,0,"ERROR: Failed to stat Input File: %s", gstrInFile);
	}
	else if (NULL == (pstr = strstr (gstrOutFile, ".gz")))
	{
		ZLOGGZ(TRUE,0,"ERROR: Input File (%s) name does not contain extention .gz", gstrInFile);
	}
	else if (S_ISDIR (tStat.st_mode)) 
	{
		ZLOGGZ(TRUE,0,"ERROR: Input File (%s) is a directory", gstrInFile);
	}
	if (! S_ISREG (tStat.st_mode)) 
	{
		ZLOGGZ(TRUE,0,"ERROR: Input File (%s) is not a regular file", gstrInFile);
	}
	else if (-1 == (glInFd = open (gstrInFile, O_RDONLY | O_BINARY, RW_USER)))
	{
		ZLOGGZ(TRUE,errno,"ERROR: Failed to open Input File (%s)", gstrInFile);
	}
	else
	{
		ClearBuffer ();

		glTimeStamp	= 0;
		glPartNB 		= 0;
		*pstr				= 0;

		if (0 > GetMethod ())
		{
			ZLOGGZ(TRUE,0,"ERROR: Failed to get method");
		}
		else if (-1 == (glOutFd = open (gstrOutFile, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, RW_USER)))
		{
			ZLOGGZ(TRUE,errno,"ERROR: Failed to open Output File (%s)", gstrOutFile);
		}
		else
		{
			for (;;) 
			{
				if (TRUE != (*pMainFunc) ())
				{
					break;
				}
				if (TRUE == ReachEOF ())
				{
					break;
				}
				glBytesOut = 0;
			}

			if (-1 == close (glInFd))
			{
				ZLOGGZ(FALSE,errno,"ERROR: Failed to close Input File (%s)", gstrInFile);
			}
			else if (-1 == close (glOutFd)) 
			{
				ZLOGGZ(TRUE,errno,"ERROR: Failed to close Output File (%s)", gstrOutFile);
			}
			else
			{
				SetOutputFileStat (&tStat);
			}
		}
	}
}



/*
** =================================================================================
** ZLEM: Main
** =================================================================================
*/
extern BooleanT DrGz (char *pstrFile)
{
	ALLOC(uch, gBufferIn,  INBUFSIZ +INBUF_EXTRA);
	ALLOC(uch, gBufferOut, OUTBUFSIZ+OUTBUF_EXTRA);
	ALLOC(uch, gBufferWin, 2L*WSIZE);

	ProcessInputFile (pstrFile);

	FREE(gBufferIn);
	FREE(gBufferOut);
	FREE(gBufferWin);

	return TRUE;
}






