
#include "drifcfg.h"
#include "drtrace.h"

#define TRUE		1
#define FALSE		0

#define GET_MASK_ADDR(x) (((struct sockaddr_in *)&((x).rt_genmask))->sin_addr.s_addr)

MODULE("IFCFG")

typedef enum
{
	eRouteAdd = 0,
	eRouteDel = 1,
	eUnkown		= 2
} RouteActionE;



/*
** ==================================================================================
** ==================================================================================
*/
static int OpenSocket (int lFamily)
{
  NetDomainT 		**pptDomain;
  int 					lSock 		= -1;
	BooleanT			bAccess		= FALSE;
	BooleanT			bContinue	= TRUE;

	if (0 == access ("/proc/net", R_OK))
	{
		bAccess = TRUE;
	}

  for (pptDomain = gtNetDomain; *pptDomain && TRUE == bContinue; pptDomain++)
	{
		NetDomainT *ptDomain = *pptDomain;

		if (ptDomain->lFd != -1)
		{
			lSock = ptDomain->lFd;
		}
		else if (0 == lFamily && TRUE == bAccess && NULL != ptDomain->pstrFfile)
		{
			if (0 != access (ptDomain->pstrFfile, R_OK))
			{
				bContinue = FALSE;
			}
		}
		else
		{
			if (-1 == (ptDomain->lFd = socket (ptDomain->lDomain, SOCK_DGRAM, 0)))
			{
				TRACE(("ERROR: Failed to create Socket"));
			}
			else
			{
				lSock 		= ptDomain->lFd;
				bContinue	= FALSE;
			}
		}
	}
  return lSock;
}




/*
** ==================================================================================
** ==================================================================================
*/
static BooleanT SetSocket (int lSock, char *pstrIfName, short lFlag)
{
	BooleanT	bSuccess	= FALSE;
	struct 		ifreq tIfreq;

	strncpy (tIfreq.ifr_name, pstrIfName, IFNAMSIZ);

	if (-1 == ioctl (lSock, SIOCGIFFLAGS, &tIfreq)) 
	{
		TRACE(("ERROR(1): Failed to set SIOCGIFFLAGS for interface: %s. %s", 
		pstrIfName, strerror(errno)));
	}
	else
	{
		strncpy (tIfreq.ifr_name, pstrIfName, IFNAMSIZ);
		tIfreq.ifr_flags |= lFlag;

		if (-1 == ioctl (lSock, SIOCSIFFLAGS, &tIfreq)) 
		{
			TRACE(("ERROR(2): Failed to set SIOCSIFFLAGS for interface: %s. %s", 
			pstrIfName, strerror(errno)));
		}
		else
		{
			bSuccess = TRUE;
		}
	}
	return bSuccess;
}




/*
** ==================================================================================
** ==================================================================================
*/
static BooleanT ClearSocket (int lSock, char *pstrIfName, short lFlag)
{
	BooleanT	bSuccess	= FALSE;
	struct 		ifreq tIfreq;
	
	strncpy (tIfreq.ifr_name, pstrIfName, IFNAMSIZ);

	if (-1 == ioctl (lSock, SIOCGIFFLAGS, &tIfreq)) 
	{
		TRACE(("ERROR(1): Failed to clear SIOCGIFFLAGS for interface: %s. %s", 
		pstrIfName, strerror(errno)));
	}
	else
	{
		strncpy (tIfreq.ifr_name, pstrIfName, IFNAMSIZ);
		tIfreq.ifr_flags &= ~lFlag;

		if (-1 == ioctl (lSock, SIOCSIFFLAGS, &tIfreq)) 
		{
			TRACE(("ERROR(2): Failed to clear SIOCSIFFLAGS for interface: %s. %s", 
			pstrIfName, strerror (errno)));
		}
		else
		{
			bSuccess = TRUE;
		}
	}
	return bSuccess;
}




/*
** ==================================================================================
** ==================================================================================
*/
static BooleanT SetNetworkMask (
								int 						lSock, 
								struct ifreq 		*ptIfReq, 
								struct sockaddr *ptSockAddr)
{
	BooleanT	bSuccess = FALSE;

	memcpy ((char *) &ptIfReq->ifr_netmask, (char *) ptSockAddr, sizeof (struct sockaddr));

	if (-1 == ioctl (lSock, SIOCSIFNETMASK, ptIfReq)) 
	{
		TRACE(("ERROR: Failed to set NetMask: SIOCSIFNETMASK: %s",strerror(errno)));
	}
	else
	{
		bSuccess = TRUE;
	}
	return bSuccess;
}



/*
** ==================================================================================
** ==================================================================================
*/
extern BooleanT DrIfcfg (char *pstrInput)
{
	char			*pstrArg[10];
  char 			**ppstrArg 		= pstrArg;
	char 			*pstrBuf 			= (char *) malloc (128);
	char			*pstrInit			= pstrBuf;
	char			*pstr					= pstrBuf;
	int				lSock					= -1;
	BooleanT	bSuccess			= TRUE;

	TRACE(("Configure Interface Begin: %s", pstrInput));

	strncpy (pstrBuf, pstrInput, 128);

	while (*pstrBuf != 0)
	{
		if (0x20 == *pstrBuf)
		{
			*pstrBuf 		= 0;
			*ppstrArg++ = pstr;
			pstr 				= ++pstrBuf;
		}
		pstrBuf++;
		
		if (0 == *pstrBuf)
		{
			*ppstrArg++ = pstr;
		}
	}
	*ppstrArg = NULL;
	ppstrArg	=	 pstrArg;
		
	if (-1 == (lSock = OpenSocket (0)))
	{
		TRACE(("ERROR: Failed to create Socket"));
	}
	else
	{
		char				strIp [64] 	= {0};
		struct 			sockaddr 		tSockAddr;
		struct 			ifreq 			tIfreq;
		NetDomainT 							*ptDomain;

		strncpy (tIfreq.ifr_name, *ppstrArg++, IFNAMSIZ);
	
		TRACE(("Start configure interface: %s", tIfreq.ifr_name));

		if (NULL != (ptDomain = GetNetDomain ("inet")))
		{
			lSock = ptDomain->lFd;
		}
		
		while (NULL != *ppstrArg && TRUE == bSuccess)
		{
			if (0 == strcmp (*ppstrArg, "up"))
			{
				bSuccess = SetSocket (lSock, tIfreq.ifr_name, (IFF_UP | IFF_RUNNING));
			}
			else if (0 == strcmp (*ppstrArg, "down"))
			{
				bSuccess = ClearSocket (lSock, tIfreq.ifr_name, IFF_UP);
			}
			else if (0 == strcmp (*ppstrArg, "broadcast"))
			{
				if (NULL == *++ppstrArg)
				{
					bSuccess = SetSocket (lSock, tIfreq.ifr_name, IFF_BROADCAST);
					TRACE(("WARN: Set Broadcast Flag without set Broadcast Address"));
				}
				else
				{
					strncpy (strIp, *ppstrArg, (sizeof (strIp)));

					if (-1 == ptDomain->InputCB (0, strIp, &tSockAddr))
					{
						TRACE(("ERROR: Failed to CallBack Network Domain for Broadcast"));
						bSuccess = FALSE;
					}
					else
					{
						memcpy ((char *) &tIfreq.ifr_broadaddr, (char *) &tSockAddr, sizeof (struct sockaddr));

						if (-1 == ioctl (ptDomain->lFd, SIOCSIFBRDADDR, &tIfreq))
						{
							TRACE(("ERROR: Failed to set Broadcast Address: SIOCSIFBRDADDR: %s",strerror (errno)));
							bSuccess = FALSE;
						}
					}
				}
			}
			else if (0 == strcmp (*ppstrArg, "netmask"))
			{
				if (NULL == *++ppstrArg)
				{
					TRACE(("ERROR: NetMask IP address not provided"));
				}
				else
				{
					strncpy (strIp, *ppstrArg, (sizeof (strIp)));

					if (-1 == ptDomain->InputCB (0, strIp, &tSockAddr))
					{
						TRACE(("ERROR: Failed to CallBack Network Domain for NetMask"));
						bSuccess = FALSE;
					}
					else
					{
						bSuccess = SetNetworkMask (ptDomain->lFd, &tIfreq, &tSockAddr);
					}
				}
			}
			else
			{
				strncpy (strIp, *ppstrArg, (sizeof (strIp)));

				if (NULL == ptDomain->InputCB)
				{
					TRACE(("ERROR: Cannot set address for this protocol family"));
					bSuccess = FALSE;
				}
				else if (-1 == ptDomain->InputCB (0, strIp, &tSockAddr))
				{
					TRACE(("ERROR: Failed to CallBack Network Domain for Main IP"));
					bSuccess = FALSE;
				}
				else
				{
					memcpy ((char *) &tIfreq.ifr_addr, (char *) &tSockAddr, sizeof (struct sockaddr));

					if (AF_INET != ptDomain->lDomain)
					{
						TRACE(("ERROR: Invalid Network Domain: %d", ptDomain->lDomain));
						bSuccess = FALSE;
					}
					else
					{
						long lSockLocal = GetNetDomainSock (AF_INET);

						if (-1 == lSockLocal)
						{
							TRACE(("ERROR: Network Domain AF_INET is not support on this system"));
							bSuccess = FALSE;
						}
						else if (-1 == ioctl (lSockLocal, SIOCSIFADDR, &tIfreq))
						{
							TRACE(("ERROR: Failed to Set Interface IP: SIOCSIFADDR"));
							bSuccess = FALSE;
						}
						else
						{
							TRACE(("ioctl Set IP Address for interface successfully"));
						}
					}
				}
			}
			ppstrArg++;
		}
	}
	free (pstrInit);
	
	TRACE(("Configure Interface Ended. %s", bSuccess ? "SUCCESS" : "ERROR"));

  return bSuccess;
}




/*
** ==================================================================================
** ==================================================================================
*/
extern BooleanT DrIfRoute (char *pstrInput)
{
	char				*pstrArg[10];
  char 				**ppstrArg 		= pstrArg;
	char 				*pstrBuf 			= (char *) malloc (128);
	char				*pstrInit			= pstrBuf;
	char				*pstr					= pstrBuf;
	BooleanT		bSuccess			= TRUE;
	NetDomainT	*ptDomain			= NULL;

	TRACE(("Add/Del Route Interface Begin: %s", pstrInput));

	strncpy (pstrBuf, pstrInput, 128);

	while (*pstrBuf != 0)
	{
		if (0x20 == *pstrBuf)
		{
			*pstrBuf 		= 0;
			*ppstrArg++ = pstr;
			pstr 				= ++pstrBuf;
		}
		pstrBuf++;
		
		if (0 == *pstrBuf)
		{
			*ppstrArg++ = pstr;
		}
	}
	*ppstrArg = NULL;
	ppstrArg	=	 pstrArg;

	if (NULL == (ptDomain = GetNetDomain ("inet")))
	{
		TRACE(("ERROR: Failed to get INET domain"));
	}
	else if (0 != strcmp ("add", *ppstrArg) && 0 != strcmp ("del", *ppstrArg))
	{
		TRACE(("ERROR: Unkown Route Action: %s", *ppstrArg));
	}
	else
	{
		RouteActionE		eAction					= eRouteAdd;
		int							lRC							= 0;
		char						strTarget[32]		= {0};
		char						strNetMask[32]	= "default";
		char						strGateway[32]	= "NONE";
		struct rtentry	tRentry;
		struct sockaddr tSockAddrMask;
	
		if (0 == strcmp ("del", *ppstrArg++))
		{
			eAction = eRouteDel;
		}
		strncpy (strTarget, *ppstrArg++, sizeof (strTarget));
		memset  ((char *) &tRentry, 0, sizeof (struct rtentry));

		if (-1 == (lRC = ptDomain->GetMaskCB (strTarget, &tSockAddrMask, strNetMask)))
		{
			TRACE(("ERROR: Failed to get Netmask Callback")); 
			bSuccess = FALSE;
		}
		else if (lRC)
		{
			tRentry.rt_genmask = tSockAddrMask;
		}
		if (-1 == (lRC = ptDomain->InputCB (0, strTarget, &tRentry.rt_dst)))
		{
			TRACE(("ERROR: Failed to get Input Callback-1")); 
			bSuccess = FALSE;
		}
		tRentry.rt_flags = (RTF_UP | RTF_HOST);

		if (0 != lRC)
		{
			tRentry.rt_flags &= ~RTF_HOST;
		}

		while (NULL != *ppstrArg && TRUE == bSuccess)
		{
			if (0 == strcmp (*ppstrArg, "metric"))
			{
				ppstrArg++;

				if (NULL == *ppstrArg || 0 == isdigit (**ppstrArg))
				{
					TRACE(("ERROR: Invalid Metric value: %s", *ppstrArg));
					bSuccess = FALSE;
				}
				else
				{
					int lMetric	= atoi (*ppstrArg);

					tRentry.rt_metric = lMetric + 1;
				}
			}
			else if (0 == strcmp (*ppstrArg, "netmask"))
			{
				ppstrArg++;

				if (NULL == *ppstrArg || GET_MASK_ADDR (tRentry))
				{
					TRACE(("ERROR: Invalid NetMask value: %s", *ppstrArg));
					bSuccess = FALSE;
				}
				else
				{
					struct sockaddr tSockAddr;

					strncpy (strNetMask, *ppstrArg, sizeof (strNetMask));

					if (-1 == (lRC = ptDomain->InputCB (0, strNetMask, &tSockAddr)))
					{
						TRACE(("ERROR: Failed to get Input Callback-2")); 
						bSuccess = FALSE;
					}
					tRentry.rt_genmask = tSockAddr;
				}
			}
			else if (0 == strcmp (*ppstrArg, "gw") || 0 == strcmp (*ppstrArg, "gateway"))
			{
				ppstrArg++;

				if (NULL == *ppstrArg)
				{
					TRACE(("ERROR: Invalid Gateway: %s", *ppstrArg));
					bSuccess = FALSE;
				}
				else if (tRentry.rt_flags & RTF_GATEWAY)
				{
					TRACE(("ERROR: Invalid Gateway flags"));
					bSuccess = FALSE;
				}
				else
				{
					strncpy (strGateway, *ppstrArg, sizeof (strGateway));

					if (-1 == (lRC = ptDomain->InputCB (256, strGateway, &tRentry.rt_gateway)))
					{
						TRACE(("ERROR: Failed to get Input Callback-3")); 
						bSuccess = FALSE;
					}
					else if (lRC)
					{
						TRACE(("ERROR: Cannot use a NETWORK as gateway"));
						bSuccess = FALSE;
					}
					else
					{
						tRentry.rt_flags |= RTF_GATEWAY;
					}
				}
			}
			else if (0 == strcmp (*ppstrArg, "mss") || 0 == strcmp (*ppstrArg, "mtu"))
			{
				ppstrArg++;
				tRentry.rt_flags |= RTF_MSS;

				if (NULL == *ppstrArg)
				{
					TRACE(("ERROR: Invalid MSS/MTU value"));
					bSuccess = FALSE;
				}
				else
				{
					tRentry.rt_mss = atoi (*ppstrArg);

					if (tRentry.rt_mss < 64 || tRentry.rt_mss > 65536)
					{
						TRACE(("ERROR: MSS value out of range"));
						bSuccess = FALSE;
					}
				}
			}
			else if (0 == strcmp (*ppstrArg, "window"))
			{
				ppstrArg++;

				if (NULL == *ppstrArg)
				{
					TRACE(("ERROR: Invalid Window value"));
					bSuccess = FALSE;
				}
				else
				{
					tRentry.rt_flags |= RTF_WINDOW;
					tRentry.rt_window = atoi (*ppstrArg);

					if (tRentry.rt_window < 128)
					{
						TRACE(("ERROR: Window value is out of range"));
						bSuccess = FALSE;
					}
				}
			}
			else if (0 == strcmp (*ppstrArg, "irtt"))
			{
				ppstrArg++;

				if (NULL == *ppstrArg)
				{
					TRACE(("ERROR: Invalid IRTT value"));
					bSuccess = FALSE;
				}
				else
				{
					tRentry.rt_flags |= RTF_IRTT;
					tRentry.rt_irtt = atoi (*(ppstrArg - 1));
					tRentry.rt_irtt *= (HZ / 100);
				}
			}
			else if (0 == strcmp (*ppstrArg, "reject"))
			{
				tRentry.rt_flags |= RTF_REJECT;
			}
			else if (0 == strcmp (*ppstrArg, "mod"))
			{
				tRentry.rt_flags |= RTF_MODIFIED;
			}
			else if (0 == strcmp (*ppstrArg, "dyn"))
			{
				tRentry.rt_flags |= RTF_DYNAMIC;
			}
			else if (0 == strcmp (*ppstrArg, "reinstate"))
			{
				tRentry.rt_flags |= RTF_REINSTATE;
			}
			else if (0 == strcmp (*ppstrArg, "device") || 0 == strcmp (*ppstrArg, "dev"))
			{
				ppstrArg++;

				if (tRentry.rt_dev || NULL == *ppstrArg)
				{
					TRACE(("ERROR: Invalid Device value"));
					bSuccess = FALSE;
				}
				tRentry.rt_dev = *ppstrArg;
			}
			ppstrArg++;
		}

		if ((tRentry.rt_flags & RTF_REJECT) && ! tRentry.rt_dev)
		{
			tRentry.rt_dev = "lo";
		}

		if (TRUE == bSuccess && GET_MASK_ADDR (tRentry))
		{
			__u32 lNetMask = ~ntohl (GET_MASK_ADDR (tRentry));

			if ((tRentry.rt_flags & RTF_HOST) && lNetMask != 0xffffffff)
			{
				TRACE(("ERROR: Netmask %.8x doesn't make sense with host route", lNetMask));
				bSuccess = FALSE;
			}
			else if (lNetMask & (lNetMask + 1))
			{
				TRACE(("ERROR: Bogus netmask %s", strNetMask));
				bSuccess = FALSE;
			}
			lNetMask = ((struct sockaddr_in *) &tRentry.rt_dst)->sin_addr.s_addr;

			if (lNetMask & ~GET_MASK_ADDR (tRentry))
			{
				TRACE(("ERROR: Netmask doesn't match route address"));
				bSuccess = FALSE;
			}
		}

		if (TRUE == bSuccess)
		{
			int lSock = -1;

			if (-1 == (lSock = socket (AF_INET, SOCK_DGRAM, 0)))
			{
				TRACE(("ERROR: Failed to create Socket. %s", strerror(errno)));
				bSuccess = FALSE;
			}
			else
			{
				if (eRouteDel == eAction)
				{
					if (-1 == ioctl (lSock, SIOCDELRT, &tRentry))
					{
						TRACE(("ERROR: Failed to IOCTL to Del-Route. %s", strerror(errno)));
						bSuccess = FALSE;
					}
				}
				else if (eRouteAdd == eAction)
				{
					if (eRouteAdd == eAction && tRentry.rt_flags & RTF_HOST)
					{
						GET_MASK_ADDR (tRentry) = 0xffffffff;
					}
					if (-1 == ioctl (lSock, SIOCADDRT, &tRentry))
					{
						TRACE(("ERROR: Failed to IOCTL to Add-Route. %s", strerror(errno)));
						bSuccess = FALSE;
					}
				}
				close (lSock);
			}
		}
	}
	free (pstrInit);

	TRACE(("Add/Del Route Interface Ended. %s", bSuccess ? "SUCCESS" : "ERROR"));

	return bSuccess;
}





