
#include "drifcfg.h"
#include <arpa/inet.h>


struct AddrItemT 
{
    struct sockaddr_in 	tSockAddr;
    char 								*pstrName;
    int 								lHost;
    struct AddrItemT 		*next;
};

static struct AddrItemT  *gptAddrItem = NULL;



/*
** ============================================================================
** ============================================================================
*/
static int ResolveLocal (char *pstrName, struct sockaddr_in *ptSockAddr, int lHost)
{
	struct hostent 	*ptHostEntry	= NULL;
	struct netent 	*ptNetEntry		= NULL;

	ptSockAddr->sin_family 	= AF_INET;
	ptSockAddr->sin_port 		= 0;

	if (0 == strcmp (pstrName, "default")) 
	{
		ptSockAddr->sin_addr.s_addr = INADDR_ANY;
		return (1);
	}
	if (inet_aton (pstrName, &ptSockAddr->sin_addr)) 
	{
		return 0;
	}
	if (lHost && (ptHostEntry = gethostbyname(pstrName)) != (struct hostent *) NULL) 
	{
		memcpy ((char *) &ptSockAddr->sin_addr, 
						(char *) ptHostEntry->h_addr_list[0], 
						sizeof(struct in_addr));
		return 0;
	}
	if ((ptNetEntry = getnetbyname(pstrName)) != (struct netent *) NULL) 
	{
		ptSockAddr->sin_addr.s_addr = htonl(ptNetEntry->n_net);
		return 1;
	}
	if (lHost) 
	{
		return -1;
	}

	if ((ptHostEntry = gethostbyname(pstrName)) == (struct hostent *) NULL) 
	{
		return -1;
	}
	memcpy ((char *) &ptSockAddr->sin_addr, 
					(char *) ptHostEntry->h_addr_list[0], 
					sizeof(struct in_addr));
	
	return 0;
}



/*
** ============================================================================
** ============================================================================
*/
static int 	ReResolveLocal (
						char 								*pstrName, 
						size_t 							lLength, 
						struct sockaddr_in 	*ptSockAddr, 
			 			int 								lNumeric, 
						unsigned int 				lNetMask)
{
	struct hostent 		*ptHostEntry	= NULL;
	struct netent 		*ptNetEntry		= NULL;
	struct AddrItemT 	*ptAddrItem		= NULL;
	unsigned long 		lAddrNet			= 0;
	unsigned long 		lAddrHost			= 0;
	int 							lHost 				= 0;

	if (AF_INET != ptSockAddr->sin_family) 
	{
		errno = EAFNOSUPPORT;
		return -1;
	}
	lAddrNet = (unsigned long) ptSockAddr->sin_addr.s_addr;

	if (lAddrNet == INADDR_ANY) 
	{
		if ((lNumeric & 0x0fff) == 0)
		{
			if (lNumeric & 0x8000) 
			{
				strncpy (pstrName, "default", lLength);
			}
			else
			{
				strncpy (pstrName, "*", lLength);
			}
			return (0);
		}
	}
	if (lNumeric & 0x0fff) 
	{
		strncpy (pstrName, inet_ntoa (ptSockAddr->sin_addr), lLength);
		return (0);
	}

	if ((lAddrNet & (~lNetMask)) != 0 || (lNumeric & 0x4000))
	{
		lHost = 1;
	}
	ptAddrItem = gptAddrItem;

	while (ptAddrItem != NULL) 
	{
		if (ptAddrItem->tSockAddr.sin_addr.s_addr == lAddrNet && ptAddrItem->lHost == lHost)
		{
			strncpy (pstrName, ptAddrItem->pstrName, lLength);
			return (0);
		}
		ptAddrItem = ptAddrItem->next;
	}
	lAddrHost 			= ntohl(lAddrNet);
	ptNetEntry 			= NULL;
	ptHostEntry 		= NULL;

	if (lHost) 
	{
		ptHostEntry = gethostbyaddr ((char *) &lAddrNet, 4, AF_INET);

		if (ptHostEntry != NULL)
		{
			strncpy (pstrName, ptHostEntry->h_name, lLength);
		}
	}
	else 
	{
		ptNetEntry = getnetbyaddr (lAddrHost, AF_INET);

		if (ptNetEntry != NULL)
		{
			strncpy (pstrName, ptNetEntry->n_name, lLength);
		}
	}
	if ((ptHostEntry == NULL) && (ptNetEntry == NULL))
	{
		strncpy(pstrName, inet_ntoa(ptSockAddr->sin_addr), lLength);
	}
	ptAddrItem 						= (struct AddrItemT *) malloc (sizeof (struct AddrItemT));
	ptAddrItem->tSockAddr	= *ptSockAddr;
	ptAddrItem->next 			= gptAddrItem;
	ptAddrItem->lHost 		= lHost;
	ptAddrItem->pstrName	= (char *) malloc (strlen (pstrName) + 1);
	strcpy (ptAddrItem->pstrName, pstrName);
	gptAddrItem = ptAddrItem;

	return (0);
}




/*
** ============================================================================
** ============================================================================
*/
static char *PrintCB (unsigned char *pstr)
{
	return (inet_ntoa ((*(struct in_addr *) pstr)));
}



/*
** ============================================================================
** ============================================================================
*/
static char *SprintCB (struct sockaddr *ptSockAddr, int lNumeric)
{
	static char strBuf[128] = {0};

	if (ptSockAddr->sa_family == 0xFFFF || ptSockAddr->sa_family == 0)
	{
		return strncpy(strBuf, "[NONE SET]", sizeof (strBuf));
	}
	if (0 != 	ReResolveLocal (
						strBuf, 
						sizeof(strBuf), 
						(struct sockaddr_in *) ptSockAddr, 
						lNumeric, 
						0xffffff00))
	{
		return NULL;
	}
	return strBuf;
}



/*
** ============================================================================
** ============================================================================
*/
static int GetSocketLocal (char *pstrBuf, struct sockaddr *ptSockAddrIn)
{
	char 			*pstr 		= pstrBuf;
	char 			*pstrTmp	= NULL;
	unsigned 	lTmp			= 0;
	int 			i					= 0;
	struct sockaddr_in *ptSockAddr;

	ptSockAddr = (struct sockaddr_in *) ptSockAddrIn;
	ptSockAddr->sin_family = AF_INET;
	ptSockAddr->sin_port = 0;

	lTmp = 0;
	pstrTmp = (char *) &lTmp;

	for (i=0; i<sizeof (ptSockAddr->sin_addr.s_addr); i++) 
	{
		*pstr = toupper (*pstr);

		if ((*pstr >= 'A') && (*pstr <= 'F'))
		{
			pstrTmp[i] |= (int) (*pstr - 'A') + 10;
		}
		else if ((*pstr >= '0') && (*pstr <= '9'))
		{
			pstrTmp[i] |= (int) (*pstr - '0');
		}
		else
		{
			return -1;
		}
		pstrTmp[i] <<= 4;
		pstr++;
		*pstr = toupper (*pstr);

		if ((*pstr >= 'A') && (*pstr <= 'F'))
		{
			pstrTmp[i] |= (int) (*pstr - 'A') + 10;
		}
		else if ((*pstr >= '0') && (*pstr <= '9'))
		{
			pstrTmp[i] |= (int) (*pstr - '0');
		}
		else
		{
			return -1;
		}
		pstr++;
	}
	ptSockAddr->sin_addr.s_addr = htonl(lTmp);

	return (pstr - pstrBuf);
}




/*
** ============================================================================
** ============================================================================
*/
static int InputCB (int lType, char *pstrBuf, struct sockaddr *ptSockAddr)
{
	switch (lType) 
	{
		case 1: 	return (GetSocketLocal 	(pstrBuf, ptSockAddr));
		case 256: return (ResolveLocal 		(pstrBuf, (struct sockaddr_in *) ptSockAddr, 1));
		default: 	return (ResolveLocal 		(pstrBuf, (struct sockaddr_in *) ptSockAddr, 0));
	}
}


/*
** ============================================================================
** ============================================================================
*/
static int GetMaskCB (char *pstrAddr, struct sockaddr *ptSockAddrIn, char *pstrName)
{
	struct sockaddr_in *ptSockAddr = (struct sockaddr_in *) ptSockAddrIn;
	char 	*pstrSlash	= NULL;
	char 	*pstrEnd		= NULL;
	int 	lPrefix			= 0;

	if ((pstrSlash = strchr (pstrAddr, '/')) == NULL)
	{
		return 0;
	}
	*pstrSlash++ = '\0';
	lPrefix = strtoul (pstrSlash, &pstrEnd, 0);

	if (*pstrEnd != '\0')
	{
		return -1;
	}
	if (pstrName) 
	{
		sprintf(pstrName, "/%d", lPrefix);
	}
	ptSockAddr->sin_family 			= AF_INET;
	ptSockAddr->sin_addr.s_addr = htonl (~(0xffffffffU >> lPrefix));

	return 1;
}



/*
** ============================================================================
** ============================================================================
*/
NetDomainT inet_aftype =
{
    "inet", 
		"DARPA Internet",
		AF_INET, 
		sizeof (unsigned long),
    PrintCB, 
		SprintCB, 
		InputCB, 
		NULL,
    NULL, 
		NULL,
    GetMaskCB ,
    -1,
    NULL
};

NetDomainT *gtNetDomain[] = { &inet_aftype, NULL };



/*
** ============================================================================
** ============================================================================
*/
extern NetDomainT *GetNetDomain (const char *pstrName)
{
	NetDomainT **ppDomain = gtNetDomain;
	
	while (*ppDomain != NULL) 
	{
		if (0 == strcmp ((*ppDomain)->pstrName, pstrName))
		{
			return (*ppDomain);
		}
		ppDomain++;
	}
	return (NULL);
}



/*
** ============================================================================
** ============================================================================
*/
extern int GetNetDomainSock (int lDomain)
{
	NetDomainT **ppDomain = gtNetDomain;

	while (*ppDomain != NULL) 
	{
		if ((*ppDomain)->lDomain == lDomain) 
		{
			return (*ppDomain)->lFd;
		}
		ppDomain++;
	}
	return -1;
}







