
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <dirent.h>
#include <sys/mount.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>


#define	TRUE		1
#define	FALSE		0


typedef	long					BooleanT;
typedef	unsigned char	ByteT;

extern BooleanT DrGz 		(char *);
extern BooleanT DrIo 		(char *);
extern BooleanT DrIfcfg (char *);



/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT ExtractFloppyFS (char *pstrFile, char *pstrDev, char *pstrMp)
{
	BooleanT	bSuccess 		= FALSE;
	char			strFile[64] = {0};
	struct 		stat tStat;

	umount  (pstrMp);
	umount  (pstrDev);
	memset  (&tStat, 0, sizeof (tStat));
	memset  (strFile,0, 64);
	sprintf (strFile, "%s/%s", pstrMp, pstrFile);
				
	if (-1 == mount (pstrDev, pstrMp, "ext2", MS_RDONLY, NULL))
	{
		long lError = errno;
		fprintf(stderr,"\n");
		fprintf(stderr,"ERROR: Failed to mount: %s to %s. %s\n", pstrDev, pstrMp, strerror(lError)); 
		fprintf(stderr,"ERROR: Verify if drive contain valid Floppy Disk\n");
	}
	else if (-1 == stat (strFile, &tStat))
	{
		long lError = errno;
		fprintf(stderr,"\n");
		fprintf(stderr,"ERROR: Floppy Disk does not contain file: %s. %s\n", pstrFile, strerror(lError)); 
		fprintf(stderr,"ERROR: Verify if drive contain valid Floppy Disk\n");
	}
	else if (0 == tStat.st_size)
	{
		fprintf(stderr,"\n");
		fprintf(stderr,"ERROR: Floppy Disk contain empty file: %s\n", pstrFile); 
		fprintf(stderr,"ERROR: Verify if drive contain valid Floppy Disk\n");
	}
	else
	{
		int	lFdInput	=	-1;
		int	lFdOutput	=	-1;

		if (-1 == (lFdInput	=	open (strFile, O_RDONLY)))
		{
			long lError = errno;
			fprintf(stderr,"ERROR: Failed to open file: %s. %s\n", pstrFile, strerror(lError));
		}
		else if (-1 == (lFdOutput	=	open ("/drfstmp.gz", O_WRONLY | O_CREAT | O_TRUNC, 0666)))
		{
			long lError = errno;
			fprintf(stderr,"ERROR: Failed to create DRFSTMP file. %s\n", strerror(lError));
		}
		else
		{
			BooleanT	bContinue			= TRUE;
			long			lRead 				= 0;
			long			lWrite				= 0;
			ByteT			strBuf[32768]	= {0};

			bSuccess = TRUE;
			
			while (TRUE == bContinue)
			{
				if (-1 == (lRead = read (lFdInput, strBuf, 32768)))
				{
					long lError = errno;
					fprintf(stderr,"ERROR: Failed to read file: %s. %s\n", pstrFile, strerror(lError));
					bSuccess	= FALSE;
					bContinue	= FALSE;
				}
				else if (0 == lRead)
				{
					bSuccess	= TRUE;
					bContinue	= FALSE;
				}
				else if (32768 >= lRead)
				{
					if (-1 == (lWrite = write (lFdOutput, strBuf, lRead)))
					{
						long lError = errno;
						fprintf(stderr,"ERROR: Failed to write to DRFSTMP file. %s\n", strerror(lError));
						bSuccess	= FALSE;
						bContinue	= FALSE;
					}
					else if (lWrite != lRead)
					{
						long lError = errno;
						fprintf(stderr,"ERROR: Write short to new DRFSTMP file. %s\n", strerror(lError));
						bSuccess	= FALSE;
						bContinue	= FALSE;
					}
				}
			}
		}
		close (lFdInput);
		close (lFdOutput);

		if (TRUE == bSuccess)
		{
			if (TRUE != DrGz ("/drfstmp.gz"))
			{
				fprintf(stderr,"ERROR: Failed to decompress DRFSTMP file\n");
				bSuccess	= FALSE;
			}
			else if (TRUE != DrIo ("/drfstmp"))
			{
				fprintf(stderr,"ERROR: Failed to extract DRFSTMP file\n");
				bSuccess	= FALSE;
			}
			else
			{
				int lFdDelete = open ("/drfstmp", O_WRONLY | O_TRUNC, 0666);

				if (-1 != lFdDelete)
				{
					fsync (lFdDelete);
					close (lFdDelete);
				}
			}
		}
	}
	umount (pstrMp);
	
	return bSuccess;
}



/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT ExtractCdromFS (char *pstrFile, char *pstrDev, char *pstrMp)
{
#ifdef DR_BACKWARD_COMP

	BooleanT	bSuccess 	= FALSE;
	char			strFile[64] = {0};
	struct 		stat tStat;

	umount  (pstrMp);
	umount  (pstrDev);
	memset  (&tStat, 0, sizeof (tStat));
	memset  (strFile,0, 64);
	sprintf (strFile, "%s/fs/%s", pstrMp, pstrFile);

	if (-1 == mount (pstrDev, pstrMp, "iso9660", MS_RDONLY, NULL))
	{
		long lError = errno;
		fprintf(stderr,"\n");
		fprintf(stderr,"ERROR: Failed to mount: %s to %s. %s\n", pstrDev, pstrMp, strerror(lError)); 
		fprintf(stderr,"ERROR: Check CDROM drive for a Valid ValutDR Media\n\n");
	}
	else if (-1 == stat (strFile, &tStat))
	{
		long lError = errno;
		fprintf(stderr,"\n");
		fprintf(stderr,"ERROR: CDROM media does not contain file: %s. %s\n", pstrFile, strerror(lError)); 
		fprintf(stderr,"ERROR: Check CDROM drive for a Valid ValutDR Media\n\n");
	}
	else if (0 == tStat.st_size)
	{
		fprintf(stderr,"\n");
		fprintf(stderr,"ERROR: CDROM media contain empty file: %s\n", pstrFile); 
		fprintf(stderr,"ERROR: Check CDROM drive for a Valid ValutDR Media\n\n");
	}
	else
	{
		int	lFdInput	=	-1;
		int	lFdOutput	=	-1;

		if (-1 == (lFdInput	=	open (strFile, O_RDONLY)))
		{
			long lError = errno;
			fprintf(stderr,"ERROR: Failed to open file: %s. %s\n", pstrFile, strerror(lError));
		}
		else if (-1 == (lFdOutput	=	open ("/drfstmp.gz", O_WRONLY | O_CREAT | O_TRUNC, 0666)))
		{
			long lError = errno;
			fprintf(stderr,"ERROR: Failed to create DRFSTMP file. %s\n", strerror(lError));
		}
		else
		{
			BooleanT	bContinue			= TRUE;
			long			lRead 				= 0;
			long			lWrite				= 0;
			ByteT			strBuf[32768]	= {0};

			bSuccess = TRUE;
			
			while (TRUE == bContinue)
			{
				if (-1 == (lRead = read (lFdInput, strBuf, 32768)))
				{
					long lError = errno;
					fprintf(stderr,"ERROR: Failed to read file: %s. %s\n", pstrFile, strerror(lError));
					bSuccess	= FALSE;
					bContinue	= FALSE;
				}
				else if (0 == lRead)
				{
					bSuccess	= TRUE;
					bContinue	= FALSE;
				}
				else if (32768 >= lRead)
				{
					if (-1 == (lWrite = write (lFdOutput, strBuf, lRead)))
					{
						long lError = errno;
						fprintf(stderr,"ERROR: Failed to write to DRFSTMP file. %s\n", strerror(lError));
						bSuccess	= FALSE;
						bContinue	= FALSE;
					}
					else if (lWrite != lRead)
					{
						long lError = errno;
						fprintf(stderr,"ERROR: Write short to DRFSTMP file. %s\n", strerror(lError));
						bSuccess	= FALSE;
						bContinue	= FALSE;
					}
				}
			}
		}
		close (lFdInput);
		close (lFdOutput);

		if (TRUE == bSuccess)
		{
			if (TRUE != DrGz ("/drfstmp.gz"))
			{
				fprintf(stderr,"ERROR: Failed to decompress DRFSTMP file\n");
				bSuccess	= FALSE;
			}
			else if (TRUE != DrIo ("/drfstmp"))
			{
				fprintf(stderr,"ERROR: Failed to extract DRFSTMP file\n");
				bSuccess	= FALSE;
			}
			else
			{
				int lFdDelete = open ("/drfstmp", O_WRONLY | O_TRUNC, 0666);

				if (-1 != lFdDelete)
				{
					fsync (lFdDelete);
					close (lFdDelete);
				}
			}
		}
	}
	umount (pstrMp);
	umount (pstrDev);
	
	return bSuccess;

#else
	return TRUE;
#endif
}



/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT MediaIsCdrom (char **ppstrDev)
{
#ifdef DR_BACKWARD_COMP

	BooleanT	bSuccess 		= FALSE;
	BooleanT	bContinue 	= TRUE;
	char			*pstrChars	= "abcdefghijklmn\0";
	char			strBuf[64]	= {0};
	int				i						= 0;
	FILE			*pFd				= NULL;
	
	for (i=0; pstrChars[i] != '\0' && TRUE == bContinue; i++)
	{
		sprintf (strBuf,"/proc/ide/hd%c/media", pstrChars[i]); 
		
		pFd = fopen (strBuf, "r");

		if (NULL != pFd && NULL != fgets (strBuf, sizeof (strBuf), pFd))
		{
			if (0 == strncmp (strBuf, "cdrom", 5))
			{
				bContinue	= FALSE;
				memset  (strBuf, 0, 64);
				sprintf (strBuf, "/dev/hd%c", pstrChars[i]);
					
				umount ("/mp");
				umount (strBuf);
					
				if (-1 != mount (strBuf, "/mp", "iso9660", MS_RDONLY, NULL))
				{
					struct stat tStat;
					int					lFd		= -1;
					long				lRead	= 0;
					ByteT				strReadBuf[4];
					
					memset (&tStat, 0, sizeof (tStat));
					memset (strReadBuf, 0, 4);

					if (-1 != stat ("/mp/sig/sig.bin", &tStat) &&
							-1 != (lFd = open ("/mp/sig/sig.bin", O_RDONLY)) &&
							-1 != (lRead = read (lFd, strReadBuf, 4)) && 4 == lRead)
					{
						if (0x0a == strReadBuf[0] && 0x0f == strReadBuf[1] &&
								0x1a == strReadBuf[2] && 0x1f == strReadBuf[3])
						{
							*ppstrDev = (char *) malloc (64);
							memset 	(*ppstrDev, 0, 64);
							strncpy (*ppstrDev, strBuf, strlen (strBuf));
							bSuccess 	= TRUE;
						}
					}
					umount ("/mp");
				}
			}
		}
		if (NULL != pFd)
		{
			fclose (pFd);
		}
	}
	return bSuccess;
#else
	return TRUE;
#endif
}


/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT MediaIsFloppy ()
{
	BooleanT	bSuccess 	= FALSE;
  int       lFd       = -1;

	if (-1 != (lFd =  open ("/dev/fd0", O_RDONLY)))
	{
		ByteT strBuf[512] = {0};
		long  lRead       = 0;

		if (-1 != lseek (lFd, 0, 0) && -1 != (lRead = read (lFd, strBuf, 512)) && 512 == lRead)
		{
			if ((0x55 == strBuf[510] && 0xaa == strBuf[511]) ||
					(0xeb == strBuf[0]   && 0x90 == strBuf[2])   || 0xe9 == strBuf[0])
			{
				bSuccess  = TRUE;
			}
		}
		close (lFd);
	}
	return bSuccess;
}




/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT MediaIsUsbFloppy (char **ppstrDev)
{
	BooleanT	bSuccess 	= FALSE;
	BooleanT	bContinue	= TRUE;
	char			pstrDev[]	= "/dev/sd ";
	char			cDev			= 0x61;
	int				lRetry		= 0;
	int				lRetryCnt	= 1;
	int				i					= 0;
	int				lFd				= -1;
	
	for (i=0; i<8 && TRUE == bContinue; i++)
	{
		pstrDev[7] 	= cDev + i;
		lRetryCnt		= 1;

		(0 == i) ? (lRetry = 10) : (lRetry = 2);

		while (lRetryCnt++ <= lRetry)
		{
			sleep (2);

			if (-1 != (lFd = open (pstrDev, O_RDONLY)))
			{
				ByteT strBuf[512] = {0};
				long  lRead       = 0;

				if (-1 != lseek (lFd, 0, 0) && -1 != (lRead = read (lFd, strBuf, 512)) && 512 == lRead)
				{
					if ((0x55 == strBuf[510] && 0xaa == strBuf[511]) ||
							(0xeb == strBuf[0]   && 0x90 == strBuf[2])   || 0xe9 == strBuf[0])
					{
						bSuccess  = TRUE;
						bContinue	= FALSE;
						lRetryCnt	= lRetry + 1;

						if (NULL != (*ppstrDev = (char *) malloc (64)))
						{
							memset 	(*ppstrDev, 0, 64);
							strncpy (*ppstrDev, pstrDev, strlen (pstrDev));
						}
					}
				}
				close (lFd);
			}
		}
	}
	return bSuccess;
}




/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT ContinueLoad (char *pstrDev)
{
	BooleanT	bSuccess1 = FALSE;
	BooleanT	bSuccess2 = FALSE;
	int				i;
	int				j;

	if (NULL == pstrDev)
	{
		pstrDev = "/dev/fd0";
	}

	for (i=0; i<3 && TRUE != bSuccess1; i++)
	{
		fprintf(stderr,"Insert Floppy Disk #2 and press ENTER\n");
		getchar();
		fprintf(stderr,"Loading VaultDR Filesystem. Please wait...\n");

		if (TRUE != (bSuccess1 = ExtractFloppyFS ("drfs2", pstrDev, "mp")))
		{
			fprintf(stderr,"ERROR: Failed to extract Floppy DRFS2\n\n");
		}
		else
		{
			sleep(3);
			fprintf(stderr,"Loading VaultDR Filesystem completed.\n\n");

			for (j=0; j<3 && TRUE != bSuccess2; j++)
			{
				fprintf(stderr,"Insert Floppy Disk #3 and press ENTER\n");
				getchar();
				fprintf(stderr,"Loading VaultDR Device Drivers. Please wait...\n");

				if (TRUE != (bSuccess2 = ExtractFloppyFS ("drfs3", pstrDev, "mp")))
				{
					fprintf(stderr,"ERROR: Failed to extract Floppy DRFS3\n\n");
				}
				else
				{
					fprintf(stderr,"Loading VaultDR Device Drivers completed.\n");
				}
			}
		}
	}
	return (bSuccess1 & bSuccess2);
}




/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT ContinueLoadNcr (char *pstrDev)
{
	BooleanT	bSuccess = FALSE;
	int				i;

	if (NULL == pstrDev)
	{
		pstrDev = "/dev/fd0";
	}

	for (i=0; i<3 && TRUE != bSuccess; i++)
	{
		fprintf(stderr,"Insert Floppy Disk #2 and press ENTER\n");
		getchar();
		fprintf(stderr,"Loading VaultDR Filesystem. Please wait...\n");

		if (TRUE != (bSuccess = ExtractFloppyFS ("drfs2", pstrDev, "mp")))
		{
			fprintf(stderr,"ERROR: Failed to extract Floppy DRFS2\n\n");
		}
		else
		{
			fprintf(stderr,"Loading VaultDR Filesystem completed.\n\n");
		}
	}
	return bSuccess;
}




/*
** =======================================================================================
** =======================================================================================
*/
int main (int argc, char **argv) 
{
	BooleanT	bSuccess	= FALSE;
	long 			lError 		= -1;
	char			*pstrDev	= NULL;

	if (-1 == mount ("/dev/ram0", "/", "ext2", MS_REMOUNT | S_WRITE, NULL))
	{
		lError = errno;
		fprintf(stderr,"ERROR: Failed to remount ROOT filesystem. %s\n", strerror(lError));
	}
	else if (-1 == mount ("/proc", "/proc", "proc", MS_RDONLY | S_WRITE, NULL))
	{
		lError = errno;
		fprintf(stderr,"ERROR: Failed to mount PROC filesystem. %s\n", strerror(lError));
	}
	else if (TRUE == MediaIsCdrom (&pstrDev))
	{
		fprintf(stderr,"\rLoading VaultOS completed.\n\n");
		fprintf(stderr,"\rLoading VaultDR Filesystem. Please wait...\n");

		if (TRUE != ExtractCdromFS ("drfs2", pstrDev, "/mp"))
		{
			fprintf(stderr,"ERROR: Failed to extract Cdrom DRFS2\n");
		}
		else
		{
			fprintf(stderr,"\rLoading VaultDR Filesystem completed.\n");
			bSuccess = TRUE;
		}
		umount	("/mp");
		umount  (pstrDev);
		free 		(pstrDev);
	}
	else if (TRUE == MediaIsFloppy ())
	{
		sleep(3);
		fprintf(stderr,"\rLoading VaultOS completed.\n\n");

		if (NULL == argv[1])
		{
			bSuccess = ContinueLoad (NULL);
		}
		else if (0 == strcmp (argv[1], "ncr"))
		{
			bSuccess = ContinueLoadNcr (NULL);
		}
	}
	else if (TRUE == MediaIsUsbFloppy (&pstrDev))
	{
		fprintf(stderr,"\rLoading VaultOS completed.\n\n");

		bSuccess = ContinueLoad (pstrDev);

		free (pstrDev);
	}
	else
	{
		fprintf(stderr,"ERROR: Unkown Media. Failed to extract DRFS2\n");
	}

	if (TRUE == bSuccess)
	{
		if (TRUE != DrIfcfg ("lo 127.0.0.1 netmask 255.0.0.0 up\0"))
		{
			fprintf(stderr,"ERROR: Failed to set Loop Back Interface\n");
		}
		else if (-1 == sethostname ("x86dr", 5))
		{
			fprintf(stderr,"ERROR: Failed to set Host Name. %s\n", strerror(errno));
		}
		else
		{
			if (NULL == argv[1])
			{
				system ("/etc/rc");
			}
			else if (0 == strcmp (argv[1], "ncr"))
			{
				system ("/etc/rc_ncr");
			}
		}
	}
	else
	{
		fprintf(stderr,"ERROR: Fatal error occurred. System required reboot\n");

		while (TRUE)
		{
			sleep (5);
		}
	}
	return 0;
}


