
#include "driosys.h"
#include "drio.h"



/*
** =============================================================================
** Extern Data
** =============================================================================
*/
extern char *gpstrInBuf;
extern char *gInBuf;
extern long glInBufSize;
extern long glInSize;
extern long long gllInBytes;
extern int 	glBlockSize;
extern char *gOutBuf;
extern char *gpstrOutBuf;
extern long glOutBufSize;
extern long glOutSize;
extern long long gllOutBytes;
extern BooleanT	gbCreateDir;




/*
** =============================================================================
** =============================================================================
*/
extern void ZLOG (int lErrorNo, const char *pstrMsg, ...)
{
  va_list args;

	fflush		(stdout);
	fflush		(stderr);
  va_start	(args, pstrMsg);
  vfprintf 	(stderr, pstrMsg, args);
  va_end 		(args);

  if (0 != lErrorNo)
	{
		fprintf (stderr, ". %s", strerror (lErrorNo));
	}
  putc ('\n', stderr);
  fflush (stderr);
}



/*
** =============================================================================
** =============================================================================
*/
static void	MakeDirRecursive (
						char		*pstrDirName,
						int 		lMode,
						int 		lPMode,
						uid_t 	lOwner,
						gid_t 	lGroup)
{
  char 					*pstrDirPath	= NULL;
  int 					lOldMask 			= umask (0);
  struct stat 	tStat;

  pstrDirPath = alloca (strlen (pstrDirName) + 1);
  strcpy (pstrDirPath, pstrDirName);

  if (0 == stat (pstrDirPath, &tStat))
	{
		if (0 == S_ISDIR (tStat.st_mode))
		{
			ZLOG(0,"WARN: Path: (%s) already exists but is not a directory", pstrDirPath);
		}
		else if (-1 == chmod (pstrDirPath, lMode))
		{
			ZLOG(errno,"ERROR: Failed to chmod: (%s)", pstrDirPath);
		}
	}
	else
	{
		char 			*pstrSlash	= NULL;
		int 			lTmpMode		= 0700;
		BooleanT	bProtect		= FALSE;
		BooleanT	bContinue		= TRUE;

		struct tDirList
		{
			char 		*pstrDirNameEnd;
			struct 	tDirList *pNext;
		};
		struct tDirList *ptList 		= NULL;
		struct tDirList *ptTopList 	= NULL;

		if (((lPMode & 0300) != 0300) || (lOwner != - 1 && lGroup != - 1 && (lPMode & 0700) != 0))
		{
			lTmpMode = 0700;
			bProtect = TRUE;
		}
		else
		{
			lTmpMode = lPMode;
			bProtect = FALSE;
		}
		pstrSlash = pstrDirPath;

		while (*pstrSlash == '/') pstrSlash++;

		while (TRUE == bContinue && NULL != (pstrSlash = index (pstrSlash, '/')))
		{
			*pstrSlash = '\0';
		
			if (0 == stat (pstrDirPath, &tStat))
			{
				if (0 == S_ISDIR (tStat.st_mode))
				{
					ZLOG(0,"WARN: Path: (%s) already exists but is not a directory", pstrDirPath);
					bContinue = FALSE;
				}
			}
			else
			{
				if (-1 == mkdir (pstrDirPath, lTmpMode))
				{
					ZLOG(errno,"ERROR: Failed to mkdir: %s", pstrDirPath);
					bContinue = FALSE;
				}
				else if (TRUE == bProtect)
				{
					struct tDirList *ptNewList 	= (struct tDirList *) alloca (sizeof (struct tDirList));
					ptNewList->pstrDirNameEnd 	= pstrSlash;
					ptNewList->pNext 						= ptTopList;
					ptTopList 									= ptNewList;
				}
			}
			*pstrSlash++ = '/';

			while (*pstrSlash == '/') pstrSlash++;
		}

		if (-1 == mkdir (pstrDirPath, lMode))
		{
			ZLOG(errno,"ERROR: Failed to mkdir: %s", pstrDirPath);
		}
		else
		{
			if ((lMode & 0700) != 0 && -1 == chmod (pstrDirPath, lMode))
			{
				ZLOG(errno,"ERROR: Failed to chmod: %s", pstrDirPath);
			}
			for (ptList = ptTopList; ptList != NULL; ptList = ptList->pNext)
			{
				*(ptList->pstrDirNameEnd) = '\0';
			}
		}
	}
  umask (lOldMask);
}



/*
** =============================================================================
** =============================================================================
*/
static char *GetDirectoryName (char *pstrPath)
{
  char	*pstrSlash 		= NULL;
  char	*pstrNewPath	= NULL;
  int		lLength				= 0;

  pstrSlash = (char *) strrchr (pstrPath, '/');

  if (NULL == pstrSlash)
	{
		pstrPath 	= ".";
		lLength 	= 1;
	}
  else
	{
		while (pstrSlash > pstrPath && *pstrSlash == '/')
		{
			--pstrSlash;
		}
		lLength = pstrSlash - pstrPath + 1;
	}
  if (NULL != (pstrNewPath = (char *) malloc (lLength + 1)))
	{
		strncpy (pstrNewPath, pstrPath, lLength);
		pstrNewPath [lLength] = '\0';
	}
  return pstrNewPath;
}




/*
** ===========================================================================
** ===========================================================================
*/
extern void CreateDirs (char *pstrPath)
{
  char	*pstrDirName	= NULL;
  int		lMode	= 0700;

  if (NULL == (pstrDirName = GetDirectoryName (pstrPath)))
	{
		ZLOG(0,"ERROR: Out of memory");
	}
	else
	{
		if (pstrDirName[0] != '.' || pstrDirName[1] != '\0')
		{
			MakeDirRecursive (pstrDirName, lMode, 0700, -1, -1);
		}
		free (pstrDirName);
	}
}





/*
** ===========================================================================
** ===========================================================================
*/
static void FillBuffer (int lFd, int lNumBytes)
{
  gpstrInBuf 	= gInBuf;
  lNumBytes 	= (lNumBytes < glBlockSize) ? lNumBytes : glBlockSize;
  glInSize 		= read (lFd, gInBuf, lNumBytes);

  if (glInSize < 0)	 ZLOG (errno,"read error");
  if (glInSize == 0) ZLOG (0, "premature end of file");

  gllInBytes += glInSize;
}



/*
** ===========================================================================
** ===========================================================================
*/
extern void ReadBuffer (char *pstrBuf, int lFd, long lInBytesRead)
{
  register long lBytesRead = lInBytesRead;
  register long lBytesLeft;

  while (lBytesRead > 0)
	{
		if (glInSize == 0)
		{
			FillBuffer (lFd, glBlockSize);
		}
		if (lBytesRead < glInSize)
		{
			lBytesLeft = lBytesRead;
		}
		else
		{
			lBytesLeft = glInSize;
		}
		bcopy (gpstrInBuf, pstrBuf, (unsigned) lBytesLeft);

		gpstrInBuf 	+= lBytesLeft;
		pstrBuf 		+= lBytesLeft;
		glInSize 		-= lBytesLeft;
		lBytesRead 	-= lBytesLeft;
	}
}





/*
** ===========================================================================
** ===========================================================================
*/
extern void BypassBuffer (int lFd, long lInNumBytes)
{
  register long lNumBytes = lInNumBytes;
  register long lBytesLeft;

  while (lNumBytes > 0)
	{
		if (glInSize == 0)
		{
			FillBuffer (lFd, glBlockSize);
		}
		if (lNumBytes < glInSize)
		{
			lBytesLeft = lNumBytes;
		}
		else
		{
			lBytesLeft = glInSize;
		}
		gpstrInBuf 	+= lBytesLeft;
		glInSize 		-= lBytesLeft;
		lNumBytes 	-= lBytesLeft;
	}
}




/*
** ===========================================================================
** ===========================================================================
*/
extern void FlushBuffer (int lFdOut)
{
  int lWrite = write (lFdOut, gOutBuf, glOutSize);

  if (lWrite != glOutSize)
	{
		ZLOG(errno,"write error");
	}
  gllOutBytes 	+= glOutSize;
  gpstrOutBuf 	= gOutBuf;
  glOutSize 		= 0;
}




/*
** ===========================================================================
** ===========================================================================
*/
static void WrittingBuffer (char *pstrBuf, int lFdOut, long lNumBytes)
{
  register long lBytesLeft 	= lNumBytes;
  register long lSpaceLeft	= 0;

  while (lBytesLeft > 0)
	{
		lSpaceLeft = DISK_IO_BLOCK_SIZE - glOutSize;

		if (lSpaceLeft == 0)
		{
			FlushBuffer (lFdOut);
		}
		else
		{
			if (lBytesLeft < lSpaceLeft) 
			{
				lSpaceLeft = lBytesLeft;
			}
			bcopy (pstrBuf, gpstrOutBuf, (unsigned) lSpaceLeft);

			gpstrOutBuf 	+= lSpaceLeft;
			glOutSize 		+= lSpaceLeft;
			pstrBuf 			+= lSpaceLeft;
			lBytesLeft 		-= lSpaceLeft;
		}
	}
}





/*
** ===========================================================================
** ===========================================================================
*/
extern void WriteToBuffer (int lFd, int lFdOut, long lNumBytes)
{
  long lSize = 0;

  while (lNumBytes > 0)
	{
		if (glInSize == 0)
		{
			FillBuffer (lFd, glBlockSize);
		}
		lSize = (glInSize < lNumBytes) ? glInSize : lNumBytes;

		WrittingBuffer (gpstrInBuf, lFdOut, lSize);

		lNumBytes 	-= lSize;
		glInSize 		-= lSize;
		gpstrInBuf 	+= lSize;
	}
}




/*
** ===========================================================================
** ===========================================================================
*/
typedef struct
{
  unsigned long 	lInode;
  unsigned long 	lMajor;
  unsigned long 	lMinor;
  char 						*pstrFileName;
} InodeT;

static InodeT **ppHash 	= NULL;
static int lHashSize 		= 22;
static int lHashNum			= 0;



/*
** ===========================================================================
** ===========================================================================
*/
static void HashInsert (InodeT *ptInode)
{
  int lStart 	= ptInode->lInode % lHashSize;
  int lTmp		= 0;

  if (NULL == ppHash [lStart])
	{
		ppHash [lStart] = ptInode;
	}
	else
	{
		lTmp = (lStart + 1) % lHashSize;

		while (ppHash[lTmp] != NULL) 
		{
			lTmp = (lTmp + 1) % lHashSize;
		}
		ppHash[lTmp] = ptInode;
	}
}



/*
** ===========================================================================
** ===========================================================================
*/
static char	*FindInode (
						unsigned long lInode,
						unsigned long lMajor,
						unsigned long lMinor)
{
  int lStart	= 0;
  int lTmp		= 0;

  if (NULL != ppHash)
	{
		lStart = lInode % lHashSize;

		if (NULL == ppHash [lStart])
		{
			return NULL;
		}
		if (ppHash[lStart]->lInode == lInode && 
				ppHash[lStart]->lMajor == lMajor && 
				ppHash[lStart]->lMinor == lMinor)
		{
			return ppHash[lStart]->pstrFileName;
		}

		for (lTmp = (lStart + 1) % lHashSize; 
				ppHash[lTmp] != NULL && lTmp != lStart; 
				lTmp = (lTmp + 1) % lHashSize)
		{
			if (ppHash[lTmp]->lInode 		== lInode && 
					ppHash[lStart]->lMajor 	== lMajor && 
					ppHash[lStart]->lMinor 	== lMinor)
			{
				return ppHash[lTmp]->pstrFileName;
			}
		}
	}
  return NULL;
}



/*
** ===========================================================================
** ===========================================================================
*/
static void	AddInode (
						char 					*pstrFileName,
						unsigned long lInode,
						unsigned long lMajor,
						unsigned long lMinor)
{
  InodeT *ptInode;

  ptInode 							= (InodeT *) malloc (sizeof (InodeT));
  ptInode->lInode 			= lInode;
  ptInode->lMajor 			= lMajor;
  ptInode->lMinor 			= lMinor;
  ptInode->pstrFileName = (char *) strdup (pstrFileName);

  if (lHashNum == lHashSize || NULL == ppHash)
	{
		InodeT 	**ppOldInode;
		int 		i;
		ppOldInode = ppHash;

		if (NULL == ppOldInode) 
		{
			lHashNum = 0;
		}
		lHashSize = 2 * lHashSize + 3;
		ppHash 		= (InodeT **) malloc (lHashSize * sizeof (InodeT *));
		bzero (ppHash, lHashSize * sizeof (InodeT *));

		for (i = 0; i < lHashNum; i++) 
		{
			HashInsert (ppOldInode[i]);
		}
		if (NULL != ppOldInode) 
		{
			free (ppOldInode);
		}
	}
  HashInsert (ptInode);
  lHashNum++;
}




/*
** =============================================================================
** =============================================================================
*/
extern int CreateLink (char *pstrSource, char *pstrTarget)
{
  int lRC = link (pstrTarget, pstrSource);

  if (-1 == lRC && TRUE == gbCreateDir)
	{
		CreateDirs (pstrSource);
		lRC = link (pstrTarget, pstrSource);
	}
  return lRC;
}



/*
** =============================================================================
** =============================================================================
*/
extern int CreateLinkSpecial (char *pstrFile, int lMajor, int lMinor, int lInode)
{
  int		lRC						= -1;
  char 	*pstrLinkName = FindInode  (lInode, lMajor, lMinor);

  if (NULL == pstrLinkName)
	{
    AddInode (pstrFile, lInode, lMajor, lMinor);
	}
  else
	{
    lRC = CreateLink (pstrFile, pstrLinkName);
	}
  return lRC;
}

