
#include "driosys.h"
#include "drio.h"

/*
** ====================================================================================
** Extern Data
** ====================================================================================
*/
extern void (*gMainFunc) ();

extern int glFd;
extern int glBlockSize;
extern enum AFormatT  geFormat;

extern char *gInBuf;
extern char *gpstrInBuf;
extern long glInBufSize;
extern long glInSize;
extern long long gllInBytes;

extern char *gOutBuf;
extern char *gpstrOutBuf;
extern long glOutBufSize;
extern long glOutSize;
extern long long gllOutBytes;

extern BooleanT gbOverWriteExist;
extern BooleanT gbCreateDir;
extern BooleanT gbRetainTime;

extern void ZLOG              (int, const char *, ...);
extern int  CreateLinkSpecial (char *, int, int, int);
extern int  CreateLink        (char *, char *);
extern void BypassBuffer      (int, long);
extern void ReadBuffer        (char *, int, long);
extern void MainFuncExtract   ();
extern void CreateDirs        (char *);
extern void FlushBuffer       (int);
extern void WriteToBuffer     (int, int, long);


static struct tDelay *gptTopDelay = NULL;



/*
** ====================================================================================
** ====================================================================================
*/
static void DelayFree (struct tDelay *ptDelay)
{
  free (ptDelay->tHdr.c_name);
  free (ptDelay);
}



/*
** ====================================================================================
** ====================================================================================
*/
static struct tDelay *DelayCreate (zcpHeaderT *ptHdr)
{
  struct tDelay *ptDelay = NULL;

	if (NULL != (ptDelay = (struct tDelay *) malloc (sizeof (struct tDelay))))
	{
		ptDelay->tHdr = *ptHdr;
		ptDelay->tHdr.c_name = (char *) malloc (strlen (ptHdr->c_name) + 1);
		strcpy (ptDelay->tHdr.c_name, ptHdr->c_name);
	}
  return ptDelay;
}



/*
** ====================================================================================
** ====================================================================================
*/
static void DeplayPut (zcpHeaderT *ptHdr)
{
  struct tDelay *ptDelay = NULL;

  if (NULL != (ptDelay = DelayCreate (ptHdr)))
	{
		ptDelay->pNext 	= gptTopDelay;
		gptTopDelay 		= ptDelay;
	}
}




/*
** ====================================================================================
** ====================================================================================
*/
static void DeplayCreateLink (zcpHeaderT *ptHdr)
{
  struct 	tDelay *ptDelay 		= gptTopDelay;
  struct 	tDelay *ptDelayPre	= NULL;
  int			lInode							= ptHdr->c_ino;
  int			lMajor 							= ptHdr->c_dev_maj;
  int			lMinor							= ptHdr->c_dev_min;

  while (ptDelay != NULL)
	{
		if (lInode == ptDelay->tHdr.c_ino && 
				lMajor == ptDelay->tHdr.c_dev_maj && 
				lMinor == ptDelay->tHdr.c_dev_min)
		{
			struct tDelay *ptDelayFree = NULL;

			if (-1 == CreateLink (ptDelay->tHdr.c_name, ptHdr->c_name))
			{
				ZLOG(errno,"ERROR: Failed to link %s to %s", ptDelay->tHdr.c_name, ptHdr->c_name);
			}
			if (ptDelayPre != NULL)
			{
				ptDelayPre->pNext = ptDelay->pNext;
			}
			else
			{
				gptTopDelay = ptDelay->pNext;
			}
			ptDelayFree = ptDelay;
			ptDelay 		= ptDelay->pNext;
			DelayFree 	(ptDelayFree);
		}
		else
		{
			ptDelayPre 	= ptDelay;
			ptDelay 		= ptDelay->pNext;
		}
	}
}



/*
** ====================================================================================
** ====================================================================================
*/
static void DelayCompleteLink ()
{
  struct utimbuf tTimeBuf;
  struct tDelay 	*ptDelay	= NULL;
  int							lFdOut		= -1;

  bzero (&tTimeBuf, sizeof (struct utimbuf));
  
  for (ptDelay = gptTopDelay; ptDelay != NULL; ptDelay = ptDelay->pNext)
	{
		ptDelay = gptTopDelay;

		if (0 == 	CreateLinkSpecial (
							ptDelay->tHdr.c_name,
							ptDelay->tHdr.c_dev_maj, 
							ptDelay->tHdr.c_dev_min,
							ptDelay->tHdr.c_ino)) 
		{
			continue;
		}
		lFdOut = open (ptDelay->tHdr.c_name, O_CREAT | O_WRONLY | O_BINARY, 0600);

		if (-1 == lFdOut && TRUE == gbCreateDir)
		{
			CreateDirs (ptDelay->tHdr.c_name);
			lFdOut = open (ptDelay->tHdr.c_name, O_CREAT | O_WRONLY | O_BINARY, 0600);
		}
		if (-1 == lFdOut)
		{
			ZLOG(errno,"%s", ptDelay->tHdr.c_name);
			continue;
		}
		if (-1 == close (lFdOut))
		{
			ZLOG(errno,"%s", ptDelay->tHdr.c_name);
		}
		if (-1 == chown (ptDelay->tHdr.c_name, ptDelay->tHdr.c_uid, ptDelay->tHdr.c_gid) && errno != EPERM)
		{
			ZLOG(errno,"%s", ptDelay->tHdr.c_name);
		}
		if (-1 == chmod (ptDelay->tHdr.c_name, (int) ptDelay->tHdr.c_mode))
		{
			ZLOG(errno,"%s", ptDelay->tHdr.c_name);
		}
		if (TRUE == gbRetainTime)
		{
			tTimeBuf.actime = tTimeBuf.modtime = ptDelay->tHdr.c_mtime;

			if (-1 == utime (ptDelay->tHdr.c_name, &tTimeBuf))
			{
				ZLOG(errno,"%s", ptDelay->tHdr.c_name);
			}
		}
	}
}



/*
** ====================================================================================
** ====================================================================================
*/
static void StripSlashR (char *pstrPath)
{
  int lLength	= strlen (pstrPath) - 1;

  while (lLength > 0 && pstrPath[lLength] == '/')
	{
    pstrPath[lLength--] = '\0';
	}
}



/*
** ====================================================================================
** ====================================================================================
*/
static void SkipPadding (int lFd, int lNumBytes)
{
  int lPad = 0;

  geFormat == arf_newascii ? (lPad = (4 - (lNumBytes % 4)) % 4) : (lPad = 0);
  
	if (lPad != 0)
	{
    BypassBuffer (lFd, lPad);
	}
}



/*
** ====================================================================================
** ===================================================================================
*/
static void ExtractHdr (zcpHeaderT *ptHdr, int lFd)
{
  char strHdr [112] = {0};

  ReadBuffer (strHdr , lFd, 104L);
  strHdr [104] = '\0';

  sscanf (strHdr , "%8lx%8lx%8lx%8lx%8lx%8lx%8lx%8lx%8lx%8lx%8lx%8lx%8lx",
					&ptHdr->c_ino, 
					&ptHdr->c_mode, 
					&ptHdr->c_uid,
					&ptHdr->c_gid, 
					&ptHdr->c_nlink, 
					&ptHdr->c_mtime,
					&ptHdr->c_filesize, 
					&ptHdr->c_dev_maj, 
					&ptHdr->c_dev_min,
					&ptHdr->c_rdev_maj, 
					&ptHdr->c_rdev_min, 
					&ptHdr->c_namesize,
					&ptHdr->c_chksum);

  if (ptHdr->c_name != NULL)
	{
    free (ptHdr->c_name);
	}
  ptHdr->c_name = (char *) malloc (ptHdr->c_namesize);

  ReadBuffer  (ptHdr->c_name, lFd, (long) ptHdr->c_namesize);
  SkipPadding (lFd, ptHdr->c_namesize + 110);
}
																										


/*
** ====================================================================================
** ====================================================================================
*/
static void ReadHdr (zcpHeaderT *ptHdr, int lFd)
{
	BooleanT	bContinue		= TRUE;
  long 			lBytesSkip 	= 0;

  ptHdr->c_tar_linkname = NULL;

  ReadBuffer ((char *) ptHdr, lFd, 6L);

  while (TRUE == bContinue)
	{
		if (geFormat == arf_newascii && 0 == strncmp ((char *) ptHdr, "070701", 6))
		{
			if (lBytesSkip > 0)
			{
				ZLOG(0,"WARN: skipped %ld bytes of junk", lBytesSkip);
			}
			ExtractHdr (ptHdr, lFd);
			bContinue = FALSE;
		}
		else
		{
			lBytesSkip++;
			bcopy ((char *) ptHdr + 1, (char *) ptHdr, 5);
			ReadBuffer ((char *) ptHdr + 5, lFd, 1L);
		}
	}
}



/*
** ====================================================================================
** ====================================================================================
*/
extern void MainFuncExtract ()
{
	BooleanT				bContinue = TRUE;
  BooleanT				bDirExist	= FALSE;
  BooleanT				bByPass		= FALSE;
  int 						lFdOut		= -1;
  char 						*pstrLink	= NULL;
  zcpHeaderT 			tHdr;
  struct stat 		tStat;
  struct utimbuf 	tTimeBuf;

  bzero (&tTimeBuf, sizeof (struct utimbuf));

  tHdr.c_name = NULL;

  while (TRUE == bContinue)
	{
		pstrLink 	= NULL;
		bDirExist = FALSE;
		bByPass 	= FALSE;
		
		ReadHdr (&tHdr, glFd);

		if (0 == strcmp ("TRAILER!!!", tHdr.c_name))
		{
			bContinue = FALSE;
		}
		else
		{
			if (0 == lstat (tHdr.c_name, &tStat))
			{
				if (S_ISDIR (tStat.st_mode) && ((tHdr.c_mode & CP_IFMT) == CP_IFDIR))
				{
					bDirExist = TRUE;
				}
				else if (TRUE != gbOverWriteExist && tHdr.c_mtime <= tStat.st_mtime)
				{
					ZLOG(0,"%s not created: newer or same age version exists", tHdr.c_name);
					BypassBuffer (glFd, tHdr.c_filesize);
					SkipPadding  (glFd, tHdr.c_filesize);
					bByPass = TRUE;
				}
				else if (S_ISDIR (tStat.st_mode) ? rmdir (tHdr.c_name) : unlink (tHdr.c_name))
				{
					ZLOG(errno,"cannot remove current %s", tHdr.c_name);
					BypassBuffer (glFd, tHdr.c_filesize);
					SkipPadding  (glFd, tHdr.c_filesize);
					bByPass = TRUE;
				}
			}

			if (TRUE != bByPass)
			{
				switch (tHdr.c_mode & CP_IFMT)
				{
					case CP_IFREG:
					{
						if (tHdr.c_nlink > 1 && arf_newascii == geFormat)
						{
							if (tHdr.c_filesize == 0)
							{
								DeplayPut			(&tHdr);
								BypassBuffer	(glFd, tHdr.c_filesize);
								SkipPadding		(glFd, tHdr.c_filesize);
								break;
							}
							if (0 == CreateLinkSpecial (tHdr.c_name, tHdr.c_dev_maj, tHdr.c_dev_min, tHdr.c_ino))
							{
								BypassBuffer (glFd, tHdr.c_filesize);
								SkipPadding  (glFd, tHdr.c_filesize);
								break;
							}
						}

						if (pstrLink == NULL)
						{
							lFdOut = open (tHdr.c_name, O_CREAT | O_WRONLY | O_BINARY, 0600);

							if (-1 == lFdOut && TRUE == gbCreateDir)
							{
								CreateDirs (tHdr.c_name);
								lFdOut = open (tHdr.c_name, O_CREAT | O_WRONLY | O_BINARY, 0600);
							}
							if (-1 == lFdOut)
							{
								ZLOG(errno,"%s", tHdr.c_name);
								BypassBuffer (glFd, tHdr.c_filesize);
								SkipPadding  (glFd, tHdr.c_filesize);
							}
							else
							{
								WriteToBuffer	(glFd, lFdOut, tHdr.c_filesize);
								FlushBuffer		(lFdOut);

								if (-1 == close (lFdOut))
								{
									ZLOG(errno,"%s", tHdr.c_name);
								}
								if (-1 == chown (tHdr.c_name, tHdr.c_uid, tHdr.c_gid) && errno != EPERM)
								{
									ZLOG(errno,"%s", tHdr.c_name);
								}
								if (-1 == chmod (tHdr.c_name, (int) tHdr.c_mode))
								{
									ZLOG(errno, "%s", tHdr.c_name);
								}
								if (TRUE == gbRetainTime)
								{
									tTimeBuf.actime = tTimeBuf.modtime = tHdr.c_mtime;

									if (-1 == utime (tHdr.c_name, &tTimeBuf))
									{
										ZLOG(errno, "%s", tHdr.c_name);
									}
								}
								SkipPadding (glFd, tHdr.c_filesize);

								if (tHdr.c_nlink > 1 && geFormat == arf_newascii)
								{
									DeplayCreateLink (&tHdr);
								}
							}
						}
						break;
					}

					case CP_IFDIR:
					{
						long	lRC = 0;

						StripSlashR (tHdr.c_name);

						if (tHdr.c_name[0] == '.' && tHdr.c_name[1] == '\0') 
						{
							break;
						}
						TRUE == bDirExist ? (lRC = 0) : (lRC = mkdir (tHdr.c_name, tHdr.c_mode));

						if (-1 == lRC && TRUE == gbCreateDir)
						{
							CreateDirs (tHdr.c_name);
							lRC = mkdir (tHdr.c_name, tHdr.c_mode);
						}
						if (-1 == lRC && (errno != EEXIST || 0 != lstat (tHdr.c_name, &tStat) || 0 == S_ISDIR (tStat.st_mode)))
						{
							ZLOG(errno,"%s", tHdr.c_name);
							continue;
						}
						else
						{
							if (-1 == chown (tHdr.c_name, tHdr.c_uid, tHdr.c_gid) && errno != EPERM)
							{
								ZLOG(errno,"%s", tHdr.c_name);
							}
							if (-1 == chmod (tHdr.c_name, (int) tHdr.c_mode))
							{
								ZLOG(errno,"%s", tHdr.c_name);
							}
							if (TRUE == gbRetainTime)
							{
								tTimeBuf.actime = tTimeBuf.modtime = tHdr.c_mtime;

								if (-1 == utime (tHdr.c_name, &tTimeBuf))
								{
									ZLOG(errno,"%s", tHdr.c_name);
								}
							}
						}
						break;
					}

					case CP_IFCHR:
					case CP_IFBLK:
					case CP_IFSOCK:
					case CP_IFIFO:
					{
						long	lRC = 0;

						if (tHdr.c_nlink > 1 && geFormat == arf_newascii)
						{
							if (0 == CreateLinkSpecial (tHdr.c_name, tHdr.c_dev_maj, tHdr.c_dev_min, tHdr.c_ino))
							{
								break;
							}
						}
						lRC = mknod (tHdr.c_name, tHdr.c_mode, makedev (tHdr.c_rdev_maj, tHdr.c_rdev_min));

						if (-1 == lRC && TRUE == gbCreateDir)
						{
							CreateDirs (tHdr.c_name);
							lRC = mknod (tHdr.c_name, tHdr.c_mode, makedev (tHdr.c_rdev_maj, tHdr.c_rdev_min));
						}
						if (-1 == lRC)
						{
							ZLOG(errno,"%s", tHdr.c_name);
						}
						else
						{
							if (-1 == chown (tHdr.c_name, tHdr.c_uid, tHdr.c_gid) && errno != EPERM)
							{
								ZLOG(errno,"%s", tHdr.c_name);
							}
							if (-1 == chmod (tHdr.c_name, tHdr.c_mode))
							{
								ZLOG(errno,"%s", tHdr.c_name);
							}
							if (TRUE == gbRetainTime)
							{
								tTimeBuf.actime = tTimeBuf.modtime = tHdr.c_mtime;

								if (-1 == utime (tHdr.c_name, &tTimeBuf))
								{
									ZLOG(errno,"%s", tHdr.c_name);
								}
							}
						}
						break;
					}

					case CP_IFLNK:
					{
						long	lRC = 0;

						if (geFormat == arf_newascii)
						{
							pstrLink = (char *) malloc ((unsigned int) tHdr.c_filesize + 1);
							pstrLink[tHdr.c_filesize] = '\0';
							ReadBuffer (pstrLink, glFd, tHdr.c_filesize);
							SkipPadding (glFd, tHdr.c_filesize);
						}
						else
						{
							pstrLink = (char *) strdup (tHdr.c_tar_linkname);
						}
						lRC = UMASKED_SYMLINK (pstrLink, tHdr.c_name, tHdr.c_mode);

						if (-1 == lRC && TRUE == gbCreateDir)
						{
							CreateDirs (tHdr.c_name);
							lRC = UMASKED_SYMLINK (pstrLink, tHdr.c_name, tHdr.c_mode);
						}
						if (-1 == lRC)
						{
							ZLOG(errno,"%s", tHdr.c_name);
							free (pstrLink);
							pstrLink = NULL;
						}
						else
						{
							if (-1 == lchown (tHdr.c_name, tHdr.c_uid, tHdr.c_gid) && errno != EPERM)
							{
								ZLOG(errno,"%s", tHdr.c_name);
							}
							free (pstrLink);
							pstrLink = NULL;
						}
						break;
					}

					default:
						ZLOG(0,"%s: unknown file type", tHdr.c_name);
						BypassBuffer (glFd, tHdr.c_filesize);
						SkipPadding (glFd, tHdr.c_filesize);
				}
			}
		}
	}
  if (geFormat == arf_newascii)
	{
    DelayCompleteLink ();
	}
}

