static const char drmod_c[] = "$Header: /faststuff/CVS/dastuff190503/vaultos/unix/linux/x86/config/drmod.c,v 1.7 2007/01/16 17:59:42 sreenivasa Exp $" ;

#include <sys/types.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <alloca.h>
#include <limits.h>
#include <ctype.h>
#include <errno.h>
#include <stddef.h>
#include <stdarg.h>
#include <getopt.h>
#include <sys/stat.h>
#include <sys/file.h>
#include <sys/utsname.h>


#include "drmod.h"
#include "drmodobj.h"

MODULE("DRMOD")


#define STRVERSIONLEN	32


extern int	init_module 	(const char *, DrModT *);
extern int	query_module	(const char *, int, void *, size_t, size_t *);
extern long	create_module	(const char *, size_t);
extern int	delete_module	(const char *);



static int 				glHasErr			= 0;
static BooleanT		bHasInfo			= FALSE;
static BooleanT		bSysCall 			= FALSE;
static size_t 		glNumModStat	= 0;
static size_t			glNumKsym			= 0;
static int 				glModUse;
static ModStatT 	*gptModStat		= NULL;
static ModSymT 		*gptKsym			= NULL;
static char 			*gpstrModBuf	= NULL;



/*
** =======================================================================
** =======================================================================
*/
extern void eLog (const char *pstrFormat,...)
{
	char		strBuf[256];
	va_list args;

	memset		(strBuf, 0, sizeof (strBuf));
	strncpy		(strBuf, "ERROR: ", 7);
	va_start	(args, pstrFormat);
	vsprintf	(strBuf + 7, pstrFormat, args);
	va_end		(args);
	TRACE((strBuf));
	glHasErr++;
}


/*
** =======================================================================
** =======================================================================
*/
extern void iLog (const char *pstrFormat,...)
{
	char		strBuf[256];
	va_list args;

	memset		(strBuf, 0, sizeof (strBuf));
	va_start	(args, pstrFormat);
	vsprintf	(strBuf, pstrFormat, args);
	va_end		(args);
	TRACE((strBuf));
}




/*
** =======================================================================
** =======================================================================
*/
static ObjFileT *ObjLoad (int lFd, Elf32_Half lElfType, const char *pstrModFile)
{
  ObjFileT 		*ptObjFile;
  ElfW(Shdr)	*ptSecHdr;
  int					i						= 0;
  int					lShNum			= 0;
  long				lHdrSize		= 0;
  long				lShSize			= 0;
  char				*pstrShTab	= NULL;

  ptObjFile = ArNewFile ();

  memset (ptObjFile, 0, sizeof (*ptObjFile));
  ptObjFile->symbol_cmp		= strcmp;
  ptObjFile->symbol_hash	= ObjHash;
  ptObjFile->ppStart 			= &ptObjFile->load_order;
	lHdrSize								= sizeof (ptObjFile->header);

  lseek (lFd, 0, SEEK_SET);

  if (lHdrSize != read (lFd, &ptObjFile->header, lHdrSize))
	{
		eLog("Failed to read ELF header from: %s", pstrModFile);
		return NULL;
	}

  if (ptObjFile->header.e_ident[EI_MAG0] != ELFMAG0 || 
			ptObjFile->header.e_ident[EI_MAG1] != ELFMAG1 || 
			ptObjFile->header.e_ident[EI_MAG2] != ELFMAG2 || 
			ptObjFile->header.e_ident[EI_MAG3] != ELFMAG3)
	{
		eLog("Mod file is not ELF: %s",pstrModFile);
		return NULL;
	}

  if (ptObjFile->header.e_ident[EI_CLASS] != ELFCLASS32 || 
			ptObjFile->header.e_ident[EI_DATA] != ELFDATA2LSB || 
			ptObjFile->header.e_ident[EI_VERSION] != EV_CURRENT || 
			! MATCH_MACHINE (ptObjFile->header.e_machine))
	{
		eLog("ELF Mod file is not for this architecture: %s", pstrModFile);
		return NULL;
	}

  if (ptObjFile->header.e_type != lElfType && lElfType != ET_NONE)
	{
		eLog("ELF file not a Exe/Reloc. Expecting: %d Got: %d File: %s",
		lElfType, ptObjFile->header.e_type, pstrModFile);
		return NULL;
	}

  if (ptObjFile->header.e_shentsize != sizeof (ElfW (Shdr)))
	{
		eLog("SecHdr size mismatch: %lu != %lu, File: %s", 
		(ulong) ptObjFile->header.e_shentsize, 
		(ulong) sizeof (ElfW (Shdr)),
		pstrModFile);
		return NULL;
	}
  lShNum = ptObjFile->header.e_shnum;
  ptObjFile->sections = malloc (sizeof (ObjSecT *) * lShNum);

  memset (ptObjFile->sections, 0, sizeof (ObjSecT *) * lShNum);
  ptSecHdr	= alloca (sizeof (ElfW (Shdr)) * lShNum);
  lShSize		= sizeof (ElfW (Shdr)) * lShNum;

  lseek (lFd, ptObjFile->header.e_shoff, SEEK_SET);

  if (lShSize != read (lFd, ptSecHdr, lShSize))
	{
		eLog ("Failed to read ELF SecHdr: %s",pstrModFile);
		return NULL;
	}

  for (i=0; i<lShNum; ++i)
	{
		ObjSecT *ptObjSec;

		ptObjFile->sections[i] = ptObjSec = ArNewSec ();

		memset (ptObjSec, 0, sizeof (*ptObjSec));
		ptObjSec->header 	= ptSecHdr[i];
		ptObjSec->idx 		= i;

		switch (ptObjSec->header.sh_type)
		{
			case SHT_NULL:
			case SHT_NOTE:
			case SHT_NOBITS:
			{
				break;
			}

			case SHT_PROGBITS:
			case SHT_SYMTAB:
			case SHT_STRTAB:
			case SHT_REL:
			{
				if (ptObjSec->header.sh_size > 0)
				{
					long	lSize = 0;

					ptObjSec->contents = malloc (ptObjSec->header.sh_size);
					lseek (lFd, ptObjSec->header.sh_offset, SEEK_SET);
					lSize = ptObjSec->header.sh_size;

					if (lSize != read (lFd, ptObjSec->contents, lSize))
					{
						eLog("Failed to read ELF SecData: %s", pstrModFile);
						return NULL;
					}
				}
				else
				{
					ptObjSec->contents = NULL;
				}
				break;
			}

			case SHT_RELA:
			{
				if (ptObjSec->header.sh_size)
				{
					eLog("Relocations not supported on this arch: %s",pstrModFile);
					return NULL;
				}
				break;
			}

			default:
			{
				if (ptObjSec->header.sh_type >= SHT_LOPROC)
				{
					ptObjSec->contents = 0;
					break;
				}
				eLog("Unkown Sec Type. File: %s",pstrModFile);
				return NULL;
			}
		}
	}
  pstrShTab = ptObjFile->sections[ptObjFile->header.e_shstrndx]->contents;

  for (i=0; i<lShNum; ++i)
	{
		ObjSecT *ptObjSec = ptObjFile->sections[i];

		ptObjSec->name = pstrShTab + ptObjSec->header.sh_name;
	}

  for (i=0; i<lShNum; ++i)
	{
		ObjSecT *ptObjSec = ptObjFile->sections[i];

		if (0 == strcmp (ptObjSec->name, ".modinfo") || 0 == strcmp (ptObjSec->name, ".modstring"))
		{
			ptObjSec->header.sh_flags &= ~SHF_ALLOC;
		}

		if (ptObjSec->header.sh_flags & SHF_ALLOC)
		{
			ObjInsertSec (ptObjFile, ptObjSec);
		}

		if (SHT_SYMTAB == ptObjSec->header.sh_type)
		{
			ulong 			j 				= 0;
			ulong 			lNumSym		= 0;
			char 				*pstrTab	= NULL;
			ElfW(Sym)		*ptSym;

			if (ptObjSec->header.sh_entsize != sizeof (ElfW (Sym)))
			{
				eLog("Symbol size mismatch: %lu != %lu File: %s",
				(ulong) ptObjSec->header.sh_entsize,
				(ulong) sizeof (ElfW (Sym)),
				pstrModFile);
				return NULL;
			}
			lNumSym = ptObjSec->header.sh_size / sizeof (ElfW (Sym));
			pstrTab = ptObjFile->sections[ptObjSec->header.sh_link]->contents;
			ptSym 	= (ElfW (Sym) *) ptObjSec->contents;
			j 			= ptObjFile->local_symtab_size = ptObjSec->header.sh_info;

			ptObjFile->local_symtab = malloc (j *= sizeof (ObjSymT *));
			memset (ptObjFile->local_symtab, 0, j);

			for (j=1, ++ptSym; j<lNumSym; ++j, ++ptSym)
			{
				const char *pstrName;

				if (ptSym->st_name)
				{
					pstrName = pstrTab + ptSym->st_name;
				}
				else
				{
					pstrName = ptObjFile->sections[ptSym->st_shndx]->name;
				}
				ObjAddSymbol (
						ptObjFile, 
						pstrName, 
						j, 
						ptSym->st_info, 
						ptSym->st_shndx, 
						ptSym->st_value, 
						ptSym->st_size);
			}
		}
	}

  for (i=0; i<lShNum; ++i)
	{
		ObjSecT *ptObjSec = ptObjFile->sections[i];

		if (SHT_REL == ptObjSec->header.sh_type)
		{
			char					*pstrTab	= NULL;
			ulong 				lShSize		= 0;
			ulong 				j					= 0;
			ElfW(RelM)		*ptRel;
			ObjSecT				*ptObjSec1;

			if (ptObjSec->header.sh_entsize != sizeof (ElfW (RelM)))
			{
				eLog("Reloc entry size mismatch: %lu != %lu File: %s",
				(ulong) ptObjSec->header.sh_entsize,
				(ulong) sizeof (ElfW (RelM)),
				pstrModFile);
				return NULL;
			}
			lShSize		= ptObjSec->header.sh_size / sizeof (ElfW (RelM));
			ptRel			= (ElfW (RelM) *) ptObjSec->contents;
			ptObjSec1	= ptObjFile->sections[ptObjSec->header.sh_link];
			pstrTab 	= ptObjFile->sections[ptObjSec1->header.sh_link]->contents;

			for (j=0; j<lShSize; ++j, ++ptRel)
			{
				ElfW(Sym)		*ptXsym;
				ObjSymT			*ptIsym;
				ulong				lSymNum	= ELFW (R_SYM) (ptRel->r_info);

				if (0 != lSymNum)
				{
					ptXsym = ((ElfW (Sym) *) ptObjSec1->contents) + lSymNum;

					if (ELFW (ST_BIND) (ptXsym->st_info) == STB_LOCAL)
					{
						ptIsym = ptObjFile->local_symtab[lSymNum];
					}
					else
					{
						const char *pstrName;

						if (ptXsym->st_name)
						{
							pstrName = pstrTab + ptXsym->st_name;
						}
						else
						{
							pstrName = ptObjFile->sections[ptXsym->st_shndx]->name;
						}
						ptIsym = ObjFindSymbol (ptObjFile, pstrName);
					}
					ptIsym->r_type = ELFW (R_TYPE) (ptRel->r_info);
				}
			}
		}
	}
  ptObjFile->filename = strdup (pstrModFile);

  return ptObjFile;
}



/*
** =======================================================================
** =======================================================================
*/
static void ObjFree (ObjFileT *ptObjFile)
{
	ObjSecT *ptObjSec;
	ObjSymT *ptObjSym;
	ObjSymT *ptObjSymNext;
	int i;
	int n;

	if (ptObjFile->sections) 
	{
		n = ptObjFile->header.e_shnum;

		for (i=0; i<n; ++i) 
		{
			if (NULL != (ptObjSec = ptObjFile->sections[i])) 
			{
				if (ptObjSec->contents)
				{
					free (ptObjSec->contents);
				}
				free (ptObjSec);
			}
		}
		free (ptObjFile->sections);
	}

	for (i=0; i<HASH_BUCKETS; ++i) 
	{
		for (ptObjSym = ptObjFile->symtab[i]; ptObjSym; ptObjSym = ptObjSymNext) 
		{
			ptObjSymNext = ptObjSym->next;
			free (ptObjSym);
		}
	}

	if (ptObjFile->local_symtab)
	{
		free (ptObjFile->local_symtab);
	}
	if (ptObjFile->filename)
	{
		free ((char *)(ptObjFile->filename));
	}
	if (ptObjFile->persist)
	{
		free ((char *)(ptObjFile->persist));
	}
	free (ptObjFile);
}




/*
** =======================================================================
** =======================================================================
*/
static void LocalFree ()
{
	if (NULL != gptModStat) 
	{
		ModStatT *ptMod;
		int i;

		for (i=0, ptMod=gptModStat; i<glNumModStat; ++i, ++ptMod) 
		{
			if (ptMod->syms) 
			{
				free(ptMod->syms);
			}

			if (ptMod->refs) 
			{
				free(ptMod->refs);
			}
		}
		free (gptModStat);
		gptModStat 		= NULL;
		glNumModStat 	= 0;
	}

	if (NULL != gpstrModBuf) 
	{
		free (gpstrModBuf);
		gpstrModBuf = NULL;
	}
	if (NULL != gptKsym) 
	{
		free (gptKsym);
		gptKsym 	= NULL;
		glNumKsym = 0;
	}
}



/*
** =======================================================================
** =======================================================================
*/
static BooleanT GetAllModInfo (int lType)
{
	char 		*pstrBuf	= NULL;
	size_t 	lBufSize	= 0;
	size_t 	lRetSize	= 0;
	int			lRC				= 0;

	bSysCall = ! query_module (NULL, 0, NULL, 0, NULL);

	LocalFree ();

	if (NULL == (pstrBuf = (char *) malloc (lBufSize = 256)))
	{
		eLog("Failed to allocate memory-1");
	}
	else if (0 != (lRC = query_module (NULL, QM_MODULES, pstrBuf, lBufSize, &lRetSize)) && ENOSPC != lRC) 
	{
		eLog("Failed to Query Module. %s",strerror(errno));
	}
	else
	{
		if (lRC == ENOSPC && NULL == (pstrBuf = realloc (pstrBuf, lBufSize = lRetSize)))
		{
			eLog("Failed to allocate memory-2");
		}
		else
		{
			ModStatT 	*ptMod		= NULL;
			ModStatT 	*ptModBuf	= NULL;
			size_t 		lNumMod		= 0;
			size_t 		i					= 0;
			char 			*pstr			= NULL;

			gpstrModBuf 	= pstrBuf;
			glNumModStat	= lNumMod		= lRetSize;
			gptModStat 		= ptModBuf 	= malloc (lNumMod * sizeof (ModStatT));

			memset (ptModBuf, 0, lNumMod * sizeof (ModStatT));
			
			for (i=0, pstr=pstrBuf, ptMod=ptModBuf; i<lNumMod; ++i, ++ptMod, pstr += strlen (pstr) + 1) 
			{
				ModInfoT tModInfo;

				ptMod->name = pstr;

				if (0 != query_module (pstr, QM_INFO, &tModInfo, sizeof (tModInfo), &lRetSize)) 
				{
					if (errno == ENOENT) 
					{
						ptMod->flags = NEW_MOD_DELETED;
						continue;
					}
					eLog("Failed to query module info: %s. %s", pstr, strerror(errno));
					return FALSE;
				}
				ptMod->addr = tModInfo.addr;

				if (lType & K_SYMBOLS) 
				{
					ModSymT	*ptModSymBuf	= NULL;
					ModSymT	*ptModSym			= NULL;
					size_t	lNumModSym		= 0;
					size_t	j							= 0;
					BooleanT bContinue    = TRUE;

					ptModSymBuf = malloc (lBufSize = 1024);

					while (TRUE == bContinue && 0 != query_module (pstr, QM_SYMBOLS, ptModSymBuf, lBufSize, &lRetSize)) 
					{
						if (errno == ENOSPC) 
						{
							ptModSymBuf = realloc (ptModSymBuf, lBufSize = lRetSize);
							continue;
						}
						if (errno == ENOENT) 
						{
							ptMod->flags = NEW_MOD_DELETED;
							free (ptModSymBuf);
							bContinue = FALSE;
						} 
						else 
						{
							eLog("Failed query module symbols: %s. %s", pstr, strerror(errno));
							return FALSE;
						}
					}
					if (TRUE == bContinue)
					{
						lNumModSym 		= lRetSize;
						ptMod->nsyms 	= lNumModSym;
						ptMod->syms 	= ptModSymBuf;

						for (j=0, ptModSym=ptModSymBuf; j<lNumModSym; ++j, ++ptModSym)
						{
							ptModSym->name += (ulong) ptModSymBuf;
						}
					}
				}
			}
		}
	}

	if (lType & K_SYMBOLS) 
	{
		ModSymT	*ptModSymBuf	= NULL;
		ModSymT	*ptModSym			= NULL;
		size_t lNumModSym			= 0;
		size_t j							= 0;

		ptModSymBuf = malloc (lBufSize = 16 * 1024);

		while (0 != query_module (NULL, QM_SYMBOLS, ptModSymBuf, lBufSize, &lRetSize)) 
		{
			if (errno != ENOSPC) 
			{
				eLog("Failed query module symbols: %s. %s", "NULL", strerror(errno));
				return FALSE;
			}
			ptModSymBuf = realloc (ptModSymBuf, lBufSize = lRetSize);
		}
		glNumKsym = lNumModSym = lRetSize;
		gptKsym 	= ptModSymBuf;

		for (j=0, ptModSym=ptModSymBuf; j<lNumModSym; ++j, ++ptModSym)
		{
			ptModSym->name += (ulong) ptModSymBuf;
		}
	}
	return TRUE;
}




/*
** =======================================================================
** =======================================================================
*/
static int GetKernelVersion (char *pstrVersion)
{
	int							lVersion	= -1;
	struct utsname	tUname;

	if (-1 == uname (&tUname))
	{
		eLog("Failed to get Kernel Version - Uname. %s",strerror(errno));
	}
	else
	{
		char 	*pstr1 	= NULL;
		char	*pstr2	= NULL;
		int a, b, c;

		strncpy (pstrVersion, tUname.release, STRVERSIONLEN);

		pstr1 = tUname.release;

		a = strtoul (pstr1, &pstr1, 10);

		if (*pstr1 != '.')
		{
			return lVersion;
		}
		b = strtoul (pstr1 + 1, &pstr1, 10);

		if (*pstr1 != '.')
		{
			return lVersion;
		}
		c = strtoul (pstr1 + 1, &pstr2, 10);

		if (pstr1 + 1 == pstr2)
		{
			return lVersion;
		}
		lVersion = a << 16 | b << 8 | c;
	}
	return lVersion;
}





/*
** =======================================================================
** =======================================================================
*/
static int NcvStrCmp (const char *pstr1, const char *pstr2)
{
	size_t lLen1 = strlen(pstr1), lLen2 = strlen(pstr2);

	if (lLen2 == lLen1 + 10
			&& pstr2[lLen1] == '_' 
			&& pstr2[lLen1 + 1] == 'R') 
	{
		return strncmp (pstr1, pstr2, lLen1);
	} 
	else if (lLen1 == lLen2 + 10
					 && pstr1[lLen2] == '_' 
					 && pstr1[lLen2 + 1] == 'R') 
	{
		return strncmp (pstr1, pstr2, lLen2);
	} 
	else
	{
		return strcmp (pstr1, pstr2);
	}
}




/*
** =======================================================================
** =======================================================================
*/
static ulong NcvHash (const char *pstr)
{
	size_t lLen = strlen (pstr);

	if (lLen > 10
			&& pstr [lLen - 10] == '_' 
			&& pstr [lLen - 9] == 'R') 
	{
		lLen -= 10;
	}
	return ObjHashNcv (pstr, lLen);
}




/*
** =======================================================================
** =======================================================================
*/
static int 	AddSym (
						ObjFileT	*ptObjFile, 
						int				lIdx,
						ModSymT		*ptModSym, 
						size_t		lNumSym)
{
	ModSymT *ptMod;
	int 		lRC = 0;
	size_t 	i;
		
	for (i=0, ptMod = ptModSym; i < lNumSym; ++i, ++ptMod) 
	{
		ObjSymT *ptObjSym = ObjFindSymbol (ptObjFile, (char *) ptMod->name);

		if (ptObjSym && !ELFW(ST_BIND) (ptObjSym->info) == STB_LOCAL) 
		{
			ptObjSym = ObjAddSymbol (
									ptObjFile, 
									(char *) ptMod->name, 
									-1, 
									ELFW(ST_INFO) (STB_GLOBAL, STT_NOTYPE), 
									lIdx, 
									ptMod->value, 
									0);

			if (ptObjSym->secidx == lIdx)
			{
				lRC = 1;
			}
		}
	}
	return lRC;
}





/*
** =======================================================================
** =======================================================================
*/
static void AddKernSym (ObjFileT *ptObjFile)
{
	ModStatT 	*ptModStat;
	size_t		i 		= 0;
	size_t		lUsed = 0;

	for (i = 0, ptModStat = gptModStat; i < glNumModStat; ++i, ++ptModStat)
	{
		if (ptModStat->nsyms 
				&& AddSym (ptObjFile, SHN_HIRESERVE + 2 + i, ptModStat->syms, ptModStat->nsyms))
		{
			ptModStat->status = 1 , ++lUsed;
		}
	}
	glModUse = lUsed;

	if (glNumKsym)
	{
		AddSym (ptObjFile, SHN_HIRESERVE + 1, gptKsym, glNumKsym);
	}
}




/*
** =======================================================================
** =======================================================================
*/
static void HideSym (ObjFileT *ptObjFile)
{
	ObjSymT	*ptObjSym;
	const 	char *const *pstr;
	static 	const char *const pstrSpec[] = 
	{ 
		"cleanup_module", 
		"init_module", 
		"kernel_version", 
		NULL 
	};

	for (pstr = pstrSpec; *pstr; ++pstr)
	{
		if ((ptObjSym = ObjFindSymbol (ptObjFile, *pstr)) != NULL)
		{
			ptObjSym->info = ELFW(ST_INFO) (STB_LOCAL, ELFW(ST_TYPE) (ptObjSym->info));
		}
	}
}





/*
** =======================================================================
** =======================================================================
*/
static char *GetModVal (ObjFileT *ptObjFile, const char *pstrKey)
{
	ObjSecT	*ptObjSec		= ObjFindSec (ptObjFile, ".modinfo");
	size_t	lLen 				= strlen (pstrKey);
	char		*pstr				= NULL;
	char		*pstrEqual	= NULL;
	char		*pstrTmp		= NULL;
	char		*pstrNull		= NULL;

	if (NULL == ptObjSec)
	{
		return NULL;
	}
	pstr 		= ptObjSec->contents;
	pstrTmp	= pstr + ptObjSec->header.sh_size;

	while (pstr < pstrTmp) 
	{
		pstrEqual = strchr (pstr, '=');
		pstrNull 	= strchr (pstr, '\0');

		if (pstrEqual) 
		{
			if (pstrEqual - pstr == lLen && strncmp(pstr, pstrKey, lLen) == 0)
			{
				return pstrEqual + 1;
			}
		} 
		else 
		{
			if (pstrNull - pstr == lLen && strcmp (pstr, pstrKey) == 0)
			{
				return pstrNull;
			}
		}
		pstr = pstrNull + 1;
	}
	return NULL;
}





/*
** =======================================================================
** =======================================================================
*/
static void CreateMod (ObjFileT *ptObjFile, const char *pstrModName)
{
	ObjSecT *ptObjSec = ObjCreateSecF (ptObjFile, ".this", sizeof (long), sizeof (DrModT));

	memset (ptObjSec->contents, 0, sizeof(DrModT));

	ObjAddSymbol (
				ptObjFile, 
				"__this_module", 
				-1, 
				ELFW(ST_INFO) (STB_LOCAL, STT_OBJECT),
				ptObjSec->idx, 
				0, 
				sizeof(DrModT));

	ObjStrPt (ptObjFile, ptObjSec->idx, offsetof(DrModT, name), pstrModName);
}




/*
** =======================================================================
** =======================================================================
*/
static void AddKsymTab (ObjFileT *ptObjFile, ObjSymT *ptObjSym)
{
	ObjSecT			*ptObjSec = ObjFindSec (ptObjFile, "__ksymtab");
	ElfW(Addr)	lShSize;

	if (ptObjSec && ! (ptObjSec->header.sh_flags & SHF_ALLOC)) 
	{
		*((char *)(ptObjSec->name)) = 'x';
		ptObjSec = NULL;
	}
	if (! ptObjSec)
	{
		ptObjSec = ObjCreateSec (ptObjFile, "__ksymtab", sizeof (void *), 0);
	}
	if (! ptObjSec)
	{
		return;
	}
	ptObjSec->header.sh_flags |= SHF_ALLOC;

	lShSize = ptObjSec->header.sh_size;
	ObjSymPt (ptObjFile, ptObjSec->idx, lShSize, ptObjSym);
	ObjStrPt (ptObjFile, ptObjSec->idx, lShSize + sizeof (void *), ptObjSym->name);
	ObjExtendSec (ptObjSec, 2 * sizeof (char *));
}




/*
** =======================================================================
** =======================================================================
*/
static int CreateModKsym (ObjFileT *ptObjFile)
{
	ObjSecT *ptObjSec;
	int i;

	if (glModUse) 
	{
		ModRefT *ptModRef;
		ObjSymT *ptObjSym;

		ptObjSec = 	ObjCreateSec (
								ptObjFile, 
								".kmodtab", 
								sizeof (void *), 
								(sizeof(ModRefT) * glModUse));

		if (! ptObjSec)
		{
			return 0;
		}
		ptObjSym = ObjFindSymbol (ptObjFile, "__this_module");
		ptModRef = (ModRefT *) ptObjSec->contents;

		for (i=0; i<glNumModStat; ++i)
		{
			if (gptModStat[i].status) 
			{
				ptModRef->dep = gptModStat[i].addr;
				ObjSymPt (ptObjFile, ptObjSec->idx, (char *) &ptModRef->ref - ptObjSec->contents, ptObjSym);
				ptModRef->next_ref = 0;
				++ptModRef;
			}
		}
	}
	if (! ObjFindSec (ptObjFile, "__ksymtab")) 
	{
		int *plLoad = alloca (sizeof (int) * (i = ptObjFile->header.e_shnum));

		while (--i >= 0)
		{
			plLoad[i] = (ptObjFile->sections[i]->header.sh_flags & SHF_ALLOC) != 0;
		}

		for (i = 0; i < HASH_BUCKETS; ++i) 
		{
			ObjSymT *ptObjSym;

			for (ptObjSym = ptObjFile->symtab[i]; ptObjSym; ptObjSym = ptObjSym->next) 
			{
				if (ELFW(ST_BIND) (ptObjSym->info) != STB_LOCAL 
						&& ptObjSym->secidx <= SHN_HIRESERVE 
						&& (ptObjSym->secidx >= SHN_LORESERVE || plLoad[ptObjSym->secidx])) 
				{
					AddKsymTab (ptObjFile, ptObjSym);
				}
			}
		}
	}
	return 1;
}




/*
** =======================================================================
** =======================================================================
*/
static int GetModVersion (ObjFileT *ptObjFile, char str[STRVERSIONLEN])
{
	int a, b, c;
	char *pstrTmp;
	char *pstrVal;

	if ((pstrVal = GetModVal (ptObjFile, "kernel_version")) == NULL) 
	{
		ObjSymT *ptObjSym;

		bHasInfo = FALSE;

		if ((ptObjSym = ObjFindSymbol (ptObjFile, "kernel_version")) == NULL)
		{
			ptObjSym = ObjFindSymbol (ptObjFile, "__module_kernel_version");
		}
		if (ptObjSym == NULL)
		{
			return -1;
		}
		pstrVal = ptObjFile->sections[ptObjSym->secidx]->contents + ptObjSym->value;
	} 
	else
	{
		bHasInfo = TRUE;
	}
	strncpy (str, pstrVal, STRVERSIONLEN);

	a = strtoul (pstrVal, &pstrVal, 10);

	if (*pstrVal != '.')
	{
		return -1;
	}
	b = strtoul (pstrVal + 1, &pstrVal, 10);

	if (*pstrVal != '.')
	{
		return -1;
	}
	c = strtoul (pstrVal + 1, &pstrTmp, 10);

	if (pstrVal + 1 == pstrTmp)
	{
		return -1;
	}
	return a << 16 | b << 8 | c;
}



/*
** =======================================================================
** =======================================================================
*/
static int KernCheckSum ()
{
	ModSymT *ptModSym;
	size_t	i;

	for (i=0, ptModSym=gptKsym; i<glNumKsym; ++i, ++ptModSym)
	{
		if (0 == strcmp ((char *) ptModSym->name, "Using_Versions"))
		{
			return ptModSym->value;
		}
	}
	return 0;
}




/*
** =======================================================================
** =======================================================================
*/
static int ModCheckSum (ObjFileT *ptObjFile)
{
	if (TRUE == bHasInfo) 
	{
		const char *pstrVal = GetModVal (ptObjFile, "using_checksums");

		if (pstrVal)
		{
			return atoi (pstrVal);
		}
		else
		{
			return 0;
		}
	} 
	else
	{
		return ObjFindSymbol (ptObjFile, "Using_Versions") != NULL;
	}
}




/*
** =======================================================================
** =======================================================================
*/
static void AddKsymOop (
						ObjFileT		*ptObjFile, 
						const char	*pstrFile,
						const char	*pstrModName)
{
	struct stat tStat;
	ObjSecT			*ptObjSec;
	ObjSymT 		*ptObjSym;
	char				*pstrAbsFile;
	char				*pstrName;
	int					lAbsLen;	
	int					lModLen;	
	int					lModVer;	
	int					lKsymTab;
	int					lLen;
	int 				i;
	char				strRealName [PATH_MAX];
	char				strVer [STRVERSIONLEN];

	static const char pstrSymPre[] = "__insmod__";
	static const char *ppstrSecName[] = { ".text", ".rodata", ".data", ".bss" };

	if (NULL == realpath (pstrFile, strRealName)) 
	{
		int lError = errno;
		eLog("Cannot get realpath for: %s", pstrFile);
		errno 			= lError;
		pstrAbsFile = strdup (pstrFile);
	}
	else
	{
		pstrAbsFile = strdup (strRealName );
	}
	lModLen 	= strlen (pstrModName);
	lAbsLen 	= strlen (pstrAbsFile);
	lKsymTab 	= ObjFindSec (ptObjFile, "__ksymtab") || 0;

	if ((ptObjSec = ObjFindSec  (ptObjFile, ".this"))) 
	{
		lLen = sizeof(pstrSymPre) + lModLen + 2 + lAbsLen + 2 + 2 * sizeof(tStat.st_mtime) + 2 + 8 + 1; 

		pstrName = malloc (lLen);

		if (0 != stat (pstrAbsFile, &tStat))
		{
			tStat.st_mtime = 0;
		}
		lModVer = GetModVersion (ptObjFile, strVer);

		snprintf (pstrName, lLen, "%s%s_O%s_M%0*lX_V%d",
							pstrSymPre, 
							pstrModName, 
							pstrAbsFile,
							2 * (int) sizeof (tStat.st_mtime), 
							tStat.st_mtime,
							lModVer);
		
		ptObjSym = 	ObjAddSymbol (
								ptObjFile, 
								pstrName, 
								-1,
								ELFW(ST_INFO) (STB_GLOBAL, STT_NOTYPE), 
								ptObjSec->idx, 
								ptObjSec->header.sh_addr, 
								0);

		if (lKsymTab)
		{
			AddKsymTab (ptObjFile, ptObjSym);
		}
	}
	free (pstrAbsFile);

	if (ptObjFile->persist) 
	{
		lLen = sizeof (pstrSymPre) + lModLen + 2 + strlen(ptObjFile->persist) + 1;
		pstrName = malloc (lLen);
		snprintf (pstrName, lLen, "%s%s_P%s", pstrSymPre, pstrModName, ptObjFile->persist);

		ptObjSym =	ObjAddSymbol (
								ptObjFile, 
								pstrName, 
								-1, 
								ELFW(ST_INFO) (STB_GLOBAL, STT_NOTYPE),
								ptObjSec->idx, 
								ptObjSec->header.sh_addr, 
								0);

		if (lKsymTab)
		{
			AddKsymTab (ptObjFile, ptObjSym);
		}
	}

	for (i = 0; i < sizeof (ppstrSecName) / sizeof (ppstrSecName[0]); ++i) 
	{
		if ((ptObjSec = ObjFindSec (ptObjFile, ppstrSecName[i])) && ptObjSec->header.sh_size) 
		{
			lLen = sizeof(pstrSymPre) + lModLen + 2 + strlen(ptObjSec->name) + 2 + 8 + 1;
			pstrName = malloc (lLen);

			snprintf (pstrName, lLen, "%s%s_S%s_L%ld", 
								pstrSymPre, 
								pstrModName, 
								ptObjSec->name, 
								(long)ptObjSec->header.sh_size);

			ptObjSym =	ObjAddSymbol (
									ptObjFile, 
									pstrName, 
									-1, 
									ELFW(ST_INFO) (STB_GLOBAL, STT_NOTYPE),
									ptObjSec->idx, 
									ptObjSec->header.sh_addr, 
									0);

			if (lKsymTab)
			{
				AddKsymTab (ptObjFile, ptObjSym);
			}
		}
	}
}




/*
** =======================================================================
** =======================================================================
*/
static BooleanT	AddKallSym  (
								ObjFileT	*ptObjFile,
								ObjSecT		**pptObjSec, 
								int				lForce)
{
	ObjFileT		*ptObjFileSym;
	ObjSecT			*ptObjSec;
	ModSymT			*ptModSym;
	ulong				lStart 	= 0;
	ulong				lStop 	= 0;
	size_t			i				= 0;
	int 				lLen		= 0;
	const char 	*pstr		= NULL;
	const char 	*pstrR;

	for (i=0, ptModSym=gptKsym; i<glNumKsym; ++i, ++ptModSym) 
	{
		pstr 	= (char *) ptModSym->name;
		pstrR = strstr (pstr, "_R");

		if (pstrR)
		{
			lLen = pstrR - pstr;
		}
		else
		{
			lLen = strlen (pstr);
		}
		if (0 == strncmp (pstr, "__start_" KALLSYMS_SEC_NAME, lLen))
		{
			lStart = ptModSym->value;
		}
		else if (0 == strncmp (pstr, "__stop_" KALLSYMS_SEC_NAME, lLen))
		{
			lStop = ptModSym->value;
		}
	}

	if (lStart >= lStop && ! lForce)
	{
		return TRUE;
	}

	for (i=0; i<ptObjFile->header.e_shnum; ++i) 
	{
		if (0 == strcmp (ptObjFile->sections[i]->name, KALLSYMS_SEC_NAME)) 
		{
			*pptObjSec = ptObjFile->sections[i];
			break;
		}
	}
	if (! *pptObjSec)
	{
		*pptObjSec = ObjCreateSec (ptObjFile, KALLSYMS_SEC_NAME, 0, 0);
	}
	ObjKallSym (ptObjFile, &ptObjFileSym);

	ptObjSec = ptObjFileSym->sections[KALLSYMS_IDX];
	(*pptObjSec)->header.sh_addralign = ptObjSec->header.sh_addralign;
	(*pptObjSec)->header.sh_size = ptObjSec->header.sh_size;
	free((*pptObjSec)->contents);
	(*pptObjSec)->contents = ptObjSec->contents;
	ptObjSec->contents = NULL;
	ObjFree (ptObjFileSym);

	return TRUE;
}



/*
** =======================================================================
** =======================================================================
*/
static void ArAddData (ObjFileT *ptObjFile, ObjSecT **pptObjSec)
{
	size_t i;

	*pptObjSec = NULL;

	for (i=0; i<ptObjFile->header.e_shnum; ++i) 
	{
		if (0 == strcmp (ptObjFile->sections[i]->name, "__archdata")) 
		{
			*pptObjSec = ptObjFile->sections[i];
			break;
		}
	}
	if (! *pptObjSec)
	{
		*pptObjSec = ObjCreateSec (ptObjFile, "__archdata", 16, 0);
	}
}


/*
** =======================================================================
** =======================================================================
*/
static int	InitMod (
						const char 	*pstrModName, 
						ObjFileT 		*ptObjFile, 
						ulong 			lSize, 
						const char 	*pstrName)
{
	ObjSecT	*ptObjSec;
	DrModT	*ptDrMod;
	void		*pstrBuf;
	long		lAddr;
	int			lRC = 0;

	ptObjSec 	= ObjFindSec (ptObjFile, ".this");
	ptDrMod 	= (DrModT *) ptObjSec->contents;
	lAddr			= ptObjSec->header.sh_addr;

	ptDrMod->size_of_struct = sizeof (*ptDrMod);
	ptDrMod->size 					= lSize;
	ptDrMod->flags 					= 0;

	ptObjSec = ObjFindSec (ptObjFile, "__ksymtab");

	if (ptObjSec && ptObjSec->header.sh_size) 
	{
		ptDrMod->syms = ptObjSec->header.sh_addr;
		ptDrMod->nsyms = ptObjSec->header.sh_size / (2 * sizeof (char *));
	}
	if (glModUse) 
	{
		ptObjSec 				= ObjFindSec (ptObjFile, ".kmodtab");
		ptDrMod->deps 	= ptObjSec->header.sh_addr;
		ptDrMod->ndeps 	= glModUse;
	}
	ptDrMod->init 		= ObjSymbolValue (ptObjFile, ObjFindSymbol (ptObjFile, "init_module"));
	ptDrMod->cleanup 	= ObjSymbolValue (ptObjFile, ObjFindSymbol (ptObjFile, "cleanup_module"));

	if ((ptObjSec = ObjFindSec (ptObjFile, "__ex_table")))
	{
		ptDrMod->ex_table_start = ptObjSec->header.sh_addr;
		ptDrMod->ex_table_end 	= ptObjSec->header.sh_addr + ptObjSec->header.sh_size;
	}

	if ((ptObjSec = ObjFindSec (ptObjFile, ".text.init")))
	{
		ptDrMod->runsize = ptObjSec->header.sh_addr - lAddr;
	}
	if ((ptObjSec = ObjFindSec (ptObjFile, ".data.init")))
	{
		if (!ptDrMod->runsize || ptDrMod->runsize > ptObjSec->header.sh_addr - lAddr)
		{
			ptDrMod->runsize = ptObjSec->header.sh_addr - lAddr;
		}
	}
	ptObjSec = ObjFindSec (ptObjFile, "__archdata");

	if (ptObjSec && ptObjSec->header.sh_size) 
	{
		ptDrMod->archdata_start = ptObjSec->header.sh_addr;
		ptDrMod->archdata_end 	= ptDrMod->archdata_start + ptObjSec->header.sh_size;
	}
	ptObjSec = ObjFindSec (ptObjFile, KALLSYMS_SEC_NAME);

	if (ptObjSec && ptObjSec->header.sh_size) 
	{
		ptDrMod->kallsyms_start = ptObjSec->header.sh_addr;
		ptDrMod->kallsyms_end 	= ptDrMod->kallsyms_start + ptObjSec->header.sh_size;
	}
	pstrBuf = malloc (lSize);
	ObjCreateImg (ptObjFile, pstrBuf);

	if (pstrName) 
	{
		int lFd = open (pstrName, O_WRONLY | O_CREAT | O_TRUNC, S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
		int lWrite;

		if (-1 == lFd) 
		{
			eLog("Failed to open: %s", pstrName);
			lRC = -1;
		}
		else 
		{
			if ((lWrite = write (lFd, pstrBuf, lSize)) != lSize) 
			{
				eLog("Failed to write: %s", pstrName);
				lRC = -1;
			}
			close (lFd);
		}
	}

	if (lRC == 0) 
	{
		fflush (stdout);

		lRC = init_module (pstrModName, (DrModT *) pstrBuf);

		if (-1 == lRC) 
		{
			eLog("Call init_module: %m");

			delete_module (pstrModName);
		}
	}
	free (pstrBuf);

	return lRC == 0;
}




/*
** =======================================================================
** =======================================================================
*/
static int 	LocalCheckModParm (
						ObjFileT 	*ptObjFile, 
						char 			*pstrKey, 
						char 			*pstrVal, 
						int 			*plPersist)
{
	ObjSymT *ptObjSym;
	char		*pstr = pstrVal;
	int			lMin;
	int			lMax;

	ptObjSym = ObjFindSymbol (ptObjFile, pstrKey);

	if (ptObjSym == NULL) 
	{
		eLog("WARN: Symbol for parameter not found: %s", pstrKey);
		return 1;
	}
	if (isdigit (*pstr)) 
	{
		lMin = strtoul (pstr, &pstr, 10);

		if (*pstr == '-')
		{
			lMax = strtoul (pstr + 1, &pstr, 10);
		}
		else
		{
			lMax = lMin;
		}
	} 
	else
	{
		lMin = lMax = 1;
	}

	if (lMax < lMin) 
	{
		eLog("WARN: Parameter %s has max < min", pstrKey);
		return 1;
	}

	switch (*pstr) 
	{
		case 'c':
		{
			if (! isdigit (pstr[1])) 
			{
				eLog("Parameter %s has no size after 'c'", pstrKey);
				return 1;
			}
			while (isdigit(pstr[1]))
			{
				++pstr;
			}
			break;
		}

		case 'b':
		case 'h':
		case 'i':
		case 'l':
		case 's':
		{
			break;
		}

		case '\0':
		{
			eLog("Parameter %s has no format character", pstrKey);
			return 1;
		}

		default:
		{
			eLog("Parameter %s has unknown format character '%c'", pstrKey, *pstr);
			return 1;
		}
	}
	switch (*++pstr) 
	{
		case 'p':
		{
			if (*(pstr-1) == 's') 
			{
				eLog ("Parameter %s is invalid persistent string", pstrKey);
				return 1;
			}
			*plPersist = 1;
			break;
		}

		case '\0':
		{
			break;
		}

		default:
		{
			eLog("Parameter %s has unknown format modifier '%c'", pstrKey, *pstr);
			return 1;
		}
	}
	return 0;
}

/*
** =======================================================================
** =======================================================================
*/
static void CheckModParm (ObjFileT *ptObjFile, int *plPersist)
{
	ObjSecT		*ptObjSec;
	char			*pstrVal;
	char			*pstr;
	char			*pstrEnd;
	char			*pstrNull;
	int				lLen;
	int				lError	= 0;

	ptObjSec = ObjFindSec (ptObjFile, ".modinfo");

	if (ptObjSec == NULL) 
	{
		return;
	}
	pstr 		= ptObjSec->contents;
	pstrEnd = pstr + ptObjSec->header.sh_size;

	while (pstr < pstrEnd && ! lError) 
	{
		pstrVal 	= strchr (pstr, '=');
		pstrNull 	= strchr (pstr, '\0');

		if (pstrVal) 
		{
			lLen = pstrVal - pstr;

			if (lLen >= 5 
					&& strncmp (pstr, "parm_", 5) == 0 
					&& ! (lLen > 10 && strncmp (pstr, "parm_desc_", 10) == 0)) 
			{
				char *pstrName = malloc (lLen + 1);

				strncpy (pstrName, pstr + 5, lLen - 5);
				pstrName[lLen - 5] = '\0';
				lError = LocalCheckModParm (ptObjFile, pstrName, pstrVal+1, plPersist);
				free (pstrName);
			}
		} 
		else 
		{
			if (pstrNull - pstr >= 5 && strncmp (pstr, "parm_", 5) == 0) 
			{
				eLog("Parameter %s found with no value", pstr);
				lError = 1;
			}
		}
		pstr = pstrNull + 1;
	}

	if (lError)
	{
		*plPersist = 0;
	}
	return;
}




/*
** =======================================================================
** =======================================================================
*/
static BooleanT ModExist (char *pstrMod)
{
	BooleanT 	bSuccess = TRUE;
	int				i;

	for (i=0; i<glNumModStat && TRUE == bSuccess; ++i) 
	{
		if (0 == strcmp (gptModStat[i].name, pstrMod))
		{
			bSuccess = FALSE;
		}
	}
	return bSuccess;
}


/*
** =======================================================================
** =======================================================================
*/
static BooleanT GetModName (char *pstrFile, char **ppstrModName)
{
	BooleanT bSuccess = FALSE;

	if (NULL == pstrFile)
	{
		eLog("Invalid input Driver Object file");
	}
	else
	{
		char *pstr = strrchr (pstrFile, 0x2f);

		NULL == pstr ?  pstr = pstrFile : pstr++;
		NULL == (*ppstrModName = malloc (strlen (pstr))) ?
		eLog("Failed to allocate memory-1") :
		strncpy (*ppstrModName, pstr, strlen (pstr));

		if (NULL != (pstr = strrchr (*ppstrModName, '.')))
		{
			*pstr 		= 0;
			bSuccess 	= TRUE;
		}
	}
	return bSuccess;
}




/*
** =======================================================================
** =======================================================================
*/
extern BooleanT DrCmdLoadModule (char *pstrFile, int *plRC)
{
	BooleanT	bSuccess			= FALSE;
	ObjFileT *ptObjFile			= NULL;
	char 			*pstrModName	= NULL;
	char 			*pstrModFile	= NULL;
	int 			lFd						= -1;
	int				lKernVer			= -1;
	int 			lModVer				= -1;
	char 			strKernVer[64];
	char			strModVer[64];
	char      strCmd [512];

	memset (strCmd, 0x0, 512);

	/* 
	 * modprobe requires modules name withtout .ko extension
	*/
	if ( NULL == strstr(".ko",pstrFile))
	{
		pstrFile [ strlen (pstrFile) - 3 ] = '\0';
	}

	sprintf(strCmd, "/bin/modprobe %s", pstrFile);

	if (0 != system (strCmd))
	{
		eLog("DrCmdLoadModule Failed. Module: %s", pstrFile);
		*plRC	= DRMOD_ERR_FATAL;
	}
	else
	{
		iLog("DrCmdLoadModule Success. Module: %s", pstrFile);
		bSuccess = TRUE;
		*plRC	= DRMOD_ERR_NONE;
	}
	return bSuccess;



	pstrModFile = pstrFile;
	*plRC				= DRMOD_ERR_FATAL;

	iLog("Begin load module: %s", pstrModFile);

	if (TRUE != GetModName (pstrFile, &pstrModName))
	{
		eLog("Failed to get mod filename from file: ",pstrModFile);
	}
	else if (-1 == (lFd = open (pstrModFile, O_RDONLY))) 
	{
		eLog("Failed to open Module File: %s. %s", pstrModFile, strerror(errno));
	}
	else if (TRUE != GetAllModInfo (K_SYMBOLS))
	{
		eLog("Failed to get Mod Info");
	}
	else if (TRUE != ModExist (pstrModName))
	{
		eLog("Module already loaded: %s", pstrModName);
		*plRC = DRMOD_ERR_EXIST;
	}
	else if (NULL == (ptObjFile = ObjLoad (lFd, ET_REL, pstrModFile)))
	{
		eLog("Failed to Load Obj");
	}
	else if (-1 == (lKernVer = GetKernelVersion (strKernVer)))
	{
		eLog("Failed to get Kernel Version");
	}
	else if (-1 == (lModVer = GetModVersion (ptObjFile, strModVer)))
	{
		eLog("Failed to get Module Version");
	}
	else
	{
		int	lKernSum	= KernCheckSum ();
		int	lModSum		= ModCheckSum (ptObjFile);

		iLog("Kernel Version: %s (%d), CheckSum: %d", strKernVer, lKernVer, lKernSum);
		iLog("Module Version: %s (%d), CheckSum: %d", strModVer,  lModVer,	lModSum);

		if (0 != strncmp (strKernVer, strModVer, 64))
		{
			iLog("WARN: Kernel and Module Version mismatch");
		}

		if (lKernSum != lModSum)
		{
			ObjSymCompare (ptObjFile, NcvStrCmp, NcvHash);
		}
		AddKernSym	(ptObjFile);
		CreateMod		(ptObjFile, pstrModName);
		ArCreate		(ptObjFile);  

		if (TRUE != ObjUndefV (ptObjFile))	
		{
			eLog("Failed to validate unresolved symbol");
			eLog("Perhap try to compile module with CONFIG_MODVERSIONS disable");
		}
		else
		{
			int 		lPersist 	= 0;
			ObjSecT *ptSecAr	= NULL;
			ObjSecT *ptSecSym = NULL;

			ObjAllocCom 	(ptObjFile);	
			CheckModParm  (ptObjFile, &lPersist);
			HideSym 			(ptObjFile);
			AddKsymOop 		(ptObjFile, pstrModFile, pstrModName);
			
			iLog("Validate symbols successfully");

			if (TRUE == bSysCall)
			{
				CreateModKsym (ptObjFile);
			}
			ArAddData (ptObjFile, &ptSecAr);

			if (TRUE != AddKallSym (ptObjFile, &ptSecSym, 0))
			{
				eLog("Failed to Add Kallsym(1)");
			}
			else
			{
				ulong 			lModSize 	= ObjLoadSize (ptObjFile);	
				ElfW(Addr)	lModAddr	= create_module (pstrModName, lModSize);
				long				lError		= errno;
			
				iLog("Add Kallsyms successfully");

				if (-1 == lModAddr && EEXIST == lError)
				{
					eLog("Module already exist: %s", pstrModName);
					*plRC = DRMOD_ERR_EXIST;
				}
				else if (-1 == lModAddr && ENOMEM == lError)
				{
					eLog("Failed to allocate memory for module: %s", pstrModName);
				}
				else if (TRUE != ObjReloc (ptObjFile, lModAddr)) 
				{	
					eLog("Failed to relocate Obj Module");
					delete_module (pstrModName);
				}
				else
				{
					ArAddData (ptObjFile, &ptSecAr);
				
					iLog("Start add Kallsyms and Initialize module");

					if (TRUE != AddKallSym (ptObjFile, &ptSecSym, 0))
					{
						eLog("Failed to add Kallsym(2)");
						delete_module (pstrModName);
					}
					else
					{
						char *pstrBname	= NULL;

						(bSuccess = InitMod (pstrModName, ptObjFile, lModSize, pstrBname)) ?
						(*plRC	= DRMOD_ERR_NONE) : 
						(*plRC	= DRMOD_ERR_FATAL);
					}
				}
			}
		}
	}
	if (0 != glHasErr)
	{
		bSuccess = FALSE;
	}
	iLog("End load module: %s (%s)", pstrModFile, bSuccess ? "TRUE" : "FALSE");

	return bSuccess;
}
