
#ifndef DRMOD_H
#define DRMOD_H

#include <elf.h>
#include "drtrace.h"

#define K_SYMBOLS						1
#define K_INFO							2
#define K_REFS							4
#define ElfW(x)  						Elf32_ ## x
#define ELFW(x)  						ELF32_ ## x
#define QM_MODULES					1
#define QM_INFO							5
#define NEW_MOD_DELETED			2
#define QM_SYMBOLS					4
#define KALLSYMS_IDX 				2			
#define KALLSYMS_SEC_NAME 	"__kallsyms"
#define DRMOD_ERR_NONE			0
#define DRMOD_ERR_EXIST			1
#define DRMOD_ERR_FATAL			2

#define TRUE    1
#define FALSE   0


typedef struct
{
	int						size;
	ElfW(Word)		total_size;	
	int						sections;	
	ElfW(Off)			section_off;	
	int						section_size;	
	int						symbols;	
	ElfW(Off)			symbol_off;	
	int						symbol_size;	
	ElfW(Off)			string_off;	
	ElfW(Addr)		start;		
	ElfW(Addr)		end;		
} KallSymHdrT;


typedef struct
{
	ElfW(Addr)	start;		
	ElfW(Word)	size;		
	ElfW(Off)		name_off;	
	ElfW(Word)	flags;		
} KallSymSecT;


typedef struct
{
	ElfW(Off)		section_off;	
	ElfW(Addr)	symbol_addr;	
	ElfW(Off)		name_off;	
} KallSymT;


typedef struct
{
  ulong 	value;
  ulong 	name;
} ModSymT;


typedef struct
{
  ulong 	dep;
  ulong 	ref;
  ulong 	next_ref;
} ModRefT;


typedef struct
{
  ulong 	addr;
  ulong 	size;
  ulong 	flags;
	long		usecount;
} ModInfoT;


struct ModStatS
{
	char							*name;
	ulong 						addr;
	ulong 						modstruct;
	ulong 						size;
	ulong 						flags;
	long 							usecount;
	size_t 						nsyms;
	ModSymT						*syms;
	size_t 						nrefs;
	ulong							status;
	struct ModStatS		**refs;
};
typedef struct ModStatS ModStatT;


typedef struct
{
  ulong 		size_of_struct;
  ulong 		next;
  ulong 		name;
  ulong			size;
  long 			usecount;
  ulong			flags;
  unsigned 	nsyms;
  unsigned 	ndeps;
  ulong 		syms;
  ulong 		deps;
  ulong 		refs;
  ulong 		init;
  ulong 		cleanup;
  ulong 		ex_table_start;
  ulong 		ex_table_end;
  ulong 		read_start;
  ulong 		read_end;
  ulong 		can_unload;
  ulong 		runsize;
  ulong 		kallsyms_start;
  ulong 		kallsyms_end;
  ulong 		archdata_start;
  ulong 		archdata_end;
  ulong 		kernel_data;
} DrModT;


extern BooleanT DrCmdLoadModule (char *, int *);

#endif
