#ifndef DRMOD_OBJ_H
#define DRMOD_OBJ_H

#include <stdio.h>
#include <sys/types.h>
#include <elf.h>
#include "drmod.h"

#define MATCH_MACHINE(x)    (x == EM_386)
#define Elf32_RelM          Elf32_Rel
#define HASH_BUCKETS  			521
#define ElfW(x)  						Elf32_ ## x
#define ELFW(x)  						ELF32_ ## x


struct ObjSecS
{
  ElfW(Shdr)			header;
  const char 			*name;
  char 						*contents;
  int 						idx;
  struct ObjSecS 	*load_next;
};
typedef struct ObjSecS ObjSecT;


struct ObjSymS
{
  const char 			*name;
  ulong 					value;
  ulong 					size;
  int 						secidx;
  int 						info;
  int 						ksymidx;
  int 						r_type;
  struct ObjSymS 	*next;
};
typedef struct ObjSymS ObjSymT;


struct ObjPtStrS
{
  int 							reloc_secidx;
  ElfW(Addr) 				reloc_offset;
  ElfW(Addr) 				string_offset;
  struct ObjPtStrS	*next;
};
typedef struct ObjPtStrS ObjPtStrT;


struct ObjPtSymS
{
  int 							reloc_secidx;
  ElfW(Addr) 				reloc_offset;
  ObjSymT 					*sym;
  struct ObjPtSymS	*next;
};
typedef struct ObjPtSymS ObjPtSymT;


typedef struct
{
  ElfW(Ehdr) 				header;
  ElfW(Addr) 				baseaddr;
  ObjSecT 					**sections;
  ObjSecT 					*load_order;
  ObjSecT 					**ppStart;
  ObjPtStrT 				*string_patches;
  ObjPtSymT					*symbol_patches;
  int (*symbol_cmp)	(const char *, const char *);
  ulong 						(*symbol_hash)(const char *);
  ulong 						local_symtab_size;
  ObjSymT 					**local_symtab;
  ObjSymT 					*symtab[HASH_BUCKETS];
  const char 				*filename;
  char 							*persist;
} ObjFileT;


typedef enum ObjRelocE
{
  eRelocOk,
  eRelocOverflow,
  eRelocDanger,
  eRelocUnhandle,
  eRelocGp,
} ObjRelocE;


struct i386_got_entry
{
	int offset;
	unsigned offset_done : 1;
	unsigned reloc_done : 1;
};

struct i386_file
{
	ObjFileT root;
	ObjSecT *got;
};

struct i386_symbol
{
	ObjSymT root;
	struct i386_got_entry gotent;
};


extern ulong			ObjHash 				(const char *);
extern ulong			ObjHashNcv			(const char *, ulong);
extern ObjSymT		*ObjAddSymbol 	(ObjFileT *, const char *, ulong, int, int, ElfW(Addr), ulong);
extern ObjSymT		*ObjFindSymbol 	(ObjFileT *, const char *);
extern ElfW(Addr)	ObjSymbolValue 	(ObjFileT *, ObjSymT *);
extern ObjSecT		*ObjFindSec			(ObjFileT *, const char *);
extern void				ObjInsertSec		(ObjFileT *, ObjSecT *);
extern ObjSecT		*ObjCreateSec		(ObjFileT *, const char *, ulong, ulong);
extern ObjSecT		*ObjCreateSecF	(ObjFileT *, const char *, ulong, ulong);
extern void				*ObjExtendSec		(ObjSecT *, ulong);
extern void				ObjStrPt				(ObjFileT *, int, ElfW(Addr), const char *);
extern void				ObjSymPt 				(ObjFileT *, int, ElfW(Addr), ObjSymT *);
extern BooleanT		ObjUndefV 			(ObjFileT *);
extern void				ObjAllocCom 		(ObjFileT *);
extern ulong			ObjLoadSize			(ObjFileT *);
extern BooleanT		ObjReloc 				(ObjFileT *, ElfW(Addr));
extern void				ObjCreateImg 		(ObjFileT *, char *);
extern ObjFileT		*ArNewFile 			();
extern ObjSecT		*ArNewSec 			();
extern ObjSymT		*ArNewSym 			();
extern void				ArCreate 				(ObjFileT *);
extern void				ObjKallSym			(ObjFileT *, ObjFileT **);
extern void				ObjSymCompare 	(ObjFileT *, 
																	int (*cmp)(const char *, const char *), 
																	ulong (*hash)(const char *));


#endif
