
#include <sys/param.h>
#include <sys/socket.h>
#include <sys/file.h>
#include <sys/time.h>
#include <signal.h>
#include <netinet/in_systm.h>
#include <netinet/in.h>
#include <netinet/ip.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <errno.h>
#include <limits.h>
#include "drpg.h"
#include "drtrace.h"


MODULE("PG")

#define	TRUE		1
#define	FALSE		0

static PingT 				*gptPing			= NULL;
static u_char 			*gpBuffer			= NULL;
static size_t 			glBufLen 			= PG_DATA_LEN;
static long					glProbeCount	= 0;
static BooleanT			gbSuccess			= FALSE;




/*
** ======================================================================================
** ======================================================================================
*/
static u_short PgIcmpSum (u_char *pAddr, int lLength)
{
  register int 	lSum 		= 0;
  u_short 			lRC 		= 0;
  u_short 			*pShort	= NULL;

  for (pShort = (u_short *) pAddr; lLength > 1; pShort++, lLength -= 2)
	{
    lSum += *pShort;
	}
  if (lLength == 1)
	{
		*(u_char *) &lRC = *(u_char *) pShort;
		lSum += lRC;
	}
  lSum 	= (lSum >> 16) + (lSum & 0xffff);
  lSum 	+= (lSum >> 16);
  lRC 	= ~lSum;

  return lRC;
}



/*
** ======================================================================================
** ======================================================================================
*/
static int PgEncode (u_char *pBuf, size_t lBufSize, int lType, int lIdent, int lSeq)
{
  icmphdr_t *ptIcmp	= NULL;

  if (lBufSize < 8)
	{
    return -1;
	}
  ptIcmp 							= (icmphdr_t *)pBuf;
  ptIcmp->icmp_type 	= lType;
  ptIcmp->icmp_code 	= 0;
  ptIcmp->icmp_cksum 	= 0;
  ptIcmp->icmp_seq 		= lSeq;
  ptIcmp->icmp_id 		= lIdent;
  ptIcmp->icmp_cksum 	= PgIcmpSum (pBuf, lBufSize);

  return 0;
}



/*
** ======================================================================================
** ======================================================================================
*/
static int PgDecode (u_char *pBuf, size_t lBufSize, struct ip **pptIp, icmphdr_t **pptIcmp)
{
  size_t 			lLength	= 0;
  u_short 		lSum		= 0;
  struct ip 	*ptIp		= NULL;
  icmphdr_t 	*ptIcmp	= NULL;

  ptIp 		= (struct ip *) pBuf;
  lLength = ptIp->ip_hl << 2;

  if (lBufSize < lLength + ICMP_MINLEN)
	{
    return -1;
	}
  ptIcmp 		= (icmphdr_t*)(pBuf + lLength);
  *pptIp 		= ptIp;
  *pptIcmp 	= ptIcmp;
  lSum 			= ptIcmp->icmp_cksum;

  ptIcmp->icmp_cksum = PgIcmpSum ((u_char *) ptIcmp, lBufSize-lLength);

  if (ptIcmp->icmp_cksum != lSum)
	{
    return 1;
	}
  return 0;
}



/*
** ======================================================================================
** ======================================================================================
*/
static int PgComplete ()
{
  if (gptPing->lNumXmit > 0 && gptPing->lNumRecv > 0)
	{
		gbSuccess = TRUE;
	}
  return 0;
}



/*
** ======================================================================================
** ======================================================================================
*/
static BooleanT PgSetBuffer ()
{
	BooleanT bSuccess = TRUE;

  if (NULL == gptPing->pBuffer)
	{
		if (NULL == (gptPing->pBuffer = malloc (PG_BUFFER_LEN (gptPing))))
		{
			TRACE(("ERROR: Ping Buffer: Out of memory"));
		}
	}
  if (NULL == gptPing->pBufferCk)
	{
		if (NULL == (gptPing->pBufferCk = malloc (gptPing->lBufCkLen)))
		{
			TRACE(("ERROR: Ping cktab: Out of memory"));
		}
		else
		{
			memset (gptPing->pBufferCk, 0, gptPing->lBufCkLen);
		}
	}
  return bSuccess;
}



/*
** ======================================================================================
** ======================================================================================
*/
static int PgSetData (void *pData, size_t lOffset, size_t lLength)
{
	BooleanT	bSuccess = TRUE;

  if (TRUE != PgSetBuffer ())
	{
		TRACE(("ERROR: Failed to set Ping Buffer-1"));
		bSuccess = FALSE;
	}
	else
	{
		if (gptPing->lBufLen < lOffset + lLength)
		{
			bSuccess = FALSE;
		}
		else
		{
			icmphdr_t *ptHdrIcmp = (icmphdr_t *) gptPing->pBuffer;

			memcpy (ptHdrIcmp->icmp_data + lOffset, pData, lLength);
		}
	}
  return bSuccess;
}





/*
** ======================================================================================
** ======================================================================================
*/
static int PgXmit ()
{
	if (TRUE != PgSetBuffer ())
	{
		TRACE(("ERROR: Failed to set Ping Buffer-2"));
	}
	else
	{
		long	lBufLen = 8 + gptPing->lBufLen;

		PG_C_CLR (gptPing, gptPing->lNumXmit % gptPing->lBufCkLen);

		PgEncode (gptPing->pBuffer, lBufLen, ICMP_ECHO, gptPing->lIdent, gptPing->lNumXmit);

		if (-1 == sendto (
							gptPing->lSock, 
							(char *) gptPing->pBuffer, 
							lBufLen, 
							0, 
							(struct sockaddr*) &gptPing->tSockAddrDest, 
							sizeof (struct sockaddr_in)))
		{
			TRACE(("WARN: Failed to send echo"));
		}
		else
		{
			gptPing->lNumXmit++;
		}
	}
  return 0;
}


/*
** ======================================================================================
** ======================================================================================
*/
static int PgRecv ()
{
  int 			lLegnth 		= sizeof (gptPing->tSockAddrPeer);
  int 			lRC					= 0;
	int				lBytesRecv 	= 0;
  icmphdr_t *ptHdrIcmp	= NULL;
  struct ip *ptIp				= NULL;

  if (-1 == (lBytesRecv = recvfrom (
													gptPing->lSock, 
													(char *) gptPing->pBuffer, 
													PG_BUFFER_LEN (gptPing), 
													0,
													(struct sockaddr *) &gptPing->tSockAddrPeer, 
													&lLegnth)))
  {
		TRACE(("WARN: Failed to recvfrom"));
  }
	else if (-1 == (lRC = PgDecode (gptPing->pBuffer, lBytesRecv, &ptIp, &ptHdrIcmp)))
	{
		TRACE(("WARN: Recv short packet"));
	}
	else
	{
		switch (ptHdrIcmp->icmp_type)
		{
			case ICMP_ECHOREPLY:
			case ICMP_TIMESTAMPREPLY:
			case ICMP_ADDRESSREPLY:
			{
				if (ptHdrIcmp->icmp_id != gptPing->lIdent)
				{
					return -1;
				}
				gptPing->lNumRecv++;

				if (PG_C_TST (gptPing, ptHdrIcmp->icmp_seq % gptPing->lBufCkLen))
				{
					gptPing->lNumRecv--;
				}
				else
				{
					PG_C_SET (gptPing, ptHdrIcmp->icmp_seq % gptPing->lBufCkLen);
				}
				break;
			}
		}
	} 
  return 0;
}




/*
** ======================================================================================
** ======================================================================================
*/
static int PgSendEcho ()
{
  int lOffSet = 0;
	int	lRC			= 0;
	
	glProbeCount++;
	
  if (glBufLen >= PG_HDR_LEN)
	{
		struct timeval tTime;
		gettimeofday (&tTime, NULL);
		PgSetData (&tTime, 0, sizeof (tTime));
		lOffSet += sizeof (tTime);
	}
  if (NULL != gpBuffer)
	{
		PgSetData (gpBuffer, lOffSet, glBufLen > PG_HDR_LEN ?  glBufLen-PG_HDR_LEN : glBufLen);
	}
  lRC = PgXmit ();
	
  return lRC;
}

  


/*
** ======================================================================================
** ======================================================================================
*/
static int PgRun ()
{
  fd_set 		tFdSet;
  int 			lSockMax	= gptPing->lSock + 1;
  BooleanT	bDone			= FALSE;
  struct 		timeval 	tTimeOut;
  struct 		timeval 	tTimeNow;
  struct 		timeval 	tTimeInt;
	struct 		timeval 	tTimeLast;
  struct 		timeval 	*ptTime = NULL;
	

	tTimeInt.tv_sec 	= gptPing->lInterval;
	tTimeInt.tv_usec 	= 0;
  
  gettimeofday (&tTimeLast, NULL);

  PgSendEcho ();

  while (4 > glProbeCount)
	{
		int lRC = -1;

		FD_ZERO (&tFdSet);
		FD_SET  (gptPing->lSock, &tFdSet);

		gettimeofday (&tTimeNow, NULL);

		tTimeOut.tv_sec 	= tTimeLast.tv_sec  + tTimeInt.tv_sec  - tTimeNow.tv_sec;
		tTimeOut.tv_usec 	= tTimeLast.tv_usec + tTimeInt.tv_usec - tTimeNow.tv_usec;

		while (tTimeOut.tv_usec < 0)
		{
			tTimeOut.tv_usec += 1000000;
			tTimeOut.tv_sec--;
		}
		while (tTimeOut.tv_usec >= 1000000)
		{
			tTimeOut.tv_usec -= 1000000;
			tTimeOut.tv_sec++;
		}
		if (tTimeOut.tv_sec < 0)
		{
			tTimeOut.tv_sec = tTimeOut.tv_usec = 0;
		}

		if (-1 == (lRC = select (lSockMax, &tFdSet, NULL, NULL, &tTimeOut)))
		{
			if (errno != EINTR) 
			{
				TRACE(("WARN: Failed to select Socket"));
				continue;
			}
		}
		else if (1 == lRC)
		{
			PgRecv ();

			if (NULL == ptTime)
			{
				gettimeofday (&tTimeNow, NULL);
				ptTime = &tTimeNow;
			}
			if (gptPing->lCount && gptPing->lNumRecv >= gptPing->lCount) 
			{
				break;
			}
		}
		else
		{
			if (! gptPing->lCount || gptPing->lNumRecv < gptPing->lCount)
			{
				PgSendEcho ();
			}
			else if (TRUE == bDone)
			{
				break;
			}
			else
			{
				bDone = TRUE;

				tTimeInt.tv_sec = MAX_WAIT_SECS;
			}
			gettimeofday (&tTimeLast, NULL);
		}
	}
  PgComplete ();

  return 0;
}



/*
** ======================================================================================
** ======================================================================================
*/
static PingT *PgInitialization (int lType, int lIdent)
{
  struct 	protoent *ptProtocol = NULL;
  int 		lSock			= -1;
  PingT 	*ptPing		= NULL;

	if (NULL == (ptProtocol = getprotobyname ("icmp"))) 
	{
		TRACE(("ERROR: Failed to get Protocol ICMP from /etc/protocols file"));
	}
	else if (-1 == (lSock = socket (AF_INET, SOCK_RAW, ptProtocol->p_proto)))
	{
		TRACE(("ERROR: Failed to open socket"));
	}
	else if (NULL == (ptPing = malloc (sizeof (PingT))))
	{
		TRACE(("ERROR: Failed to allocate memory for Ping Structure"));
		close (lSock);
	}
	else
	{
		memset (ptPing, 0, sizeof (PingT));

		ptPing->lSock 		= lSock;
		ptPing->lType 		= lType;
		ptPing->lCount 		= 3;
		ptPing->lInterval	= 3;
		ptPing->lBufLen  	= glBufLen;
		ptPing->lIdent 		= lIdent & 0xffff;
		ptPing->lBufCkLen = PING_CKTABSIZE;
	}
  return ptPing;
}



/*
** ======================================================================================
** ======================================================================================
*/
static BooleanT SetDestination (char *pstrHost)
{
	BooleanT bSuccess = TRUE;
  struct 	sockaddr_in *ptSockAddr = &gptPing->tSockAddrDest;

  ptSockAddr->sin_family = AF_INET;

  if (0 != inet_aton (pstrHost, &ptSockAddr->sin_addr))
	{
		gptPing->pstrHost = strdup (pstrHost);
	}
  else
	{
		struct hostent *ptHost = gethostbyname (pstrHost);

		if (NULL == ptHost)
		{
			TRACE(("ERROR: Failed to gethostbyname Host: %s", pstrHost));
			bSuccess = FALSE;
		}
		else
		{
			ptSockAddr->sin_family = ptHost->h_addrtype;

			if (ptHost->h_length > (int) sizeof (ptSockAddr->sin_addr))
			{
				ptHost->h_length = sizeof (ptSockAddr->sin_addr);
			}
			memcpy (&ptSockAddr->sin_addr, ptHost->h_addr, ptHost->h_length);

			gptPing->pstrHost = strdup (ptHost->h_name);
		}
	}
	return bSuccess;
}




/*
** ======================================================================================
** ======================================================================================
*/
extern BooleanT DrPg (char *pstrHost)
{
	TRACE(("DRPG begin with host: %s", pstrHost));

	if (NULL == (gpBuffer = malloc (glBufLen)))
	{
		TRACE(("ERROR: Failed to allocate memory for Ping Buffer"));
	}
	else if (NULL == (gptPing = PgInitialization (ICMP_ECHO, getpid ())))
	{
		TRACE(("ERROR: Failed to Initialize Ping Data Structure"));
	}
	else
	{
		int 	lSockOpt		= 1;
		long	i 					= 0;
		
		gbSuccess			= FALSE;
		glProbeCount	= 0;
		
		setuid (getuid ());

		for (i=0; i<glBufLen; i++) 
		{
			gpBuffer[i] = i;
		}
		if (-1 == setsockopt (gptPing->lSock, SOL_SOCKET, SO_BROADCAST, (char*) &lSockOpt, sizeof (lSockOpt)))
		{
			TRACE(("ERROR: Failed to set Socket Opt to Broadcast"));
		}
		else if (TRUE != SetDestination (pstrHost))
		{
			TRACE(("ERROR: Failed to set Destination IP: %s", pstrHost));
		}
		else
		{
			PgRun ();
		}
	}
	free (gpBuffer);

	TRACE(("DRPG End: %s", gbSuccess ? "SUCCESS" : "ERROR"));

	return gbSuccess;
}


