static const char drprobe_c[] = "$Header: /faststuff/CVS/dastuff190503/vaultos/unix/linux/x86/config/drprobe.c,v 1.12 2007/01/16 17:59:42 sreenivasa Exp $";

#include <ncurses.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/mount.h>
#include <unistd.h>
#include <dirent.h>
#include <fcntl.h>
#include <net/if.h>
#include <errno.h>
#include <curses.h>
#include "kudzu.h"
#include "drtrace.h"
#include "drmod.h"
#include "drloadmod.h"

MODULE("DRPROBE")

#define TRUE									1
#define FALSE									0
#define	MAX_NUM_IF						10

#define STR_BAKBONE				"BakBone Software, Inc."
#define STR_TITLE					"Module Drivers Installation"
#define STR_NO_DEVICE			"Unable to detect hardware. Manual loading drivers required"
#define STR_CONFIG_IF     "Configuration Interface (%s)"
#define STR_AVAIL_NIC     "Available Network Interfaces"
#define STR_OK						"[  OK  ]"
#define STR_NIC						"[  NIC   ]" 
#define STR_SCSI					"[  SCSI  ]" 
#define STR_EXIT					"[  Exit  ]"
#define STR_CONFIG        "[ Config ]"
#define STR_AUTO          "[  Auto  ]"
#define STR_CANCEL        "[ Cancel ]"


typedef unsigned char			ByteT;

static ModListT		gtList [MAX_MOD_ARRAY_SIZE];
static long				glCount		= 0;
static BooleanT		gbExit		= FALSE;
static BooleanT		gbLoad		= FALSE;
static attr_t			gtAttribute[6];


extern BooleanT DrIfcfg 	(char *);
extern BooleanT DrIfRoute (char *);
extern BooleanT DrPg 			(char *);


/*
** =======================================================================================
** =======================================================================================
*/
static void WinInit ()
{
  WINDOW 	*pWin = NULL;
  int 		lRC 	= 0;
  int 		i 		= 0;

  pWin = initscr ();

  if (NULL != pWin)
	{
		lRC = has_colors ();

		if (0 != lRC)
		{
			start_color ();

			init_pair (1, COLOR_WHITE, 	COLOR_WHITE);
			init_pair (2, COLOR_BLACK, 	COLOR_WHITE);
			init_pair (3, COLOR_WHITE, 	COLOR_RED);
			init_pair (4, COLOR_BLUE, 	COLOR_WHITE);
			init_pair (5, COLOR_YELLOW, COLOR_BLUE);
			init_pair (6, COLOR_WHITE, 	COLOR_BLUE);

			for (i = 0; i < 6; i++)
			{
				gtAttribute[i] = COLOR_PAIR (i + 1) | ((!(i & 1)) ? A_BOLD : 0);
			}
		}
		else
		{
			gtAttribute[0] = gtAttribute[1] = gtAttribute[3] =
			gtAttribute[4] = gtAttribute[5] = A_NORMAL;
			gtAttribute[2] = A_REVERSE;
		}
	}
}




/*
** =======================================================================================
** =======================================================================================
*/
static void DisplayInfo (char *pstrMsg)
{
	WINDOW		*pWin				= NULL;
	int				lX					= strlen (pstrMsg) + 10;
	int				lY					= 5;
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	int				i						= 0;

	clear 		();
	refresh		();
	getmaxyx 	(stdscr, lMaxY, lMaxX);

	pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);
	
	wborder (pWin, 
					ACS_VLINE 		| gtAttribute[0], 
					ACS_VLINE 		| gtAttribute[1], 
					ACS_HLINE 		| gtAttribute[0], 
					ACS_HLINE 		| gtAttribute[1], 
					ACS_ULCORNER 	| gtAttribute[0], 
					ACS_URCORNER 	| gtAttribute[0], 
					ACS_LLCORNER 	| gtAttribute[1], 
					ACS_LRCORNER 	| gtAttribute[1]);

	wattrset (pWin, gtAttribute[1]);

	for (i=1; i<lY-1; i++)
	{
		wmove 	(pWin, i, 1);
		whline 	(pWin, ' ', lX-2);
	}
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 2, (lX-strlen (pstrMsg)) / 2, pstrMsg);
	wrefresh	(pWin);
	redrawwin (pWin);
}


/*
** =======================================================================================
** QueryUserLoadDrivers - Asks the user whether they would like VaultOS to scan
** for the hardware, or whether they would like to manually load them.
** =======================================================================================
*/
static BooleanT QueryUserLoadDrivers ()
{
	WINDOW		*pWin				= NULL;
	int				lX					= 70;
	int				lY					= 7;
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	int				i						= 0;
	BooleanT	bExit				= FALSE;
	BooleanT	bLoadDrivers= TRUE;	/* Default to auto-load */
	char     *pstrMsg 		= "Automatically Scan for Drivers and Load?";

	clear 		();
	refresh		();
	getmaxyx 	(stdscr, lMaxY, lMaxX);

	pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);
	
	wborder (pWin, 
					ACS_VLINE 		| gtAttribute[0], 
					ACS_VLINE 		| gtAttribute[1], 
					ACS_HLINE 		| gtAttribute[0], 
					ACS_HLINE 		| gtAttribute[1], 
					ACS_ULCORNER 	| gtAttribute[0], 
					ACS_URCORNER 	| gtAttribute[0], 
					ACS_LLCORNER 	| gtAttribute[1], 
					ACS_LRCORNER 	| gtAttribute[1]);

	wattrset (pWin, gtAttribute[1]);

	/* Draw the bounding box */
	for (i=1; i<lY-1; i++)
	{
		wmove 	(pWin, i, 1);
		whline 	(pWin, ' ', lX-2);
	}
	/* Set the 'Cancel' Button as initially highlighted */
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 2, (lX-strlen (pstrMsg)) / 2, pstrMsg);
	wmove 		(pWin, 3, 1);
	whline 		(pWin, ACS_HLINE, lX-2);
	wattrset 	(pWin, gtAttribute[2]);
	mvwaddstr (pWin, 5, (lX/3) - (strlen (STR_OK)/2), STR_OK);
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 5, (lX*2/3) - (strlen (STR_CANCEL)/2), STR_CANCEL);

	noecho 		();
	cbreak 		();
	keypad 		(pWin, 1);
	/* Loop until user hits the 'ENTER' key*/
	do
	{
		int lRC = wgetch(pWin);
		wrefresh 		(pWin);
		redrawwin 	(pWin);

		switch (lRC)
		{
			case '\t':
			case KEY_RIGHT:
			{
				bLoadDrivers = FALSE;
				break;
			}
			case KEY_LEFT:
			{
				bLoadDrivers = TRUE;
				break;
			}
			case '\n':
			{
				bExit	= TRUE;
				break;
			}
		}
		if (TRUE == bLoadDrivers)
			wattrset 	(pWin, gtAttribute[2]);
		else
			wattrset 	(pWin, gtAttribute[3]);
		wmove 		(pWin, 5, (lX/3) - (strlen (STR_OK)/2));
		whline 		(pWin, ' ', strlen (STR_OK));
		mvwaddstr (pWin, 5, (lX/3) - (strlen (STR_OK)/2), STR_OK);

		if (TRUE == bLoadDrivers)
			wattrset 	(pWin, gtAttribute[3]);
		else
			wattrset 	(pWin, gtAttribute[2]);
		wattrset 	(pWin, gtAttribute[bLoadDrivers?3:2]);
		wmove 		(pWin, 5, (lX*2/3) - (strlen (STR_CANCEL)/2));
		whline 		(pWin, ' ', strlen (STR_CANCEL));
		mvwaddstr (pWin, 5, (lX*2/3) - (strlen (STR_CANCEL)/2), STR_CANCEL);
	} while (TRUE != bExit);

	wrefresh	(pWin);
	redrawwin (pWin);
	return bLoadDrivers;
}




/*
** =======================================================================================
** =======================================================================================
*/
static void DisplayMsg (char *pstrMsg)
{
	WINDOW		*pWin				= NULL;
	int				lX					= strlen (pstrMsg) + 10;
	int				lY					= 7;
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	int				RC					= 0;
	int				i						= 0;

	clear 		();
	refresh		();
	getmaxyx 	(stdscr, lMaxY, lMaxX);

	pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);
	
	wborder (pWin, 
					ACS_VLINE 		| gtAttribute[0], 
					ACS_VLINE 		| gtAttribute[1], 
					ACS_HLINE 		| gtAttribute[0], 
					ACS_HLINE 		| gtAttribute[1], 
					ACS_ULCORNER 	| gtAttribute[0], 
					ACS_URCORNER 	| gtAttribute[0], 
					ACS_LLCORNER 	| gtAttribute[1], 
					ACS_LRCORNER 	| gtAttribute[1]);

	wattrset (pWin, gtAttribute[1]);

	for (i=1; i<lY-1; i++)
	{
		wmove 	(pWin, i, 1);
		whline 	(pWin, ' ', lX-2);
	}
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 2, (lX-strlen (pstrMsg)) / 2, pstrMsg);
	wattrset 	(pWin, gtAttribute[2]);
	mvwaddstr (pWin, 4, (lX-strlen (STR_OK)) /2, STR_OK);
		
	do
	{
		RC = wgetch	(pWin);
	} while (RC != '\n');
	
	clear 	();
	refresh	();
	delwin 	(pWin);
}




/*
** =======================================================================================
** =======================================================================================
*/
static void PrintUsage ()
{
	fprintf(stderr,"\n");
	fprintf(stderr,"Usage: drprobe [-n | -l | -v]\n");
	fprintf(stderr,"       -n Do not load device drivers if found\n");
	fprintf(stderr,"       -l Load device drivers if found\n");
	fprintf(stderr,"       -p Print all devices\n");
	fprintf(stderr,"       -v Display version of drprobe\n\n");
	exit (-1);
}




/*
** =======================================================================================
** =======================================================================================
*/
static long CfgGetNegotiationTime ()
{
  long  lRC = 1;
  int   lFd = open ("/bin/drnic.cfg", O_RDONLY);
  char  strBuf[16];

  memset (strBuf, 0, sizeof (strBuf));

  if (-1 != lFd && -1 != read (lFd, strBuf, sizeof (strBuf)))
  {
    lRC = atoi (strBuf);
    close (lFd);
  }
  TRACE(("CfgGetNegotiationTime: %ld", lRC));

  return lRC;
}





/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT GetBcastAddr (char *pstrIp, char *pstrMask, char *pstrBcast)
{
  BooleanT  bSuccess	= FALSE;
  long      lIpA    	= 0;
  long      lIpB    	= 0;
  long      lIpC    	= 0;
  long      lIpD    	= 0;

	TRACE(("Get Bcast for IP: %s  NetMask: %s", pstrIp, pstrMask));

  if (4 != sscanf (pstrIp, "%ld.%ld.%ld.%ld", &lIpA, &lIpB, &lIpC, &lIpD))
  {
    TRACE(("ERROR: Failed to sscanf to convert IP to Bcast"));
  }
  else
  {
    long  lIp = (lIpA << 24) | (lIpB << 16) | (lIpC << 8) | (lIpD);

    if (4 != sscanf (pstrMask, "%ld.%ld.%ld.%ld", &lIpA, &lIpB, &lIpC, &lIpD))
    {
      TRACE(("ERROR: Failed to sscanf to convert NetMask to Bcast"));
    }
    else
    {
      long  lMask   = (lIpA << 24) | (lIpB << 16) | (lIpC << 8) | (lIpD);
      long  lBcast  = ~lMask | lIp;

      sprintf (pstrBcast,"%ld.%ld.%ld.%ld",
              ((lBcast >> 24) & 0xff),
              ((lBcast >> 16) & 0xff),
              ((lBcast >> 8)  & 0xff),
              (lBcast & 0xff));

      bSuccess = TRUE;
    }
  }
	TRACE(("End get Bcast for IP: %s  NetMask: %s. %s", 
	pstrIp, pstrMask, bSuccess ? "SUCCESS" : "ERROR"));

  return bSuccess;
}




/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT ValidateIfParm (char *pstr)
{
  int   bSuccess  = FALSE;
  long  lIpA   		= 0;
  long  lIpB   		= 0;
  long  lIpC   		= 0;
  long  lIpD   		= 0;

	TRACE(("Validate Interface Parm: %s", pstr));

  if (4 == sscanf (pstr, "%ld.%ld.%ld.%ld", &lIpA, &lIpB, &lIpC, &lIpD))
  {
    if (0x00 <= lIpA && 0xff >= lIpA &&
        0x00 <= lIpB && 0xff >= lIpB &&
        0x00 <= lIpC && 0xff >= lIpC &&
        0x00 <= lIpD && 0xff >= lIpD)
    {
      bSuccess = TRUE;
    }
  }
	TRACE(("End validate Interface Parm: %s. %s", 
	pstr, bSuccess ? "SUCCESS" : "ERROR"));

  return bSuccess;
}





/*
** =======================================================================================
** =======================================================================================
*/
static long GetNicInterfaces ()
{
	long	lInt	= 0;
	FILE	*pFd	= fopen ("/proc/net/dev", "r");

	TRACE(("Get all NIC interfaces from file: /proc/net/dev"));

	if (NULL == pFd)
	{
		TRACE(("ERROR: Failed to open file /proc/net/dev. %s", strerror(errno)));
	}
	else
	{
		char	strBuf[256] = {0};
		char	*pstr	= NULL;

		while (NULL != fgets (strBuf, sizeof (strBuf), pFd))
		{
			pstr = (char *) strtok (strBuf, " ");

			if (NULL != pstr && NULL != strstr (pstr, "eth"))
			{
				TRACE(("Got NIC interface: eth%ld", lInt));
				lInt++;
			}
		}
		fclose (pFd);
	}
	TRACE(("Found (%ld) NIC interfaces from /proc/net/dev. %s",
	lInt, (lInt > 0) ? "SUCCESS" : "ERROR"));

	return lInt;
}



/*
** =======================================================================================
** Checks to see if a device's appropriate module is already marked for loading.
** If not, then it creates a new entry in 'gtList' to get it loaded.
** =======================================================================================
*/
static void LoadStateData (struct device *ptDev)
{
	int				i 			= 0;
	BooleanT	bExist	= FALSE;

	for (i=0; i<glCount && TRUE != bExist; i++)
	{
		if (0 == strcmp (gtList[i].pstrMods[0], ptDev->driver))
		{
			bExist = TRUE;
		}
	}

	if (TRUE != bExist)
	{
		TRACE(("WARNING: Couldn't find this module in our drnic.txt or drscsi.txt"));
		if (NULL == ptDev->desc)
		{
			gtList[glCount].pstrDesc = "Unknown Description";
		}
		else
		{
			NULL == (gtList[glCount].pstrDesc = (char *) malloc (strlen (ptDev->desc) + 1)) ?
			gtList[glCount].pstrDesc = "No memory" :
			strcpy (gtList[glCount].pstrDesc, ptDev->desc);
		}
		NULL == (gtList[glCount].pstrMods[0] = (char *) malloc (strlen (ptDev->driver) + 1)) ?
		gtList[glCount].pstrMods[0] = "No memory" :
		strcpy (gtList[glCount].pstrMods[0], ptDev->driver);
			
		gtList[glCount].bModNet 	= (CLASS_NETWORK == ptDev->type);
		gtList[glCount].lRC				= DRMOD_ERR_FATAL;
		gtList[glCount].lModCount	= 1;

		TRACE(("Index: %ld  Module: %s  Desc: %s",
			glCount,
			gtList[glCount].pstrMods[0],
			gtList[glCount].pstrDesc));
		glCount++;
	}
}



/*
** =======================================================================================
** =======================================================================================
 */
static void AutoLoadMiscModules()
{
	struct    dirent  *pDirEnt  = NULL;
	DIR       *pDir             = NULL;
	
	DisplayInfo ("Auto loading basic drivers. Please wait...");
	TRACE(("Enter AutoLoadMiscModules"));
	if (NULL == (pDir = opendir ("/mods/autoload")))
	{
		TRACE(("ERROR: Failed to opendir /mods/autoload"));
	}
	else
	{
		while (NULL != (pDirEnt = readdir (pDir)))
		{
			if (0 != strcmp (pDirEnt->d_name, ".")  &&
					0 != strcmp (pDirEnt->d_name, "..") &&
					0 != strstr (pDirEnt->d_name, DR_MOD_EXT))
			{
				char strModFile[256] = {0};
				int  lRC             = DRMOD_ERR_FATAL;

				TRACE(("Loading device driver '%s' ...", pDirEnt->d_name));
				snprintf(strModFile, 256, "%s", pDirEnt->d_name);
				if (TRUE != DrCmdLoadModule (strModFile, &lRC) && DRMOD_ERR_FATAL == lRC)
				{
					TRACE(("ERROR: Failed to load module: %s", strModFile));
				}
				else if (DRMOD_ERR_EXIST == lRC)
				{
					TRACE(("Module already loaded: %s", strModFile));
				}
				else if (DRMOD_ERR_NONE == lRC)
				{
					TRACE(("Loaded module successfully: %s", strModFile));
				}
			}
		}
	}
	TRACE(("Exit AutoLoadMiscModules"));
}



/*
** =======================================================================================
** =======================================================================================
*/
static void ProbeAndLoadModule ()
{
	struct device **pptDev	= NULL;
	
	gbLoad ?
	DisplayInfo ("Auto detecting hardware and loading device drivers. Please wait...") :
	DisplayInfo ("Auto detecting hardware device drivers. Please wait...");

	TRACE(("Begin ProbeAndLoadModule: Auto probing devices"));

	if (NULL == (pptDev = probeDevices (CLASS_UNSPEC, BUS_UNSPEC, PROBE_ALL)))
	{
		TRACE(("Unable to auto probe hardware devices"));
		DisplayMsg ("Unable to detect hardware configuration");
	}
	else
	{
		int			lRC	= DRMOD_ERR_FATAL;
		int 		i		= 0;
		char		strModFile[64];
		char		strMsg[128];
		struct 	device	*ptDev = NULL;

		for (i=0; (ptDev = pptDev[i]); i++)
		{
			TRACE(("mod=%s; class=%ld; desc=%s", ptDev->driver, ptDev->type, ptDev->desc));

			if (CLASS_NETWORK == ptDev->type || CLASS_SCSI == ptDev->type || CLASS_RAID == ptDev->type)
			{
				TRACE(("LOAD mod=%s", ptDev->driver));
				LoadStateData (ptDev);
			}
			else
			{
				TRACE(("IGNORE mod=%s", ptDev->driver));
			}
		}

		if (TRUE == gbLoad)
		{
			for (i=0; i<glCount; i++)
			{
				memset (strModFile, 0, sizeof (strModFile));
				memset (strMsg, 		0, sizeof (strMsg));

				
				if (TRUE == gtList[i].bModNet)
				{
					sprintf (strModFile, "%s%s", gtList[i].pstrMods[0], DR_MOD_EXT);
					snprintf(strMsg,70,"Loading (%s) %s", gtList[i].pstrMods[0], gtList[i].pstrDesc);
					DisplayInfo (strMsg);
					sleep(1);

					if (TRUE != DrCmdLoadModule (strModFile, &lRC) && DRMOD_ERR_FATAL == lRC)
					{
						TRACE(("ERROR: Failed to load module: %s", strModFile));
					}
					else if (DRMOD_ERR_EXIST == lRC)
					{
						TRACE(("Module already loaded: %s", strModFile));
					}
					else if (DRMOD_ERR_NONE == lRC)
					{
						TRACE(("Load module successfully: %s", strModFile));
					}
				}
				else
				{
					if (0 == strcmp (gtList[i].pstrMods[0], "mptscsih"))
					{
						BooleanT	bFirst = FALSE;
						
						snprintf(strMsg,70,"Loading (%s) %s", gtList[i].pstrMods[0], gtList[i].pstrDesc);
						DisplayInfo (strMsg);
						sleep(1);

						sprintf(strModFile, "%s%s", "mptbase", DR_MOD_EXT);
					
						if (TRUE != DrCmdLoadModule (strModFile, &lRC) && DRMOD_ERR_FATAL == lRC)
						{
							TRACE(("ERROR: Failed to load module: %s", strModFile));
						}
						else if (DRMOD_ERR_EXIST == lRC)
						{
							TRACE(("Module already loaded: %s", strModFile));
						}
						else if (DRMOD_ERR_NONE == lRC)
						{
							TRACE(("Load module successfully: %s", strModFile));
							bFirst = TRUE;
						}

						if (TRUE == bFirst)
						{
							sprintf(strModFile, "%s%s", "mptscsih", DR_MOD_EXT);

							if (TRUE != DrCmdLoadModule (strModFile, &lRC) && DRMOD_ERR_FATAL == lRC)
							{
								TRACE(("ERROR: Failed to load module: %s", strModFile));
							}
							else if (DRMOD_ERR_EXIST == lRC)
							{
								TRACE(("Module already loaded: %s", strModFile));
							}
							else if (DRMOD_ERR_NONE == lRC)
							{
								TRACE(("Load module successfully: %s", strModFile));
							}
						}
					}
					else if (0 == strcmp (gtList[i].pstrMods[0], "megaraid_mbox"))
					{
						BooleanT	bFirst = FALSE;
						
						snprintf(strMsg,70,"Loading (%s) %s", gtList[i].pstrMods[0], gtList[i].pstrDesc);
						DisplayInfo (strMsg);
						sleep(1);

						sprintf(strModFile, "%s%s", "megaraid_mm", DR_MOD_EXT);
					
						if (TRUE != DrCmdLoadModule (strModFile, &lRC) && DRMOD_ERR_FATAL == lRC)
						{
							TRACE(("ERROR: Failed to load module: %s", strModFile));
						}
						else if (DRMOD_ERR_EXIST == lRC)
						{
							TRACE(("Module already loaded: %s", strModFile));
						}
						else if (DRMOD_ERR_NONE == lRC)
						{
							TRACE(("Load module successfully: %s", strModFile));
							bFirst = TRUE;
						}

						if (TRUE == bFirst)
						{
							sprintf(strModFile, "%s%s", "megaraid_mbox", DR_MOD_EXT);

							if (TRUE != DrCmdLoadModule (strModFile, &lRC) && DRMOD_ERR_FATAL == lRC)
							{
								TRACE(("ERROR: Failed to load module: %s", strModFile));
							}
							else if (DRMOD_ERR_EXIST == lRC)
							{
								TRACE(("Module already loaded: %s", strModFile));
							}
							else if (DRMOD_ERR_NONE == lRC)
							{
								TRACE(("Load module successfully: %s", strModFile));
							}
						}
					}
					else
					{
						sprintf(strModFile, "%s%s", gtList[i].pstrMods[0], DR_MOD_EXT);
						snprintf(strMsg,70,"Loading (%s) %s", gtList[i].pstrMods[0], gtList[i].pstrDesc);
						DisplayInfo (strMsg);
						sleep(1);
						
						if (TRUE != DrCmdLoadModule (strModFile, &lRC) && DRMOD_ERR_FATAL == lRC)
						{
							TRACE(("ERROR: Failed to load module: %s", strModFile));
						}
						else if (DRMOD_ERR_EXIST == lRC)
						{
							TRACE(("Module already loaded: %s", strModFile));
						}
						else if (DRMOD_ERR_NONE == lRC)
						{
							TRACE(("Load module successfully: %s", strModFile));
						}
					}
				}
				gtList[i].lRC = lRC;
			}
		}
	}
	TRACE(("End ProbeAndLoadModule"));
}





/*
** =======================================================================================
** =======================================================================================
*/
static void DisplayMenu (WINDOW *pWin, int lX, int lY) 
{
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	int				lRC					= 0;
	int				lKeyPos			= 1;
	BooleanT	bExit				= FALSE;

	getmaxyx 	(pWin, lMaxY, lMaxX);
	noecho 		();
	keypad 		(pWin, 1);
	cbreak 		();

	do
	{
		wrefresh 		(pWin);
		redrawwin 	(pWin);
		
		/* Highlight the different buttons depending on what lKepPos is pointing to */
		if (lKeyPos == 1)
			wattrset 	(pWin, gtAttribute[2]); // Turn on NIC button
		else 
			wattrset 	(pWin, gtAttribute[3]); // Turn off NIC button
		wmove 		(pWin, 20, (lX/4) - (strlen (STR_NIC)/2) - 5);
		whline 		(pWin, ' ', strlen (STR_NIC));
		mvwaddstr (pWin, 20, (lX/4) - (strlen (STR_NIC)/2) - 5, STR_NIC);
				
		if (lKeyPos == 2)
			wattrset 	(pWin, gtAttribute[2]); // Turn on SCSI button
		else 
			wattrset 	(pWin, gtAttribute[3]); // Turn off SCSI button
		wmove 		(pWin, 20, (lX/2) - (strlen (STR_SCSI)/2));
		whline 		(pWin, ' ', strlen (STR_SCSI));
		mvwaddstr (pWin, 20, (lX/2) - (strlen (STR_SCSI)/2), STR_SCSI);
		
		if (lKeyPos == 3)
			wattrset 	(pWin, gtAttribute[2]); // Turn on EXIT button
		else 
			wattrset 	(pWin, gtAttribute[3]); // Turn off EXIT button
		wmove 		(pWin, 20, (3*lX/4) - (strlen (STR_EXIT)/2) + 5);
		whline 		(pWin, ' ', strlen (STR_EXIT));
		mvwaddstr (pWin, 20, (3*lX/4) - (strlen (STR_EXIT)/2) + 5, STR_EXIT);

		/* Get the user input*/
		lRC = wgetch(pWin);
		switch (lRC)
		{
			case '\t':
			case KEY_RIGHT:
			{
				lKeyPos++;
				if (lKeyPos > 3)
					lKeyPos = 1;
				break;
			}
			
			case KEY_LEFT:
			{
				lKeyPos--;
				if (lKeyPos < 1)
					lKeyPos = 3;
				break;
			}

			case '\n':
			{
				switch (lKeyPos)
				{
					case 1:
					{
						glCount = BeginNICUserGUI(gtList, glCount);
						TRACE(("Finished Manual Loading of NIC modules.")); 
						break;
					}
					case 2:
					{
						glCount = BeginSCSIUserGUI(gtList, glCount);
						TRACE(("Finished Manual Loading of SCSI modules.")); 
						break;
					}
					case 3:
					{
						gbExit = TRUE;
						break;
					}
				}
				bExit	= TRUE;
				break;
			}
		}

	} while (TRUE != bExit);
}



/*
** =======================================================================================
** =======================================================================================
*/
static void DisplayMainMenu ()
{
	WINDOW	*pWin			= NULL;
	int			lX				= 78;
	int			lY				= 22;
	int			lPos			= 1;
	int			lMaxX			= 0;
	int			lMaxY			= 0;
	int			i					= 0;

	while (TRUE != gbExit)
	{
		lPos = 1;

		clear 	 ();
		refresh	 ();
		getmaxyx (stdscr, lMaxY, lMaxX);

		pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);

		wborder (pWin, 
						ACS_VLINE 		| gtAttribute[0], 
						ACS_VLINE 		| gtAttribute[1], 
						ACS_HLINE 		| gtAttribute[0], 
						ACS_HLINE 		| gtAttribute[1], 
						ACS_ULCORNER 	| gtAttribute[0], 
						ACS_URCORNER 	| gtAttribute[0], 
						ACS_LLCORNER 	| gtAttribute[1], 
						ACS_LRCORNER 	| gtAttribute[1]);

		wattrset (pWin, gtAttribute[1]);

		for (i=1; i<lY-1; i++)
		{
			wmove 	(pWin, i, 1);
			whline 	(pWin, ' ', lX-2);
		}
		wattrset 	(pWin, gtAttribute[3]);
		mvwaddstr (pWin, lPos++, (lX/2) - (strlen (STR_BAKBONE) / 2), STR_BAKBONE);
		mvwaddstr (pWin, lPos++, (lX/2) - (strlen (STR_TITLE) / 2), STR_TITLE);
		wattrset 	(pWin, gtAttribute[1]);
		wmove 		(pWin, lPos++, 1);
		whline 		(pWin, ACS_HLINE, lX-2);
		
		if (0 == glCount)
		{
			TRACE(("glCount = %ld", glCount)); 
			mvwaddstr (pWin, 1+lPos++, (lX/2) - (strlen (STR_NO_DEVICE) / 2), STR_NO_DEVICE);
		}
		else
		{
			char strMsg[128];

			if (glCount > 13)
			{
				glCount = 13;
			}

			for (i=0; i<glCount; i++)
			{
				memset (strMsg,	0, sizeof (strMsg));

				if (TRUE == gbLoad)
				{
					if (DRMOD_ERR_NONE == gtList[i].lRC)
					{
						snprintf(strMsg,70,"[Success] Mod: (%s) %s", gtList[i].pstrMods[0], gtList[i].pstrDesc);
					}
					else if (DRMOD_ERR_EXIST == gtList[i].lRC)
					{
						snprintf(strMsg,70,"[Loaded] Mod: (%s) %s", gtList[i].pstrMods[0], gtList[i].pstrDesc);
					}
					else if (DRMOD_ERR_FATAL == gtList[i].lRC)
					{
						snprintf(strMsg,70,"[Failed] Mod: (%s) %s", gtList[i].pstrMods[0], gtList[i].pstrDesc);
					}
				}
				else
				{
					snprintf(strMsg,70,"Found module: (%s) %s", gtList[i].pstrMods[0], gtList[i].pstrDesc);
				}
				mvwaddstr (pWin, 1+lPos++, 2, strMsg);
			}
		}
		wmove 		(pWin, 19, 1);
		whline 		(pWin, ACS_HLINE, lX-2);
		wattrset 	(pWin, gtAttribute[3]);
		mvwaddstr (pWin, 20, (lX/4) - (strlen (STR_NIC)/2) - 5, STR_NIC);
		mvwaddstr (pWin, 20, (lX/2) - (strlen (STR_SCSI)/2), STR_SCSI);
		mvwaddstr (pWin, 20, (3*lX/4) - (strlen (STR_EXIT)/2) + 5, STR_EXIT);

		DisplayMenu (pWin, lX, lY);

		delwin		(pWin);
		redrawwin	(stdscr);
		refresh		();
	}
}




/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT GetIfParms (char *pstrIf, char **ppstrTags, char **ppstrParms, long lCount)
{
	WINDOW		*pWin				= NULL;
	int				lX					= 45;
	int				lY					= 14;
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	int				i						= 0;
	int				lTagLen			= strlen (*ppstrTags);
	int				lParmLen		= (lX - lTagLen) - 5;
	int				lParmStart	= lTagLen + 2;
	int				lPos				= 0;
	int				lCursePos		= 0;
	int				lInput			= 0;
	BooleanT	bContinue		= TRUE;
	BooleanT	bSuccess		= FALSE;
	char			strBuf[64];

	clear 	 ();
	refresh	 ();
	getmaxyx (stdscr, lMaxY, lMaxX);

	(NULL == pstrIf) ? (lY = 16) : (lY = 14);

	pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);

	wborder (pWin, 
					ACS_VLINE 		| gtAttribute[0], 
					ACS_VLINE 		| gtAttribute[1], 
					ACS_HLINE 		| gtAttribute[0], 
					ACS_HLINE 		| gtAttribute[1], 
					ACS_ULCORNER 	| gtAttribute[0], 
					ACS_URCORNER 	| gtAttribute[0], 
					ACS_LLCORNER 	| gtAttribute[1], 
					ACS_LRCORNER 	| gtAttribute[1]);

	wattrset (pWin, gtAttribute[1]);

	for (i=1; i<lY-1; i++)
	{
		wmove 	(pWin, i, 1);
		whline 	(pWin, ' ', lX-2);
	}
	memset 		(strBuf, 0, sizeof (strBuf));
	(NULL == pstrIf) ?
	sprintf		(strBuf, STR_CONFIG_IF, "ALL") :
	sprintf		(strBuf, STR_CONFIG_IF, pstrIf);
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 1, (lX/2) - (strlen (STR_BAKBONE) / 2), STR_BAKBONE);
	mvwaddstr (pWin, 2, (lX/2) - (strlen (strBuf) / 2), strBuf);
	wattrset 	(pWin, gtAttribute[1]);
	wmove 		(pWin, 3, 1);
	whline 		(pWin, ACS_HLINE, lX-2);
	
	for (i=0; i<lCount; i++)
	{
		wattrset 	(pWin, gtAttribute [1]);
		mvwaddstr (pWin, 5+2*i, 2, ppstrTags[i]);
		wattrset 	(pWin, gtAttribute[5]);
		whline 		(pWin, ' ', lParmLen);
		memset    (ppstrParms[i], 0, lParmLen);
	}
	wattrset 	(pWin, gtAttribute[1]);
	wmove 		(pWin, lY-3, 1);
	whline 		(pWin, ACS_HLINE, lX-2);
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, lY-2, (lX/4) - (strlen (STR_CONFIG)/2), STR_CONFIG);
	mvwaddstr (pWin, lY-2, (3*lX/4) - (strlen (STR_CANCEL)/2), STR_CANCEL);
	
	wrefresh	(pWin);
	noecho 		();
	keypad 		(pWin, 1);
	cbreak 		();
	
	while (TRUE == bContinue)
	{
		if (lPos < lCount)
		{
			wattrset 	(pWin, gtAttribute[3]);
			mvwaddstr (pWin, lY-2, (3*lX/4) - (strlen (STR_EXIT)/2), STR_CANCEL);
			mvwaddstr (pWin, lY-2, (lX/4) - (strlen (STR_CONFIG)/2), STR_CONFIG);

			wmove 		(pWin, 5 + 2 * lPos, lParmStart);
			wattrset 	(pWin, gtAttribute[4]);
			whline 		(pWin, ' ', lParmLen);
			waddstr 	(pWin, ppstrParms[lPos]);
			wmove 		(pWin, 5 + 2 * lPos, lParmStart + lCursePos);

			lInput = wgetch (pWin);

			wmove 		(pWin, 5 + 2 * lPos, lParmStart);
			wattrset 	(pWin, gtAttribute[5]);
			whline 		(pWin, ' ', lParmLen);
			waddstr 	(pWin, ppstrParms[lPos]);

			switch (lInput)
			{
				case KEY_LEFT:
				{
					if (lCursePos > 0)
					{
						lCursePos--;
					}
					break;
				}
				
				case KEY_RIGHT:
				{
					if (lCursePos < strlen (ppstrParms[lPos]))
					{
						lCursePos++;
					}
					break;
				}

				case KEY_HOME:
				{
					lCursePos = 0;
					break;
				}
				
				case KEY_END:
				{
					lCursePos = strlen (ppstrParms[lPos]);
					break;
				}
				
				case KEY_BACKSPACE:
				{
					if (lCursePos > 0)
					{
						lCursePos--;
						memmove	(
								ppstrParms[lPos] + lCursePos, 
								ppstrParms[lPos] + lCursePos + 1, 
								strlen (ppstrParms[lPos]) - lCursePos);
					}
					break;
				}
				
				case KEY_DC:
				{
					if (lCursePos < strlen (ppstrParms[lPos]))
					{
						memmove (
								ppstrParms[lPos] + lCursePos, 
								ppstrParms[lPos] + lCursePos + 1, 
								strlen (ppstrParms[lPos]) - lCursePos);
					}
					break;
				}
				
				case '\n':
				{
					lInput = KEY_DOWN;
					break;
				}

				case ' '...'~':
				{
					if (strlen (ppstrParms[lPos]) < lMaxX - lTagLen - 6)
					{
						memmove (
								ppstrParms[lPos] + lCursePos + 1, 
								ppstrParms[lPos] + lCursePos, 
								strlen (ppstrParms[lPos]) - lCursePos + 1);

						ppstrParms[lPos][lCursePos++] = lInput;
					}
					break;
				}
			}
		}
		else
		{
			int lTmp	=	(NULL == pstrIf) ? 4 : 3;

			if (lTmp == lPos)
			{
				wattrset 	(pWin, gtAttribute[3]);
				mvwaddstr (pWin, lY-2, (3*lX/4) - (strlen (STR_CANCEL)/2), STR_CANCEL);
				wattrset 	(pWin, gtAttribute[2]);
				mvwaddstr (pWin, lY-2, (lX/4) - (strlen (STR_CONFIG)/2), STR_CONFIG);
			}
			else if (lTmp + 1 == lPos)
			{
				wattrset 	(pWin, gtAttribute[3]);
				mvwaddstr (pWin, lY-2, (lX/4) - (strlen (STR_CONFIG)/2), STR_CONFIG);
				wattrset 	(pWin, gtAttribute[2]);
				mvwaddstr (pWin, lY-2, (3*lX/4) - (strlen (STR_CANCEL)/2), STR_CANCEL);
			}
			lInput = wgetch (pWin);
		}

		switch (lInput)
		{
			case '\t':
			case KEY_DOWN:
			{
				lCursePos = 0;
				lPos++;
				lPos %= (lCount + 2);
				break;
			}

			case KEY_UP:
			{
				lCursePos = 0;

				if (--lPos < 0)
				{
					lPos = lCount + 1;
				}
				break;
			}
				
			case '\n':
			{
				int lTmp	=	(NULL == pstrIf) ? 4 : 3;

				if (lTmp == lPos)
				{
					bSuccess = TRUE;
				}
				bContinue = FALSE;
				break;
			}
		}
	}
	delwin		(pWin);
	redrawwin	(stdscr);
	refresh		();

	return bSuccess;
}





/*
** =======================================================================================
** =======================================================================================
*/
static void ValidateAndConfigIf (char *pstrIf, char **ppstrParms, long lNumIf)
{
	char strMsg[128] = {0};

	if (TRUE != ValidateIfParm (ppstrParms[0]))
	{
		sprintf(strMsg,"ERROR: Invalid input IP Address: %s", ppstrParms[0]);
		DisplayMsg (strMsg);
	}
	else if (TRUE != ValidateIfParm (ppstrParms[1]))
	{
		sprintf(strMsg,"ERROR: Invalid input NetMask: %s", ppstrParms[1]);
		DisplayMsg (strMsg);
	}
	else if (0 < strlen (ppstrParms[2]) && TRUE != ValidateIfParm (ppstrParms[2]))
	{
		sprintf(strMsg,"ERROR: Invalid input GateWay: %s", ppstrParms[2]);
		DisplayMsg (strMsg);
	}
	else if (NULL == pstrIf && TRUE != ValidateIfParm (ppstrParms[3]))
	{
		sprintf(strMsg,"ERROR: Invalid input Server IP: %s", ppstrParms[3]);
		DisplayMsg (strMsg);
	}
	else if (NULL == pstrIf)
	{
		BooleanT	bSuccess				= FALSE;
		char 			strConfig[128] 	= {0};
		char 			strMsg[128]			= {0};
		char			strBcast[32]		= {0};
		int				i;
		int				j;

		TRACE(("Entering Config Auto for all NIC interfaces"));

		memset (strBcast, 0, sizeof (strBcast));
		GetBcastAddr (ppstrParms[0], ppstrParms[1], strBcast);

		for (i=0; i<lNumIf && FALSE == bSuccess; i++)
		{
			memset (strConfig, 0, sizeof (strConfig));
			(0 != strBcast[0]) ? 
			sprintf(strConfig,"eth  %s netmask %s broadcast %s down",ppstrParms[0],ppstrParms[1],strBcast) :
			sprintf(strConfig,"eth  %s netmask %s down",ppstrParms[0],ppstrParms[1]);

			for (j=0; j<lNumIf; j++)
			{
				strConfig[3] = 0x30 + j;
				TRACE(("Bring NIC Interface: eth%c (down)", 0x30 + j));

				if (TRUE != DrIfcfg (strConfig))
				{
					TRACE(("ERROR: Failed to bring NIC Interface: eth%c (down)", 0x30 + j));
				}
			}
			memset (strConfig, 0, sizeof (strConfig));
			(0 != strBcast[0]) ? 
			sprintf(strConfig,"eth  %s netmask %s broadcast %s up",ppstrParms[0],ppstrParms[1],strBcast) :
			sprintf(strConfig,"eth  %s netmask %s up",ppstrParms[0],ppstrParms[1]);

			strConfig[3] = 0x30 + i;
			TRACE(("Bring NIC Interface: eth%c (up)", 0x30 + i));

			if (TRUE != DrIfcfg (strConfig))
			{
				TRACE(("ERROR: Failed to bring NIC Interface: eth%c (up)", 0x30 + i));
			}
			else
			{
				if (0 < strlen (ppstrParms[2]))
				{
					memset (strConfig, 0, sizeof (strConfig));
					sprintf(strConfig, "add default gw %s", ppstrParms[2]);
				
					TRACE(("Add Default Gateway for without specific Dev: %s", ppstrParms[2]));
					
					/* Attempt to remove any previous default gateways from the route table*/
					DrIfRoute("del default");
					if (TRUE != DrIfRoute (strConfig))
					{
						TRACE(("ERROR: Failed to Add Default Gateway: %s", ppstrParms[2]));
					}
					else
					{
						TRACE(("Add Default Gateway successfully: %s", ppstrParms[2]));
					}
				}

				sprintf(strMsg,"Contact Server: %s using Interface: eth%c. Please Wait...",
				ppstrParms[3], 0x30 + i);

				DisplayInfo (strMsg);

				sleep (CfgGetNegotiationTime ());

				if (TRUE == DrPg (ppstrParms[3]))
				{
					sprintf(strMsg,"Contact Server: %s Successfully using Interface: eth%c",
					ppstrParms[3], 0x30 + i);
					TRACE((strMsg));
					bSuccess = TRUE;
				}
				else
				{
					TRACE(("WARN: Failed to contact Server: %s Using Interface: eth%c",
					ppstrParms[3], 0x30 + i));
				}
			}
		}
		if (TRUE == bSuccess)
		{
			DisplayMsg (strMsg);
		}
		else
		{
			sprintf(strMsg,"ERROR: Failed contact Server: %s using all interfaces",ppstrParms[3]);
			DisplayMsg (strMsg);
		}
	}
	else if (NULL != pstrIf)
	{
		char 	strConfig[128] 	= {0};
		char	strBcast[32]		= {0};

		memset (strConfig, 0, sizeof (strConfig));
		memset (strBcast,  0, sizeof (strBcast));

		if (TRUE == GetBcastAddr (ppstrParms[0], ppstrParms[1], strBcast))
		{
			sprintf(strConfig, "%s %s netmask %s broadcast %s up",
			pstrIf, ppstrParms[0], ppstrParms[1], strBcast);
		}
		else
		{
			sprintf(strConfig, "%s %s netmask %s up",
			pstrIf, ppstrParms[0], ppstrParms[1]);
		}

		if (TRUE != DrIfcfg (strConfig))
		{
			sprintf(strMsg,"ERROR: Failed to config Interface: %s", pstrIf);
			DisplayMsg (strMsg);
		}
		else
		{
			sprintf(strMsg,"Config Interface: %s successfully", pstrIf);
			DisplayMsg (strMsg);
		}
		
		if (0 < strlen (ppstrParms[2]))
		{
			/* Attempt to remove any previous default gateways from the route table*/
			memset (strConfig, 0, sizeof (strConfig));
			sprintf(strConfig, "del default dev %s", pstrIf);
			DrIfRoute(strConfig);
			
			memset (strConfig, 0, sizeof (strConfig));
			sprintf(strConfig, "add default gw %s dev %s", ppstrParms[2], pstrIf);
			
			if (TRUE != DrIfRoute (strConfig))
			{
				sprintf(strMsg,"ERROR: Failed to Add Default Gateway: %s", ppstrParms[2]);
				DisplayMsg (strMsg);
			}
			else
			{
				sprintf(strMsg,"Add Default Gateway successfully: %s", ppstrParms[2]);
				DisplayMsg (strMsg);
			}
		}
	}
}





/*
** =======================================================================================
** =======================================================================================
*/
static void DisplayIfMenu (WINDOW *pWin, int lX, int lY, long lNumIf) 
{
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	int				lInitY			= 4;
	int				lInitX			= 4;
	int				lPos				= 1;
	int				lIdx				= 1;
	int				lRC					= 0;
	int				i						= 0;
	int				lKeyPos			= 1;
	BooleanT	bExit				= FALSE;
	char			*pstrIfs[MAX_NUM_IF];

	getmaxyx (pWin, lMaxY, lMaxX);
	wattrset (pWin, gtAttribute [1]);

	for (i=0; i<lNumIf; i++)
	{
		pstrIfs[i] = (char *) malloc (5);
		sprintf		(pstrIfs[i], "eth%d", i);
		wmove 		(pWin, lPos+i+lInitY, lInitX);
		mvwaddstr (pWin, lPos+i+lInitY, lInitX, pstrIfs[i]);
	}
	noecho 	();
	keypad 	(pWin, 1);
	cbreak 	();

	do
	{
		wattrset 		(pWin, gtAttribute [2]);
		wmove 			(pWin, lPos+lInitY, 2);
		whline 			(pWin, ' ', lMaxX - 4);
		mvwaddstr 	(pWin, lPos+lInitY, lInitX, pstrIfs[lIdx-1]);
		whline			(pWin, ' ',(lInitX - strlen (pstrIfs[lIdx-1])+1) / 2 - 2);
		wrefresh 		(pWin);
		redrawwin 	(pWin);
		lRC = wgetch(pWin);
		wattrset 		(pWin, gtAttribute [1]);
		wmove 			(pWin, lPos+lInitY, 2);
		whline 			(pWin, ' ', lMaxX - 4);
		mvwaddstr 	(pWin, lPos+lInitY, lInitX, pstrIfs[lIdx-1]);

		switch (lRC)
		{
			case KEY_UP:
			{
				if (1 < lPos)
				{
					lPos--;
				}
				if (1 < lIdx)
				{
					lIdx--;
				}
				break;
			}

			case KEY_DOWN:
			{
				if (lNumIf > lPos)
				{
					lPos++;
				}
				if (lNumIf > lIdx)
				{
					lIdx++;
				}
				break;
			}
			
			case '\t':
			case KEY_RIGHT:
			{
				if (1 == lKeyPos)
				{
					lKeyPos++;
					wattrset 	(pWin, gtAttribute[2]);
					wmove 		(pWin, 17, (lX/4) - (strlen (STR_CONFIG)/2) - 5);
					whline 		(pWin, ' ', strlen (STR_CONFIG));
					mvwaddstr (pWin, 17, (lX/4) - (strlen (STR_CONFIG)/2) - 5, STR_CONFIG);

				}
				else if (2 == lKeyPos)
				{
					lKeyPos++;
					wattrset 	(pWin, gtAttribute[3]);
					wmove 		(pWin, 17, (lX/4) - (strlen (STR_CONFIG)/2) - 5);
					whline 		(pWin, ' ', strlen (STR_CONFIG));
					mvwaddstr (pWin, 17, (lX/4) - (strlen (STR_CONFIG)/2) - 5, STR_CONFIG);
	
					wattrset 	(pWin, gtAttribute[2]);
					wmove 		(pWin, 17, (lX/2) - (strlen (STR_AUTO)/2));
					whline 		(pWin, ' ', strlen (STR_AUTO));
					mvwaddstr (pWin, 17, (lX/2) - (strlen (STR_AUTO)/2), STR_AUTO);
				}
				else if (3 == lKeyPos)
				{
					lKeyPos++;
					
					wattrset 	(pWin, gtAttribute[3]);
					wmove 		(pWin, 17, (lX/2) - (strlen (STR_AUTO)/2));
					whline 		(pWin, ' ', strlen (STR_AUTO));
					mvwaddstr (pWin, 17, (lX/2) - (strlen (STR_AUTO)/2), STR_AUTO);

					wattrset 	(pWin, gtAttribute[2]);
					wmove 		(pWin, 17, (3*lX/4) - (strlen (STR_EXIT)/2) + 5);
					whline 		(pWin, ' ', strlen (STR_EXIT));
					mvwaddstr (pWin, 17, (3*lX/4) - (strlen (STR_EXIT)/2) + 5, STR_EXIT);
				}
				else
				{
					lKeyPos = 1;
					wattrset 	(pWin, gtAttribute[3]);
					wmove 		(pWin, 17, (3*lX/4) - (strlen (STR_EXIT)/2) + 5);
					whline 		(pWin, ' ', strlen (STR_EXIT));
					mvwaddstr (pWin, 17, (3*lX/4) - (strlen (STR_EXIT)/2) + 5, STR_EXIT);
				}
				break;
			}
			
			case KEY_LEFT:
			{
				if (1 == lKeyPos)
				{
					lKeyPos	= 4;
					wattrset 	(pWin, gtAttribute[2]);
					wmove 		(pWin, 17, (3*lX/4) - (strlen (STR_EXIT)/2) + 5);
					whline 		(pWin, ' ', strlen (STR_EXIT));
					mvwaddstr (pWin, 17, (3*lX/4) - (strlen (STR_EXIT)/2) + 5, STR_EXIT);
				}
				else if (4 == lKeyPos)
				{
					lKeyPos--;

					wattrset 	(pWin, gtAttribute[3]);
					wmove 		(pWin, 17, (3*lX/4) - (strlen (STR_EXIT)/2) + 5);
					whline 		(pWin, ' ', strlen (STR_EXIT));
					mvwaddstr (pWin, 17, (3*lX/4) - (strlen (STR_EXIT)/2) + 5, STR_EXIT);

					wattrset 	(pWin, gtAttribute[2]);
					wmove 		(pWin, 17, (lX/2) - (strlen (STR_AUTO)/2));
					whline 		(pWin, ' ', strlen (STR_AUTO));
					mvwaddstr (pWin, 17, (lX/2) - (strlen (STR_AUTO)/2), STR_AUTO);
				}
				else if (3 == lKeyPos)
				{
					lKeyPos--;
					
					wattrset 	(pWin, gtAttribute[3]);
					wmove 		(pWin, 17, (lX/2) - (strlen (STR_AUTO)/2));
					whline 		(pWin, ' ', strlen (STR_AUTO));
					mvwaddstr (pWin, 17, (lX/2) - (strlen (STR_AUTO)/2), STR_AUTO);
					
					wattrset 	(pWin, gtAttribute[2]);
					wmove 		(pWin, 17, (lX/4) - (strlen (STR_CONFIG)/2) - 5);
					whline 		(pWin, ' ', strlen (STR_CONFIG));
					mvwaddstr (pWin, 17, (lX/4) - (strlen (STR_CONFIG)/2) - 5, STR_CONFIG);
				}
				else if (2 == lKeyPos)
				{
					lKeyPos--;
					
					wattrset 	(pWin, gtAttribute[3]);
					wmove 		(pWin, 17, (lX/4) - (strlen (STR_CONFIG)/2) - 5);
					whline 		(pWin, ' ', strlen (STR_CONFIG));
					mvwaddstr (pWin, 17, (lX/4) - (strlen (STR_CONFIG)/2) - 5, STR_CONFIG);
				}
				break;
			}
			case '\n':
			{
				if (4 == lKeyPos)
				{
					bExit = TRUE;
				}
				else if (3 == lKeyPos)
				{
					char	*pstrTags[] = { "  IP Address: ", "Network Mask: ", "     Gateway: ", "   Server IP: " };
					long	lTagCnt			= sizeof (pstrTags) / sizeof (pstrTags[0]);
					char	strParms[lTagCnt][32];
					char	*pstrParms[lTagCnt];
					int		i;

					for (i=0; i<lTagCnt; i++)
					{
						memset (strParms[i], 0, sizeof (strParms[i]));
						pstrParms[i] = strParms[i];
					}
					if (TRUE == GetIfParms (NULL, pstrTags, pstrParms, lTagCnt))
					{
						ValidateAndConfigIf (NULL, pstrParms, lNumIf);
					}
				}
				else if (2 == lKeyPos || 1 == lKeyPos)
				{
					char	*pstrTags[] = { "  IP Address: ", "Network Mask: ", "     Gateway: " };
					long	lTagCnt			= sizeof (pstrTags) / sizeof (pstrTags[0]);
					char	strParms[lTagCnt][32];
					char	*pstrParms[lTagCnt];
					int		i;

					for (i=0; i<lTagCnt; i++)
					{
						memset (strParms[i], 0, sizeof (strParms[i]));
						pstrParms[i] = strParms[i];
					}
					if (TRUE == GetIfParms (pstrIfs[lIdx-1], pstrTags, pstrParms, lTagCnt))
					{
						ValidateAndConfigIf (pstrIfs[lIdx-1], pstrParms, lNumIf);
					}
				}
				break;
			}
		}
	} while (TRUE != bExit);

	for (i=0; i<lNumIf; i++)
	{
		free (pstrIfs[i]);
	}
}






/*
** =======================================================================================
** =======================================================================================
*/
static void DisplayAvailIf (long lNumIf)
{
	WINDOW	*pWin			= NULL;
	int			lX				= 45;
	int			lY				= 19;
	int			lMaxX			= 0;
	int			lMaxY			= 0;
	int			i					= 0;

	clear 	 ();
	refresh	 ();
	getmaxyx (stdscr, lMaxY, lMaxX);

	pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);

	wborder (pWin, 
					ACS_VLINE 		| gtAttribute[0], 
					ACS_VLINE 		| gtAttribute[1], 
					ACS_HLINE 		| gtAttribute[0], 
					ACS_HLINE 		| gtAttribute[1], 
					ACS_ULCORNER 	| gtAttribute[0], 
					ACS_URCORNER 	| gtAttribute[0], 
					ACS_LLCORNER 	| gtAttribute[1], 
					ACS_LRCORNER 	| gtAttribute[1]);

	wattrset (pWin, gtAttribute[1]);

	for (i=1; i<lY-1; i++)
	{
		wmove 	(pWin, i, 1);
		whline 	(pWin, ' ', lX-2);
	}
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 1, (lX/2) - (strlen (STR_BAKBONE) / 2), STR_BAKBONE);
	mvwaddstr (pWin, 2, (lX/2) - (strlen (STR_AVAIL_NIC) / 2), STR_AVAIL_NIC);
	wattrset 	(pWin, gtAttribute[1]);
	wmove 		(pWin, 3, 1);
	whline 		(pWin, ACS_HLINE, lX-2);
	wmove 		(pWin, 16, 1);
	whline 		(pWin, ACS_HLINE, lX-2);
	
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 17, (lX/4) - (strlen (STR_CONFIG)/2) - 5, STR_CONFIG);
	mvwaddstr (pWin, 17, (lX/2) - (strlen (STR_AUTO)/2), STR_AUTO);
	mvwaddstr (pWin, 17, (3*lX/4) - (strlen (STR_EXIT)/2) + 5, STR_EXIT);

	DisplayIfMenu (pWin, lX, lY, lNumIf);

	delwin		(pWin);
	redrawwin	(stdscr);
	refresh		();
}



/*
** =======================================================================================
** =======================================================================================
*/
static void ConfigAllNic ()
{
	long lNumIf = 0;

	if (0 == (lNumIf = GetNicInterfaces ()))
	{
		DisplayMsg("ERROR: No Network Interfaces available");
	}
	else if (lNumIf > MAX_NUM_IF)
	{
		lNumIf = MAX_NUM_IF;
		DisplayAvailIf (lNumIf);
	}
	else
	{
		DisplayAvailIf (lNumIf);
	}
}


/*
** =======================================================================================
** Function to clean up memory allocated in global variables
** =======================================================================================
*/
extern void CleanupMemory()
{
	int i = 0;
	int iMod = 0;
	for (i = 0; i < MAX_MOD_ARRAY_SIZE; i++)
	{
		if (NULL != gtList[i].pstrComp)
			free(gtList[i].pstrComp);
		if (NULL != gtList[i].pstrDesc)
			free(gtList[i].pstrDesc);
		
		for (iMod = 0; iMod < gtList[i].lModCount; iMod++)
		{
			if (NULL != gtList[i].pstrMods[iMod])
				free(gtList[i].pstrMods[iMod]);
		}
	}
}




/*
** =======================================================================================
** =======================================================================================
*/
static void DrPrintDevices ()
{
	struct device  **pptDev = NULL;
	struct device  *ptDev   = NULL;
	int    i = 0;
	
	pptDev = probeDevices (CLASS_UNSPEC, BUS_UNSPEC, PROBE_ALL);
	
	if (NULL == pptDev)
	{
		fprintf(stderr,"ERROR: Failed to probeDevices\n");
		fflush (stderr);
		return;
	}

	for (i=0; (ptDev = pptDev[i]); i++)
	{
		fprintf(stderr,"%-14s  [%s]\n", ptDev->driver, ptDev->desc);
		fflush (stderr);
	}
	exit (0);
}


/*
** =======================================================================================
** =======================================================================================
*/
int main (int argc, char **argv) 
{
	BooleanT bVer = FALSE;
	BooleanT bOnlyConfigNics = FALSE;

	if (0 == --argc)
	{
		PrintUsage ();
	}

  while (0 != argc && '-' == **++argv)
  {
    argc--;

    if (NULL == strchr ("cnlpv", (*argv)[1]))
    {
      PrintUsage ();
    }
    else if ('c' == (*argv)[1])
    {
      bOnlyConfigNics = TRUE;
		}
    else if ('n' == (*argv)[1])
    {
      gbLoad = FALSE;
    }
    else if ('l' == (*argv)[1])
    {
      gbLoad = TRUE;
    }
    else if ('p' == (*argv)[1])
    {
      DrPrintDevices ();
    }
    else if ('v' == (*argv)[1])
    {
			fprintf(stderr,"\ndrprobe: Version 1.1\n\n");
      bVer = TRUE;
    }
  }

	if (TRUE != bVer)
	{
		DrTraceInit (TRUE);
  	
		TRACE(("NetVault DRPROBE: Built on %s", __DATE__));
		TRACE(("DRPROBE process Begin"));

		WinInit ();

		/* Autoload any modules present in the /mods/autoload directory at startup */
		AutoLoadMiscModules();

		/* Ask User whether they want to load the found driver.*/
		if (TRUE == QueryUserLoadDrivers())
		{
			ProbeAndLoadModule ();
		}

		DisplayMainMenu ();

		/* Extra option to allow user to change their NIC settings without manual 
			 intervention SH: 20040627*/
		if (TRUE == gbLoad || TRUE == bOnlyConfigNics)
		{
			ConfigAllNic ();
		}
		system ("tset -Q");

		TRACE(("DRPROBE process Ended"));
	}

	/* Clean up memory allocated in the global lists.*/
	CleanupMemory();
	endwin();
	return 0;
}

