
#include <ncurses.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mount.h>
#include <unistd.h>
#include <dirent.h>
#include <fcntl.h>
#include <net/if.h>
#include <errno.h>
#include <curses.h>

#define		TRUE										1
#define		FALSE										0
#define		STR_BAKBONE							"BakBone Software, Inc."
#define		STR_STATION_TITLE				"Station Selections"
#define		STR_HOST_TITLE					"Host Selections"
#define		STR_OK									"[  OK  ]"
#define		STR_YES									"[ YES ]"
#define		STR_NO									"[ NO ]"
#define		STR_SERVER_IP						"192.168.58.6"
#define		MAX_NUM_AREA						10
#define		MAX_NUM_HOST						10

typedef		long					BooleanT;
typedef		unsigned char	ByteT;

typedef struct
{
	char	*pstrName;
	char	*pstrIp;
} HostT;

typedef struct
{
	char	*pstrName;
	long	lHostCount;
	HostT	tHost[MAX_NUM_HOST];
}	AreaT;


typedef enum
{
	DrYes	= 0,
	DrNo	= 1
} DrRequestE;


/*
** =======================================================================================
** 0 = Grey
** 1 = Grey
** 2 = Red
** 3 = Grey
** 4 = Blue
** 5 = Blue
** 6 = Black
** =======================================================================================
*/
static	attr_t	gtAttribute [6];
static	long		glSelArea		=	0;
static	long		glSelHost		=	0;
static	long		glNumArea		= 0;
static	AreaT		gtArea[MAX_NUM_AREA];
static	char		*gpstrHost[] = { "isp", "jw1", "jw2", "saps01", "saps02" };

#define NUM_OF_HOST	(sizeof (gpstrHost) / sizeof (gpstrHost[0]))



/*
** =======================================================================================
** =======================================================================================
*/
WINDOW *WinInit ()
{
  WINDOW 	*pWin = NULL;
  int 		lRC 	= 0;
  int 		i 		= 0;

  pWin = initscr ();

  if (NULL != pWin)
	{
		lRC = has_colors ();

		if (0 != lRC)
		{
			start_color ();

			init_pair (1, COLOR_WHITE, 	COLOR_WHITE);
			init_pair (2, COLOR_BLACK, 	COLOR_WHITE);
			init_pair (3, COLOR_WHITE, 	COLOR_RED);
			init_pair (4, COLOR_BLUE, 	COLOR_WHITE);
			init_pair (5, COLOR_YELLOW, COLOR_BLUE);
			init_pair (6, COLOR_WHITE, 	COLOR_BLUE);

			for (i = 0; i < 6; i++)
			{
				gtAttribute[i] = COLOR_PAIR (i + 1) | ((!(i & 1)) ? A_BOLD : 0);
			}
		}
		else
		{
			gtAttribute[0] = gtAttribute[1] = gtAttribute[3] =
			gtAttribute[4] = gtAttribute[5] = A_NORMAL;
			gtAttribute[2] = A_REVERSE;
		}
	}
  return pWin;
}




/*
** =======================================================================================
** =======================================================================================
*/

static void DisplayInfo (char *pstrMsg)
{
  WINDOW    *pWin       = NULL;
  int       lX          = strlen (pstrMsg) + 10;
  int       lY          = 5;
  int       lMaxX       = 0;
  int       lMaxY       = 0;
  int       RC          = 0;
  int       i           = 0;

  clear     ();
  refresh   ();
  getmaxyx  (stdscr, lMaxY, lMaxX);

  pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);

  wborder (pWin,
          ACS_VLINE     | gtAttribute[0],
          ACS_VLINE     | gtAttribute[1],
          ACS_HLINE     | gtAttribute[0],
          ACS_HLINE     | gtAttribute[1],
          ACS_ULCORNER  | gtAttribute[0],
          ACS_URCORNER  | gtAttribute[0],
          ACS_LLCORNER  | gtAttribute[1],
          ACS_LRCORNER  | gtAttribute[1]);

  wattrset (pWin, gtAttribute[1]);

  for (i=1; i<lY-1; i++)
  {
    wmove   (pWin, i, 1);
    whline  (pWin, ' ', lX-2);
  }
  wattrset  (pWin, gtAttribute[3]);
  mvwaddstr (pWin, 2, (lX-strlen (pstrMsg)) / 2, pstrMsg);

  wrefresh (pWin);
  redrawwin (pWin);
}




/*
** =======================================================================================
** =======================================================================================
*/
static char *GetBroadcastAddr (char *pstrIp, char *pstrMask)
{
	BooleanT	bSuccess		= FALSE;
	char			*pstrBcast	= (char *) malloc (32);
	long			lIpa 				= 0;
	long			lIpb 				= 0;
	long			lIpc 				= 0;
	long			lIpd 				= 0;

	if (NULL == pstrBcast)
	{
		fprintf(stderr,"ERROR: Failed to allocate memory - 1\n");
	}
	else if (4 != sscanf (pstrIp, "%ld.%ld.%ld.%ld", &lIpa, &lIpb, &lIpc, &lIpd))
	{
		fprintf(stderr,"ERROR: Failed to sscanf to convert IP to Bcast\n");
	}
	else
	{
		long	lIp = (lIpa << 24) | (lIpb << 16) | (lIpc << 8) | (lIpd);

		if (4 != sscanf (pstrMask, "%ld.%ld.%ld.%ld", &lIpa, &lIpb, &lIpc, &lIpd))
		{
			fprintf(stderr,"ERROR: Failed to sscanf to convert NetMask to Bcast\n");
		}
		else
		{
			long 	lMask 	= (lIpa << 24) | (lIpb << 16) | (lIpc << 8) | (lIpd);
			long	lBcast	= ~lMask | lIp;

			sprintf (pstrBcast,"%ld.%ld.%ld.%ld",
							((lBcast >> 24) & 0xff),
							((lBcast >> 16) & 0xff),
							((lBcast >> 8)  & 0xff),
							(lBcast & 0xff));

			bSuccess = TRUE;
		}
	}
	return (bSuccess ? pstrBcast : NULL);
}



/*
** =======================================================================================
** =======================================================================================
*/
static DrRequestE	DisplayRequest (
									char				*pstrYes,
									char				*pstrNo,
									char 				*pstrMsg, 
									DrRequestE 	eDefaultRequest)
{
	WINDOW		*pWin				= NULL;
	int				lX					= strlen (pstrMsg) + 10;
	int				lY					= 7;
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	int				RC					= 0;
	int				i						= 0;
	int				lKeyPos			= 0;
	BooleanT	bExit				= FALSE;

	clear 		();
	refresh		();
	getmaxyx 	(stdscr, lMaxY, lMaxX);

	pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);
	
	wborder (pWin, 
					ACS_VLINE 		| gtAttribute[0], 
					ACS_VLINE 		| gtAttribute[1], 
					ACS_HLINE 		| gtAttribute[0], 
					ACS_HLINE 		| gtAttribute[1], 
					ACS_ULCORNER 	| gtAttribute[0], 
					ACS_URCORNER 	| gtAttribute[0], 
					ACS_LLCORNER 	| gtAttribute[1], 
					ACS_LRCORNER 	| gtAttribute[1]);

	wattrset (pWin, gtAttribute[1]);

	for (i=1; i<lY-1; i++)
	{
		wmove 	(pWin, i, 1);
		whline 	(pWin, ' ', lX-2);
	}
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 2, (lX-strlen (pstrMsg)) / 2, pstrMsg);

	if (DrYes == eDefaultRequest)
	{
		lKeyPos	= 0;
		wattrset 	(pWin, gtAttribute[2]);
		mvwaddstr (pWin, 4, (lX/4) - (strlen(pstrYes)/2), pstrYes);
		wattrset 	(pWin, gtAttribute[3]);
		mvwaddstr (pWin, 4, (3*lX)/4 - (strlen(pstrNo)/2), pstrNo);
	}
	else
	{
		lKeyPos	= 1;
		wattrset 	(pWin, gtAttribute[3]);
		mvwaddstr (pWin, 4, (lX/4) - (strlen(pstrYes)/2), pstrYes);
		wattrset 	(pWin, gtAttribute[2]);
		mvwaddstr (pWin, 4, (3*lX)/4 - (strlen(pstrNo)/2), pstrNo);
	}
	
	noecho 		();
	keypad 		(pWin, 1);
	cbreak 		();
		
	do
	{
		wrefresh 		(pWin);
		redrawwin 	(pWin);
		RC = wgetch	(pWin);

		switch (RC)
		{
			case '\t':
			case KEY_RIGHT:
			case KEY_LEFT:
			{
				if (0 == lKeyPos)
				{
					lKeyPos = 1;
					wattrset 	(pWin, gtAttribute[3]);
					mvwaddstr (pWin, 4, (lX/4) - (strlen(pstrYes)/2), pstrYes);
					
					wattrset 	(pWin, gtAttribute[2]);
					mvwaddstr (pWin, 4, (3*lX)/4 - (strlen(pstrNo)/2), pstrNo);
				}
				else if (1 == lKeyPos)
				{
					lKeyPos = 0;
					wattrset 	(pWin, gtAttribute[2]);
					mvwaddstr (pWin, 4, (lX/4) - (strlen(pstrYes)/2), pstrYes);
					wattrset 	(pWin, gtAttribute[3]);
					mvwaddstr (pWin, 4, (3*lX)/4 - (strlen(pstrNo)/2), pstrNo);
				}
				break;
			}
			
			case '\n':
			{
				bExit = TRUE;
				break;
			}
		}
	} while (TRUE != bExit);
	
	clear 	();
	refresh	();
	delwin 	(pWin);

	return lKeyPos;
}




/*
** =======================================================================================
** =======================================================================================
*/
static void DisplayMsg (char *pstrMsg)
{
	WINDOW		*pWin				= NULL;
	int				lX					= strlen (pstrMsg) + 10;
	int				lY					= 7;
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	int				RC					= 0;
	int				i						= 0;

	clear 		();
	refresh		();
	getmaxyx 	(stdscr, lMaxY, lMaxX);

	pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);
	
	wborder (pWin, 
					ACS_VLINE 		| gtAttribute[0], 
					ACS_VLINE 		| gtAttribute[1], 
					ACS_HLINE 		| gtAttribute[0], 
					ACS_HLINE 		| gtAttribute[1], 
					ACS_ULCORNER 	| gtAttribute[0], 
					ACS_URCORNER 	| gtAttribute[0], 
					ACS_LLCORNER 	| gtAttribute[1], 
					ACS_LRCORNER 	| gtAttribute[1]);

	wattrset (pWin, gtAttribute[1]);

	for (i=1; i<lY-1; i++)
	{
		wmove 	(pWin, i, 1);
		whline 	(pWin, ' ', lX-2);
	}
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 2, (lX-strlen (pstrMsg)) / 2, pstrMsg);
	wattrset 	(pWin, gtAttribute[2]);
	mvwaddstr (pWin, 4, (lX-strlen (STR_OK)) /2, STR_OK);
		
	do
	{
		RC = wgetch	(pWin);
	} while (RC != '\n');
	
	clear 	();
	refresh	();
	delwin 	(pWin);
}


/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT ValidateNetworkNum (char *pstrNum)
{
	int		bSuccess 	= FALSE;
	long	lClassZ		= 0;
	long	lClassA		= 0;
	long	lClassB		= 0;
	long	lClassC		= 0;

	if (4 == sscanf (pstrNum, "%ld.%ld.%ld.%ld", &lClassZ, &lClassA, &lClassB, &lClassC))
	{
		if (0 <= lClassZ && 255 >= lClassZ &&
				0 <= lClassA && 255 >= lClassA &&
				0 <= lClassB && 255 >= lClassB &&
				0 <= lClassC && 255 >= lClassC)
		{
			bSuccess = TRUE;
		}
	}
	return bSuccess;
}



/*
** =======================================================================================
** =======================================================================================
*/
static long GetNicInterfaces ()
{
	long	lInterface	= 0;
	FILE	*pFd				= fopen ("/proc/net/dev", "r");

	if (NULL != pFd)
	{
		char	strBuffer [512] = {0};
		char	*pstr	= NULL;

		while (NULL != fgets (strBuffer, sizeof (strBuffer), pFd))
		{
			pstr = (char *) strtok (strBuffer, " ");

			if (NULL != pstr)
			{
				if (NULL != strstr (pstr, "eth"))
				{
					lInterface++;
				}
			}
		}
		fclose (pFd);
	}
	return lInterface;
}



/*
** =======================================================================================
** =======================================================================================
*/
static void DrawMenu (
						WINDOW 	*pWin, 
						long 		lCount,
						char		*pstrArr[])
{
	int		lInitY	= 4;
	int		lInitX	= 4;
	int		i 			= 1;

	wattrset (pWin, gtAttribute [1]);

	for (i=1; i<=lCount; i++)
	{
		wmove 		(pWin, i+lInitY, lInitX);
		whline 		(pWin, ' ', 50-lInitX*2);
		mvwaddstr (pWin, i+lInitY, lInitX, pstrArr [i-1]);
	}
}




/*
** =======================================================================================
** =======================================================================================
*/
static void DisplayMenu (
						WINDOW 		*pWin, 
						int				lX,
						int				lY,
						long 			lCount,
						char			*pstrArr[],
						BooleanT	bStation)
{
	int				lInitY			= 4;
	int				lInitX			= 4;
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	int				lPos				= 1;
	int				lIdx				= 1;
	int				RC					= 0;
	int				i						= 0;
	int				lKeyPos			= 1;
	BooleanT	bExit				= FALSE;

	getmaxyx 	(pWin, lMaxY, lMaxX);
	DrawMenu	(pWin, lCount, pstrArr);
	noecho 		();
	keypad 		(pWin, 1);
	cbreak 		();
		
	do
	{
		wattrset 		(pWin, gtAttribute [2]);
		wmove 			(pWin, lPos+lInitY, 2);
		whline 			(pWin, ' ', lMaxX - 4);
		mvwaddstr 	(pWin, lPos+lInitY, lInitX, pstrArr [lIdx-1]);
		whline			(pWin, ' ',(lInitX - strlen (pstrArr [lIdx-1])+1) / 2 - 2);
		wrefresh 		(pWin);
		redrawwin 	(pWin);
		RC = wgetch	(pWin);
		wattrset 		(pWin, gtAttribute [1]);
		wmove 			(pWin, lPos+lInitY, 2);
		whline 			(pWin, ' ', lMaxX - 4);
		mvwaddstr 	(pWin, lPos+lInitY, lInitX, pstrArr [lIdx-1]);

		switch (RC)
		{
			case KEY_UP:
			{
				if (1 < lPos)
				{
					lPos--;
				}
				if (1 < lIdx)
				{
					lIdx--;
				}
				break;
			}

			case KEY_DOWN:
			{
				if (lCount > lPos)
				{
					lPos++;
				}
				if (lCount > lIdx)
				{
					lIdx++;
				}
				break;
			}
	
			case '\t':
			case KEY_RIGHT:
			{
				if (1 == lKeyPos)
				{
					lKeyPos++;
					wattrset 	(pWin, gtAttribute[2]);
					mvwaddstr (pWin, lY-2, (lX - strlen (STR_OK)) / 2, STR_OK);
				}
				else
				{
					lKeyPos = 1;
					wattrset 	(pWin, gtAttribute[3]);
					mvwaddstr (pWin, lY-2, (lX - strlen (STR_OK)) / 2, STR_OK);
				}
				break;
			}
			
			case KEY_LEFT:
			{
				if (1 == lKeyPos)
				{
					lKeyPos++;
					wattrset 	(pWin, gtAttribute[2]);
					mvwaddstr (pWin, lY-2, (lX - strlen (STR_OK)) / 2, STR_OK);
				}
				else
				{
					lKeyPos = 1;
					wattrset 	(pWin, gtAttribute[3]);
					mvwaddstr (pWin, lY-2, (lX - strlen (STR_OK)) / 2, STR_OK);
				}
				break;
			}

			case '\n':
			{
				if (1 == lKeyPos || 2 == lKeyPos)
				{
					if (TRUE == bStation)
					{
						glSelArea = lIdx - 1;
					}
					else
					{
						glSelHost = lIdx - 1;
					}
					bExit	= TRUE;
				}
				break;
			}
		}
	} while (TRUE != bExit);
}




/*
** =======================================================================================
** =======================================================================================
*/
static void CreateMenu (
						long 			lCount,
						char			*pstrArr[],
						BooleanT	bStation)
{
	WINDOW		*pWin				= NULL;
	int				lX					= 40;
	int				lY					= 17;
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	long			i						= 0;

	getmaxyx (stdscr, lMaxY, lMaxX);

	pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);

	wborder (pWin, 
					ACS_VLINE 		| gtAttribute[0], 
					ACS_VLINE 		| gtAttribute[1], 
					ACS_HLINE 		| gtAttribute[0], 
					ACS_HLINE 		| gtAttribute[1], 
					ACS_ULCORNER 	| gtAttribute[0], 
					ACS_URCORNER 	| gtAttribute[0], 
					ACS_LLCORNER 	| gtAttribute[1], 
					ACS_LRCORNER 	| gtAttribute[1]);

	wattrset (pWin, gtAttribute[1]);

	for (i=1; i<lY-1; i++)
	{
		wmove 	(pWin, i, 1);
		whline 	(pWin, ' ', lX-2);
	}
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 1, (lX-strlen (STR_BAKBONE)) / 2, STR_BAKBONE);
	mvwaddstr (pWin, 2, (lX-strlen (bStation ? STR_STATION_TITLE: STR_HOST_TITLE )) / 2, 
	(bStation ? STR_STATION_TITLE : STR_HOST_TITLE));
	wattrset 	(pWin, gtAttribute[1]);
	wmove 		(pWin, 3, 1);
	whline 		(pWin, ACS_HLINE, lX-2);
	wmove 		(pWin, lY-3, 1);
	whline 		(pWin, ACS_HLINE, lX-2);
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, lY-2, (lX - strlen (STR_OK)) / 2, STR_OK);
	
	DisplayMenu (pWin, lX, lY, lCount, pstrArr, bStation);

	delwin		(pWin);
	redrawwin	(stdscr);
	refresh		();
}


/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT DeviceIsCdrom (char *pstrDevice)
{
	BooleanT	bCdrom 		= FALSE;
	FILE			*pFd 			= NULL;
	char			strBuffer	[512];
	struct 		stat  		tStat;

	if (0 == strncmp ("/dev/hd", pstrDevice, 7))
	{
		snprintf (strBuffer, sizeof(strBuffer), "/proc/ide/%s/media", pstrDevice+5);

		pFd = fopen (strBuffer, "r");

		if (NULL != pFd && NULL != fgets (strBuffer, sizeof (strBuffer), pFd))
		{
			if (0 == strncmp (strBuffer, "cdrom", 5))
			{
				bCdrom = TRUE;
			}
		}
		else
		{
			if (0 == stat (pstrDevice, &tStat))
			{
				if (0 == (tStat.st_mode & 0222))
				{
					bCdrom = TRUE;
				}
			}
		}
		fclose(pFd);
	}
	return bCdrom;
}


/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT	DevGetDevices (char *pstrDevices[], ulong *plCount)
{
	BooleanT	bSuccess 	= FALSE;
	FILE			*pFd 			= fopen ("/proc/partitions", "r");
	
	if (NULL == pFd)
	{
		long	lError = errno;
		char	strMsg [512] = {0};

		sprintf(strMsg,"Failed to open /proc/partitions file. %s", strerror(errno));
		DisplayMsg (strMsg);
	}
	else
	{
		char 		strBuf	[128] = {0};
		char 		strName	[64] 	= {0};
		long		lMajor 	= 0;
		long		lMinor 	= 0;
		long		lSize 	= 0;
		ulong		lCount	= 0;
	
		while (NULL != fgets (strBuf, sizeof (strBuf), pFd))
		{
			if (4 == sscanf (strBuf, " %d %d %d %[^\n ]", &lMajor, &lMinor, &lSize, strName))
			{
				BooleanT	bDeviceFound	= FALSE;
				char			*pstr 				= NULL;
				char			strDev	[64]	= {0};
				
				snprintf (strDev, 64, "/dev/%s\0", strName);

				for (pstr=strName; *pstr; pstr++);

        if (NULL != strstr (strName, "ida/"))
        {
          if ('p' != pstr[-2])
          {
            bDeviceFound = TRUE;
          }
        }
        else if (NULL != strstr (strName, "rd/"))
        {
          if ('p' != pstr[-2])
          {
            bDeviceFound = TRUE;
          }
        }
        else if (NULL != strstr (strName, "cciss/"))
        {
          if ('p' != pstr[-2])
          {
            bDeviceFound = TRUE;
          }
        }
        else
        {
          if (! isdigit (pstr[-1]))
          {
            bDeviceFound = TRUE;
          }
        }

				if (TRUE == bDeviceFound && TRUE != DeviceIsCdrom (strDev))
				{
					pstrDevices [lCount] = (char *) malloc (sizeof (strDev));
					strcpy (pstrDevices [lCount], strDev);
					lCount++;
					bSuccess = TRUE;
				}
			}
		}
		*plCount = lCount;
	}
	fclose (pFd);
	
	return bSuccess;
}



/*
** =======================================================================================
** =======================================================================================
*/
static void WriteInfoToMBR (char *pstrIp)
{
	BooleanT	bSuccess	= TRUE;
	char			*pstrDevs [10];
	long			lDevCount	= 0;
	int				lFd				= -1;
	ByteT			strMbr [512] = {0};

	if (TRUE != DevGetDevices (pstrDevs, &lDevCount))
	{
		DisplayMsg("ERROR: Failed to get Disk Device to write Signature Info");
	}
	else if (-1 == (lFd = open (pstrDevs[0], O_RDONLY)))
	{
		long	lError = errno;
		char	strMsg [512] = {0};
		sprintf(strMsg,"ERROR: Failed to open-r %s. %s\0", pstrDevs[0], strerror(lError));
		DisplayMsg(strMsg);
	}
	else if (512 != read (lFd, strMbr, 512))
	{
		long	lError = errno;
		char	strMsg [512] = {0};
		sprintf(strMsg,"ERROR: Failed to read MBR from %s. %s\0", pstrDevs[0], strerror(lError));
		DisplayMsg(strMsg);
	}
	else
	{
		long i = 0;

		for (i=0x01ee; i<0x01fe && TRUE == bSuccess; i++)
		{
			if (0x00 != strMbr[i])
			{
				bSuccess = FALSE;
			}
		}
	}
	close (lFd);
	
	if (TRUE != bSuccess)
	{
		DisplayMsg("ERROR: MBR's Partition 4th is not empty. Failed to write Signature Info");
	}
	else if (-1 == (lFd = open (pstrDevs[0], O_WRONLY)))
	{
		long	lError = errno;
		char	strMsg [512] = {0};
		sprintf(strMsg,"ERROR: Failed to open-w %s. %s\0", pstrDevs[0], strerror(lError));
		DisplayMsg(strMsg);
	}
	else
	{
		long lClassA 	= 0;
		long lClassB 	= 0;
		long lClassC 	= 0;
		long lClassD 	= 0;

		if (4 != sscanf (pstrIp, "%ld.%ld.%ld.%ld", &lClassA, &lClassB, &lClassC, &lClassD))
		{
			DisplayMsg("ERROR: Failed to sscanf Input IP Address");
		}
		else
		{
			strMbr[0x01fa] = 0xc0;
			strMbr[0x01fb] = 0xa8;
			strMbr[0x01fc] = 0x3a;
			strMbr[0x01fd] = lClassD;

			if (512 != write (lFd, strMbr, 512))
			{
				long	lError = errno;
				char	strMsg [512] = {0};
				sprintf(strMsg,"ERROR: Failed to write Signature Info to MBR %s. %s\0", pstrDevs[0], 
				strerror(lError));
				DisplayMsg(strMsg);
			}
		}
	}
	close (lFd);
}




/*
** =======================================================================================
** =======================================================================================
*/
static void LoadStationData ()
{
	long	i	= 0;
	long	j	= 0;

	for (i=0; i<6; i++)
	{
		NULL == (gtArea[i].pstrName = (char *) malloc (15)) ?
		gtArea[i].pstrName = "Unkown" :
		(char *) sprintf (gtArea[i].pstrName, "Station %ld\0", i+1);
		gtArea[i].lHostCount = NUM_OF_HOST;

		for (j=0; j<NUM_OF_HOST; j++)
		{
			NULL == (gtArea[i].tHost[j].pstrName = (char *) malloc (10)) ?
			gtArea[i].tHost[j].pstrName = "Unkown" :
			strncpy (gtArea[i].tHost[j].pstrName, gpstrHost[j], 10);

			NULL == (gtArea[i].tHost[j].pstrIp = (char *) malloc (20)) ?
			gtArea[i].tHost[j].pstrIp = "Unkown" : 
			(char *) sprintf (gtArea[i].tHost[j].pstrIp, "192.168.58.1%ld%ld\0",j+2, i+1);
		}
	}
	glNumArea	= 6;
}





/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT ProbeAndConfigNic ()
{
	char  		*pstrArg[10];
	BooleanT	bSuccess	= FALSE;
	char			*pstrIfIp	= NULL;
	long			lBufLen		= 20;
	char  		**ppstr   = pstrArg;
	long			lArgCount	= sizeof (pstrArg) / sizeof (pstrArg[0]);
	char  		*pstrBuf  = (char *) malloc (lBufLen * lArgCount);
	long			lNumIf		= GetNicInterfaces ();

	if (NULL == pstrBuf)
	{
		DisplayMsg ("ERROR: Failed to allocate memory - 2");
	}
	else if (0 == lNumIf)
	{
		DisplayMsg ("ERROR: No NIC availables");
	}
	else
	{
		char			*pstrProbe[3];
		char			*pstrBcast	= NULL;
		long			i, j;
		
		pstrProbe[0] = (char *) malloc (lBufLen);
		pstrProbe[1] = (char *) malloc (lBufLen);
		pstrProbe[2] = NULL;

		for (i=0; i<lArgCount-1; i++)
		{ 
			pstrArg[i] = pstrBuf + lBufLen * i;
		}
		memset (pstrBuf, '\0', lBufLen * lArgCount);
					
		pstrIfIp = gtArea[glSelArea].tHost[glSelHost].pstrIp;

		if (NULL == (pstrBcast = GetBroadcastAddr (pstrIfIp, "255.255.255.0")))
		{
			pstrBcast = "255.255.255.255";
		}
		strncpy (pstrProbe[0], "zProbe", lBufLen);
		strncpy (pstrProbe[1], STR_SERVER_IP, lBufLen);

		strncpy (*ppstr++, "zNetCfg", lBufLen);
		strncpy (*ppstr++, "eth ", lBufLen);
		strncpy (*ppstr++, pstrIfIp, lBufLen);
		strncpy (*ppstr++, "netmask", lBufLen);
		strncpy (*ppstr++, "255.255.255.0", lBufLen);
		strncpy (*ppstr++, "broadcast", lBufLen);
		strncpy (*ppstr++, pstrBcast, lBufLen);
		strncpy (*ppstr++, "down", lBufLen);
		*ppstr	=	NULL;
		ppstr		= pstrArg;

		for (i=0; i<lNumIf && FALSE == bSuccess; i++)
		{
			strncpy (*ppstr+20*7, "down", lBufLen);

			for (j=0; j<lNumIf; j++)
			{
				*(*ppstr+23) = 0x30 + j;
				zNetCfg (8, pstrArg);
			}
			strncpy (*ppstr+20*7, "up", lBufLen);
			*(*ppstr+23) = 0x30 + i;
			zNetCfg (8, pstrArg);
			
			if (TRUE == zProbe (2, pstrProbe))
			{
				bSuccess = TRUE;
			}
		}
	}
	return bSuccess;
}




/*
** =======================================================================================
** =======================================================================================
*/
int main (int argc, char *argv[]) 
{
	BooleanT	bExit	= FALSE;
	WINDOW		*pWin	= NULL;
	long			i 		= 0;
	char			*pstrArea	[MAX_NUM_AREA];
	char			*pstrHost	[MAX_NUM_HOST];
	char			strMsg [64];

	LoadStationData ();
	
	pWin = WinInit ();

	for (i=0; i<glNumArea; i++)
	{
		pstrArea[i] = gtArea[i].pstrName;
	}
	for (i=0; i<MAX_NUM_HOST; i++)
	{
		pstrHost[i] = (char *) malloc (32);
	}

	while (TRUE != bExit)
	{
		CreateMenu (glNumArea, pstrArea, TRUE);
		{
			long 	lHostCount 	= gtArea[glSelArea].lHostCount;
			HostT	*ptHost			= gtArea[glSelArea].tHost; 

			for (i=0; i<lHostCount; i++)
			{
				memset	(pstrHost[i], 0, 32);
				sprintf	(pstrHost[i],"%-8s %s\0", ptHost->pstrName, ptHost->pstrIp);
				ptHost++;
			}
			CreateMenu (lHostCount, pstrHost, FALSE);
		}
		sprintf(strMsg,"%s: %s=%s. Is this Correct?",
		gtArea[glSelArea].pstrName,
		gtArea[glSelArea].tHost[glSelHost].pstrName,
		gtArea[glSelArea].tHost[glSelHost].pstrIp);

		if (DrYes == 	DisplayRequest (
									STR_YES,
									STR_NO,
									strMsg,
									DrYes))
		{
			WriteInfoToMBR (gtArea[glSelArea].tHost[glSelHost].pstrIp);
			bExit = TRUE;
		}
	}
	bExit = FALSE;

	while (TRUE != bExit)
	{
		DisplayInfo ("Contacting remote Server. Please wait...");

		if (TRUE == ProbeAndConfigNic ())
		{
			DisplayInfo ("Contacting remote Server Successfully");

			sleep (5);

			bExit = TRUE;
		}
		else
		{
			if (DrNo ==	DisplayRequest (
									STR_YES,
									STR_NO,
									"Failed to contact server 192.168.58.6. Do you want to retry",
									DrYes))
			{
				bExit = TRUE;
			}
		}
	}
	return 0;
}


