
#include <ncurses.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mount.h>
#include <unistd.h>
#include <dirent.h>
#include <fcntl.h>
#include <net/if.h>
#include <errno.h>
#include <curses.h>

#define		TRUE										1
#define		FALSE										0
#define		STR_BAKBONE							"BakBone Software, Inc."
#define		STR_STATION_TITLE				"Station Selections"
#define		STR_HOST_TITLE					"Host Selections"
#define		STR_OK									"[  OK  ]"
#define		STR_YES									"[ YES ]"
#define		STR_NO									"[ NO ]"
#define		STR_SERVER_IP						"192.168.58.6"
#define		STR_AREA_FILE_NAME			"/floppy/station.txt"
#define		MAX_NUM_HOST						10


typedef		long					BooleanT;
typedef		unsigned char	ByteT;



typedef struct
{
	char	*pstrName;
	char	*pstrIp;
} HostT;

static struct
{
	char	*pstrName;
	long	lHostCount;
	HostT	tHost[MAX_NUM_HOST];
}	gtArea[] = 
{
	{ NULL, 0, NULL }, { NULL, 0, NULL }, { NULL, 0, NULL }, { NULL, 0, NULL }, { NULL, 0, NULL }, 
	{ NULL, 0, NULL }, { NULL, 0, NULL }, { NULL, 0, NULL }, { NULL, 0, NULL }, { NULL, 0, NULL }, 
};

#define	MAX_NUM_AREA	(sizeof (gtArea) / sizeof (gtArea[0]))



typedef enum
{
	DrYes	= 0,
	DrNo	= 1
} DrRequestE;


/*
** =======================================================================================
** 0 = Grey
** 1 = Grey
** 2 = Red
** 3 = Grey
** 4 = Blue
** 5 = Blue
** 6 = Black
** =======================================================================================
*/
static	attr_t	gtAttribute [6];
static	long		glSelArea		=	0;
static	long		glSelHost		=	0;
static	long		glNumArea		= 0;




/*
** =======================================================================================
** =======================================================================================
*/
WINDOW *WinInit ()
{
  WINDOW 	*pWin = NULL;
  int 		lRC 	= 0;
  int 		i 		= 0;

  pWin = initscr ();

  if (NULL != pWin)
	{
		lRC = has_colors ();

		if (0 != lRC)
		{
			start_color ();

			init_pair (1, COLOR_WHITE, 	COLOR_WHITE);
			init_pair (2, COLOR_BLACK, 	COLOR_WHITE);
			init_pair (3, COLOR_WHITE, 	COLOR_RED);
			init_pair (4, COLOR_BLUE, 	COLOR_WHITE);
			init_pair (5, COLOR_YELLOW, COLOR_BLUE);
			init_pair (6, COLOR_WHITE, 	COLOR_BLUE);

			for (i = 0; i < 6; i++)
			{
				gtAttribute[i] = COLOR_PAIR (i + 1) | ((!(i & 1)) ? A_BOLD : 0);
			}
		}
		else
		{
			gtAttribute[0] = gtAttribute[1] = gtAttribute[3] =
			gtAttribute[4] = gtAttribute[5] = A_NORMAL;
			gtAttribute[2] = A_REVERSE;
		}
	}
  return pWin;
}




/*
** =======================================================================================
** =======================================================================================
*/

static void DisplayInfo (char *pstrMsg)
{
  WINDOW    *pWin       = NULL;
  int       lX          = strlen (pstrMsg) + 10;
  int       lY          = 5;
  int       lMaxX       = 0;
  int       lMaxY       = 0;
  int       RC          = 0;
  int       i           = 0;

  clear     ();
  refresh   ();
  getmaxyx  (stdscr, lMaxY, lMaxX);

  pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);

  wborder (pWin,
          ACS_VLINE     | gtAttribute[0],
          ACS_VLINE     | gtAttribute[1],
          ACS_HLINE     | gtAttribute[0],
          ACS_HLINE     | gtAttribute[1],
          ACS_ULCORNER  | gtAttribute[0],
          ACS_URCORNER  | gtAttribute[0],
          ACS_LLCORNER  | gtAttribute[1],
          ACS_LRCORNER  | gtAttribute[1]);

  wattrset (pWin, gtAttribute[1]);

  for (i=1; i<lY-1; i++)
  {
    wmove   (pWin, i, 1);
    whline  (pWin, ' ', lX-2);
  }
  wattrset  (pWin, gtAttribute[3]);
  mvwaddstr (pWin, 2, (lX-strlen (pstrMsg)) / 2, pstrMsg);

  wrefresh (pWin);
  redrawwin (pWin);
}




/*
** =======================================================================================
** =======================================================================================
*/
static char *GetBroadcastAddr (char *pstrIp, char *pstrMask)
{
	BooleanT	bSuccess		= FALSE;
	char			*pstrBcast	= (char *) malloc (32);
	long			lIpa 				= 0;
	long			lIpb 				= 0;
	long			lIpc 				= 0;
	long			lIpd 				= 0;

	if (NULL == pstrBcast)
	{
		fprintf(stderr,"ERROR: Failed to allocate memory - 1\n");
	}
	else if (4 != sscanf (pstrIp, "%ld.%ld.%ld.%ld", &lIpa, &lIpb, &lIpc, &lIpd))
	{
		fprintf(stderr,"ERROR: Failed to sscanf to convert IP to Bcast\n");
	}
	else
	{
		long	lIp = (lIpa << 24) | (lIpb << 16) | (lIpc << 8) | (lIpd);

		if (4 != sscanf (pstrMask, "%ld.%ld.%ld.%ld", &lIpa, &lIpb, &lIpc, &lIpd))
		{
			fprintf(stderr,"ERROR: Failed to sscanf to convert NetMask to Bcast\n");
		}
		else
		{
			long 	lMask 	= (lIpa << 24) | (lIpb << 16) | (lIpc << 8) | (lIpd);
			long	lBcast	= ~lMask | lIp;

			sprintf (pstrBcast,"%ld.%ld.%ld.%ld",
							((lBcast >> 24) & 0xff),
							((lBcast >> 16) & 0xff),
							((lBcast >> 8)  & 0xff),
							(lBcast & 0xff));

			bSuccess = TRUE;
		}
	}
	return (bSuccess ? pstrBcast : NULL);
}



/*
** =======================================================================================
** =======================================================================================
*/
static DrRequestE	DisplayRequest (
									char				*pstrYes,
									char				*pstrNo,
									char 				*pstrMsg, 
									DrRequestE 	eDefaultRequest)
{
	WINDOW		*pWin				= NULL;
	int				lX					= strlen (pstrMsg) + 10;
	int				lY					= 7;
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	int				RC					= 0;
	int				i						= 0;
	int				lKeyPos			= 0;
	BooleanT	bExit				= FALSE;

	clear 		();
	refresh		();
	getmaxyx 	(stdscr, lMaxY, lMaxX);

	pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);
	
	wborder (pWin, 
					ACS_VLINE 		| gtAttribute[0], 
					ACS_VLINE 		| gtAttribute[1], 
					ACS_HLINE 		| gtAttribute[0], 
					ACS_HLINE 		| gtAttribute[1], 
					ACS_ULCORNER 	| gtAttribute[0], 
					ACS_URCORNER 	| gtAttribute[0], 
					ACS_LLCORNER 	| gtAttribute[1], 
					ACS_LRCORNER 	| gtAttribute[1]);

	wattrset (pWin, gtAttribute[1]);

	for (i=1; i<lY-1; i++)
	{
		wmove 	(pWin, i, 1);
		whline 	(pWin, ' ', lX-2);
	}
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 2, (lX-strlen (pstrMsg)) / 2, pstrMsg);

	if (DrYes == eDefaultRequest)
	{
		lKeyPos	= 0;
		wattrset 	(pWin, gtAttribute[2]);
		mvwaddstr (pWin, 4, (lX/4) - (strlen(pstrYes)/2), pstrYes);
		wattrset 	(pWin, gtAttribute[3]);
		mvwaddstr (pWin, 4, (3*lX)/4 - (strlen(pstrNo)/2), pstrNo);
	}
	else
	{
		lKeyPos	= 1;
		wattrset 	(pWin, gtAttribute[3]);
		mvwaddstr (pWin, 4, (lX/4) - (strlen(pstrYes)/2), pstrYes);
		wattrset 	(pWin, gtAttribute[2]);
		mvwaddstr (pWin, 4, (3*lX)/4 - (strlen(pstrNo)/2), pstrNo);
	}
	
	noecho 		();
	keypad 		(pWin, 1);
	cbreak 		();
		
	do
	{
		wrefresh 		(pWin);
		redrawwin 	(pWin);
		RC = wgetch	(pWin);

		switch (RC)
		{
			case '\t':
			case KEY_RIGHT:
			case KEY_LEFT:
			{
				if (0 == lKeyPos)
				{
					lKeyPos = 1;
					wattrset 	(pWin, gtAttribute[3]);
					mvwaddstr (pWin, 4, (lX/4) - (strlen(pstrYes)/2), pstrYes);
					
					wattrset 	(pWin, gtAttribute[2]);
					mvwaddstr (pWin, 4, (3*lX)/4 - (strlen(pstrNo)/2), pstrNo);
				}
				else if (1 == lKeyPos)
				{
					lKeyPos = 0;
					wattrset 	(pWin, gtAttribute[2]);
					mvwaddstr (pWin, 4, (lX/4) - (strlen(pstrYes)/2), pstrYes);
					wattrset 	(pWin, gtAttribute[3]);
					mvwaddstr (pWin, 4, (3*lX)/4 - (strlen(pstrNo)/2), pstrNo);
				}
				break;
			}
			
			case '\n':
			{
				bExit = TRUE;
				break;
			}
		}
	} while (TRUE != bExit);
	
	clear 	();
	refresh	();
	delwin 	(pWin);

	return lKeyPos;
}




/*
** =======================================================================================
** =======================================================================================
*/
static void DisplayMsg (char *pstrMsg)
{
	WINDOW		*pWin				= NULL;
	int				lX					= strlen (pstrMsg) + 10;
	int				lY					= 7;
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	int				RC					= 0;
	int				i						= 0;

	clear 		();
	refresh		();
	getmaxyx 	(stdscr, lMaxY, lMaxX);

	pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);
	
	wborder (pWin, 
					ACS_VLINE 		| gtAttribute[0], 
					ACS_VLINE 		| gtAttribute[1], 
					ACS_HLINE 		| gtAttribute[0], 
					ACS_HLINE 		| gtAttribute[1], 
					ACS_ULCORNER 	| gtAttribute[0], 
					ACS_URCORNER 	| gtAttribute[0], 
					ACS_LLCORNER 	| gtAttribute[1], 
					ACS_LRCORNER 	| gtAttribute[1]);

	wattrset (pWin, gtAttribute[1]);

	for (i=1; i<lY-1; i++)
	{
		wmove 	(pWin, i, 1);
		whline 	(pWin, ' ', lX-2);
	}
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 2, (lX-strlen (pstrMsg)) / 2, pstrMsg);
	wattrset 	(pWin, gtAttribute[2]);
	mvwaddstr (pWin, 4, (lX-strlen (STR_OK)) /2, STR_OK);
		
	do
	{
		RC = wgetch	(pWin);
	} while (RC != '\n');
	
	clear 	();
	refresh	();
	delwin 	(pWin);
}


/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT ValidateNetworkNum (char *pstrNum)
{
	int		bSuccess 	= FALSE;
	long	lClassZ		= 0;
	long	lClassA		= 0;
	long	lClassB		= 0;
	long	lClassC		= 0;

	if (4 == sscanf (pstrNum, "%ld.%ld.%ld.%ld", &lClassZ, &lClassA, &lClassB, &lClassC))
	{
		if (0 <= lClassZ && 255 >= lClassZ &&
				0 <= lClassA && 255 >= lClassA &&
				0 <= lClassB && 255 >= lClassB &&
				0 <= lClassC && 255 >= lClassC)
		{
			bSuccess = TRUE;
		}
	}
	return bSuccess;
}



/*
** =======================================================================================
** =======================================================================================
*/
static long GetNicInterfaces ()
{
	long	lInterface	= 0;
	FILE	*pFd				= fopen ("/proc/net/dev", "r");

	if (NULL != pFd)
	{
		char	strBuffer [512] = {0};
		char	*pstr	= NULL;

		while (NULL != fgets (strBuffer, sizeof (strBuffer), pFd))
		{
			pstr = (char *) strtok (strBuffer, " ");

			if (NULL != pstr)
			{
				if (NULL != strstr (pstr, "eth"))
				{
					lInterface++;
				}
			}
		}
		fclose (pFd);
	}
	return lInterface;
}



/*
** =======================================================================================
** =======================================================================================
*/
static void DrawMenu (
						WINDOW 	*pWin, 
						long 		lCount,
						char		*pstrArr[])
{
	int		lInitY	= 4;
	int		lInitX	= 4;
	int		i 			= 1;

	wattrset (pWin, gtAttribute [1]);

	for (i=1; i<=lCount; i++)
	{
		wmove 		(pWin, i+lInitY, lInitX);
		whline 		(pWin, ' ', 50-lInitX*2);
		mvwaddstr (pWin, i+lInitY, lInitX, pstrArr [i-1]);
	}
}




/*
** =======================================================================================
** =======================================================================================
*/
static void DisplayMenu (
						WINDOW 		*pWin, 
						int				lX,
						int				lY,
						long 			lCount,
						char			*pstrArr[],
						BooleanT	bStation)
{
	int				lInitY			= 4;
	int				lInitX			= 4;
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	int				lPos				= 1;
	int				lIdx				= 1;
	int				RC					= 0;
	int				i						= 0;
	int				lKeyPos			= 1;
	BooleanT	bExit				= FALSE;

	getmaxyx 	(pWin, lMaxY, lMaxX);
	DrawMenu	(pWin, lCount, pstrArr);
	noecho 		();
	keypad 		(pWin, 1);
	cbreak 		();
		
	do
	{
		wattrset 		(pWin, gtAttribute [2]);
		wmove 			(pWin, lPos+lInitY, 2);
		whline 			(pWin, ' ', lMaxX - 4);
		mvwaddstr 	(pWin, lPos+lInitY, lInitX, pstrArr [lIdx-1]);
		whline			(pWin, ' ',(lInitX - strlen (pstrArr [lIdx-1])+1) / 2 - 2);
		wrefresh 		(pWin);
		redrawwin 	(pWin);
		RC = wgetch	(pWin);
		wattrset 		(pWin, gtAttribute [1]);
		wmove 			(pWin, lPos+lInitY, 2);
		whline 			(pWin, ' ', lMaxX - 4);
		mvwaddstr 	(pWin, lPos+lInitY, lInitX, pstrArr [lIdx-1]);

		switch (RC)
		{
			case KEY_UP:
			{
				if (1 < lPos)
				{
					lPos--;
				}
				if (1 < lIdx)
				{
					lIdx--;
				}
				break;
			}

			case KEY_DOWN:
			{
				if (lCount > lPos)
				{
					lPos++;
				}
				if (lCount > lIdx)
				{
					lIdx++;
				}
				break;
			}
	
			case '\t':
			case KEY_RIGHT:
			{
				if (1 == lKeyPos)
				{
					lKeyPos++;
					wattrset 	(pWin, gtAttribute[2]);
					mvwaddstr (pWin, lY-2, (lX - strlen (STR_OK)) / 2, STR_OK);
				}
				else
				{
					lKeyPos = 1;
					wattrset 	(pWin, gtAttribute[3]);
					mvwaddstr (pWin, lY-2, (lX - strlen (STR_OK)) / 2, STR_OK);
				}
				break;
			}
			
			case KEY_LEFT:
			{
				if (1 == lKeyPos)
				{
					lKeyPos++;
					wattrset 	(pWin, gtAttribute[2]);
					mvwaddstr (pWin, lY-2, (lX - strlen (STR_OK)) / 2, STR_OK);
				}
				else
				{
					lKeyPos = 1;
					wattrset 	(pWin, gtAttribute[3]);
					mvwaddstr (pWin, lY-2, (lX - strlen (STR_OK)) / 2, STR_OK);
				}
				break;
			}

			case '\n':
			{
				if (1 == lKeyPos || 2 == lKeyPos)
				{
					if (TRUE == bStation)
					{
						glSelArea = lIdx - 1;
					}
					else
					{
						glSelHost = lIdx - 1;
					}
					bExit	= TRUE;
				}
				break;
			}
		}
	} while (TRUE != bExit);
}




/*
** =======================================================================================
** =======================================================================================
*/
static void CreateMenu (
						long 			lCount,
						char			*pstrArr[],
						BooleanT	bStation)
{
	WINDOW		*pWin				= NULL;
	int				lX					= 40;
	int				lY					= 17;
	int				lMaxX				= 0;
	int				lMaxY				= 0;
	long			i						= 0;

	getmaxyx (stdscr, lMaxY, lMaxX);

	pWin = newwin (lY, lX, (lMaxY - lY) / 2, (lMaxX - lX) / 2);

	wborder (pWin, 
					ACS_VLINE 		| gtAttribute[0], 
					ACS_VLINE 		| gtAttribute[1], 
					ACS_HLINE 		| gtAttribute[0], 
					ACS_HLINE 		| gtAttribute[1], 
					ACS_ULCORNER 	| gtAttribute[0], 
					ACS_URCORNER 	| gtAttribute[0], 
					ACS_LLCORNER 	| gtAttribute[1], 
					ACS_LRCORNER 	| gtAttribute[1]);

	wattrset (pWin, gtAttribute[1]);

	for (i=1; i<lY-1; i++)
	{
		wmove 	(pWin, i, 1);
		whline 	(pWin, ' ', lX-2);
	}
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, 1, (lX-strlen (STR_BAKBONE)) / 2, STR_BAKBONE);
	mvwaddstr (pWin, 2, (lX-strlen (bStation ? STR_STATION_TITLE: STR_HOST_TITLE )) / 2, 
	(bStation ? STR_STATION_TITLE : STR_HOST_TITLE));
	wattrset 	(pWin, gtAttribute[1]);
	wmove 		(pWin, 3, 1);
	whline 		(pWin, ACS_HLINE, lX-2);
	wmove 		(pWin, lY-3, 1);
	whline 		(pWin, ACS_HLINE, lX-2);
	wattrset 	(pWin, gtAttribute[3]);
	mvwaddstr (pWin, lY-2, (lX - strlen (STR_OK)) / 2, STR_OK);
	
	DisplayMenu (pWin, lX, lY, lCount, pstrArr, bStation);

	delwin		(pWin);
	redrawwin	(stdscr);
	refresh		();
}



/*
** =======================================================================================
** =======================================================================================
*/
static void WriteInfoToFloppy ()
{
	int	lFd	= -1;

	umount ("/floppy");
	umount ("/dev/fd0");
		
	if (-1 == mount ("/dev/fd0", "/floppy", "msdos", S_WRITE, NULL))
	{
		long 	lError = errno;
		DisplayMsg ("Failed to mount Floppy to write Meta Data");
	}
	else if (-1 == (lFd = open ("/floppy/dhn.txt", O_WRONLY | O_CREAT | O_TRUNC, 0666)))
	{
		long 	lError = errno;
		DisplayMsg ("Failed to open dhn.txt to write Meta Data");
	}
	else
	{
		char 	strBuf[512] = {0};

		sprintf(strBuf,"STATION %ld\nHOST %s\nIP %s\0",
		glSelArea+1, 
		gtArea[glSelArea].tHost[glSelHost].pstrName,
		gtArea[glSelArea].tHost[glSelHost].pstrIp);

		if (strlen (strBuf) != write (lFd, strBuf, strlen (strBuf)))
		{
			DisplayMsg ("Failed to write Meta Data to dhn.txt");
		}
	}
	close	 (lFd);
	umount ("/floppy");
	umount ("/dev/fd0");
}





/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT LoadStationData (long lFileSize)
{
	BooleanT	bSuccess 	= FALSE;
	int				lFd				= open (STR_AREA_FILE_NAME, O_RDONLY);
	ByteT			*pstrBuf	= (ByteT *) malloc (lFileSize);
	long			lRead			= 0;

	if (NULL == pstrBuf)
	{
		fprintf(stderr,"ERROR: Memmory not available\n");
	}
	else if (-1 == lFd)
	{
		fprintf(stderr,"ERROR: Failed to open station.txt file. %s\n", strerror(errno));
	}
	else if (-1 == (lRead = read (lFd, pstrBuf, 4096)) || lRead == 0)
	{
		fprintf(stderr,"ERROR: Failed to read file station.txt. %s\n", strerror(errno));
	}
	else
	{
		ByteT 		*pstrInit	= pstrBuf;
		ByteT 		*pstr 		= pstrBuf;
		long			lStrip		= 0;
		long			lArea			= -1;
		char			*pstrVal	= 0;
		BooleanT	bBof			= TRUE;

		while (0 != *pstr)
		{
			if (0 == lStrip && *pstr != 0x0d && *pstr != 0x20 && *pstr != 0x09)
			{
				if (*pstr == 0x0a && TRUE == bBof) {}
				else
				{
					*pstrBuf++ = *pstr;
					bBof = FALSE;
				}
			}
			else if (1 == lStrip)
			{
				while (*pstr == 0x0a && *(pstr+1) == 0x0a)
				{
					*pstr++;
				}
				*pstrBuf++ = *pstr;
			}
			else if (*pstr == 0x5b)
			{
				char	*pstrKey = pstr + 1;

				while (0x5d != *pstr++);
				
				*(pstr-1) = 0;
				lArea++;
				
				gtArea[lArea].lHostCount = -1;
				
				NULL == (gtArea[lArea].pstrName = (char *) malloc (strlen (pstrKey))) ?
				gtArea[lArea].pstrName = "Unkown" :
				strncpy (gtArea[lArea].pstrName, pstrKey, strlen (pstrKey));

				*(pstr-1) = 0x5d;
				pstrVal = pstr + 1;
			}
			else if (*pstr == 0x3d)
			{
				long	lHost = ++gtArea[lArea].lHostCount;
				*pstr = 0;

				NULL == (gtArea[lArea].tHost[lHost].pstrName = (char *) malloc (strlen (pstrVal))) ?
				gtArea[lArea].tHost[lHost].pstrName = "Unkown" :
				strncpy (gtArea[lArea].tHost[lHost].pstrName, pstrVal, strlen (pstrVal));

				*pstr = 0x3d;
				pstrVal = pstr + 1;
			}
			else if (*pstr == 0x0a)
			{
				long	lHost = gtArea[lArea].lHostCount;
				*pstr = 0;

				NULL == (gtArea[lArea].tHost[lHost].pstrIp = (char *) malloc (strlen (pstrVal))) ?
				gtArea[lArea].tHost[lHost].pstrIp = "Unkown" :
				strncpy (gtArea[lArea].tHost[lHost].pstrIp, pstrVal, strlen (pstrVal));

				*pstr = 0x0a;
				pstrVal = pstr + 1;
			}
			*pstr++;

			if (0 == *pstr && lStrip < 2)
			{
				lStrip++;
				*pstrBuf = 0;
				pstr = pstrBuf = pstrInit;
			}
		}
		bSuccess 	= TRUE;
		glNumArea	= ++lArea;
	}
	close (lFd);

	return bSuccess;
}





/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT ProbeAndConfigNic ()
{
	char  		*pstrArg[10];
	BooleanT	bSuccess	= FALSE;
	char			*pstrIfIp	= NULL;
	long			lBufLen		= 20;
	char  		**ppstr   = pstrArg;
	long			lArgCount	= sizeof (pstrArg) / sizeof (pstrArg[0]);
	char  		*pstrBuf  = (char *) malloc (lBufLen * lArgCount);
	long			lNumIf		= GetNicInterfaces ();

	if (NULL == pstrBuf)
	{
		DisplayMsg ("ERROR: Failed to allocate memory - 2");
	}
	else if (0 == lNumIf)
	{
		DisplayMsg ("ERROR: No NIC availables");
	}
	else
	{
		char			*pstrProbe[3];
		char			*pstrBcast	= NULL;
		long			i, j;
		
		pstrProbe[0] = (char *) malloc (lBufLen);
		pstrProbe[1] = (char *) malloc (lBufLen);
		pstrProbe[2] = NULL;

		for (i=0; i<lArgCount-1; i++)
		{ 
			pstrArg[i] = pstrBuf + lBufLen * i;
		}
		memset (pstrBuf, '\0', lBufLen * lArgCount);
					
		pstrIfIp = gtArea[glSelArea].tHost[glSelHost].pstrIp;

		if (NULL == (pstrBcast = GetBroadcastAddr (pstrIfIp, "255.255.255.0")))
		{
			pstrBcast = "255.255.255.255";
		}
		strncpy (pstrProbe[0], "zProbe", lBufLen);
		strncpy (pstrProbe[1], STR_SERVER_IP, lBufLen);

		strncpy (*ppstr++, "zNetCfg", lBufLen);
		strncpy (*ppstr++, "eth ", lBufLen);
		strncpy (*ppstr++, pstrIfIp, lBufLen);
		strncpy (*ppstr++, "netmask", lBufLen);
		strncpy (*ppstr++, "255.255.255.0", lBufLen);
		strncpy (*ppstr++, "broadcast", lBufLen);
		strncpy (*ppstr++, pstrBcast, lBufLen);
		strncpy (*ppstr++, "down", lBufLen);
		*ppstr	=	NULL;
		ppstr		= pstrArg;

		for (i=0; i<lNumIf && FALSE == bSuccess; i++)
		{
			strncpy (*ppstr+20*7, "down", lBufLen);

			for (j=0; j<lNumIf; j++)
			{
				*(*ppstr+23) = 0x30 + j;
				zNetCfg (8, pstrArg);
			}
			strncpy (*ppstr+20*7, "up", lBufLen);
			*(*ppstr+23) = 0x30 + i;
			zNetCfg (8, pstrArg);
			
			if (TRUE == zProbe (2, pstrProbe))
			{
				bSuccess = TRUE;
			}
		}
	}
	return bSuccess;
}




/*
** =======================================================================================
** =======================================================================================
*/
int main (int argc, char *argv[]) 
{
	BooleanT	bContinue = TRUE;
	struct		stat	tStat;
			
	while (TRUE == bContinue)
	{
		umount ("/floppy");
		umount ("/dev/fd0");
			
		memset (&tStat, 0, sizeof (tStat));

		if (-1 == mount ("/dev/fd0", "/floppy", "msdos", MS_RDONLY, NULL))
		{
			fprintf(stderr,"\n");
			fprintf(stderr,"ERROR: Failed to mount Floppy. %s\n", strerror(errno));
			fprintf(stderr,"ERROR: Make sure floppy disk contains Station Config file.\n\n");
			fprintf(stderr,"Insert a Valid Floppy Disk and Press ENTER...\n");
			getchar();
		}
		else if (-1 == stat (STR_AREA_FILE_NAME, &tStat))
		{
			fprintf(stderr,"\n");
			fprintf(stderr,"ERROR: Floppy disk does not contains station.txt file\n\n");
			fprintf(stderr,"Insert a Valid Floppy Disk and Press ENTER...\n");
			getchar();
		}
		else if (0 == tStat.st_size)
		{
			fprintf(stderr,"\n");
			fprintf(stderr,"ERROR: The station.txt file is empty\n\n");
			fprintf(stderr,"Insert a Valid Floppy Disk and Press ENTER...\n");
			getchar();
		}
		else if (TRUE != LoadStationData (tStat.st_size))
		{
			fprintf(stderr,"\n");
			fprintf(stderr,"ERROR: Failed to load station.txt file\n\n");
			fprintf(stderr,"Insert a Valid Floppy Disk and Press ENTER...\n");
			getchar();
		}
		else
		{
			BooleanT	bExit	= FALSE;
			WINDOW		*pWin	= NULL;
			long			i;
			char			*pstrArea	[glNumArea];
			char			*pstrHost	[MAX_NUM_HOST];
			char			strMsg [64];

			pWin = WinInit ();

			for (i=0; i<glNumArea; i++)
			{
				pstrArea[i] = gtArea[i].pstrName;
			}
			for (i=0; i<MAX_NUM_HOST; i++)
			{
				pstrHost[i] = (char *) malloc (32);
			}
			while (TRUE != bExit)
			{
				CreateMenu (glNumArea, pstrArea, TRUE);
				{
					long 	lHostCount 	= gtArea[glSelArea].lHostCount;
					HostT	*ptHost			= gtArea[glSelArea].tHost; 

					for (i=0; i<=lHostCount; i++)
					{
						memset	(pstrHost[i], 0, 32);
						sprintf	(pstrHost[i],"%-8s %s\0", ptHost->pstrName, ptHost->pstrIp);
						ptHost++;
					}
					CreateMenu (lHostCount+1, pstrHost, FALSE);
				}
				sprintf(strMsg,"%s: %s=%s. Is this Correct?",
				gtArea[glSelArea].pstrName,
				gtArea[glSelArea].tHost[glSelHost].pstrName,
				gtArea[glSelArea].tHost[glSelHost].pstrIp);

				if (DrYes == 	DisplayRequest (
											STR_YES,
											STR_NO,
											strMsg,
											DrYes))
				{
					WriteInfoToFloppy ();
					bExit = TRUE;
				}
			}
			bExit = FALSE;

			while (TRUE != bExit)
			{
				DisplayInfo ("Contacting remote Server. Please wait...");

				if (TRUE == ProbeAndConfigNic())
				{
					DisplayInfo ("Contacting remote Server Successfully");

					sleep (5);

					bExit = TRUE;
				}
				else
				{
					if (DrNo ==	DisplayRequest (
											STR_YES,
											STR_NO,
											"Failed to contact server 192.168.58.6. Do you want to retry",
											DrYes))
					{
						bExit = TRUE;
					}
				}
			}
			bContinue = FALSE;
		}
	}
	umount ("/floppy");
	umount ("/dev/fd0");

	return 0;
}




