#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mount.h>
#include <sys/socket.h>
#include <unistd.h>
#include <dirent.h>
#include <fcntl.h>
#include <net/if.h>
#include <errno.h>
#include <net/if.h>
#include <netinet/in.h>
#include <netinet/ether.h>
#include <asm/types.h>
#include <linux/if_packet.h>
#include <linux/filter.h>
#include <stdarg.h>


#define	TRUE								1
#define	FALSE								0
#define	SYSERR(x)						strerror(x)
#define	STR_BOOLEAN(x)			((x)?"TRUE":"FALSE")

typedef   long                    BooleanT;
typedef   unsigned char						ByteT;
typedef   unsigned long long      int64;



/*
** =======================================================================================
** =======================================================================================
*/
static void zlog (const char *pstrFormat, ...)
{
	va_list		vargs;

	va_start 	(vargs, pstrFormat);
	vfprintf	(stderr, pstrFormat, vargs);
	va_end		(vargs);
	fprintf		(stderr, "\n");
	fflush		(stderr);
}



/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT ConnectToServer (char *pstrIp, long *plSock)
{
	BooleanT	bSuccess	= FALSE;
	long 			lIpA			= 0;
	long 			lIpB			= 0;
	long 			lIpC			= 0;
	long 			lIpD			= 0;

	zlog("Connect to Server with IP (%s)", pstrIp);

	if (4 != sscanf (pstrIp, "%ld.%ld.%ld.%ld", &lIpA, &lIpB, &lIpC, &lIpD))
	{
		zlog("ERROR: Failed to sscanf for Server IP address");
	}
	else
	{
		long 	lIpAddr	= ((lIpA << 24) & 0xff000000) | 
										((lIpB << 16) & 0xff0000) 	|
										((lIpC << 8)  & 0xff00)			|
										((lIpD)  & 0xff);

		zlog("Connect the server with IP: 0x%0lx", lIpAddr);

		if (0 == lIpAddr)
		{
			zlog("ERROR: Server IP address is zero");
		}
		else
		{
			long		lSock	= -1;
			struct	sockaddr_in	tSockAddr;

			memset (&tSockAddr, 0, sizeof (tSockAddr));

			tSockAddr.sin_family			= AF_INET;
			tSockAddr.sin_addr.s_addr	= htonl (lIpAddr);
			tSockAddr.sin_port				= htons (10666);

			if (0 > (lSock = socket (AF_INET, SOCK_STREAM, 0)))
			{
				zlog("ERROR: Failed to create TCP socket for connect to Server");
			}
			else if (0 >	connect (
										lSock,
										(struct sockaddr *) &tSockAddr,
										sizeof (tSockAddr)))
			{
				long lErr = errno;
				zlog("ERROR: Failed to connect. %s", SYSERR(lErr));
			}
			else
			{
				*plSock		= lSock;
				bSuccess 	= TRUE;
			}
		}
	}
	return bSuccess;
}




/*
** =================================================================================
** =================================================================================
*/
static BooleanT XferData (long lSock, char *pstrDev)
{
	BooleanT	bSuccess	= TRUE;
	long 			lFd				= -1;

	zlog("Start transfer data from server to device (%s)", pstrDev);

	if (-1 == (lFd = open (pstrDev, O_WRONLY | O_CREAT)))
	{
		long lErr = errno;
		zlog("ERROR: Failed to open device (%s). %s", pstrDev, SYSERR (lErr));
		bSuccess	= FALSE;
	}
	else
	{
		BooleanT	bContinue				= TRUE;
		long			lBytesRecv			= 0;
		long			lWritten				= 0;
		int64			llTotal					= 0;
		ByteT			strBuf [32768]	= {0};
	
		zlog("Start receive data from server. Please wait.....");

		while (TRUE == bContinue)
		{
			lBytesRecv = recv (lSock, strBuf, sizeof (strBuf), MSG_WAITALL);

			if (-1 == lBytesRecv)
			{
				bContinue = FALSE;
				bSuccess 	= FALSE;
				zlog("ERROR: Failed to receive data from server");
			}
			else if (32768 == lBytesRecv)
			{
				lWritten = write (lFd, strBuf, 32768);
				
				if (-1 == lWritten || 32768 != lWritten)
				{
					zlog("ERROR: Failed to write to device. Bytes written is %ld", lWritten);
					bContinue	= FALSE;
					bSuccess	= FALSE;
				}
				else
				{
					llTotal += (int64) lWritten;
					fprintf(stderr, "%lld\r", llTotal);
				}
			}
			else if (32768 > lBytesRecv && lBytesRecv > 0)
			{
				bContinue	= FALSE;
				lWritten 	= write (lFd, strBuf, lBytesRecv);
				
				if (-1 == lWritten || lBytesRecv != lWritten)
				{
					zlog("ERROR: Failed to write last buffer to device");
					bSuccess	= FALSE;
				}
				else
				{
					llTotal += (int64) lWritten;
					zlog("Total Bytes Recv = %lld", llTotal);
					zlog("EOF: Write last buffer.");
				}
			}
			else if (0 == lBytesRecv)
			{
				zlog("Total Bytes Recv = %lld", llTotal);
				zlog("EOF: Write last buffer.");
				bContinue = FALSE;
			}
		}
		close (lFd);
	}
	return bSuccess;
}



/*
** =================================================================================
** =================================================================================
*/
int main (int argc, char *argv[]) 
{
	if (argc < 3)
	{
		zlog("Usage: xrecv </dev/sda> <10.66.68.30>");
	}
	else
	{
		long	lSock = -1;

		zlog("xrecv Process Started.");
		zlog("      argc = %ld", argc);
		zlog("      arg0 = %s", argv[0]);
		zlog("      arg1 = %s", argv[1]);
		zlog("      arg2 = %s", argv[2]);

		if (0 == strlen (argv[1]))
		{
			zlog("ERROR: Argument 1, Device to write to is invalid");
		}
		else if (0 == strlen (argv[2]))
		{
			zlog("ERROR: Argument 2. IP address is invalid");
		}
		else if (TRUE != ConnectToServer (argv[2], &lSock))
		{
			zlog("ERROR: Unable to contact Server");
		}
		else
		{
			zlog("Connect to Server (%s) at port 10666 successfully", argv[2]);

			if (TRUE == XferData (lSock, argv[1]))
			{
				zlog("Transfer data from Server completed successfully");
			}
			else
			{
				zlog("ERROR: Failed to transfer data from server"); 
			}
		}
	}
	return 0;
}

