#include <ncurses.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mount.h>
#include <sys/socket.h>
#include <unistd.h>
#include <dirent.h>
#include <fcntl.h>
#include <net/if.h>
#include <errno.h>
#include <curses.h>
#include <net/if.h>
#include <netinet/in.h>
#include <netinet/ether.h>
#include <asm/types.h>
#include <linux/if_packet.h>
#include <linux/filter.h>



#define	TRUE								1
#define	FALSE								0
#define	SYSERR(x)						strerror(x)
#define	STR_BOOLEAN(x)			((x)?"TRUE":"FALSE")


typedef   long                    BooleanT;
typedef   unsigned char						ByteT;
typedef   unsigned long long      int64;



/*
** =======================================================================================
** =======================================================================================
*/
static void zlog (const char *pstrFormat, ...)
{
	va_list		vargs;

	va_start 	(vargs, pstrFormat);
	vfprintf	(stderr, pstrFormat, vargs);
	va_end		(vargs);
	fprintf		(stderr, "\n");
	fflush		(stderr);
}





/*
** =======================================================================================
** =======================================================================================
*/
static BooleanT ConnectToServer (char *pstrIp, long *plSock)
{
	BooleanT	bSuccess	= FALSE;
	long 			lIpA			= 0;
	long 			lIpB			= 0;
	long 			lIpC			= 0;
	long 			lIpD			= 0;

	zlog("Connect to Server with IP (%s)", pstrIp);

	if (4 != sscanf (pstrIp, "%ld.%ld.%ld.%ld", &lIpA, &lIpB, &lIpC, &lIpD))
	{
		zlog("ERROR: Failed to sscanf for Server IP address");
	}
	else
	{
		long 	lIpAddr	= ((lIpA << 24) & 0xff000000) | 
										((lIpB << 16) & 0xff0000) 	|
										((lIpC << 8)  & 0xff00)			|
										((lIpD)  & 0xff);

		zlog("Connect the server with IP: 0x%0lx", lIpAddr);

		if (0 == lIpAddr)
		{
			zlog("ERROR: Server IP address is zero");
		}
		else
		{
			long		lSock	= -1;
			struct	sockaddr_in	tSockAddr;

			memset (&tSockAddr, 0, sizeof (tSockAddr));

			tSockAddr.sin_family			= AF_INET;
			tSockAddr.sin_addr.s_addr	= htonl (lIpAddr);
			tSockAddr.sin_port				= htons (10666);

			if (0 > (lSock = socket (AF_INET, SOCK_STREAM, 0)))
			{
				zlog("ERROR: Failed to create TCP socket for connect to Server");
			}
			else if (0 >	connect (
										lSock,
										(struct sockaddr *) &tSockAddr,
										sizeof (tSockAddr)))
			{
				long lErr = errno;
				zlog("ERROR: Failed to connect to Server. %s", SYSERR(lErr));
			}
			else
			{
				*plSock		= lSock;
				bSuccess 	= TRUE;
			}
		}
	}
	return bSuccess;
}




/*
** =================================================================================
** =================================================================================
*/
static BooleanT XferData (long lSock, char *pstrDev, char *pstrBufCount)
{
	BooleanT	bSuccess	= TRUE;
	long 			lFd				= -1;

	zlog("Start transfer data to Server from device (%s) and count (%s)", pstrDev, pstrBufCount);

	if (-1 == (lFd = open (pstrDev, O_RDONLY)))
	{
		long lErr = errno;
		zlog("ERROR: Failed to open device (%s). %s", pstrDev, SYSERR (lErr));
		bSuccess	= FALSE;
	}
	else
	{
		BooleanT	bContinue				= TRUE;
		long			lBufCount				= atol (pstrBufCount);
		long			lRead						= 0;
		long			lSend						= 0;
		int64			llTotal					= 0;
		ByteT			strBuf [32768]	= {0};
		long			i								= 0;

		for (i=0; i<lBufCount && TRUE == bContinue; i++)
		{
			lRead = read (lFd, strBuf, 32768);
			
			if (-1 == lRead)
			{
				bContinue = FALSE;
				bSuccess 	= FALSE;
				zlog("ERROR: Failed to read data from device (%s)", pstrDev);
			}
			else if (32768 == lRead)
			{
				lSend = send (lSock, strBuf, sizeof (strBuf), MSG_WAITALL);

				if (32768 != lSend)
				{
					bContinue = FALSE;
					bSuccess 	= FALSE;
					zlog("ERROR: Failed to send data to Server");
				}
				else
				{
					llTotal += (int64) lRead;
					fprintf(stderr,"%lld\r", llTotal);
				}
			}
			else
			{
				bContinue = FALSE;
				bSuccess 	= FALSE;
				zlog("ERROR: Failed to read. Read short");
			}
		}
		close (lFd);
		close (lSock);
		zlog("Total Bytes Send: %lld", llTotal);
	}
	return bSuccess;
}



/*
** =================================================================================
** =================================================================================
*/
int main (int argc, char *argv[]) 
{
	if (argc < 4)
	{
		zlog("Usage: xsend </dev/sda> <10.66.68.30> <count 32768>");
	}
	else
	{
		long	lSock = -1;

		zlog("xsend Process Started.");
		zlog("      argc = %ld", argc);
		zlog("      arg0 = %s", argv[0]);
		zlog("      arg1 = %s", argv[1]);
		zlog("      arg2 = %s", argv[2]);
		zlog("      arg3 = %s", argv[3]);

		if (0 == strlen (argv[1]))
		{
			zlog("ERROR: Argument 1, Device is invalid");
		}
		else if (0 == strlen (argv[2]))
		{
			zlog("ERROR: Argument 2, IP address is invalid");
		}
		else if (0 == strlen (argv[3]))
		{
			zlog("ERROR: Argument 3, Buffer Count is invalid");
		}
		else if (TRUE != ConnectToServer (argv[2], &lSock))
		{
			zlog("ERROR: Unable to contact Server");
		}
		else
		{
			zlog("Connect to Server (%s) at port 10666 successfully", argv[2]);

			if (TRUE == XferData (lSock, argv[1], argv[3]))
			{
				zlog("Transfer data to Server completed successful");
			}
			else
			{
				zlog("ERROR: Failed to transfer data to Server");
			}
		}
	}
	return 0;
}
