
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <ctype.h>
#include <errno.h>
#include <getopt.h>
#include <sys/types.h>
#include <sys/file.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/mtio.h>
#include <sys/utsname.h>
#include <asm/param.h>
#include <scsi/scsi.h>
#include <linux/hdreg.h>
#include <linux/unistd.h>

#define BLKRRPART						_IO(0x12,95)
#define BLKGETSIZE					_IO(0x12,96)
#define BLKFLSBUF						_IO(0x12,97)
#define BLKSSZGET						_IO(0x12,104)
#define BLK_SECTOR					512
#define BLK_SCSI						32768
#define	TRUE								1
#define	FALSE								0

typedef	long								BooleanT;
typedef	unsigned long long	int64;
typedef	unsigned char				ByteT;


/*
** =================================================================================
** =================================================================================
*/
static BooleanT GetDeviceGeometry (
								char		*pstrDev,
								ulong		*plCylCount,
								ulong		*plHdrCount,
								ulong		*plSecCount,
								int64		*pllDevSize)
{
	BooleanT	bSuccess	= FALSE;
	int				lFd 			= open (pstrDev, O_RDONLY);

	if (-1 == lFd)
	{
		fprintf(stderr,"ERROR: Failed to open device: %s. %s\n", 
		pstrDev, strerror(errno));
	}
	else
	{
		struct	hd_geometry	tGeo;
		ulong		lRC1				= 0;
		ulong		lRC2				= 0;
		ulong		lCylCount		= 0;
		ulong		lHdrCount		= 0;
		ulong		lSecCount		= 0;
		int64		llDevSize		= 0;

		lRC1	=	ioctl (lFd, BLKGETSIZE, &llDevSize);

#ifdef HDIO_REQ
		lRC2	=	ioctl (lFd, HDIO_REQ,	&tGeo);
#else
		lRC2	=	ioctl (lFd, HDIO_GETGEO, &tGeo);
#endif

		if (-1 == lRC2)
		{
			fprintf(stderr,"ERROR: Failed to get device geometry: %s. %s\n",
			pstrDev, strerror(errno));
		}
		else
		{
			lHdrCount = tGeo.heads;
			lSecCount = tGeo.sectors;

			if (0 == lHdrCount * lSecCount)
			{
				lRC2 = -1;
			}
		}
		if (0 != lRC1)
		{
			fprintf(stderr,"ERROR: Failed to get device size: %s\n", pstrDev);
		}
		if (0 == lRC1 && 0 == lRC2)
		{
			lCylCount = llDevSize / (lHdrCount * lSecCount);
		}
		else if (0 == lRC1)
		{
			lHdrCount = lCylCount = 1;
			lSecCount = llDevSize;
		}
		*plCylCount 	= lCylCount;
		*plHdrCount 	= lHdrCount;
		*plSecCount 	= lSecCount;
		*pllDevSize		= (int64) llDevSize * (int64) BLK_SECTOR;
		bSuccess 			= TRUE;
		
		close (lFd);
	}
	return bSuccess;
}




/*
** =================================================================================
** =================================================================================
*/
static void	PrintDevGeo (char *pstrDev)
{
	ulong			lCylCount	= 0;
	ulong			lHdrCount	= 0;
	ulong			lSecCount	= 0;
	int64			llSize		= 0;

	if (TRUE == GetDeviceGeometry (
							pstrDev,
							&lCylCount,
							&lHdrCount,
							&lSecCount,
							&llSize))
	{
		fprintf(stderr,"\n");
		fprintf(stderr,"*** Device: %s ***\n", pstrDev);
		fprintf(stderr,"    lCylCount = %ld\n", lCylCount);
		fprintf(stderr,"    lHdrCount = %ld\n", lHdrCount);
		fprintf(stderr,"    lSecCount = %ld\n", lSecCount);
		fprintf(stderr,"    llDevSize = %lld\n",llSize);
		fprintf(stderr,"\n");
	}
	else
	{
		fprintf(stderr,"ERROR: Failed to get device geometry: %s\n", pstrDev);
	}
}



/*
** =================================================================================
** =================================================================================
*/
static BooleanT	ZeroDevice (char *pstrDev)
{
	BooleanT 	bSuccess 	= FALSE;
	int 			lFd 				= 0;
	
	PrintDevGeo (pstrDev);

	if (-1 == (lFd = open (pstrDev, O_WRONLY)))
	{
		fprintf(stderr,"ERROR: Failed to open device: %s\n", pstrDev);
	}
	else
	{
		ulong			lCylCount	= 0;
		ulong			lHdrCount	= 0;
		ulong			lSecCount	= 0;
		int64			llSize		= 0;
		int64			llTotal		= 0;

		if (TRUE != GetDeviceGeometry (
								pstrDev,
								&lCylCount,
								&lHdrCount,
								&lSecCount,
								&llSize))
		{
			fprintf(stderr,"ERROR: Failed to get device geometry: %s\n",pstrDev);
		}
		else
		{
			ulong 		lPercent 	= 1;
			ulong 		lWrite		= 0;
			ByteT 		strBuf [32768];
			
			fprintf(stderr,"Begin Zero device: %s\n\n", pstrDev);

			memset (strBuf, 0, BLK_SCSI);
			llSize = llSize / 20;

			fprintf(stderr,"Completion: ");
			fflush (stderr);

			while (TRUE)
			{
				lWrite = write (lFd, strBuf, BLK_SCSI);

				if (BLK_SCSI == lWrite)
				{
					llTotal += (int64) lWrite;
					
					if (llTotal >= llSize * (int64) lPercent)
					{
						fprintf(stderr,"%lu ", (lPercent++ * 10) / 2);
						fflush (stderr);
					}
				}
				else
				{
					fprintf(stderr,"100\n");
					break;
				}
			}
			fprintf(stderr,"\nStart Sync the FileSystem...\n");

			if (0 != ioctl (lFd, BLKRRPART, NULL))
			{
				fprintf(stderr,"ERROR: Failed to Sync device: %s\n", pstrDev);
			}
			fprintf(stderr,"\nZero device successfully: %s\n", pstrDev);
		}
		close (lFd);
	}
	return bSuccess;
}



/*
** =================================================================================
** =================================================================================
*/
int main (int argc, char *argv[]) 
{
	if (2 != argc)
	{
		printf("Usage: drzero [Device]\n");
		return -1;
	}
	ZeroDevice (argv[1]);

	return 0;
}


